local Constants = require("Constants")
local DFGraphics = require("DFCommon.Graphics")

local AudioBars = {}
local AudioBarBox = {}

function AudioBarBox:new()
    local Ob = {}
    
    local quadDeck = MOAIGfxQuad2D.new()
    local texture = DFGraphics.loadTexture( "Effects/Textures/AudioBarEdge", true )
    quadDeck:setTexture( texture )
    quadDeck:setRect( 0, 0, 1, -1 )
    
    Ob.mBox = MOAIProp2D.new()
    Ob.mBox.__typename = "AudioBarBox"
    Ob.mBox:setDeck( quadDeck )
    Ob.mBox:setLoc(0, 0)
    Ob.mBox:setColor(1.0, 1.0, 1.0, 1.0)
    Ob.mBox:setScl(1, 1)
    --ADDITIVE 
    --Ob.mBox:setBlendMode(MOAIProp2D.GL_ONE, MOAIProp2D.GL_ONE)
    --MULTIPLY
    --Ob.mBox:setBlendMode(MOAIProp.GL_DST_COLOR, MOAIProp.GL_ONE_MINUS_SRC_ALPHA)
    --ALPHA
    Ob.mBox:setBlendMode(MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA)
    return Ob
end

function AudioBars:Init( rRenderer )
    -- create some props
    
    AudioBars.Bars = {}
    
    AudioBars.mMount = MOAIProp2D.new()
    AudioBars.mMount:setLoc(0, 0)
    AudioBars.rLayer = rRenderer.mGameLayer
    AudioBars.offset = .25
        
    AudioBars.mDeck = QuadBatchDeck.new()
        
    for i=1,Constants.AudioSpectrumNumValues*4 do
        local bar = AudioBarBox:new()
        local barBox = bar.mBox
        table.insert(AudioBars.Bars, bar)
        barBox:setParent(AudioBars.mMount)
    end
    
    self:SetMaxScale(1000.0)
    self:SetScale(1.5)
    self:SetColor(1.0, 0.5, 0.6, 1.0)
    self:Enable(false)
end

function AudioBars:UpdateSize()
    -- we acutally do this every frame, so whatever
end
  
function AudioBars:Enable(bEnable)
    --bEnable = true

    if bEnable ~= AudioBars.mIsEnabled then
        AudioBars.mIsEnabled = bEnable
        
        if bEnable then
            for i,rBar in ipairs(AudioBars.Bars) do
                AudioBars.rLayer:insertProp( rBar.mBox )
            end
            AudioBars.rLayer:insertProp(AudioBars.mMount)        
        else
            for i,rBar in ipairs(AudioBars.Bars) do
                AudioBars.rLayer:removeProp( rBar.mBox )
            end
            AudioBars.rLayer:removeProp(AudioBars.mMount)
        end
    end
end

function AudioBars:SetColor(r, g, b, a)
    AudioBars.mMount:setColor(r, g, b, a)
    for i,rBar in ipairs(AudioBars.Bars) do
        rBar.mBox:setColor(r, g, b, a)
    end
end

function AudioBars:SetMaxScale(yMax)
    AudioBars.mMaxScale = yMax
end

function AudioBars:SetScale(yScale)
    AudioBars.mScale = yScale
end

function AudioBars:Update()
    if AudioBars.mIsEnabled then
        local barWidth = Constants.ValidDrawWidth / Constants.AudioSpectrumNumValues
        local barHeightScale = Constants.ValidDrawHeight * 8.0 * AudioBars.mScale
        
        AudioBars.mMount:setLoc(-Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight)
        
        local numBars = #AudioBars.Bars
        local numValues = Constants.AudioSpectrumNumValues
        local startBar = 1
        local endBar = Constants.AudioSpectrumNumValues
        local earlyLoopOut = 55 -- lua is too slow, don't bother doing anything with higher order spectral values
        
        
        local sub_iOffset = math.floor(numValues * (AudioBars.offset))
        local twoPi = 2.0 * 3.14159
        
        for i=startBar,(endBar - earlyLoopOut) do
            local rBar = AudioBars.Bars[i]
            local sub_i = (i-1) % Constants.AudioSpectrumNumValues + 1
            local barHeight = math.min(Constants.tAudioSpectrum[sub_i] * barHeightScale, AudioBars.mMaxScale)
            local radius = Constants.GameRingRadius
            
            sub_i = sub_i - sub_iOffset
            rBar.mBox:setLoc( Constants.ValidDrawHalfWidth + math.sin(((sub_i-1) / numValues )*twoPi)*radius, -math.cos(((sub_i-1) / numValues )*twoPi)*radius - Constants.ValidDrawHalfHeight )
            rBar.mBox:setRot(  ( (sub_i) / numValues ) * 360 ,0,0)
            rBar.mBox:setScl( 2*(twoPi * radius) / numValues , barHeight)
        end
            
        local startBar = endBar + 1
        local endBar = startBar + Constants.AudioSpectrumNumValues - 1 
        for i=startBar,(endBar - earlyLoopOut) do
            local rBar = AudioBars.Bars[i]
            local sub_i = (i-1) % Constants.AudioSpectrumNumValues + 1
            local barHeight = math.min(Constants.tAudioSpectrum[sub_i] * barHeightScale, AudioBars.mMaxScale)
            local radius = Constants.GameRingRadius
            
            sub_i = -sub_i - sub_iOffset
            rBar.mBox:setLoc( Constants.ValidDrawHalfWidth + math.sin(((sub_i-1) / numValues )*twoPi)*radius, -math.cos(((sub_i-1) / numValues )*twoPi)*radius - Constants.ValidDrawHalfHeight )
            rBar.mBox:setRot(  ( (sub_i) / numValues ) * 360 ,0,0)
            rBar.mBox:setScl( 2*(twoPi * radius) / numValues , barHeight)
        end
       
        local startBar = endBar + 1
        local endBar = startBar + Constants.AudioSpectrumNumValues - 1
        for i=startBar,(endBar - earlyLoopOut) do
            local rBar = AudioBars.Bars[i]
            local sub_i = (i-1) % Constants.AudioSpectrumNumValues + 1
            local barHeight = math.min(Constants.tAudioSpectrum[sub_i] * barHeightScale, AudioBars.mMaxScale)
            local radius = Constants.GameRingRadius
            
            sub_i = -sub_i + sub_iOffset
            rBar.mBox:setLoc( Constants.ValidDrawHalfWidth + math.sin(((sub_i-1) / numValues )*twoPi)*radius, -math.cos(((sub_i-1) / numValues )*twoPi)*radius - Constants.ValidDrawHalfHeight )
            rBar.mBox:setRot(  ( (sub_i) / numValues ) * 360 ,0,0)
            rBar.mBox:setScl( 2*(twoPi * radius) / numValues , barHeight)
        end
        
        local startBar = endBar + 1
        local endBar = startBar + Constants.AudioSpectrumNumValues - 1
        for i=startBar,(endBar - earlyLoopOut) do
            local rBar = AudioBars.Bars[i]
            local sub_i = (i-1) % Constants.AudioSpectrumNumValues + 1
            local barHeight = math.min(Constants.tAudioSpectrum[sub_i] * barHeightScale, AudioBars.mMaxScale)
            local radius = Constants.GameRingRadius
            
            sub_i = sub_i + sub_iOffset
            rBar.mBox:setLoc( Constants.ValidDrawHalfWidth + math.sin(((sub_i-1) / numValues )*twoPi)*radius, -math.cos(((sub_i-1) / numValues )*twoPi)*radius - Constants.ValidDrawHalfHeight )
            rBar.mBox:setRot(  ( (sub_i) / numValues ) * 360 ,0,0)
            rBar.mBox:setScl( 2*(twoPi * radius) / numValues , barHeight)
        end
        
      end
end

return AudioBars
