local Constants = require('Constants')
local DFMath = require('DFCommon.Math')
local RadFont = require('RadFont')
local AudioManager = require('AudioManager')
local BitFactory = {}

-- Bit Object
function BitFactory:NewBit(id)
    local Bit = {}
    
    function Bit:Init(id)
        Bit.mId = id
        Bit.mEnabled = false
        Bit.mType = Constants.BitTypeBad
        Bit.mEntering = false
        Bit.mEnterTimer = 0.0
        Bit.mEnterDuration = 0.0
        Bit.mLifetime = -1.0
        Bit.mLifeDuration = -1.0
        Bit.mHoldTimer = 0.0
        Bit.mHoldDuration = 0.0
        Bit.mHoldActiveFrames = 0
        Bit.mFanFareTimer = 0.0
        Bit.mWidth = Constants.ValidDrawHeight * .03
        Bit.mHalfWidth = Bit.mWidth * .5
        Bit.mOutline = Bit.mHalfWidth * .25
        Bit.mInitialZ = 0.0
        Bit.mPercentStart = {0.0, 0.0}
        Bit.mPercentDest = {0.0, 0.0}
        Bit.mStartPos = {0.0, 0.0, 0.0}
        Bit.mDestination = {0.0, 0.0}
        Bit.mPrimaryRadians = { 0.0, math.pi * 0.5 }
        Bit.mSecondaryRadians = { 0.0, math.pi * 0.5 }     
        Bit.mPropUpdateRad = { math.pi * 0.25, math.pi * 8.0 }
        Bit.mPointPropIndexA = -1
        Bit.mPointPropIndexB = -1
        Bit.mPointCount = 1
        Bit.mPointPropScale = 1.0
    end
    
    function Bit:CreateProp( tRenderer, nBitIndex )
        if not BitFactory.mShapeBatch then
            BitFactory.mShapeBatch = ShapeProp.new()
            BitFactory.mNumShapesInBatch = 60 -- 1 for foreground, background, and 2 point props
            BitFactory.mShapeBatch:SetNumShapes(BitFactory.mNumShapesInBatch)
            for i=1,BitFactory.mNumShapesInBatch do
                BitFactory.mShapeBatch:SetBatchIndex(i)
                BitFactory.mShapeBatch:SetVisible(false)
            end
            BitFactory.mShapeBatch:setDeck(tRenderer.mShapeDeck)
            tRenderer.mGameLayer:insertProp( BitFactory.mShapeBatch )
        end
    
    
        local fHalfPenWidth = math.max( Constants.BasePenWidth * 0.5, 2.0 )
        
        Bit.mPrimaryPropIndex = (nBitIndex * 4) - 3
        Bit.mSecondaryPropIndex = (nBitIndex * 4) - 2
        Bit.mPointPropIndexA = (nBitIndex * 4) - 1
        Bit.mPointPropIndexB = (nBitIndex * 4)
        
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex )
        BitFactory.mShapeBatch:SetNumSides( 32 )
        BitFactory.mShapeBatch:SetRadii( 0.0, Bit.mHalfWidth + fHalfPenWidth )  
        BitFactory.mShapeBatch:SetVisible( false )   
        
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex )
        BitFactory.mShapeBatch:SetNumSides( 32 )
        BitFactory.mShapeBatch:SetRadii( 0.0, Bit.mHalfWidth - fHalfPenWidth )  
        BitFactory.mShapeBatch:SetVisible( false )
  
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexA )
        BitFactory.mShapeBatch:SetNumSides( 16 )
        BitFactory.mShapeBatch:SetRadii( 0.0, Bit.mHalfWidth - fHalfPenWidth )  
        BitFactory.mShapeBatch:SetVisible( false )
        
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexB )
        BitFactory.mShapeBatch:SetNumSides( 16 )
        BitFactory.mShapeBatch:SetRadii( 0.0, Bit.mHalfWidth - fHalfPenWidth )  
        BitFactory.mShapeBatch:SetVisible( false )
    end
        
    function Bit:UpdateSize()        
        if Bit.mType < Constants.BitTypePoint then
            Bit.mWidth = Constants.ValidDrawHeight * .03
            Bit.mHalfWidth = Bit.mWidth * .5         
        elseif Bit.mType == Constants.BitTypePoint then
            Bit.mWidth = (Constants.GameRingRadius * 0.35) * Bit.mPointCount
            Bit.mHalfWidth = Bit.mWidth * 0.5  
        end      
        
        --this is the same as a quarter the standard half width
        Bit.mOutline = Constants.ValidDrawHeight * .00375
        
        Bit.mPointPropScale = Bit.mWidth / 128.0
                    
        BitFactory.mShapeBatch:SetBatchIndex(Bit.mPrimaryPropIndex)
        BitFactory.mShapeBatch:SetRadii( Bit.mHalfWidth, Bit.mHalfWidth + Bit.mOutline )     
        BitFactory.mShapeBatch:SetBatchIndex(Bit.mSecondaryPropIndex)
        BitFactory.mShapeBatch:SetRadii( Bit.mHalfWidth, Bit.mHalfWidth + Bit.mOutline )        
        
        local fXScale = Constants.ValidDrawHeight * 1.5
        Bit.mStartPos[1] = Bit.mPercentStart[1] * fXScale * Constants.PatternPositionScalar
        Bit.mStartPos[2] = Bit.mPercentStart[2] * Constants.ValidDrawHeight * Constants.PatternPositionScalar
        Bit.mDestination[1] = Bit.mPercentDest[1] * fXScale * Constants.PatternPositionScalar
        Bit.mDestination[2] = Bit.mPercentDest[2] * Constants.ValidDrawHeight * Constants.PatternPositionScalar     
       
        Bit:SetPropPositions()
    end
    
    function Bit:EnableForMenu(iType, fStartX, fStartY, fDestX, fDestY, fLifetime, fEnterTimer)
        Bit:Enable(iType, fStartX, fStartY, fDestX, fDestY, fLifetime, fEnterTimer, fEnterTimer)
        
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex )
        BitFactory.mShapeBatch:SetVisible( false )
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex )
        BitFactory.mShapeBatch:SetArcRadians( 0.0, 0.0 ) 
    end
    
    function Bit:EnableForPoint(fStartX, fStartY, fDestX, fDestY, fLifetime, fEnterTimer, fEnterDuration, iCount)             
        Bit.mPointCount = iCount
        Bit:Enable(Constants.BitTypePoint, fStartX, fStartY, fDestX, fDestY, fLifetime, fEnterTimer, fEnterDuration)
        
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexA )
        BitFactory.mShapeBatch:SetVisible( false )
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexB )
        BitFactory.mShapeBatch:SetVisible( false )        
    end
    
    function Bit:Enable(iType, fStartX, fStartY, fDestX, fDestY, fLifetime, fEnterTimer, fEnterDuration)
        Bit.mEnabled = true
        Bit.mEntering = true
        Bit.mEnterDuration = fEnterDuration
        Bit.mEnterTimer = fEnterTimer
        Bit.mType = iType
        Bit.mLifetime = fLifetime
        Bit.mLifeDuration = fLifetime
        Bit.mHoldTimer = 0.0
        Bit.mHoldDuration = 0.0
        Bit.mHoldActiveFrames = 0
                
        Bit.mInitialZ = -5.0        
        Bit.mPercentStart[1] = fStartX
        Bit.mPercentStart[2] = fStartY
        Bit.mPercentDest[1] = fDestX
        Bit.mPercentDest[2] = fDestY

        Bit.mStartPos[3] = Bit.mInitialZ
                
        Bit:UpdateSize()
        
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex )
        BitFactory.mShapeBatch:SetRadii( Bit.mHalfWidth, Bit.mHalfWidth + Bit.mOutline )     
        BitFactory.mShapeBatch:SetColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
        
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex )
        BitFactory.mShapeBatch:SetColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )
        BitFactory.mShapeBatch:SetDrawMode( ShapeProp.DrawModeArc )   
        BitFactory.mShapeBatch:SetArcRadians( -DFMath.halfPi, -DFMath.halfPi ) 
        BitFactory.mShapeBatch:SetRadii( Bit.mHalfWidth, Bit.mHalfWidth + Bit.mOutline )
        BitFactory.mShapeBatch:SetVisible( true ) 

        --type specific init
        if Bit.mType == Constants.BitTypeGood then
            --early out of this branch
        elseif Bit.mType == Constants.BitTypeBad then
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex )
            BitFactory.mShapeBatch:SetDrawMode( ShapeProp.DrawModeX )   
            BitFactory.mShapeBatch:SetVisible( true )
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex )
            BitFactory.mShapeBatch:SetRadii( Bit.mWidth, Bit.mWidth + Bit.mOutline )  
        elseif Bit.mType == Constants.BitTypeHold then
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex )
            BitFactory.mShapeBatch:SetDrawMode( ShapeProp.DrawModeCircle )
            BitFactory.mShapeBatch:SetVisible( true )
   
            Bit.mHoldDuration = Bit.mEnterDuration
            Bit.mHoldTimer = Bit.mHoldDuration
            Bit.mWidth = Constants.ValidDrawHeight * .05
            Bit.mHalfWidth = Bit.mWidth * .5 
            Bit.mOutline = Bit.mHalfWidth * .25  
        elseif Bit.mType == Constants.BitTypePoint then
            Bit.mHoldDuration = (Bit.mEnterDuration * 0.5)
            Bit.mHoldTimer = Bit.mHoldDuration       
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex ) 
            BitFactory.mShapeBatch:SetDrawMode( ShapeProp.DrawModeArc )
        end   
    end
  
    function Bit:Disable()        
        Bit.mEnabled = false
        Bit.mEntering = false
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex ) 
        BitFactory.mShapeBatch:SetVisible( false )
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex ) 
        BitFactory.mShapeBatch:SetVisible( false )        
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexA )
        BitFactory.mShapeBatch:SetVisible( false )    
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexB )
        BitFactory.mShapeBatch:SetVisible( false )            
    end
    
    function Bit:GetID()
        return self.mID
    end
    
    function Bit:SetID(sId)
        self.mID = sId
    end
        
    function Bit:GetHoldRatio( iFrames )
        Bit.mHoldActiveFrames = iFrames
        if Bit.mHoldDuration > 0.0 then
            return Bit.mHoldTimer / Bit.mHoldDuration
        end
        return 0.0
    end
    
    function Bit:UpdateMenu(iIndex, iDepth)   
        local bWasActiveThisFrame = Bit.mHoldActiveFrames > 0
        
        Bit:Update()
        
        if iDepth ~= 0 or iIndex ~= 1 then
            Bit.mWidth = Constants.ValidDrawHeight * .025
            Bit.mHalfWidth = Bit.mWidth * .5 
            Bit.mOutline = Bit.mHalfWidth * .25           
        end
        
        --bring the ring in slowly
        if Bit.mEntering then
            local fEnterRatio = (Bit.mEnterTimer / Bit.mEnterDuration) * math.pi
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex ) 
            BitFactory.mShapeBatch:SetArcRadians( -DFMath.halfPi + fEnterRatio, DFMath.halfPi + math.pi - fEnterRatio ) 
        end
        
        --stomp size after menu messes with it
        local fQuarterPenWidth = Constants.BasePenWidth * 1.5 
        local fRingRadius = Bit.mHalfWidth + fQuarterPenWidth + Bit.mOutline
        if bWasActiveThisFrame then
            if iIndex ~= 1 then
                fRingRadius = Bit.mWidth + Bit.mOutline
            else
                fRingRadius = (Bit.mWidth * 0.8) + Bit.mOutline
            end
            
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( 0.0, fRingRadius - Bit.mOutline )    
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( fRingRadius, fRingRadius + Bit.mOutline )            
        else
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( 0.0, Bit.mHalfWidth + fQuarterPenWidth )    
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( fRingRadius, fRingRadius + Bit.mOutline )         
        end        
        
        local fHoldRatio = 1.0 - (Bit.mHoldTimer / Bit.mHoldDuration)
        fHoldRatio = math.min( fHoldRatio * 2.0, 1.0 )
        local r = DFMath.lerp( Constants.Color3[1], 1.0, fHoldRatio )
        local g = DFMath.lerp( Constants.Color3[2], 1.0, fHoldRatio )
        local b = DFMath.lerp( Constants.Color3[3], 1.0, fHoldRatio )        
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex ) 
        BitFactory.mShapeBatch:SetVisible( not Bit.mEntering )
        BitFactory.mShapeBatch:SetColor( r, g, b, 1.0 )
    end
    
    --returns if still alive
    function Bit:Update()
        --if we're entering, tick down that timer
        Bit:UpdateEnteringLogic( Constants.FrameStep )

        --update look of prop according to state                
        if Bit.mType == Constants.BitTypeGood then        
            return Bit:UpdateGoodType()        
        elseif Bit.mType == Constants.BitTypeBad then        
            return Bit:UpdateBadType()
        elseif Bit.mType == Constants.BitTypeHold then        
            return Bit:UpdateHoldType()
        elseif Bit.mType == Constants.BitTypeFreeze then        
            return Bit:UpdateFreezeType()
        elseif Bit.mType == Constants.BitTypeFanFare then        
            return Bit:UpdateFanFareType()
        end
        
        return false
    end
    
    function Bit:UpdateEnteringLogic( fStep )
        if Bit.mEntering then
            Bit.mEnterTimer = Bit.mEnterTimer - fStep
            if Bit.mEnterTimer <= 0.0 then
                --done entering; we're live now
                Bit:GoLive()
            end        
            
            local fEnterRatio = 1.0 - (Bit.mEnterTimer / Bit.mEnterDuration)
            local fEnterRadians = fEnterRatio * DFMath.twoPi
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex ) 
            BitFactory.mShapeBatch:SetArcRadians( -DFMath.halfPi, -DFMath.halfPi + fEnterRadians )                    
        end
    end
    
    function Bit:UpdateHoldLogic()
        if Bit.mHoldActiveFrames > 0 then
            Bit.mHoldTimer = Bit.mHoldTimer - Constants.FrameStep
        else
            Bit.mHoldTimer = math.min( Bit.mHoldTimer + Constants.FrameStep, Bit.mHoldDuration )
        end
        Bit.mHoldActiveFrames = Bit.mHoldActiveFrames - 1   
    end
    
    function Bit:UpdateLifetime()
        Bit.mLifetime = Bit.mLifetime - Constants.FrameStep
        if Bit.mLifetime <= 0.0 then
            Bit:Disable()
        end    
    end
        
    function Bit:UpdateBadType()
        if Bit.mEntering then
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( Bit.mHalfWidth, Bit.mHalfWidth + Bit.mOutline )   
            BitFactory.mShapeBatch:SetColor( Constants.BadEffectColor[1], Constants.BadEffectColor[2], Constants.BadEffectColor[3], 1.0 )
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( Bit.mWidth, Bit.mWidth + Bit.mOutline )
            BitFactory.mShapeBatch:SetColor( Constants.BadEffectColor[1]*0.25, Constants.BadEffectColor[2]*0.25, Constants.BadEffectColor[3]*0.25, 1.0 )              
        else
            local fHalfPenWidth = Constants.BasePenWidth * 0.5
            local random =  DFMath.randomFloat( -4.0, 4.0 )
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( fHalfPenWidth + random, Bit.mHalfWidth + Constants.BasePenWidth + random)  
            BitFactory.mShapeBatch:SetColor( Constants.BadEffectColor[1], Constants.BadEffectColor[2], Constants.BadEffectColor[3], 1.0 )            
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( 0.0, Bit.mHalfWidth + random)    
            BitFactory.mShapeBatch:SetColor( Constants.BadEffectColor[1]*0.25, Constants.BadEffectColor[2]*0.25, Constants.BadEffectColor[3]*0.25, 1.0 ) 
        end
        
        if Bit.mEntering then return true end
        Bit:UpdateLifetime()    
        Bit:SetPropPositions()   
   
        return Bit.mEnabled           
    end
    
    function Bit:UpdateGoodType()
        if Bit.mEntering then    
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex ) 
            BitFactory.mShapeBatch:SetColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )
        else
            local fQuarterPenWidth = Constants.BasePenWidth * 0.25
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( 0.0, Bit.mHalfWidth + fQuarterPenWidth )    
            BitFactory.mShapeBatch:SetColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( 0.0, Bit.mHalfWidth - fQuarterPenWidth )                               
            BitFactory.mShapeBatch:SetColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )            
        end
        
        if Bit.mEntering then return true end
        Bit:UpdateLifetime()    
        Bit:SetPropPositions()   
   
        return Bit.mEnabled          
    end
    
    function Bit:UpdateFreezeType()
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex ) 
        BitFactory.mShapeBatch:SetColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1.0 )
        BitFactory.mShapeBatch:SetRadii( 0.0, Bit.mHalfWidth * 0.75 )    
    end
    
    function Bit:UpdateFanFareType()
        Bit.mFanFareTimer = math.min( Bit.mFanFareTimer + (Constants.FrameStep * 2.0), 1.0 )
        local r = DFMath.lerp( Constants.SpecialColor[1], Constants.Color1[1], Bit.mFanFareTimer )
        local g = DFMath.lerp( Constants.SpecialColor[2], Constants.Color1[2], Bit.mFanFareTimer )
        local b = DFMath.lerp( Constants.SpecialColor[3], Constants.Color1[3], Bit.mFanFareTimer )
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex ) 
        BitFactory.mShapeBatch:SetColor( r, g, b, 1.0 )
        BitFactory.mShapeBatch:SetRadii( Bit.mHalfWidth * 1.5, Bit.mHalfWidth * 3.0 )      
    end
    
    function Bit:UpdateHoldType()       
        if Bit.mEntering then   
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( 0.0, Bit.mHalfWidth - Bit.mOutline )                                    
            BitFactory.mShapeBatch:SetColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( Bit.mHalfWidth, Bit.mHalfWidth + Bit.mOutline )               
            BitFactory.mShapeBatch:SetColor( Constants.SpecialColor[1], Constants.SpecialColor[2], Constants.SpecialColor[3], 1.0 )    
        else
            local fQuarterPenWidth = Constants.BasePenWidth * 0.25
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( 0.0, Bit.mHalfWidth + fQuarterPenWidth )    
            BitFactory.mShapeBatch:SetColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )
                
            local fRingRadius = Bit.mHalfWidth + fQuarterPenWidth + Bit.mOutline
            local fHoldRatio = (1.0 - (Bit.mHoldTimer / Bit.mHoldDuration)) * math.pi            
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex )     
            BitFactory.mShapeBatch:SetRadii( fRingRadius, fRingRadius + Bit.mOutline )             
            BitFactory.mShapeBatch:SetArcRadians( -DFMath.halfPi + fHoldRatio, DFMath.halfPi + math.pi - fHoldRatio )
            BitFactory.mShapeBatch:SetColor( Constants.SpecialColor[1], Constants.SpecialColor[2], Constants.SpecialColor[3], 1.0 )           
        end
        
        if Bit.mEntering then return true end
        Bit:UpdateHoldLogic()
        Bit:UpdateLifetime()    
        Bit:SetPropPositions()   
   
        return Bit.mEnabled          
    end
    
    function Bit:UpdatePointType() 
        --point types are updated independently, but can still be frozen..
        if Bit.mType == Constants.BitTypeFreeze then
            return true
        elseif Bit.mType == Constants.BitTypeFanFare then
            Bit:UpdateFanFareType()
            return true
        end
        
        Bit:UpdateEnteringLogic( Constants.FrameStep )
        
        if Bit.mEntering then    
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( Bit.mWidth, Bit.mWidth + (Bit.mOutline * 2.0) )
            BitFactory.mShapeBatch:SetColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )
        else            
            local fHoldRatio = 1.0 - (Bit.mHoldTimer / Bit.mHoldDuration)
            local fHalfPenWidth = Constants.BasePenWidth * 0.5
            local fPrimaryWidth = Bit.mWidth - (Bit.mOutline * 2.0)
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( fPrimaryWidth * fHoldRatio, fPrimaryWidth )    
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex ) 
            BitFactory.mShapeBatch:SetRadii( Bit.mWidth, Bit.mWidth + (Bit.mOutline * 2.0) + fHalfPenWidth )  
            
            if Bit.mHoldActiveFrames > 0 then
                BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex ) 
                BitFactory.mShapeBatch:SetColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )             
                BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex ) 
                BitFactory.mShapeBatch:SetColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )            
            else
                BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex ) 
                BitFactory.mShapeBatch:SetColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )             
                BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex ) 
                BitFactory.mShapeBatch:SetColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )            
            end
                
            local fPointWidth = Bit.mHalfWidth * 0.5
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexA )
            BitFactory.mShapeBatch:SetRadii( 0.0, fPointWidth )  
            BitFactory.mShapeBatch:SetColor( Constants.BGColor[1], Constants.BGColor[2], Constants.BGColor[3], 1.0 )
            
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexB )
            BitFactory.mShapeBatch:SetRadii( 0.0, fPointWidth )  
            BitFactory.mShapeBatch:SetColor( Constants.BGColor[1], Constants.BGColor[2], Constants.BGColor[3], 1.0 )            
        end       
        
        if Bit.mEntering then return true end
        
        local fPreviousHold = Bit.mHoldTimer
        Bit:UpdateHoldLogic()
        if fPreviousHold == Bit.mHoldDuration and Bit.mHoldTimer < Bit.mHoldDuration then
            AudioManager:PlaySound( 'Radius/SFX/CircleActivate' )
        end
        
        Bit:UpdateLifetime()  
        Bit:SetPropPositions()    

        return Bit.mEnabled
    end
        
    function Bit:UpdateArcRadians()
        local fSpeed = DFMath.lerp( Bit.mPropUpdateRad[1], Bit.mPropUpdateRad[2], 0.5 )
        local fAngleStep = fSpeed * Constants.FrameStep
    
        Bit.mPrimaryRadians[1] = Bit.mPrimaryRadians[1] + fAngleStep
        Bit.mPrimaryRadians[2] = Bit.mPrimaryRadians[2] + fAngleStep
        if Bit.mPrimaryRadians[2] > DFMath.twoPi then 
            Bit.mPrimaryRadians[2] = Bit.mPrimaryRadians[2] - DFMath.twoPi
            Bit.mPrimaryRadians[1] = Bit.mPrimaryRadians[1] - DFMath.twoPi
        end

        Bit.mSecondaryRadians[1] = Bit.mSecondaryRadians[1] - fAngleStep
        Bit.mSecondaryRadians[2] = Bit.mSecondaryRadians[2] - fAngleStep
        if Bit.mSecondaryRadians[1] < 0.0 then 
            Bit.mSecondaryRadians[2] = Bit.mSecondaryRadians[2] + DFMath.twoPi
            Bit.mSecondaryRadians[1] = Bit.mSecondaryRadians[1] + DFMath.twoPi
        end
    end
    
    function Bit:SetPropPositions()
        local fCompleteRatio = 1.0 - math.max( Bit.mLifetime / Bit.mLifeDuration, 0.0 )
        local x = DFMath.lerp( Bit.mStartPos[1], Bit.mDestination[1], fCompleteRatio )
        local y = DFMath.lerp( Bit.mStartPos[2], Bit.mDestination[2], fCompleteRatio )
                
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex )           
        BitFactory.mShapeBatch:SetPosition( x, y, Bit.mStartPos[3] )
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex ) 
        BitFactory.mShapeBatch:SetPosition( x, y, Bit.mStartPos[3] )    
    
        if Bit.mPointCount == 1 then
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexA )
            BitFactory.mShapeBatch:SetPosition( x, y, 0.0 )          
        else
            local fOffset = Bit.mHalfWidth * 0.65
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexA )
            BitFactory.mShapeBatch:SetPosition( x - fOffset, y, 0.0 )              
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexB )
            BitFactory.mShapeBatch:SetPosition( x + fOffset, y, 0.0 )             
        end
    end
    
    function Bit:GetCurrentPos()
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex )  
        return BitFactory.mShapeBatch:GetPosition()
    end
    
    function Bit:GetEnteringDest()
        if Bit.mEnterTimer < 0.0 then
            return Bit.mDestination[1], Bit.mDestination[2]
        end

        local nToDest = {}
        nToDest[1] = Bit.mDestination[1] - Bit.mStartPos[1]
        nToDest[2] = Bit.mDestination[2] - Bit.mStartPos[2]            
        
        local fEnteringRatio = 1.0 - (Bit.mEnterTimer / Bit.mEnterDuration)
        nToDest[1] = nToDest[1] * fEnteringRatio
        nToDest[2] = nToDest[2] * fEnteringRatio
        nToDest[1] = nToDest[1] + Bit.mStartPos[1]
        nToDest[2] = nToDest[2] + Bit.mStartPos[2]
        
        return nToDest[1], nToDest[2]
    end
    
    function Bit:GoLive()
        Bit.mEntering = false
        Bit.mEnterTimer = 0.0    
        
        if Bit.mType == Constants.BitTypePoint then
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex )  
            BitFactory.mShapeBatch:SetDrawMode( ShapeProp.DrawModeCircle )
            BitFactory.mShapeBatch:SetVisible( true )      
            BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex )    
            BitFactory.mShapeBatch:SetArcRadians( -DFMath.halfPi, DFMath.halfPi )     
            BitFactory.mShapeBatch:SetVisible( true )      
    
            if Bit.mPointCount == 1 then
                BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexA )
                BitFactory.mShapeBatch:SetVisible( true )         
            else
                BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexA )
                BitFactory.mShapeBatch:SetVisible( true )                
                BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexB )
                BitFactory.mShapeBatch:SetVisible( true )                 
            end    
        else
            Bit.mStartPos[3] = 0.0
            
            if Bit.mType == Constants.BitTypeBad then
                BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex )  
                BitFactory.mShapeBatch:SetDrawMode( ShapeProp.DrawModeInvertedQuad )
                BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex )    
                BitFactory.mShapeBatch:SetVisible( true )
                BitFactory.mShapeBatch:SetDrawMode( ShapeProp.DrawModeDoubleTriangle )
            elseif Bit.mType == Constants.BitTypeGood then
                BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex )  
                BitFactory.mShapeBatch:SetVisible( true )
                BitFactory.mShapeBatch:SetDrawMode( ShapeProp.DrawModeCircle )
                BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex )    
                BitFactory.mShapeBatch:SetDrawMode( ShapeProp.DrawModeCircle )
            end                
        end
    end

    function Bit:FreezePoint()
        Bit.mType = Constants.BitTypeFreeze
        Bit.mStartPos[3] = Bit.mInitialZ 
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex )  
        BitFactory.mShapeBatch:SetVisible( false )
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexA )
        BitFactory.mShapeBatch:SetVisible( false )        
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPointPropIndexB )
        BitFactory.mShapeBatch:SetVisible( false )                
        Bit:UpdateSize()
    end
    
    function Bit:Freeze()
        Bit.mType = Constants.BitTypeFreeze
        Bit.mStartPos[3] = Bit.mInitialZ 
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex )  
        BitFactory.mShapeBatch:SetVisible( false )
        Bit:UpdateSize()
    end
  
    function Bit:SetFanfare()
        Bit.mType = Constants.BitTypeFanFare
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mPrimaryPropIndex )  
        BitFactory.mShapeBatch:SetDrawMode( ShapeProp.DrawModeStar )
        BitFactory.mShapeBatch:SetBatchIndex( Bit.mSecondaryPropIndex )  
        BitFactory.mShapeBatch:SetVisible( false )
        Bit.mFanFareTimer = 0.0
        
        Bit.mWidth = Constants.ValidDrawHeight * .03
        Bit.mHalfWidth = Bit.mWidth * .5 
        Bit.mOutline = Bit.mHalfWidth * .25        
    end
    
    function Bit:CanBeDestroyed()
        return Bit.mEnabled and not Bit.mEntering and Bit.mType ~= Constants.BitTypeFreeze and Bit.mType ~= Constants.BitTypeFanFare 
    end
      
    function Bit:IsPreventingQueueAdvancement()
        return Bit.mEnabled and not Bit.mEntering and (Bit.mType == Constants.BitTypeGood or Bit.mType == Constants.BitTypeHold or Bit.mType == Constants.BitTypePoint)
    end
        
    function Bit:CanAdvanceQueue()
        return Bit.mEnabled and Bit.mEntering and Bit.mType ~= Constants.BitTypeFanFare and Bit.mType ~= Constants.BitTypeBad 
    end
    
    Bit:Init(id)
    return Bit
end

-- Bit Object
function BitFactory:NewBitQueueSlot()
    local BitQueueSlot = {}

    function BitQueueSlot:Init()    
        BitQueueSlot.mEnabled = false
        BitQueueSlot.mBitType = Constants.BitTypeBad
        BitQueueSlot.mOrientation = 0
        BitQueueSlot.mBitDelay = 0.0
        BitQueueSlot.mStartX = 0.0
        BitQueueSlot.mStartY = 0.0
        BitQueueSlot.mDestX = 0.0
        BitQueueSlot.mDestY = 0.0
        BitQueueSlot.mLifetime = 0.0
        BitQueueSlot.mEnterTimer = 0.0
        BitQueueSlot.mEnterDuration = 0.0
    end
  
    function BitQueueSlot:Set(iType, fDelay, fStartX, fStartY, fLife, fEnterTimer)
        BitQueueSlot.mEnabled = true
        BitQueueSlot.mBitType = iType
        BitQueueSlot.mBitDelay = fDelay
        BitQueueSlot.mStartX = fStartX
        BitQueueSlot.mStartY = fStartY
        BitQueueSlot.mDestX = fStartX
        BitQueueSlot.mDestY = fStartY
        BitQueueSlot.mLifetime = fLife        
        BitQueueSlot.mEnterTimer = fEnterTimer
        BitQueueSlot.mEnterDuration = BitQueueSlot.mEnterTimer
    end
  
    function BitQueueSlot:SetMoving(iType, fDelay, fStartX, fStartY, fDestX, fDestY, fLife, fEnterTimer)
        BitQueueSlot.mEnabled = true
        BitQueueSlot.mBitType = iType
        BitQueueSlot.mBitDelay = fDelay
        BitQueueSlot.mStartX = fStartX
        BitQueueSlot.mStartY = fStartY
        BitQueueSlot.mDestX = fDestX
        BitQueueSlot.mDestY = fDestY
        BitQueueSlot.mLifetime = fLife 
        BitQueueSlot.mEnterTimer = fEnterTimer
        BitQueueSlot.mEnterDuration = BitQueueSlot.mEnterTimer
    end
  
    function BitQueueSlot:SetSlice(iOrientation, fDelay, fLife)
        BitQueueSlot.mEnabled = true
        BitQueueSlot.mBitType = Constants.BitTypeSlice    
        BitQueueSlot.mBitDelay = fDelay    
        BitQueueSlot.mOrientation = iOrientation
        BitQueueSlot.mLifetime = fLife
        BitQueueSlot.mEnterTimer = 0.0
    end
    
    function BitQueueSlot:CanAdvanceQueue()
        return BitQueueSlot.mEnabled and (BitQueueSlot.mBitType == Constants.BitTypeGood or BitQueueSlot.mBitType == Constants.BitTypeSlice or BitQueueSlot.mBitType == Constants.BitTypeHold)
    end
    
    BitQueueSlot:Init()
    return BitQueueSlot
end

return BitFactory
