local Constants = require('Constants')
local DFMath = require('DFCommon.Math')
local ChargeField = {}

function ChargeField:Init( tRenderer )
    ChargeField.mChargeFieldDeck = ChargeFieldDeck.new()
    
    ChargeField:VisualQualityChange()
    
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 5.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 1.0, 8.0 )
    ChargeField.mChargeFieldDeck:SetActivatedDistance( 20.0 )
    ChargeField.mChargeFieldProp = MOAIProp2D.new()
    ChargeField.mChargeFieldProp.__typename = "ChargeField"
    ChargeField.mChargeFieldProp:setDeck( ChargeField.mChargeFieldDeck )
        
    tRenderer.mBackgroundLayer:insertProp( ChargeField.mChargeFieldProp )
    
    ChargeField:Enable( false )
end
  
function ChargeField:UpdateSize()
    ChargeField.mSpacing = Constants.ValidDrawWidth / ChargeField.mTileSize
    ChargeField.mChargeFieldDeck:UpdateSpacing( ChargeField.mSpacing )    
end
  
function ChargeField:VisualQualityChange()
    ChargeField.mTileSize = 64.0

    if Constants.VisualQuality ~= Constants.VisualQualityLevelUltra then
        ChargeField.mTileSize = 28.0
    end
    
    ChargeField.mSpacing = Constants.NativeScreenWidth / ChargeField.mTileSize
    local iXDimensionCount = Constants.NativeScreenWidth / ChargeField.mSpacing
    local iYDimensionCount = Constants.NativeScreenHeight / ChargeField.mSpacing    
    ChargeField.mChargeFieldDeck:SetDimensions( iXDimensionCount, iYDimensionCount, 1, ChargeField.mSpacing )
    ChargeField.mChargeFieldDeck:SetColor( Constants.Color2[1],Constants.Color2[2], Constants.Color2[3], 1.0 )

    ChargeField:UpdateSize()
end
  
function ChargeField:Enable(bEnable)
    ChargeField.mEnabled = bEnable
    ChargeField.mChargeFieldProp:setVisible(bEnable)
end

function ChargeField:SetColor(r, g, b, a)
    ChargeField.mChargeFieldDeck:SetColor( r, g, b, a )
end

function ChargeField:Update()
    if ChargeField.mEnabled == true then
        --don't update the charge field for mobile
        --ChargeField.mChargeFieldDeck:Update( Constants.FrameStep )
    end
end
  
function ChargeField:AddImpression(fX, fY, fRadius)
    if ChargeField.mEnabled == true then
        ChargeField.mChargeFieldDeck:AddImpression( fX, fY, 0.0, fRadius )
    end
end

return ChargeField