local Constants = require('Constants')
local DFMath = require('DFCommon.Math')
local ChargeField = require('ChargeField')
local Grid = require('Grid')
local AudioManager = require('AudioManager')
local ColorManager = {}

function ColorManager:Init()    
    ColorManager.mInterruptTimer = 0.0  
    ColorManager.mInterruptDuration = 0.1
    ColorManager.mInterruptFlashCount = -1
          
    ColorManager.mStrokeWidthLow = 3.0
    ColorManager.mStrokeWidthHigh = 7.0
        
    ColorManager.BGColor = {0, 0, 0}
    ColorManager.Color1 = {1, 0, 0} --Game Ring, Title, Note Stroke, Losing Health
    ColorManager.Color2 = {0, 1, 0} --Charge Fill, Note Fill, Score, Health bar outline, Health
    ColorManager.Color3 = {0, 0, 1} --Score multiplier, Tagged notes
    ColorManager.SpecialColor = {.95, .95, .95} --Diamonds, other extras   
    
    ColorManager.BadEffectColor = {1,0,0,1}
end

function ColorManager:Update()
    if Constants.GameState ~= Constants.GameStatePost then                 
        --update stroke
        Constants.BasePenWidth = DFMath.lerp( ColorManager.mStrokeWidthLow, ColorManager.mStrokeWidthHigh, Constants.Heartbeat )
    end
        
    --overrides if we're being interrupted
    if ColorManager.mInterruptFlashCount > 0 then
        Constants.BasePenWidth = 4.0
           
        ColorManager.mInterruptTimer = ColorManager.mInterruptTimer - Constants.FrameStep
        if ColorManager.mInterruptTimer <= 0.0 then
            ColorManager.mInterruptFlashCount = ColorManager.mInterruptFlashCount - 1
            ColorManager.mInterruptTimer = ColorManager.mInterruptDuration
            ColorManager:SetColor()
        end
    end  
end
  
function ColorManager:SetColor()    
    local i = 1
    for i = 1, 3 do
        Constants.BGColor[i] = ColorManager.BGColor[i]
        Constants.Color1[i] = ColorManager.Color1[i]
        Constants.Color2[i] = ColorManager.Color2[i]
        Constants.Color3[i] = ColorManager.Color3[i]
        Constants.SpecialColor[i] = ColorManager.SpecialColor[i]      
    end
    for i = 1, 4 do
        Constants.BadEffectColor[i] = ColorManager.BadEffectColor[i]       
    end
end  
  
function ColorManager:IsInterrupted()
    return ColorManager.mInterruptFlashCount > 0
end
  
function ColorManager:Interrupt()
    if ColorManager:IsInterrupted() == false then
        ColorManager.mInterruptTimer = ColorManager.mInterruptDuration
        ColorManager.mInterruptFlashCount = 3
    end
end
     
function ColorManager:SetLineModeIntro()       
    Constants.ColorMode = Constants.ColorModeIntro
    ColorManager.mStrokeWidthLow = 3.0
    ColorManager.mStrokeWidthHigh = 7.0
    Constants.BeatScale = .25    
    ColorManager:SetColor()
end
  
function ColorManager:SetLineModeInGame()    
    Constants.ColorMode = Constants.ColorModeInGame    
    ColorManager.mStrokeWidthLow = 4.0     
    ColorManager.mStrokeWidthHigh = 25.0
    Constants.BeatScale = 2.0 
    ColorManager:SetColor()
end

function ColorManager:SetLineModeComplete(fRatio)    
    Constants.ColorMode = Constants.ColorModeInGame    
    ColorManager.mStrokeWidthLow = DFMath.lerp( 2.0, 1.0, fRatio )     
    ColorManager.mStrokeWidthHigh = DFMath.lerp( 25.0, 1.01, fRatio )
    AudioManager:SetAdvanceGameScalar( DFMath.lerp( 1.0, 0.1, fRatio ) )
end

return ColorManager
