local DebugDisplay = {}

local ParticleSystem = require("ParticleSystem")
local DFGraphics = require("DFCommon.Graphics")
local Post = require("postfx.Post")
local Constants = require("Constants")
local Renderer = require("Renderer")
local DFFile = require("DFCommon.File")
local DFMath = require("DFCommon.Math")

function DebugDisplay:new(rViewport, rCamera)
    local Ob = {}
    
    function Ob:OnScreenResize(width, height)
        local yTextLoc = ( height / 2 ) + 2
        
        -- for some dumb reason post effects show this thing flipped when I add it to that layer, instead of making that make sense
        --  I'm just flipping it for now.
        if Ob.nYScale > 0 then
            Ob.mText:setYFlip ( true )
        else
            yTextLoc = -yTextLoc + 32
        end
        
        Ob.mText:setLoc(-width/2 + 2, yTextLoc)
        
        self.mBox:setLoc(-width/2, self.nYScale * height/2, 1 )
    end
    
    function Ob:Draw()
        local sString = ""

        if Constants.ShowFullProfilingInfo then
            local kOnlyPrintDifferenes = true
            local kFramesBetweenPrints = 300
            
            if not self.mFramesUntilPrintHistogram then self.mFramesUntilPrintHistogram = kFramesBetweenPrints end
            self.mFramesUntilPrintHistogram = self.mFramesUntilPrintHistogram - 1
            if self.mFramesUntilPrintHistogram <= 0 then
                self.mFramesUntilPrintHistogram = kFramesBetweenPrints
                
                if not self.mHistogramOrig then self.mHistogramOrig = {} end
                
                local tHistogram = MOAISim.getHistogram()
                if tHistogram then                
                    print("\n\n\n------------------------------------\nMOAI Runtime Objects: \n")
                    for key,value in pairs(tHistogram) do
                        if not self.mHistogramOrig[key] then
                            self.mHistogramOrig[key] = value
                        end
                        
                        local bSame = self.mHistogramOrig[key] == value
                        
                        if not bSame or not kOnlyPrintDifferenes then
                            print(" * " .. key .. " - orig: " .. tostring(self.mHistogramOrig[key]) .. " cur: " .. tostring(value))
                        end
                    end
                    print("------------------------------------\n\n")
                end
            end       
        end
    
        for idx, tValue in ipairs(self.tValues) do
            if tValue.key == "FPS" then
                local nFPS = DFMath.round(MOAISim.getPerformance(), 2)
                
                local sFPS = tostring(nFPS)
                local sFPSLength = string.len(sFPS)
                for i=sFPSLength,4 do
                    sFPS = sFPS .. "0"
                end
                
                sString = sString .. "FPS: " .. sFPS .. " "
            else
                sString = sString .. tValue.key .. ": " .. tValue.value .. " "
            end
        end
        
        self.mText:setString(sString)
    end

    function Ob:SetValue(sKey, sValue)
        local nIndex = -1
        
        for idx,tValue in ipairs(self.tValues) do
            if tValue.key == sKey then
                nIndex = idx
                break
            end
        end
        
        if -1 == nIndex then
            nIndex = #self.tValues + 1
            
            table.insert(self.tValues, {key=sKey, value=sValue})
        end
        
        self.tValues[nIndex].value = sValue
    end
    
    Ob.nYScale = 1
    
    if Constants.UsePostEffects then
        Ob.mLayer = Post.rScreen.overlayLayer
    else
        Ob.mLayer = MOAILayer2D.new()
        Ob.mLayer:setViewport( rViewport )
        Ob.mLayer:setCamera( rCamera )
        MOAISim.pushRenderPass( Ob.mLayer )
        Ob.nYScale = -1
    end    

    Ob.tValues = {}
    Ob.tValueKeys = {}

    local screenW, screenH = Renderer.GetScreenSize()


    
    local quadDeck = MOAIGfxQuad2D.new()
    local whiteTexture = DFGraphics.loadTexture( "White", true )
    quadDeck:setTexture( whiteTexture )
    quadDeck:setRect( -1, -1, 1, 1 )
    Ob.mBox = MOAIProp2D.new()
    Ob.mBox:setDeck( quadDeck )
    Ob.mBox:setLoc(-screenW/2, Ob.nYScale * screenH/2, 1 )
    Ob.mBox:setColor(0.15, 0.15, 0.15, 0.8)
    Ob.mBox:setScl(360, 16)
    
    Ob.mLayer:insertProp(Ob.mBox)  
            
    local charcodes = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 .,:;!?()&/-'
    
    local kFONT_SIZE = 12
    
    Ob.mFont = DFGraphics.loadFont( "Fonts/DebugUI.font" )
            
    Ob.mFontStyle = MOAITextStyle.new()
    Ob.mFontStyle:setFont( Ob.mFont )
    Ob.mFontStyle:setSize( kFONT_SIZE )
            
	Ob.mText = MOAITextBox.new()
    Ob.mText:setStyle( Ob.mFontStyle )
	Ob.mText:setRect ( 0, -32, 360, 0 )
	Ob.mText:setAlignment( MOAITextBox.LEFT_JUSTIFY )
    
    Ob.mText:setString("")
    
	Ob.mLayer:insertProp( Ob.mText )
    
    Ob:SetValue("FPS", 33.3)
    
    if Constants.ShowFullProfilingInfo then
        Ob.rDebugProfileReportProp = MOAIProfileReportBox.new()
        Ob.rDebugProfileReportProp:setFont(Ob.mFont)
        Ob.rDebugProfileReportProp:setFontSize(14)
        Ob.rDebugProfileReportProp:setRect(0, 50, Constants.ValidDrawWidth, Constants.ValidDrawHeight)   
        Ob.rDebugProfileReportProp:enableProfiling(true)
        Ob.mLayer:insertProp ( Ob.rDebugProfileReportProp )
    elseif Constants.ShowGPUProfilingInfo then
        Ob.rDebugProfileReportProp = MOAIGpuProfileReportBox.new()
        Ob.rDebugProfileReportProp:setFont(Ob.mFont)
        Ob.rDebugProfileReportProp:setFontSize(14)
        Ob.rDebugProfileReportProp:setRect(0, 50, Constants.ValidDrawWidth, Constants.ValidDrawHeight)   
        Ob.rDebugProfileReportProp:enableProfiling(true)
        Ob.mLayer:insertProp ( Ob.rDebugProfileReportProp )        
    end
    
    function Ob:ShowLuaDebugMetrics(sCountMin)
        print("\n\n\n\n\n\n")

        if not sCountMin then
            sCountMin = 2
        end

        local function count_all(f)
            local seen = {}
            local count_table
            count_table = function(t)
                if seen[t] then return end
                f(t)
                seen[t] = true
                for k,v in pairs(t) do
                    if type(v) == "table" then
                        count_table(v)
                    elseif type(v) == "userdata" then
                        f(v)
                    end
                end
            end
            count_table(_G)
        end
        
        local function type_count()
            local counts = {}
            local enumerate = function (o)
                local t = type_name(o)
                counts[t] = (counts[t] or 0) + 1
            end
            count_all(enumerate)
            return counts
        end
               
        local types = type_count()
        
        if not self.startTypes then
            self.startTypes = {}
        end
        
        local sortedTypes = { }
        
        local function gt(a, b) return a.count > b.count end
        
        for k, v in pairs(types) do 
            table.insert(sortedTypes, { type = k, count = v }) 
        end
        table.sort(sortedTypes, gt)
        
        for k, v in ipairs(sortedTypes) do
            if v.count >= sCountMin then
                sDelta = ""

                if not self.startTypes[v.type] then
                    self.startTypes[v.type] = v.count
                end
                
                if self.prevTypes then
                    local prevValue = self.prevTypes[v.type]

                    if prevValue then
                        local delta = v.count - prevValue
                        if delta > 0 then
                            sDelta = "*** ( +" .. tostring(delta) .. " )"
                        elseif delta < 0 then
                            sDelta = "*** ( " .. tostring(delta) .. " )"
                        end
                    end
                end
                    
                print("   - type " .. v.type .. ": " .. v.count .. " " .. sDelta .. " ( initial=" .. self.startTypes[v.type] .. " )")
            end
        end
        
        self.prevTypes = types

        
        print("\n\n\n\n\n\n")
    end
    
    return Ob
end

function type_name(o)           
    local typeName = "Unknown"

    if type(o) then
        typeName = type(o)
    end

    if typeName == "table" or typeName == "userdata" then
        -- HACK: for some reason some of our userdata objects have this as their tostring value,
        --  and if you try and access their __index you get a crash. 
        if tostring(o) == "userdata: 0x800" then
            return "????"
        end

        if o.__typename then
            typeName = o.__typename
        elseif o.getClassName then
            typeName = o:getClassName()
        end
    end
            
	return typeName
end


return DebugDisplay
