local Constants = require('Constants')
local DFMath = require('DFCommon.Math')
local Score = require('Score')
local GameRing = {}

function GameRing:Init( tRenderer )    
    GameRing.mPos = {0, 0}
    GameRing:DetermineSize()
    GameRing.mIntensityRingExtraWidth = GameRing.mIntensityRingStart
    
    GameRing.mRingBatchProp = ShapeProp.new()
    GameRing.mRingBatchProp:SetNumShapes(7) -- 3 flavor, 3 intensity, 1 main
    GameRing.mRingBatchProp:setDeck( tRenderer.mShapeDeck )
    tRenderer.mGameLayer:insertProp( GameRing.mRingBatchProp )   
    
    GameRing.mFlavorRotations = {}
    GameRing.mFlavorRotationsInc = {}
    GameRing.mFlavorScales = {}

    -- flavor rings
    for i = 1, 3 do
        GameRing.mRingBatchProp:SetBatchIndex(i)
        GameRing.mRingBatchProp:SetNumSides( 64 )
        GameRing.mRingBatchProp:SetRadii( GameRing.mRadius, GameRing.mRadius + 2.0 )
    
        GameRing.mFlavorRotations[i] = {}
        for j = 1, 3 do
            GameRing.mFlavorRotations[i][j] = 0.0
        end
    end
    
    GameRing.mFlavorRotationsInc[1] = {-125.0, 0.0, 0.0}
    GameRing.mFlavorRotationsInc[2] = {0.0, 110.0, 0.0}
    GameRing.mFlavorRotationsInc[3] = {75.0, 165.0, 0.0}

    GameRing.mFlavorScales[1] = -1.5
    GameRing.mFlavorScales[2] = 4.5
    GameRing.mFlavorScales[3] = -4.0

    -- intensity rings
    for i = 4, 6 do
        GameRing.mRingBatchProp:SetBatchIndex(i)
        GameRing.mRingBatchProp:SetNumSides( 64 )
        GameRing.mRingBatchProp:SetRadii( GameRing.mRadius, GameRing.mRadius + 2.0 )
    end
    GameRing:UpdateIntensityRings(0.0)
    
    GameRing.mMainRingId = 7
    GameRing.mRingBatchProp:SetBatchIndex(GameRing.mMainRingId)
    GameRing.mRingBatchProp:SetNumSides( 64 )
    GameRing.mRingBatchProp:SetRadii( GameRing.mRadius, GameRing.mRadius + Constants.BasePenWidth )  
    GameRing.mRingBatchProp:SetVisible( false )
    
    tRenderer.mGameLayer:insertProp( GameRing.mRingBatchProp )
    
    GameRing.mTimerProp = ShapeProp.new()
    GameRing.mTimerProp:setDeck( tRenderer.mShapeDeck )
    GameRing.mTimerProp:SetNumSides( 64 )
    GameRing.mTimerProp:SetRadii( GameRing.mRadius, GameRing.mRadius + Constants.BasePenWidth )  
    GameRing.mTimerProp:SetDrawMode( ShapeProp.DrawModeArc )   
    GameRing.mTimerProp:SetArcRadians( -DFMath.halfPi, -DFMath.halfPi ) 
    GameRing.mTimerProp:setVisible( false )
    
    tRenderer.mGameLayer:insertProp( GameRing.mTimerProp )   
    
    GameRing.mTimerRatio = 0.0
    GameRing.mTimerState = Constants.PatternStateStandard
    
    GameRing.mPreIntroTimer = 0.0
    GameRing.mPreIntroDuration = 0.0
    
    GameRing.mDesiredRingRotation = {0.0, 0.0, 0.0}
    GameRing.mCurrentRingRotation = {0.0, 0.0, 0.0}
    GameRing.mRingRotationInc = 90.0
    
    GameRing.mVisibilityTimer = 0.0
    GameRing.mDesiredVisibility = true
end
  
function GameRing:DetermineSize()
    GameRing.mWidth = Constants.ValidDrawHeight * .7
    GameRing.mRadius = GameRing.mWidth * .5
    GameRing.mFlavorRadius = GameRing.mRadius
    GameRing.mFlavorWidth = GameRing.mRadius * 0.008
    GameRing.mFlavorDrawWidth = GameRing.mFlavorWidth
    GameRing.mPreIntroMaxScale = GameRing.mRadius * 3.0
    GameRing.mPreIntroMaxWidth = GameRing.mFlavorWidth * 20.0
    GameRing.mFlavorScalar = Constants.ValidDrawHeight * .00075
    GameRing.mIntensityRingStart = Constants.ValidDrawHeight * 0.6
    
    Constants.GameRingRadius = GameRing.mRadius
end

function GameRing:UpdateSize()
    GameRing:DetermineSize()
    
    GameRing.mRingBatchProp:SetBatchIndex(GameRing.mMainRingId)
    GameRing.mRingBatchProp:SetRadii( GameRing.mRadius, GameRing.mRadius + (Constants.BasePenWidth*Constants.GlobalScale) )    
    
    local fGameStateScale = 1.0
    if Constants.ColorMode == Constants.ColorModeIntro then
        fGameStateScale = 0.75
    end
    local fFlavorScale = Constants.BasePenWidth * GameRing.mFlavorScalar * fGameStateScale
    local fAdjust1 = fFlavorScale * GameRing.mFlavorScales[1]  
    local fAdjust2 = fFlavorScale * GameRing.mFlavorScales[2]
    local fAdjust3 = fFlavorScale * GameRing.mFlavorScales[3]
    
    -- flavor rings
    GameRing.mRingBatchProp:SetBatchIndex(1)
    GameRing.mRingBatchProp:SetRadii( GameRing.mFlavorRadius + fAdjust1, GameRing.mFlavorRadius + fAdjust1 + GameRing.mFlavorDrawWidth )
    GameRing.mRingBatchProp:SetBatchIndex(2)
    GameRing.mRingBatchProp:SetRadii( GameRing.mFlavorRadius + fAdjust2, GameRing.mFlavorRadius + fAdjust2 + GameRing.mFlavorDrawWidth )
    GameRing.mRingBatchProp:SetBatchIndex(3)
    GameRing.mRingBatchProp:SetRadii( GameRing.mFlavorRadius + fAdjust3, GameRing.mFlavorRadius + fAdjust3 + GameRing.mFlavorDrawWidth )      
        
    local fIntenseRadius1 = GameRing.mRadius + GameRing.mIntensityRingStart
    local fIntenseRadius2 = GameRing.mRadius + (GameRing.mIntensityRingStart * 4.0)
    local fIntenseRadius3 = GameRing.mRadius + (GameRing.mIntensityRingStart * 16.0)
    -- intensity rings
    GameRing.mRingBatchProp:SetBatchIndex(4)
    GameRing.mRingBatchProp:SetRadii( fIntenseRadius1, fIntenseRadius1 + GameRing.mFlavorDrawWidth )
    GameRing.mRingBatchProp:SetBatchIndex(5)
    GameRing.mRingBatchProp:SetRadii( fIntenseRadius2, fIntenseRadius2 + (GameRing.mFlavorDrawWidth*4.0) )
    GameRing.mRingBatchProp:SetBatchIndex(6)
    GameRing.mRingBatchProp:SetRadii( fIntenseRadius3, fIntenseRadius3 + (GameRing.mFlavorDrawWidth*16.0) )      
    
    GameRing.mTimerProp:SetRadii( GameRing.mRadius - GameRing.mFlavorScalar, GameRing.mRadius )    
end

function GameRing:InitTitle()
    local i = 1
    for i = 1, 7 do
        GameRing.mRingBatchProp:SetBatchIndex(i)
        GameRing.mRingBatchProp:SetVisible(false)
    end
end

function GameRing:InitPreIntro(fPreIntroDuration)
    GameRing.mRingBatchProp:SetBatchIndex(GameRing.mMainRingId)
    GameRing.mRingBatchProp:SetVisible(false)
    
    GameRing.mPreIntroDuration = fPreIntroDuration
    GameRing.mPreIntroTimer = GameRing.mPreIntroDuration
    
    local i = 1
    local j = 1
    for i = 1, 3 do
        for j = 1, 3 do
            GameRing.mFlavorRotations[i][j] = GameRing.mFlavorRotationsInc[i][j] * -GameRing.mPreIntroTimer
        end
        
        GameRing.mRingBatchProp:SetBatchIndex(i)
        GameRing.mRingBatchProp:SetVisible(true)
        GameRing.mRingBatchProp:SetRotation( GameRing.mFlavorRotations[i][1], GameRing.mFlavorRotations[i][2], GameRing.mFlavorRotations[i][3] )
    end    
        
    GameRing:UpdateIntensityRings(0.0)
    
    MOAIFmodEventMgr.setSoundCategoryVolume( "music", 0.0 )
end

function GameRing:ExitPreIntro()
    GameRing:ClearFlavorRingRotations()
    MOAIFmodEventMgr.setSoundCategoryVolume( "music", 1.0 )
end

function GameRing:InitPost()
    GameRing.mRingBatchProp:SetBatchIndex(GameRing.mMainRingId)
    GameRing.mRingBatchProp:SetVisible( true )
    GameRing.mTimerProp:setVisible( false )
    
    GameRing:SetDesiredRotation(0.0, 0.0, 0.0, 1.0)
    GameRing.mCurrentRingRotation[1] = 0.0
    GameRing.mCurrentRingRotation[2] = 0.0
    GameRing.mCurrentRingRotation[3] = 0.0
    
    local i = 1
    -- flavor rings
    for i = 1, 3 do
        GameRing.mRingBatchProp:SetBatchIndex(i)
        GameRing.mRingBatchProp:SetRotation( GameRing.mCurrentRingRotation[1], GameRing.mCurrentRingRotation[2], GameRing.mCurrentRingRotation[3] )
    end
    
    -- intensity rings
    for i = 4, 6 do
        GameRing.mRingBatchProp:SetBatchIndex(i)
        GameRing.mRingBatchProp:SetRotation( GameRing.mCurrentRingRotation[1], GameRing.mCurrentRingRotation[2], GameRing.mCurrentRingRotation[3] )
    end      
    GameRing.mRingBatchProp:SetBatchIndex(GameRing.mMainRingId)
    GameRing.mRingBatchProp:SetRotation( GameRing.mCurrentRingRotation[1], GameRing.mCurrentRingRotation[2], GameRing.mCurrentRingRotation[3] ) 
    
    GameRing.mTimerProp:setRot( GameRing.mCurrentRingRotation[1], GameRing.mCurrentRingRotation[2], GameRing.mCurrentRingRotation[3] )    
end

function GameRing:ClearFlavorRingRotations()
    local i = 1
    local j = 1
    for i = 1, 3 do
        for j = 1, 3 do
            GameRing.mFlavorRotations[i][j] = 0.0
        end
    end    
    
    for i = 1, 3 do
        GameRing.mRingBatchProp:SetBatchIndex(i)
        GameRing.mRingBatchProp:SetRotation( GameRing.mFlavorRotations[i][1], GameRing.mFlavorRotations[i][2], GameRing.mFlavorRotations[i][3] )
    end    
end

function GameRing:MakeRingVisible()
    GameRing.mRingBatchProp:SetBatchIndex(GameRing.mMainRingId)
    GameRing.mRingBatchProp:SetVisible( true )
end

function GameRing:MakeRingTimerVisible( bVisible )
    GameRing.mTimerProp:setVisible( bVisible )
    GameRing.mTimerProp:SetArcRadians( -DFMath.halfPi, -DFMath.halfPi )
end

function GameRing:SetDesiredRotation(x, y, z, fSpeed)
    GameRing.mDesiredRingRotation[1] = x
    GameRing.mDesiredRingRotation[2] = y
    GameRing.mDesiredRingRotation[3] = z
    GameRing.mRingRotationInc = fSpeed
end

function GameRing:SetVisibilityTimer( bVisible, fDelay )
    GameRing.mVisibilityTimer = fDelay
    GameRing.mDesiredVisibility = bVisible
end

function GameRing:UpdatePreIntro()
    local i = 1
    local j = 1
    if GameRing.mPreIntroTimer > 0.0 then
        GameRing.mPreIntroTimer = GameRing.mPreIntroTimer - Constants.FrameStep
        for i = 1, 3 do
            for j = 1, 3 do
                GameRing.mFlavorRotations[i][j] = GameRing.mFlavorRotations[i][j] + (Constants.FrameStep * GameRing.mFlavorRotationsInc[i][j])
            end                    
        end
    else
        GameRing:ClearFlavorRingRotations()            
        GameRing.mRingBatchProp:SetBatchIndex(GameRing.mMainRingId)
        GameRing.mRingBatchProp:SetVisible( true )
    end
    
    -- flavor rings
    for i = 1, 3 do
        GameRing.mRingBatchProp:SetBatchIndex(i)
        GameRing.mRingBatchProp:SetRotation( GameRing.mFlavorRotations[i][1], GameRing.mFlavorRotations[i][2], GameRing.mFlavorRotations[i][3] )
    end    
    
    --update radii for flavor rings
    local fIntroRatio = GameRing.mPreIntroTimer / GameRing.mPreIntroDuration
    GameRing.mFlavorRadius = GameRing.mRadius + (GameRing.mPreIntroMaxScale * fIntroRatio)
    
    --update width of flavor rings
    GameRing.mFlavorDrawWidth = GameRing.mFlavorWidth + (GameRing.mPreIntroMaxWidth * fIntroRatio)
    
    --update volume during pre-intro
    local fVolumeRatio = 1.0 - math.max( GameRing.mPreIntroTimer / GameRing.mPreIntroDuration, 0.0 )
    MOAIFmodEventMgr.setSoundCategoryVolume( "music", fVolumeRatio )    
end

function GameRing:Update()
    local fRingR = Constants.Color1[1]
    local fRingG = Constants.Color1[2]
    local fRingB = Constants.Color1[3]
    if Score:IsScoreShowingWarning() then
        fRingR = Constants.BadEffectColor[1]
        fRingG = Constants.BadEffectColor[2]
        fRingB = Constants.BadEffectColor[3]
    end    
    
    GameRing.mRingBatchProp:SetBatchIndex( GameRing.mMainRingId )
    GameRing.mRingBatchProp:SetColor( fRingR, fRingG, fRingB, 1.0 )
    GameRing.mRingBatchProp:SetRadii( GameRing.mRadius, GameRing.mRadius + (Constants.BasePenWidth*Constants.GlobalScale) )    
    
    local i = 1
    -- flavor rings and intensity rings
    for i = 1, 6 do
        GameRing.mRingBatchProp:SetBatchIndex(i)
        GameRing.mRingBatchProp:SetColor( fRingR, fRingG, fRingB, 1.0 )
    end
    
    local fGameStateScale = 1.0
    if Constants.ColorMode == Constants.ColorModeIntro then
        fGameStateScale = 0.75
    end
    local fFlavorScale = Constants.BasePenWidth * GameRing.mFlavorScalar * fGameStateScale
    local fAdjust1 = fFlavorScale * GameRing.mFlavorScales[1]  
    local fAdjust2 = fFlavorScale * GameRing.mFlavorScales[2]
    local fAdjust3 = fFlavorScale * GameRing.mFlavorScales[3]
    
    -- flavor rings
    GameRing.mRingBatchProp:SetBatchIndex(1)
    GameRing.mRingBatchProp:SetRadii( GameRing.mFlavorRadius + fAdjust1, GameRing.mFlavorRadius + fAdjust1 + GameRing.mFlavorDrawWidth )
    GameRing.mRingBatchProp:SetBatchIndex(2)
    GameRing.mRingBatchProp:SetRadii( GameRing.mFlavorRadius + fAdjust2, GameRing.mFlavorRadius + fAdjust2 + GameRing.mFlavorDrawWidth )
    GameRing.mRingBatchProp:SetBatchIndex(3)
    GameRing.mRingBatchProp:SetRadii( GameRing.mFlavorRadius + fAdjust3, GameRing.mFlavorRadius + fAdjust3 + GameRing.mFlavorDrawWidth )   

    if GameRing.mTimerState == Constants.PatternStateStandard then
        local r, g, b
        local t = math.min( 1.0, (GameRing.mTimerRatio - 0.25) * 4.0 )
        r = DFMath.lerp( Constants.BadEffectColor[1], Constants.Color1[1], t )
        g = DFMath.lerp( Constants.BadEffectColor[2], Constants.Color1[2], t )
        b = DFMath.lerp( Constants.BadEffectColor[3], Constants.Color1[3], t )
        GameRing.mTimerProp:SetColor( r, g, b, 1.0 )
    elseif GameRing.mTimerState == Constants.PatternStateFanfare then
        GameRing.mTimerProp:SetColor( Constants.SpecialColor[1], Constants.SpecialColor[2], Constants.SpecialColor[3], 1.0 )
    else        
        GameRing.mTimerProp:SetColor( Constants.BadEffectColor[1], Constants.BadEffectColor[2], Constants.BadEffectColor[3], 1.0 )
    end
    
    local fTimerRadiusInner = GameRing.mRadius - (GameRing.mFlavorDrawWidth * 5.5)
    local fTimerRadiusWidth = GameRing.mFlavorDrawWidth * 3.0
    GameRing.mTimerProp:SetRadii( fTimerRadiusInner, fTimerRadiusInner + fTimerRadiusWidth )
    
    --update visibility timer
    if GameRing.mVisibilityTimer > 0.0 then
        GameRing.mVisibilityTimer = GameRing.mVisibilityTimer - Constants.FrameStep
        if GameRing.mVisibilityTimer <= 0.0 then
            GameRing.mTimerProp:setVisible( GameRing.mDesiredVisibility )
            
            for i = 1, 7 do -- all rings
                GameRing.mRingBatchProp:SetBatchIndex(i)
                GameRing.mRingBatchProp:SetVisible( GameRing.mDesiredVisibility )
            end        
        end
    end    
end

function GameRing:UpdateRotation()    
    --update rotation
    local i = 1
    local fRotationStep = GameRing.mRingRotationInc * Constants.FrameStep
    for i = 1, 3 do
        local fRotDiff = GameRing.mCurrentRingRotation[i] - GameRing.mDesiredRingRotation[i]
        if fRotDiff > fRotationStep then
            GameRing.mCurrentRingRotation[i] = GameRing.mCurrentRingRotation[i] - fRotationStep
        elseif fRotDiff < -fRotationStep then
            GameRing.mCurrentRingRotation[i] = GameRing.mCurrentRingRotation[i] + fRotationStep
        else
            GameRing.mCurrentRingRotation[i] = GameRing.mDesiredRingRotation[i]
        end
    end
    
    for i = 1, 7 do -- all rings
        GameRing.mRingBatchProp:SetBatchIndex(i)
        GameRing.mRingBatchProp:SetRotation( GameRing.mCurrentRingRotation[1], GameRing.mCurrentRingRotation[2], GameRing.mCurrentRingRotation[3] )
    end      
    GameRing.mTimerProp:setRot( GameRing.mCurrentRingRotation[1], GameRing.mCurrentRingRotation[2], GameRing.mCurrentRingRotation[3] )
end
  
function GameRing:UpdateTimer(fRatio, iState)
    --a little boost for high values
    if fRatio > 0.98 then
        fRatio = 1.0
    end
    if fRatio < 0.0 then
        fRatio = 0.0
    end
    local fRadians = fRatio * 2.0 * math.pi
    GameRing.mTimerProp:SetArcRadians( -DFMath.halfPi, fRadians-DFMath.halfPi )     
    GameRing.mTimerState = iState
    GameRing.mTimerRatio = fRatio
end

function GameRing:UpdateIntensityRings(fRatio)
    local bRingsVisible = fRatio ~= 0.0
    local i = 1
    -- intensity rings
    for i = 4, 6 do
        GameRing.mRingBatchProp:SetBatchIndex(i)
        GameRing.mRingBatchProp:SetVisible(bRingsVisible)
    end      
    
    local fInvRatio = 1.0 - fRatio
    GameRing.mIntensityRingExtraWidth = (fInvRatio * GameRing.mIntensityRingStart)
    
    local fIntenseRadius1 = GameRing.mRadius + GameRing.mIntensityRingExtraWidth
    local fIntenseRadius2 = GameRing.mRadius + (GameRing.mIntensityRingExtraWidth * 4.0)
    local fIntenseRadius3 = GameRing.mRadius + (GameRing.mIntensityRingExtraWidth * 16.0)
    
    -- intensity rings
    GameRing.mRingBatchProp:SetBatchIndex(4)
    GameRing.mRingBatchProp:SetRadii( fIntenseRadius1, fIntenseRadius1 + GameRing.mFlavorDrawWidth )
    GameRing.mRingBatchProp:SetBatchIndex(5)
    GameRing.mRingBatchProp:SetRadii( fIntenseRadius2, fIntenseRadius2 + (GameRing.mFlavorDrawWidth*4.0) )
    GameRing.mRingBatchProp:SetBatchIndex(6)
    GameRing.mRingBatchProp:SetRadii( fIntenseRadius3, fIntenseRadius3 + (GameRing.mFlavorDrawWidth*16.0) )  
end
  
return GameRing
