local Constants = require("Constants")
local DFGraphics = require("DFCommon.Graphics")
local DFMath = require("DFCommon.Math")

local GlowRing = {}

local kNumRingSprites = 1
local kRingImageSize = 256.0

function GlowRing:Init( rRenderer )
    self.mRenderer = rRenderer
    self.mColor = { 1.0, 1.0, 1.0, 1.0 }
    self.mTargetColor = { 1.0, 1.0, 1.0, 1.0 }
end

function GlowRing:Update()
    -- lerp our color
    for i=1,3 do
        self.mColor[i] = DFMath.lerp(self.mColor[i], self.mTargetColor[i], .03)  
    end
    
    if self.mIsEnabled then
        if self.mRingSprites then
            for i,rRingSprite in ipairs(self.mRingSprites) do
                rRingSprite:setColor(self.mColor[1], self.mColor[2], self.mColor[3], self.mColor[4])
            end
        end
    end
end

function GlowRing:UpdateSize()
    local ringSize = (700.0) * (Constants.ValidDrawHeight / 720.0)
    local scaledSize = ringSize / kRingImageSize
    local offset = scaledSize * kRingImageSize * 0.5
    
    if self.mRingSprites then
        for i,rRingSprite in ipairs(self.mRingSprites) do
            rRingSprite:setLoc( -offset, -offset)
            rRingSprite:setScl( scaledSize, scaledSize )
        end
    end
end

function GlowRing:SetColor(r, g, b, a)
    self.mTargetColor = { r, g, b, a }
end

function GlowRing:Enable(bEnable)
    if nil == bEnable then bEnable = true end
    
    self.mIsEnabled = bEnable
    
    if bEnable then
        if not self.mImageDeck then
            self.mImageDeck = DFGraphics.loadSpriteSheet( 'Images/misc', true )
        end
        
        if not self.mRingSprites then
            self.mRingSprites = {}

            for i=1,kNumRingSprites do
                local rRingSprite = MOAIProp.new()
                rRingSprite:setDeck( self.mImageDeck )        
                rRingSprite:setIndex( self.mImageDeck.names["RingGlow"] )
                rRingSprite:setBlendMode(MOAIProp.GL_ONE, MOAIProp.GL_ONE)
                rRingSprite:setColor(self.mColor[1], self.mColor[2], self.mColor[3], self.mColor[4])
                self.mRenderer.mUILayer:insertProp(rRingSprite)
                table.insert(self.mRingSprites, rRingSprite)
            end
            
            self:UpdateSize()
        end
        
        for i,rRingSprite in ipairs(self.mRingSprites) do
            rRingSprite:setVisible(true)
        end
    else
        self:SetColor(0,0,0,1)
        if self.mRingSprites then
            for i,rRingSprite in ipairs(self.mRingSprites) do
                rRingSprite:setVisible(false)
            end
        end
    end
 end


return GlowRing
