local Constants = require('Constants')
local DFMath = require('DFCommon.Math')
local Grid = {}

function Grid:Init( tRenderer )
    Grid.mGridDeck = GridDeck.new()

    Grid:VisualQualityChange()
    
    Grid.mGridDeck:SetPointWidth( 2.0 )
    Grid.mGridDeck:SetSpringK( 10.0 )
    Grid.mGridDeck:SetSpringDampen( 0.025 )
    Grid.mGridProp = MOAIProp2D.new()
    Grid.mGridProp.__typename = "Grid"
    Grid.mGridProp:setDeck( Grid.mGridDeck )
        
    tRenderer.mBackgroundLayer:insertProp( Grid.mGridProp )
    
    Grid:Enable( false )
end
  
function Grid:UpdateSize()
    Grid.mSpacing = Constants.ValidDrawWidth / Grid.mTileSize
    Grid.mGridDeck:UpdateSpacing( Grid.mSpacing )    
end
  
function Grid:VisualQualityChange()
    Grid.mTileSize = 64.0

    if Constants.VisualQuality ~= Constants.VisualQualityLevelUltra then
        Grid.mTileSize = 28.0
    end
    
    Grid.mSpacing = Constants.NativeScreenWidth / Grid.mTileSize
    local iXDimensionCount = Constants.NativeScreenWidth / Grid.mSpacing
    local iYDimensionCount = Constants.NativeScreenHeight / Grid.mSpacing   
    
    --add a few more on both sides to hide the borders
    iXDimensionCount = iXDimensionCount + 4
    iYDimensionCount = iYDimensionCount + 4
    Grid.mGridDeck:SetDimensions( iXDimensionCount, iYDimensionCount, Grid.mSpacing )
    Grid.mGridDeck:SetColor( Constants.BGColor[1]-0.05, Constants.BGColor[2]-0.05, Constants.BGColor[3]-0.05, 1.0 )
    
    Grid:UpdateSize()
end
  
function Grid:Enable(bEnable)
    Grid.mEnabled = bEnable
    Grid.mGridProp:setVisible(bEnable)
end

function Grid:SetColor(r, g, b, a)
    Grid.mGridDeck:SetColor( r, g, b, a )
end

function Grid:Update()
    if Grid.mEnabled == true then
        Grid.mGridDeck:Update( 0.033 )
    end
end
  
function Grid:AddImpression(fX, fY, fDepth)
    if Grid.mEnabled == true then
        Grid.mGridDeck:AddImpression( fX, fY, Grid.mSpacing )
    end
end

return Grid