local Constants = require("Constants")
local DFGraphics = require("DFCommon.Graphics")
local DFMath = require("DFCommon.Math")

local HUDGlow = {}

local kGlowImageSize = 256.0

function HUDGlow:Init( rRenderer )
    self.mRenderer = rRenderer
    self.mColor = { 0.0, 0.0, 0.0, 1.0 }
    self.mTargetColor = { 0.0, 0.0, 0.0, 1.0 }
end

function HUDGlow:Update()
    -- lerp our color
    for i=1,3 do
        self.mColor[i] = DFMath.lerp(self.mColor[i], self.mTargetColor[i], .03)  
    end

    if self.mIsEnabled then
        if self.mLeftScoreGlow then
            self.mLeftScoreGlow:setColor(self.mColor[1], self.mColor[2], self.mColor[3], self.mColor[4])
        end
        
        if self.mRightScoreGlow then
            self.mRightScoreGlow:setColor(self.mColor[1], self.mColor[2], self.mColor[3], self.mColor[4])
        end
        
        -- use fft to make some sweet visuals
        if self.mIsBumpEnabled then
            if not self.mAudioSmoothed then
                self.mAudioSmoothed = 1.0
            end
            self.mAudioSmoothed = DFMath.lerp(self.mAudioSmoothed, Constants.AudioSpectrumAverage * 14.0, .3)   
        
            local scaleX = self.mScoreScaleX * 0.85 + self.mAudioSmoothed * self.mBumpScale
        
            if self.mLeftScoreGlow and self.mRightScoreGlow then
                self.mLeftScoreGlow:setScl( scaleX, self.mScoreScaleY)
                self.mRightScoreGlow:setScl(-scaleX, self.mScoreScaleY)
            end
        end
    end
end

function HUDGlow:UpdateSize()
    self.mScoreScaleX = 1.0 * (Constants.ValidDrawWidth / 1280.0)
    self.mScoreScaleY = 1.0 * (760.0 / kGlowImageSize) * (Constants.ValidDrawHeight / 720.0)

    if self.mLeftScoreGlow then
        self.mLeftScoreGlow:setLoc(-(760.0 * (Constants.ValidDrawWidth / 1280.0)), -(418.0 * (Constants.ValidDrawHeight / 720.0)))
        self.mLeftScoreGlow:setScl(self.mScoreScaleX, self.mScoreScaleY)
    end
    
    if self.mRightScoreGlow then
        self.mRightScoreGlow:setLoc( (760.0 * (Constants.ValidDrawWidth / 1280.0)), -(418.0 * (Constants.ValidDrawHeight / 720.0)))
        self.mRightScoreGlow:setScl(-self.mScoreScaleX, self.mScoreScaleY)
    end
end

function HUDGlow:SetColor(r, g, b, a)
    self.mTargetColor = { r, g, b, a }
end

function HUDGlow:SetScoreVisible( bEnable )
    if nil == bEnable then bEnable = true end
    
    local bGlowVisible = (bEnable and self.mIsEnabled)
    
    if self.mLeftScoreGlow then
        self.mLeftScoreGlow:setVisible(bGlowVisible)
    end
    
    if self.mRightScoreGlow then
        self.mRightScoreGlow:setVisible(bGlowVisible)
    end
end

function HUDGlow:EnableBumpin( bEnable, nScale )
    if nil == bEnable then bEnable = true end
    if nil == nScale then nScale = 1.0 end
    
    self.mIsBumpEnabled = bEnable
    self.mBumpScale = nScale
    
    if not bEnable then
        self:UpdateSize()
    end
end

function HUDGlow:Enable( bEnable )
    if nil == bEnable then bEnable = true end
    self.mIsEnabled = bEnable
    
    if bEnable then
        if not self.mImageDeck then
            self.mImageDeck = DFGraphics.loadSpriteSheet( 'Images/misc', true )
        end
        
        if not self.mLeftScoreGlow then
            local rSprite = MOAIProp.new()
            rSprite:setDeck( self.mImageDeck )        
            rSprite:setIndex( self.mImageDeck.names["ScoreGlow"] )
            rSprite:setColor(self.mColor[1], self.mColor[2], self.mColor[3], self.mColor[4])
            rSprite:setBlendMode(MOAIProp.GL_ONE, MOAIProp.GL_ONE)
            self.mRenderer.mUILayer:insertProp(rSprite)
            self.mLeftScoreGlow = rSprite
        end
        
        if not self.mRightScoreGlow then
            local rSprite = MOAIProp.new()
            rSprite:setDeck( self.mImageDeck )        
            rSprite:setIndex( self.mImageDeck.names["ScoreGlow"] )
            rSprite:setColor(self.mColor[1], self.mColor[2], self.mColor[3], self.mColor[4])
            rSprite:setBlendMode(MOAIProp.GL_ONE, MOAIProp.GL_ONE)
            self.mRenderer.mUILayer:insertProp(rSprite)
            self.mRightScoreGlow = rSprite
        end
        
        self.mLeftScoreGlow:setVisible(true)
        self.mRightScoreGlow:setVisible(true)
        self:UpdateSize()
    else
        self:EnableBumpin(false)
    
        -- hide
        if self.mLeftScoreGlow then
            self.mLeftScoreGlow:setVisible(false)
        end
        
        if self.mRightScoreGlow then
            self.mRightScoreGlow:setVisible(false)
        end
    end
end


return HUDGlow
