local Constants = require('Constants')
local DFUtil = require('DFCommon.Util')
local DFGraphics = require('DFCommon.Graphics')
local DFMath = require('DFCommon.Math')
local TextManager = require('TextManager')
local RadFont = require('RadFont')
local RadEffects = require('RadEffects')
local AudioManager = require('AudioManager')
local HowToPlay = {}

function HowToPlay:NewHowToPlayImage()
    local HowToPlayImage = {}
    
    function HowToPlayImage:Init( tIndexName, fScale )
        HowToPlayImage.mDeckIndex = HowToPlay.mImageDeck.names[ tIndexName ]
        HowToPlayImage.mWidth = HowToPlay.mImageDeck.rects[HowToPlayImage.mDeckIndex].origWidth * fScale
        HowToPlayImage.mHeight = HowToPlay.mImageDeck.rects[HowToPlayImage.mDeckIndex].origHeight * fScale
        HowToPlayImage.mCurrentAlpha = 0.0
        HowToPlayImage.mDesiredAlpha = 0.0  
        HowToPlayImage.mImageProp = nil
        HowToPlayImage.mSlideshowCount = 0
        HowToPlayImage.mSlideshowIndexes = {}
        HowToPlayImage.mSlideshowInterval = 0.0
        HowToPlayImage.mSlideshowTimer = 0.0
        HowToPlayImage.mSlideshowIndex = 1
    end
    
    function HowToPlayImage:SetSlideshowParams( iCount, fInterval )
        HowToPlayImage.mSlideshowCount = iCount
        HowToPlayImage.mSlideshowInterval = fInterval
    end
    
    function HowToPlayImage:AddSlideshowImage( iIndex, tIndexName )
        HowToPlayImage.mSlideshowIndexes[iIndex] = HowToPlay.mImageDeck.names[ tIndexName ]
    end
    
    function HowToPlayImage:CreateProp( tRenderer, tDeck, fScale )
        if HowToPlayImage.mImageProp == nil then
            HowToPlayImage.mImageProp = MOAIProp.new()
            HowToPlayImage.mImageProp:setDeck( tDeck )
            HowToPlayImage.mImageProp:setScl( fScale, -fScale ) 
            HowToPlayImage.mImageProp:setIndex( HowToPlayImage.mDeckIndex )
            HowToPlayImage.mImageProp:setVisible( false )
            HowToPlayImage.mImageProp:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA ) 
            tRenderer.mUILayer:insertProp( HowToPlayImage.mImageProp )     
        end
        
        HowToPlayImage.mCurrentAlpha = 0.0
        HowToPlayImage.mDesiredAlpha = 0.0        
    end
    
    function HowToPlayImage:UpdateVisibility()
        local fAlphaStep = Constants.FrameStep    
        if math.abs(HowToPlayImage.mCurrentAlpha - HowToPlayImage.mDesiredAlpha) < fAlphaStep then
            HowToPlayImage.mCurrentAlpha = HowToPlayImage.mDesiredAlpha
        elseif HowToPlayImage.mCurrentAlpha < HowToPlayImage.mDesiredAlpha then
            HowToPlayImage.mCurrentAlpha = HowToPlayImage.mCurrentAlpha + fAlphaStep
        else 
            HowToPlayImage.mCurrentAlpha = HowToPlayImage.mCurrentAlpha - fAlphaStep
        end    
        
        HowToPlayImage.mImageProp:setVisible( HowToPlayImage.mCurrentAlpha > 0.0 )
        HowToPlayImage.mImageProp:setColor( 1.0, 1.0, 1.0, HowToPlayImage.mCurrentAlpha )    
        
        --udpate slideshow if we have one
        if HowToPlayImage.mSlideshowCount > 0 then
            HowToPlayImage.mSlideshowTimer = HowToPlayImage.mSlideshowTimer - Constants.FrameStep    
            if HowToPlayImage.mSlideshowTimer < 0.0 then
                HowToPlayImage.mSlideshowIndex = HowToPlayImage.mSlideshowIndex + 1
                if HowToPlayImage.mSlideshowIndex > HowToPlayImage.mSlideshowCount then
                    HowToPlayImage.mSlideshowIndex = 1
                end
                
                HowToPlayImage.mImageProp:setIndex( HowToPlayImage.mSlideshowIndexes[HowToPlayImage.mSlideshowIndex] )
                
                HowToPlayImage.mSlideshowTimer = HowToPlayImage.mSlideshowInterval
            end
        end
    end
    
    return HowToPlayImage
end

function HowToPlay:UpdateThread()    
    TextManager:SetFadeAttributes( 0.0, 0.0 )

    local fAutoAdvanceTimer = HowToPlay.mAutoAdvanceDuration
    while true do
        if fAutoAdvanceTimer < 0.0 then
            HowToPlay.mDesiredPage = HowToPlay.mCurrentPage + 1
            if HowToPlay.mDesiredPage > HowToPlay.mNumPages then
                HowToPlay.mDesiredPage = 1
            end
        end
        
        --spin until the current page isn't the desired
        if HowToPlay.mCurrentPage ~= HowToPlay.mDesiredPage then
            HowToPlay.mAllowInput = false
            
            TextManager:ClearAll()
            
            --exit state for page
            if HowToPlay.mCurrentPage == 1 then
                HowToPlay.mImages[1].mDesiredAlpha = 0.0   
                HowToPlay.mImages[1].mCurrentAlpha = 0.0             
            elseif HowToPlay.mCurrentPage == 2 then
                HowToPlay.mImages[2].mDesiredAlpha = 0.0
                HowToPlay.mImages[2].mCurrentAlpha = 0.0
                HowToPlay.mImages[3].mDesiredAlpha = 0.0
                HowToPlay.mImages[3].mCurrentAlpha = 0.0
                HowToPlay.mImages[4].mDesiredAlpha = 0.0            
                HowToPlay.mImages[4].mCurrentAlpha = 0.0            
            elseif HowToPlay.mCurrentPage == 3 then
                HowToPlay.mImages[5].mDesiredAlpha = 0.0
                HowToPlay.mImages[5].mCurrentAlpha = 0.0
                HowToPlay.mImages[6].mDesiredAlpha = 0.0 
                HowToPlay.mImages[6].mCurrentAlpha = 0.0 
            elseif HowToPlay.mCurrentPage == 4 then
                HowToPlay.mImages[7].mDesiredAlpha = 0.0
                HowToPlay.mImages[7].mCurrentAlpha = 0.0
                HowToPlay.mImages[8].mDesiredAlpha = 0.0 
                HowToPlay.mImages[8].mCurrentAlpha = 0.0 
            end            
        
            --enter for page
            if HowToPlay.mDesiredPage == 1 then
                fAutoAdvanceTimer = HowToPlay:PageOne() - 1.5 
            elseif HowToPlay.mDesiredPage == 2 then
                fAutoAdvanceTimer = HowToPlay:PageTwo() - 1.5     
            elseif HowToPlay.mDesiredPage == 3 then
                fAutoAdvanceTimer = HowToPlay:PageThree() - 1.5      
            elseif HowToPlay.mDesiredPage == 4 then
                fAutoAdvanceTimer = HowToPlay:PageFour() - 1.5                 
            elseif HowToPlay.mDesiredPage == 5 then
                fAutoAdvanceTimer = HowToPlay:TipsPage() - 1.5      
            elseif HowToPlay.mDesiredPage == HowToPlay.mBackToMenuSelection then
                break
            end
            
            --update current page
            HowToPlay.mCurrentPage = HowToPlay.mDesiredPage          
            HowToPlay.mAllowInput = true
        else
            --sleep and tick it off our timer
            DFUtil.sleep(HowToPlay.mStandardSleep)
            fAutoAdvanceTimer = fAutoAdvanceTimer - HowToPlay.mStandardSleep
        end
    end 
end

function HowToPlay:PageOne()
    DFUtil.sleep(HowToPlay.mStandardSleep)

    local fPageDuration = HowToPlay.mAutoAdvanceDuration
    local fLineY = HowToPlay.mLetterSize * -4.5
    
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color2 )
    TextManager:AddLine( "controls", HowToPlay.mLetterSize * -4.0, fLineY, fPageDuration )
    fPageDuration = fPageDuration - HowToPlay.mPauseSleep
    DFUtil.sleep(HowToPlay.mPauseSleep)
       
    fLineY = fLineY - (HowToPlay.mLetterSize * 0.5)
    HowToPlay.mImages[1].mImageProp:setLoc( HowToPlay.mLetterSize, fLineY + HowToPlay.mImages[1].mHeight + (HowToPlay.mLetterSize * 2.0), 10.0 )
    HowToPlay.mImages[1].mCurrentAlpha = 0.0
    HowToPlay.mImages[1].mDesiredAlpha = 1.0

    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color1 )    
    fLineY = fLineY + (HowToPlay.mLetterSize * 5.0)    
    TextManager:AddLine( "use", HowToPlay.mLetterSize * -9.0, fLineY, fPageDuration )
    
    fLineY = fLineY + (HowToPlay.mLetterSize * 1.5)
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color3 )    
    TextManager:AddLine( "two.thumbs", HowToPlay.mLetterSize * -12.0, fLineY, fPageDuration )
    fPageDuration = fPageDuration - HowToPlay.mBreakSleep
    DFUtil.sleep(HowToPlay.mBreakSleep)
    
    return fPageDuration
end

function HowToPlay:PageTwo()
    DFUtil.sleep(HowToPlay.mStandardSleep)

    local fPageDuration = HowToPlay.mAutoAdvanceDuration  
    local fLineY = HowToPlay.mLetterSize * -4.5
    
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color2 )
    TextManager:AddLine( "controls", HowToPlay.mLetterSize * -4.0, fLineY, fPageDuration )
    fPageDuration = fPageDuration - HowToPlay.mPauseSleep
    DFUtil.sleep(HowToPlay.mPauseSleep)
       
    fLineY = fLineY + HowToPlay.mLetterSize 
    HowToPlay.mImages[2].mImageProp:setLoc( (HowToPlay.mLetterSize * -8.0) - HowToPlay.mImages[2].mWidth, fLineY + HowToPlay.mImages[2].mHeight - HowToPlay.mLetterSize, 10.0 )
    HowToPlay.mImages[2].mCurrentAlpha = 0.0
    HowToPlay.mImages[2].mDesiredAlpha = 1.0

    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color1 )    
    fLineY = fLineY + (HowToPlay.mLetterSize * 2.0)    
    TextManager:AddLine( "position", (HowToPlay.mLetterSize * -8.0), fLineY, fPageDuration )

    fLineY = fLineY + (HowToPlay.mLetterSize * 1.5)
    TextManager:AddLine( "the", (HowToPlay.mLetterSize * -8.0), fLineY, fPageDuration )
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color3 )    
    TextManager:AddLine( "beam", (HowToPlay.mLetterSize * -4.0), fLineY, fPageDuration )
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color1 )    
    TextManager:AddLine( "to", HowToPlay.mLetterSize, fLineY, fPageDuration )
    fPageDuration = fPageDuration - HowToPlay.mBreakSleep
    DFUtil.sleep(HowToPlay.mBreakSleep)

    HowToPlay.mImages[3].mImageProp:setLoc( (HowToPlay.mLetterSize * 9.0), fLineY + HowToPlay.mImages[3].mHeight + (HowToPlay.mLetterSize * 0.5), 10.0 )
    HowToPlay.mImages[3].mCurrentAlpha = 0.0
    HowToPlay.mImages[3].mDesiredAlpha = 1.0
    
    fLineY = fLineY + (HowToPlay.mLetterSize * 3.0)   
    TextManager:AddLine( "strike", HowToPlay.mLetterSize * -3.0, fLineY, fPageDuration )
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color3 )
    TextManager:AddLine( "notes", HowToPlay.mLetterSize * 4.0, fLineY, fPageDuration )
    fPageDuration = fPageDuration - (HowToPlay.mBreakSleep * 2.0)
    DFUtil.sleep((HowToPlay.mBreakSleep * 2.0))

    fLineY = fLineY + (HowToPlay.mLetterSize * 1.5)
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color1 )
    TextManager:AddLine( "and", HowToPlay.mLetterSize * 2.0, fLineY, fPageDuration )
    
    fLineY = fLineY + (HowToPlay.mLetterSize * 1.5)
    TextManager:AddLine( "avoid", HowToPlay.mLetterSize * -6.0, fLineY, fPageDuration )
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color3 )
    TextManager:AddLine( "scratches", 0.0, fLineY, fPageDuration )

    HowToPlay.mImages[4].mImageProp:setLoc( (HowToPlay.mLetterSize * 9.0), fLineY + HowToPlay.mImages[4].mHeight - (HowToPlay.mLetterSize * 2.25), 10.0 )
    HowToPlay.mImages[4].mCurrentAlpha = 0.0
    HowToPlay.mImages[4].mDesiredAlpha = 1.0
    
    return fPageDuration
end

function HowToPlay:PageThree()
    DFUtil.sleep(HowToPlay.mStandardSleep)

    local fPageDuration = HowToPlay.mAutoAdvanceDuration  
    local fLineY = HowToPlay.mLetterSize * -4.5
    
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color2 )
    TextManager:AddLine( "advanced", HowToPlay.mLetterSize * -4.0, fLineY, fPageDuration )
    fPageDuration = fPageDuration - HowToPlay.mPauseSleep
    DFUtil.sleep(HowToPlay.mPauseSleep)
       
    HowToPlay.mImages[5].mImageProp:setLoc( (HowToPlay.mLetterSize * 6.0), fLineY + HowToPlay.mImages[5].mHeight, 10.0 )
    HowToPlay.mImages[5].mCurrentAlpha = 0.0
    HowToPlay.mImages[5].mDesiredAlpha = 1.0

    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color1 )    
    fLineY = fLineY + (HowToPlay.mLetterSize * 2.5)    
    TextManager:AddLine( "strike.all.notes", HowToPlay.mLetterSize * -10.0, fLineY, fPageDuration )

    fLineY = fLineY + (HowToPlay.mLetterSize * 1.5)
    TextManager:AddLine( "without.a.scratch", HowToPlay.mLetterSize * -11.0, fLineY, fPageDuration )
    
    fLineY = fLineY + (HowToPlay.mLetterSize * 1.5)
    TextManager:AddLine( "for.a", HowToPlay.mLetterSize * -15.0, fLineY, fPageDuration )
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color3 )    
    TextManager:AddLine( "perfect.pattern", HowToPlay.mLetterSize * -9.0, fLineY, fPageDuration )    
    fPageDuration = fPageDuration - (HowToPlay.mBreakSleep * 3.0)
    DFUtil.sleep((HowToPlay.mBreakSleep * 3.0))

    HowToPlay.mImages[6].mImageProp:setLoc( (HowToPlay.mLetterSize * -7.0) - HowToPlay.mImages[6].mWidth, fLineY + HowToPlay.mImages[6].mHeight, 10.0 )
    HowToPlay.mImages[6].mCurrentAlpha = 0.0
    HowToPlay.mImages[6].mDesiredAlpha = 1.0
    
    fLineY = fLineY + (HowToPlay.mLetterSize * 3.5)
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color1 )    
    TextManager:AddLine( "perfect.patterns.add", HowToPlay.mLetterSize * -7.0, fLineY, fPageDuration )
   
    fLineY = fLineY + (HowToPlay.mLetterSize * 1.5)
    TextManager:AddLine( "to.the", HowToPlay.mLetterSize * -7.0, fLineY, fPageDuration )
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color3 )   
    TextManager:AddLine( "multiplier", 0.0, fLineY, fPageDuration )
    
    return fPageDuration
end

function HowToPlay:PageFour()
    DFUtil.sleep(HowToPlay.mStandardSleep)

    local fPageDuration = HowToPlay.mAutoAdvanceDuration 
    local fLineY = HowToPlay.mLetterSize * -4.5
    
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color2 )
    TextManager:AddLine( "advanced", HowToPlay.mLetterSize * -4.0, fLineY, fPageDuration )
    fPageDuration = fPageDuration - HowToPlay.mPauseSleep
    DFUtil.sleep(HowToPlay.mPauseSleep)
       
    HowToPlay.mImages[7].mImageProp:setLoc( (HowToPlay.mLetterSize * 1.7), fLineY + HowToPlay.mImages[7].mHeight - HowToPlay.mLetterSize-100.0, 10.0 )
    HowToPlay.mImages[7].mCurrentAlpha = 0.0
    HowToPlay.mImages[7].mDesiredAlpha = 1.0

    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color1 )    
    fLineY = fLineY + (HowToPlay.mLetterSize * 2.5)    
    TextManager:AddLine( "use.a.swipe.gesture", HowToPlay.mLetterSize * -13.0, fLineY, fPageDuration )

    fLineY = fLineY + (HowToPlay.mLetterSize * 1.5)
    TextManager:AddLine( "to", HowToPlay.mLetterSize * -10.0, fLineY, fPageDuration )
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color3 )    
    TextManager:AddLine( "spin", HowToPlay.mLetterSize * -7.0, fLineY, fPageDuration )
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color1 )    
    TextManager:AddLine( "the.beam", HowToPlay.mLetterSize * -2.0, fLineY, fPageDuration )
    fPageDuration = fPageDuration - (HowToPlay.mBreakSleep * 2.0)
    DFUtil.sleep((HowToPlay.mBreakSleep * 2.0))
    
    HowToPlay.mImages[8].mImageProp:setLoc( (HowToPlay.mLetterSize * -7.0) - HowToPlay.mImages[8].mWidth-10, fLineY + HowToPlay.mImages[6].mHeight+45, 10.0 )
    HowToPlay.mImages[8].mCurrentAlpha = 0.0
    HowToPlay.mImages[8].mDesiredAlpha = 1.0
    
    fLineY = fLineY + (HowToPlay.mLetterSize * 4.5)
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color1 )    
    TextManager:AddLine( "a.beam.spin.will", HowToPlay.mLetterSize * -7.0, fLineY, fPageDuration )
   
    fLineY = fLineY + (HowToPlay.mLetterSize * 1.5)
    TextManager:AddLine( "light.up", HowToPlay.mLetterSize * -7.0, fLineY, fPageDuration )
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color3 )   
    TextManager:AddLine( "sync.pads", HowToPlay.mLetterSize * 2.0, fLineY, fPageDuration )
    
    return fPageDuration
end

function HowToPlay:TipsPage()
    DFUtil.sleep(HowToPlay.mStandardSleep)

    local fPageDuration = HowToPlay.mAutoAdvanceDuration  
    local fLineY = HowToPlay.mLetterSize * -4.5
    
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color2 )
    TextManager:AddLine( "tips", HowToPlay.mLetterSize * -2.0, fLineY, fPageDuration )
    fPageDuration = fPageDuration - HowToPlay.mPauseSleep
    DFUtil.sleep(HowToPlay.mPauseSleep)
       
    fLineY = fLineY + (HowToPlay.mLetterSize * 2.0)
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color1 )    
    TextManager:AddLine( "_.use.the", HowToPlay.mLetterSize * -15.0, fLineY, fPageDuration )
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color3 )    
    TextManager:AddLine( "verse.progress.bar", HowToPlay.mLetterSize * -5.0, fLineY, fPageDuration )
    
    fLineY = fLineY + (HowToPlay.mLetterSize * 1.5)
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color1 )
    TextManager:AddLine( "to.monitor.progress", HowToPlay.mLetterSize * -13.0, fLineY, fPageDuration )
    fPageDuration = fPageDuration - (HowToPlay.mBreakSleep * 2.0)
    DFUtil.sleep((HowToPlay.mBreakSleep * 2.0))
    
    fLineY = fLineY + (HowToPlay.mLetterSize * 2.0)
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color1 )    
    TextManager:AddLine( "_", HowToPlay.mLetterSize * -15.0, fLineY, fPageDuration )
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color3 )    
    TextManager:AddLine( "health", HowToPlay.mLetterSize * -13.0, fLineY, fPageDuration )
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color1 )    
    TextManager:AddLine( "is.awarded.for.high", HowToPlay.mLetterSize * -6.0, fLineY, fPageDuration )

    fLineY = fLineY + (HowToPlay.mLetterSize * 1.5)
    TextManager:AddLine( "pattern.perfection", HowToPlay.mLetterSize * -13.0, fLineY, fPageDuration )
    fPageDuration = fPageDuration - (HowToPlay.mBreakSleep * 2.0)
    DFUtil.sleep((HowToPlay.mBreakSleep * 2.0))
    
    fLineY = fLineY + (HowToPlay.mLetterSize * 2.0)    
    TextManager:AddLine( "_.previously.completed.tracks", HowToPlay.mLetterSize * -15.0, fLineY, fPageDuration )

    fLineY = fLineY + (HowToPlay.mLetterSize * 1.5)
    TextManager:AddLine( "can.be.skipped.by.pressing", HowToPlay.mLetterSize * -13.0, fLineY, fPageDuration )
    
    fLineY = fLineY + (HowToPlay.mLetterSize * 1.5)
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color3 )
    TextManager:AddLine( "next", HowToPlay.mLetterSize * -13.0, fLineY, fPageDuration )
    TextManager:SetTextAttributes( HowToPlay.mLetterSize, HowToPlay.mLetterScale, Constants.Color1 )
    TextManager:AddLine( "while.paused", (HowToPlay.mLetterSize * -8.0), fLineY, fPageDuration )
    
    return fPageDuration
end

function HowToPlay:Init()
    HowToPlay.mUpdateThread = nil
    
    local fDesiredLetterSize = Constants.ValidDrawHeight / 30.0
    HowToPlay.mLetterScale = fDesiredLetterSize / 128.0
    HowToPlay.mLetterSize = fDesiredLetterSize
    
    HowToPlay.mStandardSleep = 0.125
    HowToPlay.mPauseSleep = 0.25    
    HowToPlay.mBreakSleep = 0.5    
    
    HowToPlay.mCurrentPage = 0
    HowToPlay.mDesiredPage = 0
    HowToPlay.mAutoAdvanceDuration = 99999.0
    HowToPlay.mNumPages = 5
    HowToPlay.mBackToMenuSelection = 6
    HowToPlay.mAllowInput = true
    
    HowToPlay.mSelectionColorTimer = 0.0
    HowToPlay.mSelectionColorScalar = 4.0
    HowToPlay.mSelectionColorAmount = 0.1
    
    HowToPlay.mImageScale = (Constants.ValidDrawHeight / 4.0) / 128.0
    HowToPlay.mImageDeck = DFGraphics.loadSpriteSheet( 'Images/misc', true )
    
    HowToPlay.mNumImages = 8
    HowToPlay.mImages = {}
    
    local i = 1
    for i = 1, HowToPlay.mNumImages do
        HowToPlay.mImages[i] = HowToPlay:NewHowToPlayImage()
    end
    HowToPlay.mImages[1]:Init( "controls", HowToPlay.mImageScale )
    HowToPlay.mImages[2]:Init( "beam", HowToPlay.mImageScale )
    HowToPlay.mImages[3]:Init( "note", HowToPlay.mImageScale )
    HowToPlay.mImages[4]:Init( "scratch", HowToPlay.mImageScale )
    HowToPlay.mImages[5]:Init( "perfect", HowToPlay.mImageScale )
    HowToPlay.mImages[6]:Init( "multiplier", HowToPlay.mImageScale )
    HowToPlay.mImages[7]:Init( "swipe1", HowToPlay.mImageScale )
    HowToPlay.mImages[8]:Init( "slices0", HowToPlay.mImageScale )
    
    --swipe is a slideshow!
    HowToPlay.mImages[7]:SetSlideshowParams( 12, 0.06 )
    HowToPlay.mImages[7]:AddSlideshowImage( 1, "swipe1" )
    HowToPlay.mImages[7]:AddSlideshowImage( 2, "swipe1" )
    HowToPlay.mImages[7]:AddSlideshowImage( 3, "swipe1" )
    HowToPlay.mImages[7]:AddSlideshowImage( 4, "swipe1" )
    HowToPlay.mImages[7]:AddSlideshowImage( 5, "swipe2" )
    HowToPlay.mImages[7]:AddSlideshowImage( 6, "swipe3" )
    HowToPlay.mImages[7]:AddSlideshowImage( 7, "swipe4" )
    HowToPlay.mImages[7]:AddSlideshowImage( 8, "swipe5" )
    HowToPlay.mImages[7]:AddSlideshowImage( 9, "swipe6" )
    HowToPlay.mImages[7]:AddSlideshowImage( 10, "swipe6" )
    HowToPlay.mImages[7]:AddSlideshowImage( 11, "swipe6" )
    HowToPlay.mImages[7]:AddSlideshowImage( 12, "swipe6" )
    
    --slices is also a slideshow!
    HowToPlay.mImages[8]:SetSlideshowParams( 19, 0.05 )
    HowToPlay.mImages[8]:AddSlideshowImage( 1, "slices0" )
    HowToPlay.mImages[8]:AddSlideshowImage( 2, "slices0" )
    HowToPlay.mImages[8]:AddSlideshowImage( 3, "slices0" )
    HowToPlay.mImages[8]:AddSlideshowImage( 4, "slices0" )
    HowToPlay.mImages[8]:AddSlideshowImage( 5, "slices0" )
    HowToPlay.mImages[8]:AddSlideshowImage( 6, "slices0" )
    HowToPlay.mImages[8]:AddSlideshowImage( 7, "slices1" )
    HowToPlay.mImages[8]:AddSlideshowImage( 8, "slices2" )
    HowToPlay.mImages[8]:AddSlideshowImage( 9, "slices3" )
    HowToPlay.mImages[8]:AddSlideshowImage( 10, "slices4" )
    HowToPlay.mImages[8]:AddSlideshowImage( 11, "slices5" )
    HowToPlay.mImages[8]:AddSlideshowImage( 12, "slices6" )
    HowToPlay.mImages[8]:AddSlideshowImage( 13, "slices7" )
    HowToPlay.mImages[8]:AddSlideshowImage( 14, "slices8" )
    HowToPlay.mImages[8]:AddSlideshowImage( 15, "slices8" )
    HowToPlay.mImages[8]:AddSlideshowImage( 16, "slices8" )
    HowToPlay.mImages[8]:AddSlideshowImage( 17, "slices8" )
    HowToPlay.mImages[8]:AddSlideshowImage( 18, "slices8" )
    HowToPlay.mImages[8]:AddSlideshowImage( 19, "slices8" )    
    
    HowToPlay.mSelectionTypeNone = -1
    HowToPlay.mSelectionTypeMenu = 0
    HowToPlay.mSelectionTypeForward = 1
    HowToPlay.mSelectionTypeBack = 2
    HowToPlay.mSelectionType = HowToPlay.mSelectionTypeNone
end

function HowToPlay:UpdateSize( tRenderer )
    local fDesiredLetterSize = Constants.ValidDrawHeight / 30.0
    HowToPlay.mLetterScale = fDesiredLetterSize / 128.0
    HowToPlay.mLetterSize = fDesiredLetterSize
    
    if HowToPlay:IsHowToPlayShowing() then
        HowToPlay:EndHowToPlay( tRenderer )
    end
end

function HowToPlay:BeginHowToPlay( tRenderer )
    HowToPlay.mCurrentPage = 0
    HowToPlay.mDesiredPage = 1
    HowToPlay.mAllowInput = true
    
    if HowToPlay.mUpdateThread == nil then
        HowToPlay.mUpdateThread = MOAIThread.new()
        HowToPlay.mUpdateThread:run(HowToPlay.UpdateThread, HowToPlay)
    end
    
    --create header props
    local sText = "how.to.play"
    local textWalker = string.gmatch(sText, ".")
    HowToPlay.mHeaderTextCount = #sText
    
    HowToPlay.mHeaderImageSize = Constants.ValidDrawHeight / 25.0
    HowToPlay.mHeaderImageScale = HowToPlay.mHeaderImageSize / 128.0    
    HowToPlay.mHeaderTextProps = {}
    
    local fXPos = -(HowToPlay.mHeaderImageSize * (HowToPlay.mHeaderTextCount + 2))
    local fYPos = Constants.ValidDrawHeight * -.225
    local fZPos = 2.0
    
    local i = 1
    local j = 1
    for i = 1, HowToPlay.mHeaderTextCount do
        HowToPlay.mHeaderTextProps[i] = MOAIProp.new()
        HowToPlay.mHeaderTextProps[i]:setDeck( RadFont.mFontDeck )        
        HowToPlay.mHeaderTextProps[i]:setScl( HowToPlay.mHeaderImageScale, -HowToPlay.mHeaderImageScale )
        HowToPlay.mHeaderTextProps[i]:setIndex( RadFont.mFontDeck.names[ textWalker() ] )
        HowToPlay.mHeaderTextProps[i]:setLoc( fXPos, fYPos, fZPos )
        HowToPlay.mHeaderTextProps[i]:setColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
        HowToPlay.mHeaderTextProps[i]:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
        HowToPlay.mHeaderTextProps[i]:setVisible( true )
        
        tRenderer.mUILayer:insertProp( HowToPlay.mHeaderTextProps[i] )        
        
        fXPos = fXPos + HowToPlay.mHeaderImageSize
    end
    
    for i = 1, HowToPlay.mNumImages do
        HowToPlay.mImages[i]:CreateProp( tRenderer, HowToPlay.mImageDeck, HowToPlay.mImageScale )   
    end
    
    HowToPlay.mMenuTextSize = Constants.ValidDrawHeight / 30.0
    HowToPlay.mMenuTextScale = HowToPlay.mMenuTextSize / 128.0
    HowToPlay.mMenuTextHighlightSize = HowToPlay.mMenuTextSize * 1.2
    HowToPlay.mMenuTextHighlightScale = HowToPlay.mMenuTextScale * 1.2
    HowToPlay.mMenuTextHalfDiff = (HowToPlay.mMenuTextHighlightSize - HowToPlay.mMenuTextSize) * 0.5
    
    HowToPlay.mMenuXPos = HowToPlay.mMenuTextSize * -16.0
    HowToPlay.mMenuWidth = HowToPlay.mMenuTextSize * 5.0
    HowToPlay.mMenuYPos = Constants.ValidDrawHeight * .3
    
    local menuText = "back"
    local menuTextWalker = string.gmatch(menuText, ".")
    HowToPlay.mMenuTextCount = 4
    HowToPlay.mMenuTextProps = {}
    
    for i = 1, HowToPlay.mMenuTextCount do
        HowToPlay.mMenuTextProps[i] = {}
        local iIndex = menuTextWalker()
        for j = 1, 2 do    
            HowToPlay.mMenuTextProps[i][j] = MOAIProp.new()
            HowToPlay.mMenuTextProps[i][j]:setDeck( RadFont.mFontDeck )        
            HowToPlay.mMenuTextProps[i][j]:setScl( HowToPlay.mMenuTextScale, -HowToPlay.mMenuTextScale )
            HowToPlay.mMenuTextProps[i][j]:setIndex( RadFont.mFontDeck.names[iIndex] )
            HowToPlay.mMenuTextProps[i][j]:setVisible( true )
            HowToPlay.mMenuTextProps[i][j]:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
            HowToPlay.mMenuTextProps[i][j]:setColor( 1.0, 1.0, 1.0, 1.0 )
            HowToPlay.mMenuTextProps[i][j]:setLoc( HowToPlay.mMenuXPos + ((i-1) * HowToPlay.mMenuTextSize), HowToPlay.mMenuYPos, fZPos )
            
            tRenderer.mUILayer:insertProp( HowToPlay.mMenuTextProps[i][j] )        
        end
    end      
    
    HowToPlay.mNavArrowXPos = {}
    HowToPlay.mNavArrowXPos[1] = HowToPlay.mMenuTextSize * 10.0
    HowToPlay.mNavArrowXPos[2] = HowToPlay.mMenuTextSize * 15.0
    HowToPlay.mNavArrowProps = {}
    
    for i = 1, 2 do
        HowToPlay.mNavArrowProps[i] = {}
        for j = 1, 2 do
            HowToPlay.mNavArrowProps[i][j] = MOAIProp.new()
            HowToPlay.mNavArrowProps[i][j]:setDeck( RadFont.mFontDeck )        
            HowToPlay.mNavArrowProps[i][j]:setScl( HowToPlay.mMenuTextScale, -HowToPlay.mMenuTextScale )
            HowToPlay.mNavArrowProps[i][j]:setVisible( true )
            HowToPlay.mNavArrowProps[i][j]:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
            HowToPlay.mNavArrowProps[i][j]:setColor( 1.0, 1.0, 1.0, 1.0 )
            HowToPlay.mNavArrowProps[i][j]:setLoc( HowToPlay.mNavArrowXPos[i], HowToPlay.mMenuYPos, fZPos )
        
            tRenderer.mUILayer:insertProp( HowToPlay.mNavArrowProps[i][j] )
        end
    end        
    
    HowToPlay.mNavArrowProps[1][1]:setIndex( RadFont.mFontDeck.names[ "-" ] )
    HowToPlay.mNavArrowProps[1][2]:setIndex( RadFont.mFontDeck.names[ "-" ] )
    HowToPlay.mNavArrowProps[2][1]:setIndex( RadFont.mFontDeck.names[ "^" ] )
    HowToPlay.mNavArrowProps[2][2]:setIndex( RadFont.mFontDeck.names[ "^" ] )
        
    HowToPlay.mEffectsZ = 20.0
    HowToPlay.mFingerEffects = {}
    for i = 1, 2 do
        HowToPlay.mFingerEffects[i] = RadEffects:NewFingerEffect("Effects/Particles/Fingers/HowTo.particles", false)
        if HowToPlay.mFingerEffects[i] then
            local fingerEffectProp = HowToPlay.mFingerEffects[i]:getProp()
            HowToPlay.mFingerEffects[i]:setTargetProp(fingerEffectProp)        
            HowToPlay.mFingerEffects[i]:setVisible(false)
        end        
    end  
end

function HowToPlay:Update( tGameInputs )
    --don't bother if our thread isn't active
    if HowToPlay.mUpdateThread ~= nil then
        local i = 1    
        for i = 1, HowToPlay.mNumImages do
            HowToPlay.mImages[i]:UpdateVisibility()
        end
        
        --update pointer prop
        for i = 1, 2 do
            if tGameInputs[i].IsTouched() then            
                if HowToPlay.mFingerEffects[i] then
                    local fingerEffectProp = HowToPlay.mFingerEffects[i]:getProp()
                    fingerEffectProp:setLoc( tGameInputs[i].mX, tGameInputs[i].mY, HowToPlay.mEffectsZ )
                    HowToPlay.mFingerEffects[i]:setVisible( true )
                end            
            else
                if HowToPlay.mFingerEffects[i] then
                    HowToPlay.mFingerEffects[i]:setVisible( false )
                end             
            end
        end    
        
        --see if pointer is over a selection        
        if HowToPlay.mAllowInput then
            local bSelectionValid = false
            local bPressed = false
        
            local yMin = HowToPlay.mMenuYPos - (HowToPlay.mMenuTextSize * 4.0)
            local yMax = HowToPlay.mMenuYPos + HowToPlay.mMenuTextSize
        
            for i = 1, 2 do            
                if tGameInputs[i].mY > yMin and tGameInputs[i].mY < yMax then
                    local fMenuMinX = HowToPlay.mMenuXPos - HowToPlay.mMenuTextSize
                    local fMenuMaxX = HowToPlay.mMenuXPos + HowToPlay.mMenuWidth + (HowToPlay.mMenuTextSize * 2.0)
                
                    local fNav1MinX = HowToPlay.mNavArrowXPos[1] - (HowToPlay.mMenuTextSize * 3.0)
                    local fNav1MaxX = HowToPlay.mNavArrowXPos[1] + (HowToPlay.mMenuTextSize * 2.5)

                    local fNav2MinX = HowToPlay.mNavArrowXPos[2] - (HowToPlay.mMenuTextSize * 2.5)
                    local fNav2MaxX = HowToPlay.mNavArrowXPos[2] + (HowToPlay.mMenuTextSize * 2.5)
                
                    if tGameInputs[i].mX > fMenuMinX and tGameInputs[i].mX < fMenuMaxX then
                        if tGameInputs[i]:IsTouched() then
                            HowToPlay.mSelectionType = HowToPlay.mSelectionTypeMenu
                            bSelectionValid = true
                            break                        
                        elseif tGameInputs[i]:WasTouched() then
                            HowToPlay.mSelectionType = HowToPlay.mSelectionTypeMenu
                            bPressed = true
                            break
                        end
                    elseif tGameInputs[i].mX > fNav1MinX and tGameInputs[i].mX < fNav1MaxX then
                        if tGameInputs[i]:IsTouched() then
                            HowToPlay.mSelectionType = HowToPlay.mSelectionTypeBack
                            bSelectionValid = true
                            break                        
                        elseif tGameInputs[i]:WasTouched() then
                            HowToPlay.mSelectionType = HowToPlay.mSelectionTypeBack
                            bPressed = true
                            break
                        end       
                    elseif tGameInputs[i].mX > fNav2MinX and tGameInputs[i].mX < fNav2MaxX then
                        if tGameInputs[i]:IsTouched() then
                            HowToPlay.mSelectionType = HowToPlay.mSelectionTypeForward
                            bSelectionValid = true
                            break                        
                        elseif tGameInputs[i]:WasTouched() then
                            HowToPlay.mSelectionType = HowToPlay.mSelectionTypeForward
                            bPressed = true
                            break
                        end
                    end
                end
            end
                        
            local bEscDown = bSelectionValid and HowToPlay.mSelectionType == HowToPlay.mSelectionTypeMenu
            HowToPlay:PositionEsc( bEscDown )

            local bBackDown = bSelectionValid and HowToPlay.mSelectionType == HowToPlay.mSelectionTypeBack
            HowToPlay:PositionNavArrow( 1, bBackDown )
            
            local bForwardDown = bSelectionValid and HowToPlay.mSelectionType == HowToPlay.mSelectionTypeForward
            HowToPlay:PositionNavArrow( 2, bForwardDown )
            
            if bPressed then
                if HowToPlay.mSelectionType == HowToPlay.mSelectionTypeMenu then
                    --exit!
                    HowToPlay.mDesiredPage = HowToPlay.mBackToMenuSelection
                    AudioManager:PlaySound( 'Radius/SFX/MenuSelect' )        
                elseif HowToPlay.mSelectionType == HowToPlay.mSelectionTypeForward then
                    --page forward
                    HowToPlay.mDesiredPage = HowToPlay.mCurrentPage + 1
                    if HowToPlay.mDesiredPage > HowToPlay.mNumPages then
                        HowToPlay.mDesiredPage = 1
                    end
                    AudioManager:PlaySound( 'Radius/SFX/Score_Show' )       
                elseif HowToPlay.mSelectionType == HowToPlay.mSelectionTypeBack then
                    --page back
                    HowToPlay.mDesiredPage = HowToPlay.mCurrentPage - 1
                    if HowToPlay.mDesiredPage < 1 then
                        HowToPlay.mDesiredPage = HowToPlay.mNumPages
                    end     
                    AudioManager:PlaySound( 'Radius/SFX/Score_Show' )                 
                end
            
                HowToPlay.mSelectionType = HowToPlay.mSelectionTypeNone                
            end 
        end
    end    
end

function HowToPlay:PositionEsc( bDepressed )
    local fDepressedOffset = HowToPlay.mMenuTextSize * 0.15
    if bDepressed then
        fDepressedOffset = HowToPlay.mMenuTextSize * 0.075
    end
    
    local i = 1
    local j = 1
    for i = 1, HowToPlay.mMenuTextCount do
        HowToPlay.mMenuTextProps[i][1]:setColor( 0.0, 0.0, 0.0, 0.25 )
        local x = HowToPlay.mMenuXPos + ((i-1) * HowToPlay.mMenuTextSize)
        HowToPlay.mMenuTextProps[i][1]:setLoc( x - fDepressedOffset, HowToPlay.mMenuYPos + fDepressedOffset, 0.0 )    
        HowToPlay.mMenuTextProps[i][2]:setLoc( x + fDepressedOffset, HowToPlay.mMenuYPos - fDepressedOffset, 2.0 ) 
    end
end

function HowToPlay:PositionNavArrow( iIndex, bDepressed )
    local fDepressedOffset = HowToPlay.mMenuTextSize * 0.15
    if bDepressed then
        fDepressedOffset = HowToPlay.mMenuTextSize * 0.075
    end
    
    HowToPlay.mNavArrowProps[iIndex][1]:setColor( 0.0, 0.0, 0.0, 0.25 )
    HowToPlay.mNavArrowProps[iIndex][1]:setLoc( HowToPlay.mNavArrowXPos[iIndex] + fDepressedOffset, HowToPlay.mMenuYPos + fDepressedOffset, 0.0 )
    HowToPlay.mNavArrowProps[iIndex][2]:setLoc( HowToPlay.mNavArrowXPos[iIndex] - fDepressedOffset, HowToPlay.mMenuYPos - fDepressedOffset, 2.0 )
end

function HowToPlay:IsHowToPlayShowing()
    if HowToPlay.mUpdateThread == nil then
        return false
    end
    return HowToPlay.mUpdateThread:isDone() or HowToPlay.mUpdateThread:isBusy()
end

function HowToPlay:EndHowToPlay( tRenderer )
    if HowToPlay.mUpdateThread ~= nil then
        HowToPlay.mUpdateThread:stop()
        HowToPlay.mUpdateThread = nil
    end
    
    local i = 1
    local j = 1
    for i = 1, HowToPlay.mHeaderTextCount do
        tRenderer.mUILayer:removeProp( HowToPlay.mHeaderTextProps[i] )
    end    
    
    for i = 1, HowToPlay.mNumImages do
        if HowToPlay.mImages[i].mImageProp ~= nil then
            tRenderer.mUILayer:removeProp( HowToPlay.mImages[i].mImageProp )
            HowToPlay.mImages[i].mImageProp = nil
        end
    end    
    
    for i = 1, HowToPlay.mMenuTextCount do
        for j = 1, 2 do
            tRenderer.mUILayer:removeProp( HowToPlay.mMenuTextProps[i][j] )
        end
    end
    
    tRenderer.mUILayer:removeProp( HowToPlay.mNavArrowProps[1][1] )
    tRenderer.mUILayer:removeProp( HowToPlay.mNavArrowProps[1][2] )
    tRenderer.mUILayer:removeProp( HowToPlay.mNavArrowProps[2][1] )   
    tRenderer.mUILayer:removeProp( HowToPlay.mNavArrowProps[2][2] )   
    
    for i = 1, 2 do
        if HowToPlay.mFingerEffects[i] then    
            HowToPlay.mFingerEffects[i]:stop(false)
        end
        HowToPlay.mFingerEffects[i] = nil      
    end 
end

return HowToPlay