local Constants = require('Constants')
local DFMath = require('DFCommon.Math')
local DFGraphics = require('DFCommon.Graphics')
local HUD = require('HUD')
local AudioManager = require('AudioManager')
local InputTutorial = {}

function InputTutorial:Init( tRenderer )
    InputTutorial:DetermineSize()
    
    InputTutorial.mImageDeck = DFGraphics.loadSpriteSheet( 'Images/misc', true )   
    
    InputTutorial.mNumInputProps = 2
    InputTutorial.mPropZ = -10.0
    
    InputTutorial.mInputPropsInner = {}
    InputTutorial.mInputPropsOuter = {}
    local i = 1
    for i = 1, InputTutorial.mNumInputProps do
        InputTutorial.mInputPropsOuter[i] = ShapeProp.new()
        InputTutorial.mInputPropsOuter[i]:setDeck( tRenderer.mShapeDeck )
        InputTutorial.mInputPropsOuter[i]:SetNumSides( 16 )
        InputTutorial.mInputPropsOuter[i]:SetRadii( InputTutorial.mPropRadiusOuter[1], InputTutorial.mPropRadiusOuter[2] )  
        InputTutorial.mInputPropsOuter[i]:SetColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1.0 )
        InputTutorial.mInputPropsOuter[i]:SetDrawMode( ShapeProp.DrawModeArc )
        InputTutorial.mInputPropsOuter[i]:SetArcRadians( 0.0, DFMath.twoPi ) 
        InputTutorial.mInputPropsOuter[i]:setVisible( false )

        InputTutorial.mInputPropsInner[i] = ShapeProp.new()
        InputTutorial.mInputPropsInner[i]:setDeck( tRenderer.mShapeDeck )
        InputTutorial.mInputPropsInner[i]:SetNumSides( 16 )
        InputTutorial.mInputPropsInner[i]:SetRadii( InputTutorial.mPropRadiusInner[1], InputTutorial.mPropRadiusInner[2] )  
        InputTutorial.mInputPropsInner[i]:SetColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1.0 )
        InputTutorial.mInputPropsInner[i]:SetDrawMode( ShapeProp.DrawModeArc )
        InputTutorial.mInputPropsInner[i]:SetArcRadians( 0.0, DFMath.twoPi ) 
        InputTutorial.mInputPropsInner[i]:setVisible( false )
        
        tRenderer.mGameLayer:insertProp( InputTutorial.mInputPropsOuter[i] )       
        tRenderer.mGameLayer:insertProp( InputTutorial.mInputPropsInner[i] )       
    end
    
    InputTutorial.mInputPropsOuter[1]:setLoc( InputTutorial.mPropXPos, 0.0, InputTutorial.mPropZ )
    InputTutorial.mInputPropsOuter[2]:setLoc( -InputTutorial.mPropXPos, 0.0, InputTutorial.mPropZ )
    InputTutorial.mInputPropsInner[1]:setLoc( InputTutorial.mPropXPos, 0.0, InputTutorial.mPropZ )
    InputTutorial.mInputPropsInner[2]:setLoc( -InputTutorial.mPropXPos, 0.0, InputTutorial.mPropZ )
    
    InputTutorial.mProp1Rad = { 0.0, math.pi * 1.25, 0.0, math.pi * 1.25 }
    InputTutorial.mProp2Rad = { 0.0, math.pi * 1.25, 0.0, math.pi * 1.25 }
    InputTutorial.mPropUpdateRad = { math.pi * 0.25, math.pi * 8.0 }
    InputTutorial.mPropUpdateDecay = 1.0
    InputTutorial.mPropSound = { nil, nil }
    
    InputTutorial.mPropNearbyAmount = { 0.0, 0.0 }
    
    InputTutorial.mBeamCompleteAmount = 1.0
    
    InputTutorial.mShowHUDDuration = 2.0
    InputTutorial.mShowHUDTimer = InputTutorial.mShowHUDDuration
    InputTutorial.mHUDIDs = {-1, -1}
    InputTutorial.mHUDTimer = 0.0
    InputTutorial.mHUDSpeed = 4.0
    InputTutorial.mHUDState = 0
end

function InputTutorial:UpdateSize()
    InputTutorial:DetermineSize()
    
    local i = 1
    for i = 1, InputTutorial.mNumInputProps do
        InputTutorial.mInputPropsOuter[i]:SetRadii( InputTutorial.mPropRadiusOuter[1], InputTutorial.mPropRadiusOuter[2] )  
        InputTutorial.mInputPropsInner[i]:SetRadii( InputTutorial.mPropRadiusInner[1], InputTutorial.mPropRadiusInner[2] )  
    end    
    
    InputTutorial.mInputPropsOuter[1]:setLoc( InputTutorial.mPropXPos, 0.0, InputTutorial.mPropZ )
    InputTutorial.mInputPropsOuter[2]:setLoc( -InputTutorial.mPropXPos, 0.0, InputTutorial.mPropZ )
    InputTutorial.mInputPropsInner[1]:setLoc( InputTutorial.mPropXPos, 0.0, InputTutorial.mPropZ )
    InputTutorial.mInputPropsInner[2]:setLoc( -InputTutorial.mPropXPos, 0.0, InputTutorial.mPropZ )
        
    --we do this to jiggle the handle on the text rendering
    InputTutorial.mShowHUDTimer = InputTutorial.mShowHUDDuration
end

function InputTutorial:DetermineSize()
    InputTutorial.mPropRadiusOuter = { Constants.GameRingRadius * 0.25, Constants.GameRingRadius * 0.3 }
    InputTutorial.mPropRadiusInner = { Constants.GameRingRadius * 0.17, Constants.GameRingRadius * 0.2 }
    InputTutorial.mPropXPos = Constants.GameRingRadius * 1.75
    InputTutorial.mNearbyDistance = Constants.GameRingRadius * 0.3
    InputTutorial.mImageScale = (Constants.ValidDrawHeight / 5.0) / 128.0 
end

function InputTutorial:InitForIntro(tRenderer)
    local i = 1
    for i = 1, InputTutorial.mNumInputProps do
        InputTutorial.mInputPropsOuter[i]:setVisible( true )
        InputTutorial.mInputPropsInner[i]:setVisible( true )
    end   
    InputTutorial.mShowHUDTimer = InputTutorial.mShowHUDDuration
    
    InputTutorial.mPropSound[1] = AudioManager:PlaySound( 'Radius/SFX/HoldTwirl_RIGHT' )
    if InputTutorial.mPropSound[1] ~= nil then
        InputTutorial.mPropSound[1]:setParameter( 'spinspeed', 0.0 )
    end
    
    InputTutorial.mPropSound[2] = AudioManager:PlaySound( 'Radius/SFX/HoldTwirl_LEFT' )
    if InputTutorial.mPropSound[2] ~= nil then
        InputTutorial.mPropSound[2]:setParameter( 'spinspeed', 0.0 )
    end
end

function InputTutorial:Disable(tRenderer)
    local i = 1
    for i = 1, InputTutorial.mNumInputProps do
        InputTutorial.mInputPropsOuter[i]:setVisible( false )
        InputTutorial.mInputPropsInner[i]:setVisible( false )
    end   
    
    for i = 1, 2 do
        InputTutorial.mPropNearbyAmount[i] = 0.0
    end      
    
    InputTutorial.mBeamCompleteAmount = 1.0
    
    AudioManager:StopSound( InputTutorial.mPropSound[1], true )
    AudioManager:StopSound( InputTutorial.mPropSound[2], true )
    
    InputTutorial.mHUDIDs[1] = -1
    InputTutorial.mHUDIDs[2] = -1
end

function InputTutorial:Update( tGameInputs, tChargeManager, tTextManager )
    --figure out how many inputs are active
    local iNumActiveInputs = 0
    local i = 1
    for i = 1, Constants.NumGameInputs do
        if tGameInputs[i]:IsTouched() then
            iNumActiveInputs = iNumActiveInputs + 1
        end
    end
    
    --update angles of prop 1
    local fSpeed = DFMath.lerp( InputTutorial.mPropUpdateRad[1], InputTutorial.mPropUpdateRad[2], InputTutorial.mPropNearbyAmount[1] )
    local fAngleStep = fSpeed * Constants.FrameStep
    
    InputTutorial.mProp1Rad[1] = InputTutorial.mProp1Rad[1] + fAngleStep
    InputTutorial.mProp1Rad[2] = InputTutorial.mProp1Rad[2] + fAngleStep
    if InputTutorial.mProp1Rad[2] > DFMath.twoPi then 
        InputTutorial.mProp1Rad[2] = InputTutorial.mProp1Rad[2] - DFMath.twoPi
        InputTutorial.mProp1Rad[1] = InputTutorial.mProp1Rad[1] - DFMath.twoPi
    end

    InputTutorial.mProp1Rad[3] = InputTutorial.mProp1Rad[3] - fAngleStep
    InputTutorial.mProp1Rad[4] = InputTutorial.mProp1Rad[4] - fAngleStep
    if InputTutorial.mProp1Rad[3] < 0.0 then 
        InputTutorial.mProp1Rad[4] = InputTutorial.mProp1Rad[4] + DFMath.twoPi
        InputTutorial.mProp1Rad[3] = InputTutorial.mProp1Rad[3] + DFMath.twoPi
    end
    
    --update angles of prop 2
    fSpeed = DFMath.lerp( InputTutorial.mPropUpdateRad[1], InputTutorial.mPropUpdateRad[2], InputTutorial.mPropNearbyAmount[2] )
    fAngleStep = fSpeed * Constants.FrameStep
    
    InputTutorial.mProp2Rad[1] = InputTutorial.mProp2Rad[1] - fAngleStep
    InputTutorial.mProp2Rad[2] = InputTutorial.mProp2Rad[2] - fAngleStep
    if InputTutorial.mProp2Rad[1] < 0.0 then 
        InputTutorial.mProp2Rad[2] = InputTutorial.mProp2Rad[2] + DFMath.twoPi
        InputTutorial.mProp2Rad[1] = InputTutorial.mProp2Rad[1] + DFMath.twoPi
    end
    
    InputTutorial.mProp2Rad[3] = InputTutorial.mProp2Rad[3] + fAngleStep
    InputTutorial.mProp2Rad[4] = InputTutorial.mProp2Rad[4] + fAngleStep
    if InputTutorial.mProp2Rad[4] > DFMath.twoPi then 
        InputTutorial.mProp2Rad[4] = InputTutorial.mProp2Rad[4] - DFMath.twoPi
        InputTutorial.mProp2Rad[3] = InputTutorial.mProp2Rad[3] - DFMath.twoPi
    end
    
    --set radians
    InputTutorial.mInputPropsOuter[1]:SetArcRadians( InputTutorial.mProp1Rad[1], InputTutorial.mProp1Rad[2] )
    InputTutorial.mInputPropsOuter[2]:SetArcRadians( InputTutorial.mProp2Rad[1], InputTutorial.mProp2Rad[2] )
    InputTutorial.mInputPropsInner[1]:SetArcRadians( InputTutorial.mProp1Rad[3], InputTutorial.mProp1Rad[4] )
    InputTutorial.mInputPropsInner[2]:SetArcRadians( InputTutorial.mProp2Rad[3], InputTutorial.mProp2Rad[4] )
    
    for i = 1, InputTutorial.mNumInputProps do
        local r, g, b
        r = DFMath.lerp( Constants.Color3[1], Constants.Color1[1], InputTutorial.mPropNearbyAmount[i] )
        g = DFMath.lerp( Constants.Color3[2], Constants.Color1[2], InputTutorial.mPropNearbyAmount[i] )
        b = DFMath.lerp( Constants.Color3[3], Constants.Color1[3], InputTutorial.mPropNearbyAmount[i] )
        InputTutorial.mInputPropsOuter[i]:SetColor( r, g, b, 1.0 )
        InputTutorial.mInputPropsInner[i]:SetColor( r, g, b, 1.0 )
        
        local fBeamRatio = 1.0 - InputTutorial.mBeamCompleteAmount
        InputTutorial.mInputPropsOuter[i]:SetRadii( InputTutorial.mPropRadiusOuter[1] * fBeamRatio, InputTutorial.mPropRadiusOuter[2] * fBeamRatio )
        InputTutorial.mInputPropsInner[i]:SetRadii( InputTutorial.mPropRadiusInner[1] * fBeamRatio, InputTutorial.mPropRadiusInner[2] * fBeamRatio )
    end
    
    local fDecayStep = InputTutorial.mPropUpdateDecay * Constants.FrameStep
    for i = 1, 2 do
        InputTutorial.mPropNearbyAmount[i] = math.max( InputTutorial.mPropNearbyAmount[i] - fDecayStep, 0.0 )
    end      
    
    --update sounds
    for i = 1, 2 do
        if InputTutorial.mPropSound[i] ~= nil then
            InputTutorial.mPropSound[i]:setParameter( 'spinspeed', InputTutorial.mPropNearbyAmount[i] )
        end
    end
    
    local fLowestChargePercent = 1.0
    for i = 1, 2 do
        local fPercent = tChargeManager.mCharges[i].mChargeDisabledTimer / tChargeManager.mCharges[i].mChargeDisabledDuration
        fLowestChargePercent = math.min( fLowestChargePercent, fPercent )
    end
    
    --should we show HUD text?
    InputTutorial.mShowHUDTimer = InputTutorial.mShowHUDTimer - Constants.FrameStep
    local bShouldShowText = (InputTutorial.mShowHUDTimer <= 0.0) and (fLowestChargePercent >= 0.75)
    
    --if we're not showing the hud but we should, do it!
    if InputTutorial.mHUDState == Constants.mMenuDrawStateNone then    
        if bShouldShowText then
            InputTutorial.mHUDState = Constants.mMenuDrawStateIn
            InputTutorial:ShowHUDText( tTextManager )
        end
    --if we're showing the HUD, but the player got it figured out, kill it!
    elseif InputTutorial.mHUDState == Constants.mMenuDrawStateShow and not bShouldShowText then
        if InputTutorial.mHUDState ~= Constants.mMenuDrawStateNone then
            tTextManager:ClearAll()
            InputTutorial.mHUDState = Constants.mMenuDrawStateOut
        end
    else
        --we're showing and should be showing
        if InputTutorial.mHUDState == Constants.mMenuDrawStateIn then
            InputTutorial.mHUDTimer = InputTutorial.mHUDTimer + (InputTutorial.mHUDSpeed * Constants.FrameStep)
            if InputTutorial.mHUDTimer >= 1.0 then
                InputTutorial.mHUDTimer = 1.0
                InputTutorial.mHUDState = Constants.mMenuDrawStateShow
                HUD:ShowTutorialText( InputTutorial.mHUDIDs[1], 999999.0, tTextManager )
                HUD:ShowTutorialText( InputTutorial.mHUDIDs[2], 999999.0, tTextManager )
            end
        elseif InputTutorial.mHUDState == Constants.mMenuDrawStateShow then
            --hold!
            InputTutorial.mHUDTimer = 1.0
        elseif InputTutorial.mHUDState == Constants.mMenuDrawStateOut then
            --fade out
            InputTutorial.mHUDTimer = InputTutorial.mHUDTimer - (InputTutorial.mHUDSpeed * Constants.FrameStep)
            if InputTutorial.mHUDTimer <= 0.0 then
                InputTutorial.mHUDTimer = 0.0
                InputTutorial.mHUDState = Constants.mMenuDrawStateNone
                InputTutorial.mShowHUDTimer = InputTutorial.mShowHUDDuration
            end        
        end
        
        HUD:UpdateTutorialDrawRatio( InputTutorial.mHUDIDs[1], InputTutorial.mHUDTimer, InputTutorial.mPropXPos, 0.0 )                  
        HUD:UpdateTutorialDrawRatio( InputTutorial.mHUDIDs[2], InputTutorial.mHUDTimer, -InputTutorial.mPropXPos, 0.0 )                  
    end
end

function InputTutorial:ShowHUDText( tTextManager )
    HUD:ClearTutorialText()
    InputTutorial.mHUDIDs[1] = HUD:InitTutorialText( "hold", 4, tTextManager, Constants.mHUDPositionHold1, Constants.Color1 )
    InputTutorial.mHUDIDs[2] = HUD:InitTutorialText( "hold", 4, tTextManager, Constants.mHUDPositionHold2, Constants.Color1 )                    
end

function InputTutorial:ClearHUDText()
    InputTutorial.mShowHUDTimer = InputTutorial.mShowHUDDuration
end

function InputTutorial:SetBeamCompleteAmount(fAmount)
    InputTutorial.mBeamCompleteAmount = fAmount
end

function InputTutorial:UpdateChargeNearInputProp(fPosX, fPosY, iIndex)
    local i = 1
    for i = 1, InputTutorial.mNumInputProps do
        local fPropX, fPropY = InputTutorial.mInputPropsOuter[i]:getLoc()
        local fDistance = DFMath.distance2D( fPosX, fPosY, fPropX, fPropY )
        if fDistance < InputTutorial.mNearbyDistance then
            InputTutorial.mPropNearbyAmount[i] = 1.0
            return InputTutorial.mPropNearbyAmount[i]
        end
    end       
    
    return 0.0
end

return InputTutorial