local Constants = require('Constants')
local ChargeManager = require('ChargeManager')
local Score = require('Score')
local DFMath = require('DFCommon.Math')
local RadEffects = require('RadEffects')
local AudioManager = require('AudioManager')
local InputVisuals = {}
  
function InputVisuals:Init( tRenderer )
    local i = 1
    
    InputVisuals.tGameInputs = {}
      
    InputVisuals.mFingerEffects = {}

    InputVisuals.mPropZ = -10.0
    InputVisuals.mNumInputProps = Constants.NumGameInputs
    InputVisuals.mUseAltGameScale = false
    InputVisuals.mGameComplete = false
end
      
function InputVisuals:ClearFingerEffects()
    for i = 1, InputVisuals.mNumInputProps do
        if i <= #InputVisuals.mFingerEffects then
            local fingerEffect = InputVisuals.mFingerEffects[i]
            if fingerEffect then
                fingerEffect:stop(false)
                fingerEffect:removeFromSceneLayer()
            end
        end
    end
    
    InputVisuals.mFingerEffects = {}
end

function InputVisuals:SetFingerEffects(sParticlePath, bUseParticleLayer)      
    InputVisuals:ClearFingerEffects()
    
    if not bUseParticleLayer then bUseParticleLayer = false end
    
    --only on the main inputs
    for i = 1, 2 do
        local fingerEffect = RadEffects:NewFingerEffect(sParticlePath, bUseParticleLayer)
        
        if fingerEffect then
            local fingerEffectProp = fingerEffect:getProp()
            fingerEffect:setTargetProp(fingerEffectProp)        
            fingerEffect:setVisible(false)
            table.insert(InputVisuals.mFingerEffects, fingerEffect)
        end
    end    
end
      
function InputVisuals:Reset()    
    InputVisuals.mPropZ = -10.0
end
  
function InputVisuals:UpdateSize()
end
  
function InputVisuals:SetAltGameMode( bAltGameMode )
    InputVisuals.mUseAltGameScale = bAltGameMode
end

function InputVisuals:SetGameComplete( bGameComplete )
    InputVisuals.mGameComplete = bGameComplete
end

function InputVisuals:UpdateIntro( tGameInputs )
    InputVisuals:Update( tGameInputs )
end

function InputVisuals:Update( tGameInputs )
    InputVisuals.tGameInputs = tGameInputs
        
    for i = 1, InputVisuals.mNumInputProps do
        if tGameInputs[i].IsTouched() then            
            local fPropZ = InputVisuals.mPropZ
            if InputVisuals.mUseAltGameScale then
                --if we're in the alt game mode, move depth forward
                fPropZ = 10.0
            end
                        
            if i <= #InputVisuals.mFingerEffects then
                local fingerEffect = InputVisuals.mFingerEffects[i]
                if fingerEffect then
                    local fingerEffectProp = fingerEffect:getProp()
                    if fingerEffectProp then
                        fingerEffectProp:setLoc( tGameInputs[i].mX, tGameInputs[i].mY, fPropZ )
                    end
                    InputVisuals.mFingerEffects[i]:setVisible( true )
                end
            end
        else
            if i <= #InputVisuals.mFingerEffects then
                local fingerEffect = InputVisuals.mFingerEffects[i]
                if fingerEffect then InputVisuals.mFingerEffects[i]:setVisible( false ) end
            end
        end 
    end
end
    
function InputVisuals:UpdateGameComplete( tGameInputs )
    --we've got special finger effects at the end.. when they go away, just clear them
    local i = 1
    if #InputVisuals.mFingerEffects > 0 then
        local bEffectsExist = true
        for i = 1, 2 do
            local fingerEffect = InputVisuals.mFingerEffects[i]            
            if fingerEffect then
                local fingerEffectProp = fingerEffect:getProp()
                if fingerEffectProp == nil then
                    bEffectsExist = false
                    break
                end
            else
                bEffectsExist = false
                break
            end
        end 
        
        --if they've failed, clear the table
        if not bEffectsExist then
            InputVisuals:ClearFingerEffects()
        end
    end
    
    InputVisuals:Update( tGameInputs )
end
  
return InputVisuals