local Constants = require('Constants')
local DFUtil = require('DFCommon.Util')
local DFSaveLoad = require('DFCommon.SaveLoad')
local TextManager = require('TextManager')
local AudioManager = require('AudioManager')
local RadFont = require('RadFont')
local DFMath = require('DFCommon.Math')
local RadEffects = require('RadEffects')
local Leaderboard = {}

function Leaderboard:UpdateThread()
    DFUtil.sleep(0.35)

    Leaderboard:LoadHighScores()
    
    while true do
        if Leaderboard.mCurrentPage ~= Leaderboard.mDesiredPage then
            TextManager:ClearAll()        
           
            --update current page index
            Leaderboard.mPageIndex = 1
            Leaderboard.mCurrentPage = Leaderboard.mDesiredPage   

            Leaderboard:SetupLeaders()
                        
            if Leaderboard.mDesiredPage == Leaderboard.mBackToMenuSelection then
                break
            end
        else
            --udpate page
            Leaderboard:DisplayLeaders()     
        end
    end
end

local function ScoresReported( s1, s2, s3, s4, s5, s6, s7, s8, s9, s10 )
    if s1 == nil then
        Leaderboard.DisplayLB = Constants.Leaderboards
    else
        local tScoreTable = { s1, s2, s3, s4, s5, s6, s7, s8, s9, s10 }
        local i = 1
        for i = 1, 10 do
            if tScoreTable[i] ~= nil then
                local sInitials = DFRad:ValueToInitials( tScoreTable[i].context )
                Leaderboard.DisplayLB[i][1] = sInitials
                Leaderboard.DisplayLB[i][2] = tostring( tScoreTable[i].value )
            else
                Leaderboard.DisplayLB[i][1] = "..."
                Leaderboard.DisplayLB[i][2] = "0"
            end
        end 
    end   
    
    Leaderboard.WaitForScoreReport = false
end

function Leaderboard:SetupLeaders()
    TextManager:SetFadeAttributes( 0.0, 0.0 )
    TextManager:SetTextAttributes( Leaderboard.mLeaderLetterSize, Leaderboard.mLeaderLetterScale, Constants.Color3 ) 
    
    local fTitleX = (Leaderboard.mHeaderImageSize * -2.0)
    local fLineY = Constants.ValidDrawHeight * -.2
    
    --default to the display leaderboard being the local
    local i = 1
    for i = 1, 10 do    
        Leaderboard.DisplayLB[i][1] = Constants.Leaderboards[i][1]
        Leaderboard.DisplayLB[i][2] = Constants.Leaderboards[i][2]
    end
    
    Leaderboard.WaitForScoreReport = false
    
    if Leaderboard.mCurrentPage == 1 then           
        --don't qualify the leaderboard if we're not using them
        if MOAIGameCenterIOS then 
            TextManager:AddLine( "personal", fTitleX, fLineY, Leaderboard.mLongTime )
        end
    elseif Leaderboard.mCurrentPage == 2 then       
        TextManager:AddLine( "friends", fTitleX, fLineY, Leaderboard.mLongTime )
        Leaderboard:SetupFriend()
    elseif Leaderboard.mCurrentPage == 3 then     
        TextManager:AddLine( "world", fTitleX, fLineY, Leaderboard.mLongTime )  
        Leaderboard:SetupGlobal()
    end
end

function Leaderboard:SetupFriend()
    if MOAIGameCenterIOS then
        MOAIGameCenterIOS.setGetScoresCallback( ScoresReported )
        MOAIGameCenterIOS.getScores( Constants.IOSLeaderboardID, GKLeaderboardPlayerScopeFriendsOnly )
        Leaderboard.WaitForScoreReport = true
    end
end

function Leaderboard:SetupGlobal()
    if MOAIGameCenterIOS then
        MOAIGameCenterIOS.setGetScoresCallback( ScoresReported )
        MOAIGameCenterIOS.getScores( Constants.IOSLeaderboardID )
        Leaderboard.WaitForScoreReport = true
    end
end

function Leaderboard:DisplayLeaders()
    --if we're waiting for a score report, just spin and get out
    if Leaderboard.WaitForScoreReport == true then
        DFUtil.sleep( Leaderboard.mStandardSleep )
        return 
    end
    
    TextManager:SetFadeAttributes( 0.0, 0.0 )
    local fNameX = -(Leaderboard.mLeaderLetterSize * 4.0)
    local fScoreX = Leaderboard.mLeaderLetterSize
    
    --standard score
    if Leaderboard.mPageIndex < 10 then  
        local iLeader = 11 - Leaderboard.mPageIndex
        if Constants.LeaderboardIndex == iLeader and Leaderboard.mCurrentPage == 1 then
            TextManager:SetTextAttributes( Leaderboard.mLeaderLetterSize, Leaderboard.mLeaderLetterScale, Constants.SpecialColor )
        else
            TextManager:SetTextAttributes( Leaderboard.mLeaderLetterSize, Leaderboard.mLeaderLetterScale, Constants.Color2 )            
        end
    
        local fLineY = (Constants.ValidDrawHeight * 0.225) - (Leaderboard.mLeaderLetterSize * (Leaderboard.mPageIndex-1))
        
        TextManager:AddLine( Leaderboard.DisplayLB[iLeader][1], fNameX, fLineY, Leaderboard.mLongTime )
        TextManager:AddLine( Leaderboard.DisplayLB[iLeader][2], fScoreX, fLineY, Leaderboard.mLongTime )
        AudioManager:PlaySound( 'Radius/SFX/Score_Show' )  
  
        if Leaderboard.mPageIndex == 9 then
            DFUtil.sleep( Leaderboard.mLongSleep )
        else
            DFUtil.sleep( Leaderboard.mStandardSleep )
        end
    elseif Leaderboard.mPageIndex == 10 then  
        --best score
        local iLeader = 1
        if Constants.LeaderboardIndex == iLeader and Leaderboard.mCurrentPage == 1 then
            TextManager:SetTextAttributes( Leaderboard.mProLetterSize, Leaderboard.mProLetterScale, Constants.SpecialColor )
        else
            TextManager:SetTextAttributes( Leaderboard.mProLetterSize, Leaderboard.mProLetterScale, Constants.Color1 )
        end    
        
        local fNameX = (Leaderboard.mProLetterSize * -3.0) - Leaderboard.mLeaderLetterSize
        local fLineY = (Constants.ValidDrawHeight * 0.225) - (Leaderboard.mLeaderLetterSize * 9.0)
        
        TextManager:AddLine( Leaderboard.DisplayLB[iLeader][1], fNameX, fLineY, Leaderboard.mLongTime )
        TextManager:AddLine( Leaderboard.DisplayLB[iLeader][2], fScoreX, fLineY, Leaderboard.mLongTime )
        AudioManager:PlaySound( 'Radius/SFX/Score_Leader' )

        DFUtil.sleep( Leaderboard.mProSleep )
    elseif Leaderboard.mPageIndex == 11 then  
        --pro
        TextManager:SetTextAttributes( Leaderboard.mProLetterSize, Leaderboard.mProLetterScale, Constants.SpecialColor )
        
        local fNameX = -(Leaderboard.mProLetterSize * 8.0)
        local fLineY = (Constants.ValidDrawHeight * 0.225) - (Leaderboard.mLeaderLetterSize * 9.0)
        
        TextManager:AddLine( "pro", fNameX, fLineY, Leaderboard.mLongTime )
        AudioManager:PlaySound( 'Radius/SFX/Score_Pro' )
    
        DFUtil.sleep( Leaderboard.mStandardSleep )  
        
        if Constants.TouchBotEnabled then
            -- quit!
            Leaderboard.mDesiredPage = Leaderboard.mBackToMenuSelection
        end
    else
        DFUtil.sleep( Leaderboard.mStandardSleep )
    end
    
    Leaderboard.mPageIndex = Leaderboard.mPageIndex + 1
end

function Leaderboard:Init()
    Leaderboard.mFileName = "dropchorddata"
    Leaderboard.mNumLeaders = 10
    Leaderboard.mUpdateThread = nil
    
    Leaderboard:LoadHighScores()
    
    Leaderboard.mCurrentPage = 0
    Leaderboard.mDesiredPage = 0
    Leaderboard.mNumPages = 3
    Leaderboard.mBackToMenuSelection = 4
    
    Leaderboard.mPageIndex = 0
    
    Leaderboard.mSelectionTypeNone = -1
    Leaderboard.mSelectionTypeMenu = 0
    Leaderboard.mSelectionTypeForward = 1
    Leaderboard.mSelectionTypeBack = 2
    Leaderboard.mSelectionType = Leaderboard.mSelectionTypeNone    
    
    Leaderboard.mStandardSleep = 0.1
    Leaderboard.mLongSleep = 0.5
    Leaderboard.mProSleep = 1.0
    Leaderboard.mLongTime = 9999999.0
    
    Leaderboard.DisplayLB = {}
    local i = 1
    for i = 1, 10 do
        Leaderboard.DisplayLB[i] = {}
    end
    
    Leaderboard.WaitForScoreReport = false
end

function Leaderboard:UpdateSize( tRenderer )
    if Leaderboard:AreLeadersShowing() then
        Leaderboard:HideLeaders( tRenderer )
    end
end

function Leaderboard:ShowHighScores( tRenderer )
    if Leaderboard.mUpdateThread == nil then
        Leaderboard.mUpdateThread = MOAIThread.new()
        Leaderboard.mUpdateThread:run(Leaderboard.UpdateThread, Leaderboard)
    end
    
    Leaderboard.mDesiredPage = 1
    Leaderboard.mPageIndex = 1
    
    --create header props
    local sText = "leaders"
    local textWalker = string.gmatch(sText, ".")
    Leaderboard.mHeaderTextCount = #sText
    
    Leaderboard.mHeaderImageSize = Constants.ValidDrawHeight / 20.0
    Leaderboard.mHeaderImageScale = Leaderboard.mHeaderImageSize / 128.0    
    Leaderboard.mHeaderTextProps = {}
    
    local fXPos = -(Leaderboard.mHeaderImageSize * (Leaderboard.mHeaderTextCount + 2))
    local fYPos = Constants.ValidDrawHeight * -.18
    local fZPos = 2.0
    
    local i = 1
    local j = 1
    for i = 1, Leaderboard.mHeaderTextCount do
        Leaderboard.mHeaderTextProps[i] = MOAIProp.new()
        Leaderboard.mHeaderTextProps[i]:setDeck( RadFont.mFontDeck )        
        Leaderboard.mHeaderTextProps[i]:setScl( Leaderboard.mHeaderImageScale, -Leaderboard.mHeaderImageScale )
        Leaderboard.mHeaderTextProps[i]:setIndex( RadFont.mFontDeck.names[ textWalker() ] )
        Leaderboard.mHeaderTextProps[i]:setLoc( fXPos, fYPos, fZPos )
        Leaderboard.mHeaderTextProps[i]:setColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
        Leaderboard.mHeaderTextProps[i]:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
        Leaderboard.mHeaderTextProps[i]:setVisible( true )
        
        tRenderer.mUILayer:insertProp( Leaderboard.mHeaderTextProps[i] )        
        
        fXPos = fXPos + Leaderboard.mHeaderImageSize
    end     
    
    --create pre-header props
    local sPreHeaderText = "full.mix"
    local preHeaderTextWalker = string.gmatch(sPreHeaderText, ".")
    Leaderboard.mPreHeaderTextCount = #sPreHeaderText
    
    Leaderboard.mPreHeaderImageSize = Constants.ValidDrawHeight / 35.0
    Leaderboard.mPreHeaderImageScale = Leaderboard.mPreHeaderImageSize / 128.0    
    Leaderboard.mPreHeaderTextProps = {}

    fXPos = -(Leaderboard.mHeaderImageSize * (Leaderboard.mHeaderTextCount + 3))
    fYPos = (Constants.ValidDrawHeight * -.18) - Leaderboard.mHeaderImageSize
    fZPos = 2.0
    
    for i = 1, Leaderboard.mPreHeaderTextCount do
        Leaderboard.mPreHeaderTextProps[i] = MOAIProp.new()
        Leaderboard.mPreHeaderTextProps[i]:setDeck( RadFont.mFontDeck )        
        Leaderboard.mPreHeaderTextProps[i]:setScl( Leaderboard.mPreHeaderImageScale, -Leaderboard.mPreHeaderImageScale )
        Leaderboard.mPreHeaderTextProps[i]:setIndex( RadFont.mFontDeck.names[ preHeaderTextWalker() ] )
        Leaderboard.mPreHeaderTextProps[i]:setLoc( fXPos, fYPos, fZPos )
        Leaderboard.mPreHeaderTextProps[i]:setColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
        Leaderboard.mPreHeaderTextProps[i]:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
        Leaderboard.mPreHeaderTextProps[i]:setVisible( true )
        
        tRenderer.mUILayer:insertProp( Leaderboard.mPreHeaderTextProps[i] )        
        
        fXPos = fXPos + Leaderboard.mPreHeaderImageSize
    end
    
    --additional letter sizes
    local fDesiredLetterSize = Constants.ValidDrawHeight / 30.0
    Leaderboard.mLeaderLetterScale = fDesiredLetterSize / 128.0
    Leaderboard.mLeaderLetterSize = 128.0 * Leaderboard.mLeaderLetterScale

    local fDesiredProSize = Constants.ValidDrawHeight / 20.0
    Leaderboard.mProLetterScale = fDesiredProSize / 128.0
    Leaderboard.mProLetterSize = 128.0 * Leaderboard.mProLetterScale
    
    --loading icon
    local fLoadingInner = Constants.ValidDrawHeight * 0.1
    local fLoadingOuter = Constants.ValidDrawHeight * 0.125
    Leaderboard.mLoadingIcon = ShapeProp.new()
    Leaderboard.mLoadingIcon:setDeck( tRenderer.mShapeDeck )
    Leaderboard.mLoadingIcon:SetNumSides( 16 )
    Leaderboard.mLoadingIcon:SetRadii( fLoadingInner, fLoadingOuter )  
    Leaderboard.mLoadingIcon:SetColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1.0 )
    Leaderboard.mLoadingIcon:SetDrawMode( ShapeProp.DrawModeArc )
    Leaderboard.mLoadingIcon:SetArcRadians( 0.0, DFMath.twoPi ) 
    Leaderboard.mLoadingIcon:setVisible( true )        
    Leaderboard.mLoadingIconRad = { 0.0, math.pi * 1.25, 0.0, math.pi * 1.25 }
    tRenderer.mUILayer:insertProp( Leaderboard.mLoadingIcon )     
    
    Leaderboard.mMenuTextSize = Constants.ValidDrawHeight / 30.0
    Leaderboard.mMenuTextScale = Leaderboard.mMenuTextSize / 128.0
    Leaderboard.mMenuTextHighlightSize = Leaderboard.mMenuTextSize * 1.2
    Leaderboard.mMenuTextHighlightScale = Leaderboard.mMenuTextScale * 1.2
    Leaderboard.mMenuTextHalfDiff = (Leaderboard.mMenuTextHighlightSize - Leaderboard.mMenuTextSize) * 0.5
    
    Leaderboard.mMenuXPos = Leaderboard.mMenuTextSize * -16.0
    Leaderboard.mMenuWidth = Leaderboard.mMenuTextSize * 5.0
    Leaderboard.mMenuYPos = Constants.ValidDrawHeight * .3
    
    local menuText = "back"
    local menuTextWalker = string.gmatch(menuText, ".")
    Leaderboard.mMenuTextCount = 4
    Leaderboard.mMenuTextProps = {}
    
    for i = 1, Leaderboard.mMenuTextCount do
        Leaderboard.mMenuTextProps[i] = {}
        local iIndex = menuTextWalker()
        for j = 1, 2 do    
            Leaderboard.mMenuTextProps[i][j] = MOAIProp.new()
            Leaderboard.mMenuTextProps[i][j]:setDeck( RadFont.mFontDeck )        
            Leaderboard.mMenuTextProps[i][j]:setScl( Leaderboard.mMenuTextScale, -Leaderboard.mMenuTextScale )
            Leaderboard.mMenuTextProps[i][j]:setIndex( RadFont.mFontDeck.names[iIndex] )
            Leaderboard.mMenuTextProps[i][j]:setVisible( true )
            Leaderboard.mMenuTextProps[i][j]:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
            Leaderboard.mMenuTextProps[i][j]:setColor( 1.0, 1.0, 1.0, 1.0 )
            Leaderboard.mMenuTextProps[i][j]:setLoc( Leaderboard.mMenuXPos + ((i-1) * Leaderboard.mMenuTextSize), Leaderboard.mMenuYPos, fZPos )
            
            tRenderer.mUILayer:insertProp( Leaderboard.mMenuTextProps[i][j] )        
        end
    end          
    
    Leaderboard.mNavArrowXPos = {}
    Leaderboard.mNavArrowXPos[1] = Leaderboard.mMenuTextSize * 10.0
    Leaderboard.mNavArrowXPos[2] = Leaderboard.mMenuTextSize * 15.0
    Leaderboard.mNavArrowProps = {}
    
    for i = 1, 2 do
        Leaderboard.mNavArrowProps[i] = {}
        for j = 1, 2 do
            Leaderboard.mNavArrowProps[i][j] = MOAIProp.new()
            Leaderboard.mNavArrowProps[i][j]:setDeck( RadFont.mFontDeck )        
            Leaderboard.mNavArrowProps[i][j]:setScl( Leaderboard.mMenuTextScale, -Leaderboard.mMenuTextScale )
            Leaderboard.mNavArrowProps[i][j]:setVisible( true )
            Leaderboard.mNavArrowProps[i][j]:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
            Leaderboard.mNavArrowProps[i][j]:setColor( 1.0, 1.0, 1.0, 1.0 )
            Leaderboard.mNavArrowProps[i][j]:setLoc( Leaderboard.mNavArrowXPos[i], Leaderboard.mMenuYPos, fZPos )
        
            tRenderer.mUILayer:insertProp( Leaderboard.mNavArrowProps[i][j] )
        end
    end        
    
    Leaderboard.mNavArrowProps[1][1]:setIndex( RadFont.mFontDeck.names[ "-" ] )
    Leaderboard.mNavArrowProps[1][2]:setIndex( RadFont.mFontDeck.names[ "-" ] )
    Leaderboard.mNavArrowProps[2][1]:setIndex( RadFont.mFontDeck.names[ "^" ] )
    Leaderboard.mNavArrowProps[2][2]:setIndex( RadFont.mFontDeck.names[ "^" ] )
    
    Leaderboard.mEffectsZ = 20.0
    Leaderboard.mFingerEffects = {}
    for i = 1, 2 do
        Leaderboard.mFingerEffects[i] = RadEffects:NewFingerEffect("Effects/Particles/Fingers/HowTo.particles", false)
        if Leaderboard.mFingerEffects[i] then
            local fingerEffectProp = Leaderboard.mFingerEffects[i]:getProp()
            Leaderboard.mFingerEffects[i]:setTargetProp(fingerEffectProp)        
            Leaderboard.mFingerEffects[i]:setVisible(false)
        end        
    end
end

function Leaderboard:Update( tGameInputs )
    --don't bother if our thread isn't active
    if Leaderboard.mUpdateThread ~= nil then
        local i = 1    
        for i = 1, 2 do
            if tGameInputs[i].IsTouched() then            
                if Leaderboard.mFingerEffects[i] then
                    local fingerEffectProp = Leaderboard.mFingerEffects[i]:getProp()
                    fingerEffectProp:setLoc( tGameInputs[i].mX, tGameInputs[i].mY, Leaderboard.mEffectsZ )
                    Leaderboard.mFingerEffects[i]:setVisible( true )
                end            
            else
                if Leaderboard.mFingerEffects[i] then
                    Leaderboard.mFingerEffects[i]:setVisible( false )
                end             
            end
        end
        
        --update loading icon
        local fSpeed = 4.0
        local fAngleStep = fSpeed * Constants.FrameStep       
        Leaderboard.mLoadingIconRad[1] = Leaderboard.mLoadingIconRad[1] + fAngleStep
        Leaderboard.mLoadingIconRad[2] = Leaderboard.mLoadingIconRad[2] + fAngleStep
        if Leaderboard.mLoadingIconRad[2] > DFMath.twoPi then 
            Leaderboard.mLoadingIconRad[2] = Leaderboard.mLoadingIconRad[2] - DFMath.twoPi
            Leaderboard.mLoadingIconRad[1] = Leaderboard.mLoadingIconRad[1] - DFMath.twoPi
        end        
        Leaderboard.mLoadingIcon:SetArcRadians( Leaderboard.mLoadingIconRad[1], Leaderboard.mLoadingIconRad[2] )
        Leaderboard.mLoadingIcon:setVisible( Leaderboard.WaitForScoreReport )
        
        --leaderboard specifics
        local bMultipleLeaderboards = false
        if MOAIGameCenterIOS then 
            bMultipleLeaderboards = true
        end
        
        if bMultipleLeaderboards == false then
            Leaderboard.mNavArrowProps[1][1]:setVisible( false )
            Leaderboard.mNavArrowProps[1][2]:setVisible( false )
            Leaderboard.mNavArrowProps[2][1]:setVisible( false )
            Leaderboard.mNavArrowProps[2][2]:setVisible( false )
        end
        
        --see if pointer is over a selection
        local bSelectionValid = false
        local bPressed = false
    
        local yMin = Leaderboard.mMenuYPos - (Leaderboard.mMenuTextSize * 4.0)
        local yMax = Leaderboard.mMenuYPos + Leaderboard.mMenuTextSize
            
        for i = 1, 2 do                            
            if tGameInputs[i].mY > yMin and tGameInputs[i].mY < yMax then
                local fMenuMinX = Leaderboard.mMenuXPos - Leaderboard.mMenuTextSize
                local fMenuMaxX = Leaderboard.mMenuXPos + Leaderboard.mMenuWidth + (Leaderboard.mMenuTextSize * 2.0)
            
                local fNav1MinX = Leaderboard.mNavArrowXPos[1] - (Leaderboard.mMenuTextSize * 3.0)
                local fNav1MaxX = Leaderboard.mNavArrowXPos[1] + (Leaderboard.mMenuTextSize * 2.5)

                local fNav2MinX = Leaderboard.mNavArrowXPos[2] - (Leaderboard.mMenuTextSize * 2.5)
                local fNav2MaxX = Leaderboard.mNavArrowXPos[2] + (Leaderboard.mMenuTextSize * 2.5)
                    
                if tGameInputs[i].mX > fMenuMinX and tGameInputs[i].mX < fMenuMaxX then
                    if tGameInputs[i]:IsTouched() then
                        Leaderboard.mSelectionType = Leaderboard.mSelectionTypeMenu
                        bSelectionValid = true
                        break                        
                    elseif tGameInputs[i]:WasTouched() then
                        Leaderboard.mSelectionType = Leaderboard.mSelectionTypeMenu
                        bPressed = true
                        break
                    end
                elseif tGameInputs[i].mX > fNav1MinX and tGameInputs[i].mX < fNav1MaxX then
                    if tGameInputs[i]:IsTouched() then
                        Leaderboard.mSelectionType = Leaderboard.mSelectionTypeBack
                        bSelectionValid = true
                        break                        
                    elseif tGameInputs[i]:WasTouched() then
                        Leaderboard.mSelectionType = Leaderboard.mSelectionTypeBack
                        bPressed = true
                        break
                    end       
                elseif tGameInputs[i].mX > fNav2MinX and tGameInputs[i].mX < fNav2MaxX then
                    if tGameInputs[i]:IsTouched() then
                        Leaderboard.mSelectionType = Leaderboard.mSelectionTypeForward
                        bSelectionValid = true
                        break                        
                    elseif tGameInputs[i]:WasTouched() then
                        Leaderboard.mSelectionType = Leaderboard.mSelectionTypeForward
                        bPressed = true
                        break
                    end
                end
            end
        end
                    
        local bEscDown = bSelectionValid and Leaderboard.mSelectionType == Leaderboard.mSelectionTypeMenu
        Leaderboard:PositionEsc( bEscDown )

        local bBackDown = bSelectionValid and Leaderboard.mSelectionType == Leaderboard.mSelectionTypeBack
        Leaderboard:PositionNavArrow( 1, bBackDown )
        
        local bForwardDown = bSelectionValid and Leaderboard.mSelectionType == Leaderboard.mSelectionTypeForward
        Leaderboard:PositionNavArrow( 2, bForwardDown )
        
        if bPressed then
            if Leaderboard.mSelectionType == Leaderboard.mSelectionTypeMenu then
                --exit!
                Leaderboard.mDesiredPage = Leaderboard.mBackToMenuSelection
                AudioManager:PlaySound( 'Radius/SFX/MenuSelect' )        
            elseif Leaderboard.mSelectionType == Leaderboard.mSelectionTypeForward and bMultipleLeaderboards then
                --page forward
                Leaderboard.mDesiredPage = Leaderboard.mCurrentPage + 1
                if Leaderboard.mDesiredPage > Leaderboard.mNumPages then
                    Leaderboard.mDesiredPage = 1
                end
                AudioManager:PlaySound( 'Radius/SFX/Score_Show' )       
            elseif Leaderboard.mSelectionType == Leaderboard.mSelectionTypeBack and bMultipleLeaderboards then
                --page back
                Leaderboard.mDesiredPage = Leaderboard.mCurrentPage - 1
                if Leaderboard.mDesiredPage < 1 then
                    Leaderboard.mDesiredPage = Leaderboard.mNumPages
                end     
                AudioManager:PlaySound( 'Radius/SFX/Score_Show' )                 
            end
        
            Leaderboard.mSelectionType = Leaderboard.mSelectionTypeNone                
        end 
    end    
 
    return false
end

function Leaderboard:PositionEsc( bDepressed )
    local fDepressedOffset = Leaderboard.mMenuTextSize * 0.15
    if bDepressed then
        fDepressedOffset = Leaderboard.mMenuTextSize * 0.075
    end
    
    local i = 1
    local j = 1
    for i = 1, Leaderboard.mMenuTextCount do
        Leaderboard.mMenuTextProps[i][1]:setColor( 0.0, 0.0, 0.0, 0.25 )
        local x = Leaderboard.mMenuXPos + ((i-1) * Leaderboard.mMenuTextSize)
        Leaderboard.mMenuTextProps[i][1]:setLoc( x - fDepressedOffset, Leaderboard.mMenuYPos + fDepressedOffset, 0.0 )    
        Leaderboard.mMenuTextProps[i][2]:setLoc( x + fDepressedOffset, Leaderboard.mMenuYPos - fDepressedOffset, 2.0 ) 
    end
end

function Leaderboard:PositionNavArrow( iIndex, bDepressed )
    local fDepressedOffset = Leaderboard.mMenuTextSize * 0.15
    if bDepressed then
        fDepressedOffset = Leaderboard.mMenuTextSize * 0.075
    end
    
    Leaderboard.mNavArrowProps[iIndex][1]:setColor( 0.0, 0.0, 0.0, 0.25 )
    Leaderboard.mNavArrowProps[iIndex][1]:setLoc( Leaderboard.mNavArrowXPos[iIndex] + fDepressedOffset, Leaderboard.mMenuYPos + fDepressedOffset, 0.0 )
    Leaderboard.mNavArrowProps[iIndex][2]:setLoc( Leaderboard.mNavArrowXPos[iIndex] - fDepressedOffset, Leaderboard.mMenuYPos - fDepressedOffset, 2.0 )
end

function Leaderboard:AreLeadersShowing()
    if Leaderboard.mUpdateThread == nil then
        return false
    end
    return Leaderboard.mUpdateThread:isDone() or Leaderboard.mUpdateThread:isBusy()
end

function Leaderboard:HideLeaders( tRenderer )
    if Leaderboard.mUpdateThread ~= nil then
        Leaderboard.mUpdateThread:stop()
        Leaderboard.mUpdateThread = nil
    end
    
    local i = 1
    local j = 1
    for i = 1, Leaderboard.mHeaderTextCount do
        tRenderer.mUILayer:removeProp( Leaderboard.mHeaderTextProps[i] )
    end 

    for i = 1, Leaderboard.mPreHeaderTextCount do
        tRenderer.mUILayer:removeProp( Leaderboard.mPreHeaderTextProps[i] )   
    end
    
    for i = 1, Leaderboard.mMenuTextCount do
        for j = 1, 2 do
            tRenderer.mUILayer:removeProp( Leaderboard.mMenuTextProps[i][j] )
        end
    end
    
    tRenderer.mUILayer:removeProp( Leaderboard.mLoadingIcon )  
    
    tRenderer.mUILayer:removeProp( Leaderboard.mNavArrowProps[1][1] )
    tRenderer.mUILayer:removeProp( Leaderboard.mNavArrowProps[1][2] )
    tRenderer.mUILayer:removeProp( Leaderboard.mNavArrowProps[2][1] )   
    tRenderer.mUILayer:removeProp( Leaderboard.mNavArrowProps[2][2] )
    
    for i = 1, 2 do
        if Leaderboard.mFingerEffects[i] then    
            Leaderboard.mFingerEffects[i]:stop(false)
        end
        Leaderboard.mFingerEffects[i] = nil      
    end
end

function Leaderboard:LoadHighScores()    
    if #Constants.Leaderboards == 0 then
        Constants.Leaderboards = DFSaveLoad.loadTable( Leaderboard.mFileName )
        if Constants.Leaderboards == nil then
            Leaderboard:SetDefaultLeaders()
            Constants.MaxCompletedLevel = 0
            Constants.FirstTimePlay = true
        else
            --backward compatibility
            if Constants.Leaderboards[Constants.MaxLevelIndex] == nil then
                Constants.Leaderboards[Constants.MaxLevelIndex] = { 0, 0 }
            end
            if Constants.Leaderboards[Constants.VisualQualityIndex] == nil then
                Constants.Leaderboards[Constants.VisualQualityIndex] = { Constants.VisualQualityLevelHigh, Constants.VisualQualityLevelHigh }
            end
            
            Constants.MaxCompletedLevel = Constants.Leaderboards[Constants.MaxLevelIndex][1]
            Constants.FirstTimePlay = false
        end    
    end
    
    Constants.LeaderboardHighScore = tonumber( Constants.Leaderboards[1][2] )
end

function Leaderboard:SetDefaultLeaders()
    Constants.Leaderboards = {}
    
    Constants.Leaderboards[1] = { "prh", "5000000" }
    Constants.Leaderboards[2] = { "drw", "4000000" }
    Constants.Leaderboards[3] = { "ben", "3000000" }
    Constants.Leaderboards[4] = { "pya", "2500000" }
    Constants.Leaderboards[5] = { "min", "2000000" }
    Constants.Leaderboards[6] = { "bc!", "1500000" }
    Constants.Leaderboards[7] = { "rmb", "1000000" }
    Constants.Leaderboards[8] = { "don", "800000" }
    Constants.Leaderboards[9] = { "tim", "600000" }
    Constants.Leaderboards[10] = { "tri", "400000" }    
    
    Constants.Leaderboards[Constants.MaxLevelIndex] = { 0, 0 }
    Constants.Leaderboards[Constants.VisualQualityIndex] = nil
end

--returns index of score in high score table (or zero if the score is terrible)
function Leaderboard:DetermineScoreRank( iScore )
    --leaderboards are only for full mix mode
    if Constants.FullMixMode then  
        Leaderboard:LoadHighScores()
        
        iScore = math.floor(iScore)
        Constants.PreviousGameScore = iScore
        
        local i = 1
        for i = 1, 10 do
            local iLeaderScore = tonumber( Constants.Leaderboards[i][2] )
            if iScore > iLeaderScore then
                --beat this score.. shift all the other scores down
                local j = 10
                while j ~= i do
                    Constants.Leaderboards[j][1] = Constants.Leaderboards[j-1][1]
                    Constants.Leaderboards[j][2] = Constants.Leaderboards[j-1][2]
                    j = j - 1
                end
                
                Constants.Leaderboards[i][1] = "xxx"
                Constants.Leaderboards[i][2] = tostring(iScore)
                
                Leaderboard:SaveLeaders()
                
                Constants.LeaderboardIndex = i
                return
            end
        end
    end
    
    --save leaders anyway because we may have updated our max completed level
    Leaderboard:SaveLeaders()
    Constants.LeaderboardIndex = -1
end

function Leaderboard:SaveLeaders()
    if Constants.Leaderboards[Constants.VisualQualityIndex] == nil then
        Constants.Leaderboards[Constants.VisualQualityIndex] = {Constants.VisualQuality, Constants.VisualQuality}
    else
        Constants.Leaderboards[Constants.VisualQualityIndex][1] = Constants.VisualQuality
    end
    
    Constants.Leaderboards[Constants.MaxLevelIndex][1] = math.max( Constants.Leaderboards[Constants.MaxLevelIndex][1], Constants.MaxCompletedLevel )
    DFSaveLoad.saveTable(Constants.Leaderboards, Leaderboard.mFileName, true)
end

function Leaderboard:ReportPlayerScore()
    if MOAIGameCenterIOS then
        local iScore = Constants.PreviousGameScore
        local iInitialsValue = DFRad:InitialsToValue( Constants.PreviousGameInitials )
        
        MOAIGameCenterIOS.reportScore( iScore, Constants.IOSLeaderboardID, iInitialsValue )
    end
end

return Leaderboard