local Constants = require('Constants')
local RadFont = require('RadFont')
local DFGraphics = require('DFCommon.Graphics')
local DFMath = require('DFCommon.Math')
local Renderer = require('Renderer')
local Input = require('Input')
local AudioManager = require('AudioManager')
local Leaderboard = require('Leaderboard')

local LeaderboardEntry = {}

--Leaderboard Entry Character Object
function LeaderboardEntry:NewCharacter()   
    local LeaderboardEntryChar = {}
    
    function LeaderboardEntryChar:Init()
        LeaderboardEntryChar.mPos = {}
        LeaderboardEntryChar.mProp = MOAIProp.new()
        LeaderboardEntryChar.mChar = nil
    end
    
    LeaderboardEntryChar:Init()
    return LeaderboardEntryChar
end

function LeaderboardEntry:Init()   
    LeaderboardEntry.mEnabled = false
        
    LeaderboardEntry:DetermineSize()
    
    LeaderboardEntry.mCharsString = "abcdefghijklmnopqrstuvwxyz0123456789!_-;"
    LeaderboardEntry.mNumChars = 40
    LeaderboardEntry.mChars = {}
                
    LeaderboardEntry.mEatKeyboardInput = true
    
    LeaderboardEntry.mHighlightedIndex = -1
    
    LeaderboardEntry.mHoldTimer = 0.0
    LeaderboardEntry.mHoldDuration = 1.0
    
    LeaderboardEntry.mInitials = {}
    LeaderboardEntry.mInitialIndex = 1
    LeaderboardEntry.mInitialPulse = 0.0
    LeaderboardEntry.mInitialPulseScalar = 5.0
    LeaderboardEntry.mInitialPulseUp = true
    LeaderboardEntry.mInitialHighlightTimer = 0.0
    LeaderboardEntry.mInitialHighlightDuration = 2.0
    LeaderboardEntry.mInitialHighlightIndex = -1
        
    LeaderboardEntry.mHoldTimerPropZ = 25.0
    
    LeaderboardEntry.mPointerPropMinRad = 0.0
    LeaderboardEntry.mPointerPropMaxRad = math.pi * 1.25
    LeaderboardEntry.mPointerPropUpdateRad = math.pi * 3.0
    LeaderboardEntry.mPointerPropZ = 20.0
    
    LeaderboardEntry.mAutoQuitTimer = 0.0
    LeaderboardEntry.mAutoQuitDuration = 20.0
        
    LeaderboardEntry.mCourtesyTimer = 0.0
    
    LeaderboardEntry.mCurrentState = Constants.LeaderboardEntryStateEnter
    LeaderboardEntry.mEntrySpeed = 3000.0
    LeaderboardEntry.mEntryAmount = 0.0
end

function LeaderboardEntry:DetermineSize()
    local fDesiredImageSize = Constants.ValidDrawHeight / 20.0
    LeaderboardEntry.mCharScale = fDesiredImageSize / 128.0
    LeaderboardEntry.mCharWidth = fDesiredImageSize
    LeaderboardEntry.mCharSpacing = fDesiredImageSize * 1.5
    LeaderboardEntry.mCharHighlightScale = LeaderboardEntry.mCharScale * 1.5
    LeaderboardEntry.mCharHighlightWidth = LeaderboardEntry.mCharSpacing  
    
    local fDesiredInitialSize = Constants.ValidDrawHeight / 10.0
    LeaderboardEntry.mInitialScale = fDesiredInitialSize / 128.0
    LeaderboardEntry.mInitialWidth = fDesiredInitialSize    
end

function LeaderboardEntry:UpdateSize()
    --update base letter scale
    LeaderboardEntry:DetermineSize()

    --update all props if they exist
    if LeaderboardEntry.mEnabled == true then     
        --update character props
        for i = 1, LeaderboardEntry.mNumChars do
            LeaderboardEntry.mChars[i].mProp:setScl( LeaderboardEntry.mCharScale, -LeaderboardEntry.mCharScale )        
        end

        --update initials
        for i = 1, 3 do        
            LeaderboardEntry.mInitials[i].mProp:setScl( LeaderboardEntry.mInitialScale, -LeaderboardEntry.mInitialScale )        
        end
        LeaderboardEntry:PositionInitials()
    end
end

function LeaderboardEntry:Enable()    
    local textWalker = string.gmatch(LeaderboardEntry.mCharsString, ".")
    
    local i = 1 
    for i = 1, LeaderboardEntry.mNumChars do
        local sChar = textWalker()
        
        LeaderboardEntry.mChars[i] = LeaderboardEntry:NewCharacter()   
        LeaderboardEntry.mChars[i].mProp:setDeck( RadFont.mFontDeck )
        LeaderboardEntry.mChars[i].mProp:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )
        LeaderboardEntry.mChars[i].mProp:setScl( LeaderboardEntry.mCharScale, -LeaderboardEntry.mCharScale )
        LeaderboardEntry.mChars[i].mProp:setLoc( 0.0, 0.0, 0.0 )
        LeaderboardEntry.mChars[i].mProp:setIndex( RadFont.mFontDeck.names[sChar] )
        LeaderboardEntry.mChars[i].mProp:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
        LeaderboardEntry.mChars[i].mPos = { 0.0, 0.0, 0.0 }
        LeaderboardEntry.mChars[i].mChar = sChar
    
        Renderer.mUILayer:insertProp( LeaderboardEntry.mChars[i].mProp )
    end
    LeaderboardEntry:UpdateChars(0.0)
    
    LeaderboardEntry.mCurrentState = Constants.LeaderboardEntryStateEnter
    LeaderboardEntry.mEntryAmount = (10.0 * LeaderboardEntry.mCharSpacing) + Constants.ValidDrawHalfWidth  
      
    for i = 1, 3 do        
        LeaderboardEntry.mInitials[i] = LeaderboardEntry:NewCharacter()    
        LeaderboardEntry.mInitials[i].mProp:setDeck( RadFont.mFontDeck )
        LeaderboardEntry.mInitials[i].mProp:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )
        LeaderboardEntry.mInitials[i].mProp:setScl( LeaderboardEntry.mInitialScale, -LeaderboardEntry.mInitialScale )
        LeaderboardEntry.mInitials[i].mProp:setIndex( RadFont.mFontDeck.names[ "_" ] )
        LeaderboardEntry.mInitials[i].mProp:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
        LeaderboardEntry.mInitials[i].mProp:setVisible( false )
        LeaderboardEntry.mInitials[i].mChar = "_"
    
        Renderer.mUILayer:insertProp( LeaderboardEntry.mInitials[i].mProp )    
    end
    LeaderboardEntry:PositionInitials()
        
    LeaderboardEntry.mHoldTimerProp = ShapeProp.new()
    LeaderboardEntry.mHoldTimerProp:setDeck( Renderer.mShapeDeck )
    LeaderboardEntry.mHoldTimerProp:SetNumSides( 32 )
    LeaderboardEntry.mHoldTimerProp:SetRadii( 0.0, 1.0 )  
    LeaderboardEntry.mHoldTimerProp:SetDrawMode( ShapeProp.DrawModeArc )
    LeaderboardEntry.mHoldTimerProp:SetArcRadians( 0.0, 0.0 ) 
    LeaderboardEntry.mHoldTimerProp:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
    LeaderboardEntry.mHoldTimerProp:setVisible( false )
  
    Renderer.mUILayer:insertProp( LeaderboardEntry.mHoldTimerProp ) 

    LeaderboardEntry.mPointerProp = ShapeProp.new()
    LeaderboardEntry.mPointerProp:setDeck( Renderer.mShapeDeck )
    LeaderboardEntry.mPointerProp:SetNumSides( 32 )
    LeaderboardEntry.mPointerProp:SetRadii( 0.0, 1.0 )  
    LeaderboardEntry.mPointerProp:SetDrawMode( ShapeProp.DrawModeArc )
    LeaderboardEntry.mPointerProp:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
    LeaderboardEntry.mPointerProp:setVisible( false )
    
    Renderer.mUILayer:insertProp( LeaderboardEntry.mPointerProp ) 
    
    LeaderboardEntry.mHighlightedIndex = -1
    LeaderboardEntry.mInitialIndex = 1 
    LeaderboardEntry.mHoldTimer = LeaderboardEntry.mHoldDuration
    LeaderboardEntry.mEatKeyboardInput = true
    
    LeaderboardEntry.mAutoQuitTimer = LeaderboardEntry.mAutoQuitDuration
    
    LeaderboardEntry.mEnabled = true
end

function LeaderboardEntry:Disable()
    local i = 1
    for i = 1, LeaderboardEntry.mNumChars do
        Renderer.mUILayer:removeProp( LeaderboardEntry.mChars[i].mProp )
    end 

    for i = 1, 3 do
        Renderer.mUILayer:removeProp( LeaderboardEntry.mInitials[i].mProp )
    end 
    
    Renderer.mUILayer:removeProp( LeaderboardEntry.mHoldTimerProp )
    Renderer.mUILayer:removeProp( LeaderboardEntry.mPointerProp )
    
    LeaderboardEntry.mEnabled = false
end

function LeaderboardEntry:Update( tGameInputs )
    if LeaderboardEntry.mCurrentState == Constants.LeaderboardEntryStateEnter then
        LeaderboardEntry:UpdateEnterState()
    elseif LeaderboardEntry.mCurrentState == Constants.LeaderboardEntryStateSelection then
        LeaderboardEntry:UpdateSelectionState(tGameInputs)
    elseif LeaderboardEntry.mCurrentState == Constants.LeaderboardEntryStateExit then
        LeaderboardEntry:UpdateExitState()
    end
end

function LeaderboardEntry:UpdateEnterState()
    local fMovementStep = LeaderboardEntry.mEntrySpeed * Constants.FrameStep
    LeaderboardEntry.mEntryAmount = math.max( LeaderboardEntry.mEntryAmount - fMovementStep, 0.0 )
    
    LeaderboardEntry:UpdateChars(LeaderboardEntry.mEntryAmount)
    
    if LeaderboardEntry.mEntryAmount <= 0.0 then
        LeaderboardEntry.mCurrentState = Constants.LeaderboardEntryStateSelection

        local i = 1
        for i = 1, 3 do        
            LeaderboardEntry.mInitials[i].mProp:setVisible( true )        
        end
    end
end

function LeaderboardEntry:UpdateSelectionState( tGameInputs )   
    --find letter we're hovering over
    local iHighlightedIndex = LeaderboardEntry:GetHighlightedLetterIndex(tGameInputs)
     
    --if the letter isn't the same as last frame, reset the timer   
    if iHighlightedIndex ~= LeaderboardEntry.mHighlightedIndex or iHighlightedIndex == -1 then    
        LeaderboardEntry.mHoldTimer = LeaderboardEntry.mHoldDuration
        LeaderboardEntry.mHighlightedIndex = iHighlightedIndex
        
        if iHighlightedIndex ~= -1 then
            AudioManager:PlaySound( 'Radius/SFX/ScrollOverLetter' )
        end
    else
        --else, update the timer
        LeaderboardEntry.mHoldTimer = LeaderboardEntry.mHoldTimer - Constants.FrameStep
        
        --if the timer is complete, select the letter
        if LeaderboardEntry.mHoldTimer < 0.0 then
            LeaderboardEntry:SelectLetter( LeaderboardEntry.mHighlightedIndex )
            LeaderboardEntry.mHoldTimer = LeaderboardEntry.mHoldDuration
        end
    end
        
    --update chars to update attributes
    LeaderboardEntry:UpdateChars(0.0)       
                    
    --update pointer prop
    local fAngleStep = LeaderboardEntry.mPointerPropUpdateRad * Constants.FrameStep
    LeaderboardEntry.mPointerPropMaxRad = LeaderboardEntry.mPointerPropMaxRad + fAngleStep
    LeaderboardEntry.mPointerPropMinRad = LeaderboardEntry.mPointerPropMinRad + fAngleStep
    if LeaderboardEntry.mPointerPropMaxRad > DFMath.twoPi then 
        LeaderboardEntry.mPointerPropMaxRad = LeaderboardEntry.mPointerPropMaxRad - DFMath.twoPi
        LeaderboardEntry.mPointerPropMinRad = LeaderboardEntry.mPointerPropMinRad - DFMath.twoPi
    end
    
    if tGameInputs[1].IsTouched() then
        LeaderboardEntry.mPointerProp:setLoc( tGameInputs[1].mX, tGameInputs[1].mY, LeaderboardEntry.mPointerPropZ )
        LeaderboardEntry.mPointerProp:setVisible( true )
        local fSize = LeaderboardEntry.mCharWidth * 0.4
        LeaderboardEntry.mPointerProp:SetRadii( fSize * 0.75, fSize )   
    else
        LeaderboardEntry.mPointerProp:setVisible( false )
    end
    
    LeaderboardEntry.mPointerProp:SetColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
    LeaderboardEntry.mPointerProp:SetArcRadians( LeaderboardEntry.mPointerPropMinRad, LeaderboardEntry.mPointerPropMaxRad )                
    
    --update hold prop
    local fTimerRatio = LeaderboardEntry.mHoldTimer / LeaderboardEntry.mHoldDuration
    if fTimerRatio < 0.9 then
        local fRadians = (1.0 - fTimerRatio) * 2.0 * math.pi
        local fAlpha = math.min( (1.0 - (fTimerRatio + 0.1)) * 2.0, 1.0 )
        LeaderboardEntry.mHoldTimerProp:SetColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], fAlpha )
        LeaderboardEntry.mHoldTimerProp:SetRadii( LeaderboardEntry.mCharWidth * 1.25, LeaderboardEntry.mCharWidth * 1.4 )  
        LeaderboardEntry.mHoldTimerProp:SetArcRadians( -DFMath.halfPi, fRadians-DFMath.halfPi )        
        
        local i = LeaderboardEntry:GetHighlightedLetterIndex( tGameInputs )
        if i ~= -1 then
            local x, y, z = LeaderboardEntry.mChars[i].mProp:getLoc()
            LeaderboardEntry.mHoldTimerProp:setLoc( x + (LeaderboardEntry.mCharWidth * 0.75), y - (LeaderboardEntry.mCharWidth * 0.75), LeaderboardEntry.mHoldTimerPropZ )                   
            LeaderboardEntry.mHoldTimerProp:setVisible( true )            
        end
    else
        LeaderboardEntry.mHoldTimerProp:setVisible( false )    
    end
    
    --update initials color
    local fInitialPulseStep = Constants.FrameStep * LeaderboardEntry.mInitialPulseScalar
    if LeaderboardEntry.mInitialPulseUp == true then
        LeaderboardEntry.mInitialPulse = LeaderboardEntry.mInitialPulse + fInitialPulseStep
        if LeaderboardEntry.mInitialPulse >= 1.0 then
            LeaderboardEntry.mInitialPulseUp = false
            LeaderboardEntry.mInitialPulse = 1.0
        end
    else
        LeaderboardEntry.mInitialPulse = LeaderboardEntry.mInitialPulse - fInitialPulseStep
        if LeaderboardEntry.mInitialPulse <= 0.0 then
            LeaderboardEntry.mInitialPulseUp = true
            LeaderboardEntry.mInitialPulse = 0.0
        end        
    end
    
    local i = 1
    for i = 1, 3 do        
        LeaderboardEntry.mInitials[i].mProp:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )   
    end 
    local fR = DFMath.lerp( Constants.Color2[1], Constants.SpecialColor[1], LeaderboardEntry.mInitialPulse )
    local fG = DFMath.lerp( Constants.Color2[2], Constants.SpecialColor[2], LeaderboardEntry.mInitialPulse )
    local fB = DFMath.lerp( Constants.Color2[3], Constants.SpecialColor[3], LeaderboardEntry.mInitialPulse )
    LeaderboardEntry.mInitials[ LeaderboardEntry.mInitialIndex ].mProp:setColor( fR, fG, fB, 1.0 )
    
    --if our highlight timer is set, highlight the previously set initial
    if LeaderboardEntry.mInitialHighlightTimer > 0.0 and LeaderboardEntry.mInitialHighlightIndex ~= -1 then
        local fRatio = LeaderboardEntry.mInitialHighlightTimer / LeaderboardEntry.mInitialHighlightDuration
        local fHighR = DFMath.lerp( Constants.Color2[1], Constants.SpecialColor[1], fRatio )
        local fHighG = DFMath.lerp( Constants.Color2[2], Constants.SpecialColor[2], fRatio )
        local fHighB = DFMath.lerp( Constants.Color2[3], Constants.SpecialColor[3], fRatio )
        LeaderboardEntry.mInitials[LeaderboardEntry.mInitialHighlightIndex].mProp:setColor( fHighR, fHighG, fHighB, 1.0 )
        LeaderboardEntry.mInitialHighlightTimer = LeaderboardEntry.mInitialHighlightTimer - Constants.FrameStep         
    end
    
    --update auto-quit
    if tGameInputs[1].IsTouched() then
        LeaderboardEntry.mAutoQuitTimer = LeaderboardEntry.mAutoQuitDuration
    else
        LeaderboardEntry.mAutoQuitTimer = LeaderboardEntry.mAutoQuitTimer - Constants.FrameStep 
        if LeaderboardEntry.mAutoQuitTimer <= 0.0 then
            LeaderboardEntry.mCurrentState = Constants.LeaderboardEntryStateExit
        end
    end
    
    --check for keyboard input
    LeaderboardEntry:CheckKeyboard()
end

function LeaderboardEntry:UpdateExitState()
    local fCourtesyWait = 0.5
    
    LeaderboardEntry.mHighlightedIndex = -1
    LeaderboardEntry.mPointerProp:setVisible( false )
    
    --update initials color
    local i = 1
    for i = 1, 3 do        
        LeaderboardEntry.mInitials[i].mProp:setColor( Constants.SpecialColor[1], Constants.SpecialColor[2], Constants.SpecialColor[3], 1.0 )
    end 
    
    --if our courtesy timer is set, we're just waiting to leave this state
    if LeaderboardEntry.mCourtesyTimer > 0.0 then
        LeaderboardEntry.mCourtesyTimer = LeaderboardEntry.mCourtesyTimer - Constants.FrameStep
            
        if LeaderboardEntry.mCourtesyTimer <= 0.0 then
            local sInitials = ""
            for i = 1, 3 do
                sInitials = sInitials .. LeaderboardEntry.mInitials[i].mChar
            end

            --copy to temp backup
            Constants.PreviousGameInitials = sInitials
            
            --copy initial props in to leaderboard entry
            if Constants.LeaderboardIndex ~= -1 then    
                Constants.Leaderboards[Constants.LeaderboardIndex][1] = sInitials
                Leaderboard:SaveLeaders()
            end
            
            --tell cloud about our score
            Leaderboard:ReportPlayerScore()
                
            --aaaannd, we're out
            Constants.DesiredGameState = Constants.GameStateLeaderboardShowcase        
        end
    else       
        local fMovementStep = LeaderboardEntry.mEntrySpeed * Constants.FrameStep
        LeaderboardEntry.mEntryAmount = LeaderboardEntry.mEntryAmount + fMovementStep
        LeaderboardEntry:UpdateChars(LeaderboardEntry.mEntryAmount)
    
        --if we don't have any left on screen, we're done scrolling
        local fExitAmount = (10.0 * LeaderboardEntry.mCharSpacing) + Constants.ValidDrawHalfWidth 
        if LeaderboardEntry.mEntryAmount > fExitAmount then
            LeaderboardEntry.mCourtesyTimer = fCourtesyWait
        end
    end
end

function LeaderboardEntry:CheckKeyboard()        
    if LeaderboardEntry.mCurrentState == Constants.LeaderboardEntryStateSelection then    
        local i = 1
        local keyboard = MOAIInputMgr.device.keyboard
        if keyboard then  
            local textWalker = string.gmatch(LeaderboardEntry.mCharsString, ".")
            for i = 1, LeaderboardEntry.mNumChars do
                local sChar = textWalker()
                local sCharUpper = string.upper(sChar)
                if keyboard:keyDown(sChar) or keyboard:keyDown(sCharUpper) then
                    --key was pressed, if we're not eating input, respond to it
                    if not LeaderboardEntry.mEatKeyboardInput then
                        --respond to input
                        local iCharIndex = LeaderboardEntry:GetLetterIndexFromChar(sChar)
                        LeaderboardEntry:SelectLetter( iCharIndex )
                        LeaderboardEntry.mEatKeyboardInput = true
                    end
                    
                    return
                end
            end
            
            --look for special keys            
            if keyboard:keyDown(8) or keyboard:keyDown(127) then
                --backspace or delete was hit
                local iBackspaceIndex = LeaderboardEntry:GetLetterIndexFromChar("-")            
                LeaderboardEntry:SelectLetter( iBackspaceIndex )
                LeaderboardEntry.mEatKeyboardInput = true
                return
            elseif keyboard:keyDown(13) then
                LeaderboardEntry.mCurrentState = Constants.LeaderboardEntryStateExit 
                return
            end
            
            --if we got here, there wasn't input, so reset our eat input flag
            LeaderboardEntry.mEatKeyboardInput = false
        end
    end
end

function LeaderboardEntry:HandleEsc()
    if LeaderboardEntry.mCurrentState == Constants.LeaderboardEntryStateSelection then
        LeaderboardEntry.mCurrentState = Constants.LeaderboardEntryStateExit 
    end
end

function LeaderboardEntry:SetScrollLockTimer(fDuration)
    LeaderboardEntry.mScrollLockTimer = fDuration

    --no input, clear scroll state
    LeaderboardEntry.mScrollXRegistered = false
    LeaderboardEntry.mScrollXLastPos = 0.0
    LeaderboardEntry.mScrollYLastPos = 0.0
end

function LeaderboardEntry:UpdateChars(fOffset)
    --rows are: 13, 13, 13, 1
    local i = 1 
    
    local fHighlightX = 0.0
    local fHighlightY = 0.0
    if LeaderboardEntry.mHighlightedIndex > 0 then
        fHighlightX = LeaderboardEntry.mChars[ LeaderboardEntry.mHighlightedIndex ].mPos[1]
        fHighlightY = LeaderboardEntry.mChars[ LeaderboardEntry.mHighlightedIndex ].mPos[2]
    end
    
    local fSelectedWidthDiff = (LeaderboardEntry.mCharHighlightWidth - LeaderboardEntry.mCharWidth) * 0.5
    local fNearSelectedDistance = LeaderboardEntry.mCharHighlightWidth * 2.0
    local fNearDistanceSquared = fNearSelectedDistance * fNearSelectedDistance
    
    local x, y, z
    x = -(LeaderboardEntry.mCharWidth * 1.5 * 13.0 * 0.5)
    y = LeaderboardEntry.mCharWidth * -5.0
    z = 0.0
    local fXSign = 1.0
    
    for i = 1, 40 do                              
        --set position and scale according to placement
        if i == LeaderboardEntry.mHighlightedIndex then             
            local fDrawX = x - fSelectedWidthDiff
            local fDrawY = y + fSelectedWidthDiff
            LeaderboardEntry.mChars[i].mProp:setLoc( fDrawX, fDrawY, LeaderboardEntry.mHoldTimerPropZ )
            LeaderboardEntry.mChars[i].mProp:setScl( LeaderboardEntry.mCharHighlightScale, -LeaderboardEntry.mCharHighlightScale )
            LeaderboardEntry.mChars[i].mProp:setColor( Constants.SpecialColor[1], Constants.SpecialColor[2], Constants.SpecialColor[3], 1.0 )
            
            --update position        
            LeaderboardEntry.mChars[i].mPos[1] = x
            LeaderboardEntry.mChars[i].mPos[2] = y           
        else
            local fDrawX = x
            local fDrawY = y
            
            if LeaderboardEntry.mCurrentState ~= Constants.LeaderboardEntryStateSelection then
                LeaderboardEntry.mChars[i].mPos[1] = x
                LeaderboardEntry.mChars[i].mPos[2] = y                      
            else
                --see if we're near the selected char      
                local bPosUpdated = false
                if LeaderboardEntry.mHighlightedIndex > 0 then
                    local fToChar = DFMath.distance2DSquared( fHighlightX, fHighlightY, x, y )
                    if fToChar < fNearDistanceSquared then 
                        local fToCharRatio = (1.0 - (math.sqrt(fToChar) / fNearSelectedDistance)) * 0.5
                        fDrawX = x + ((x - fHighlightX) * fToCharRatio)
                        fDrawY = y + ((y - fHighlightY) * fToCharRatio)                              
                        bPosUpdated = true
                    end     
                end
                
                --move back toward where we should be
                if not bPosUpdated then
                    local fInterpStep = Constants.FrameStep * 4.0
                    fDrawX = DFMath.lerp( LeaderboardEntry.mChars[i].mPos[1], x, fInterpStep )
                    fDrawY = DFMath.lerp( LeaderboardEntry.mChars[i].mPos[2], y, fInterpStep )
                end
                
                LeaderboardEntry.mChars[i].mPos[1] = fDrawX
                LeaderboardEntry.mChars[i].mPos[2] = fDrawY           
            end
                            
            local fOffsetX = fDrawX + (fXSign * fOffset)
            LeaderboardEntry.mChars[i].mProp:setLoc( fOffsetX, fDrawY, z )
            LeaderboardEntry.mChars[i].mProp:setScl( LeaderboardEntry.mCharScale, -LeaderboardEntry.mCharScale )

            if LeaderboardEntry.mChars[i].mChar == '-' or LeaderboardEntry.mChars[i].mChar == ';' then
                LeaderboardEntry.mChars[i].mProp:setColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1.0 )
            else
                LeaderboardEntry.mChars[i].mProp:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )
            end
        end
        
        x = x + (LeaderboardEntry.mCharWidth * 1.5)
        if i == 13 or i == 26 then
            y = y + (LeaderboardEntry.mCharWidth * 1.5)
            x = -(LeaderboardEntry.mCharWidth * 1.5 * 13.0 * 0.5)
            fXSign = fXSign * -1.0
        elseif i == 39 then
            y = y + (LeaderboardEntry.mCharWidth * 1.5)
            x = LeaderboardEntry.mCharWidth * -0.5 
        end
    end  
end

function LeaderboardEntry:SelectLetter( i )
    if i ~= -1 then
        local iSelectedIndex = LeaderboardEntry.mChars[i].mProp:getIndex()
        local iBackspaceIndex = RadFont.mFontDeck.names[ "-" ]
        local iExitIndex = RadFont.mFontDeck.names[ ";" ]
        local iUnderscoreIndex = RadFont.mFontDeck.names[ "_" ]
        
        --if we selected exit, we're done
        if iSelectedIndex == iExitIndex then
            LeaderboardEntry.mCurrentState = Constants.LeaderboardEntryStateExit 
            AudioManager:PlaySound( 'Radius/SFX/InputLetter' )
        --if we selected backspace.. back up
        elseif iSelectedIndex == iBackspaceIndex then
            LeaderboardEntry.mInitials[ LeaderboardEntry.mInitialIndex ].mProp:setIndex( iUnderscoreIndex )
            LeaderboardEntry.mInitials[ LeaderboardEntry.mInitialIndex ].mChar = "_"    
            
            LeaderboardEntry.mInitialIndex = LeaderboardEntry.mInitialIndex - 1
            if LeaderboardEntry.mInitialIndex <= 0 then
                LeaderboardEntry.mInitialIndex = 1
            end                
            
            LeaderboardEntry.mInitials[ LeaderboardEntry.mInitialIndex ].mProp:setIndex( iUnderscoreIndex )
            LeaderboardEntry.mInitials[ LeaderboardEntry.mInitialIndex ].mChar = "_"                
            
            AudioManager:PlaySound( 'Radius/SFX/DeleteLetter' )
        else
            --we're setting a letter, make it shine
            LeaderboardEntry.mInitialHighlightTimer = LeaderboardEntry.mInitialHighlightDuration 
            LeaderboardEntry.mInitialHighlightIndex = LeaderboardEntry.mInitialIndex  
            
            --set the letter we selected
            LeaderboardEntry.mInitials[ LeaderboardEntry.mInitialIndex ].mProp:setIndex( iSelectedIndex )
            LeaderboardEntry.mInitials[ LeaderboardEntry.mInitialIndex ].mChar = LeaderboardEntry.mChars[i].mChar
        
            LeaderboardEntry.mInitialIndex = LeaderboardEntry.mInitialIndex + 1
            if LeaderboardEntry.mInitialIndex > 3 then
                LeaderboardEntry.mInitialIndex = 3
            end
            
            LeaderboardEntry:SetScrollLockTimer( 0.5 )
            AudioManager:PlaySound( 'Radius/SFX/InputLetter' )
        end
    end  
end

function LeaderboardEntry:GetHighlightedLetterIndex(tGameInputs)
    if tGameInputs[1].IsTouched() then
        local fSelectionDistSquared = LeaderboardEntry.mCharWidth * LeaderboardEntry.mCharWidth
        local i = 1 
        for i = 1, LeaderboardEntry.mNumChars do
            local x, y
            x = LeaderboardEntry.mChars[i].mPos[1] + LeaderboardEntry.mCharWidth
            y = LeaderboardEntry.mChars[i].mPos[2] - (LeaderboardEntry.mCharWidth * 0.5) 
            local fToInput = DFMath.distance2DSquared( x, y, tGameInputs[1].mX, tGameInputs[1].mY )
            if fToInput < fSelectionDistSquared then 
                return i
            end
        end      
    end
    
    return -1
end

function LeaderboardEntry:GetLetterIndexFromChar( sChar )
    local i = 1 
    for i = 1, LeaderboardEntry.mNumChars do
        if LeaderboardEntry.mChars[i].mChar == sChar then
            return i
        end
    end      
    
    return -1
end

function LeaderboardEntry:PositionInitials()
    local fInitialX = LeaderboardEntry.mInitialWidth * -1.5    
    for i = 1, 3 do        
        LeaderboardEntry.mInitials[i].mProp:setLoc( fInitialX, LeaderboardEntry.mCharWidth * 2.5, 0.0 )     
        fInitialX = fInitialX + LeaderboardEntry.mInitialWidth
    end
end

return LeaderboardEntry