local Constants = require('Constants')
local RadFont = require('RadFont')
local DFGraphics = require('DFCommon.Graphics')
local DFMath = require('DFCommon.Math')
local Renderer = require('Renderer')
local Input = require('Input')
local AudioManager = require('AudioManager')
local Leaderboard = require('Leaderboard')

local LeaderboardEntry = {}

--Leaderboard Entry Character Object
function LeaderboardEntry:NewCharacter()   
    local LeaderboardEntryChar = {}
    
    function LeaderboardEntryChar:Init()
        LeaderboardEntryChar.mXPos = 0.0
        LeaderboardEntryChar.mProp = MOAIProp.new()
        LeaderboardEntryChar.mChar = nil
    end
    
    LeaderboardEntryChar:Init()
    return LeaderboardEntryChar
end

function LeaderboardEntry:Init()   
    LeaderboardEntry.mEnabled = false
        
    LeaderboardEntry:DetermineSize()
    
    LeaderboardEntry.mCharsString = "abcdefghijklmnopqrstuvwxyz0123456789!_;-"
    LeaderboardEntry.mNumChars = 40
    LeaderboardEntry.mChars = {}
    
    LeaderboardEntry.mXInputBounds = 100.0
    
    LeaderboardEntry.mHintProp = nil
    LeaderboardEntry.mHintPropYOffset = 0.0
    LeaderboardEntry.mHintPropAlpha = 0.0
    LeaderboardEntry.mHintPropAlphaScale = 4.0
    LeaderboardEntry.mHintPropAlphaDesired = 0.0
    
    LeaderboardEntry.mScrollLockTimer = 0.0
    LeaderboardEntry.mScrollXRegistered = false
    LeaderboardEntry.mScrollXRegisterHeight = 0.0
    LeaderboardEntry.mScrollXLastPos = 0.0
    LeaderboardEntry.mScrollYLastPos = 0.0
    LeaderboardEntry.mScrollXMovementScale = 1.0
    LeaderboardEntry.mScrollXSwipeMax = 25.0
    LeaderboardEntry.mScrollXSwipeScale = 1.0
    LeaderboardEntry.mScrollXVelocity = 0.0
    LeaderboardEntry.mScrollXVelocityDecay = 0.5
    
    LeaderboardEntry.mCanPullDown = false
    LeaderboardEntry.mPullDownMaxSpeed = 2.0
    LeaderboardEntry.mPullingDown = false
    LeaderboardEntry.mPullingDownAmount = 0.0
    LeaderboardEntry.mPullDownRegisterX = 0.0
    LeaderboardEntry.mPullDownRegisterY = 0.0
    LeaderboardEntry.mPullDownMovementScale = 1.0
    LeaderboardEntry.mEatKeyboardInput = true
        
    LeaderboardEntry.mInitials = {}
    LeaderboardEntry.mInitialIndex = 1
    LeaderboardEntry.mInitialPulse = 0.0
    LeaderboardEntry.mInitialPulseScalar = 5.0
    LeaderboardEntry.mInitialPulseUp = true
    LeaderboardEntry.mInitialHighlightTimer = 0.0
    LeaderboardEntry.mInitialHighlightDuration = 2.0
    LeaderboardEntry.mInitialHighlightIndex = -1
            
    LeaderboardEntry.mAutoQuitTimer = 0.0
    LeaderboardEntry.mAutoQuitDuration = 20.0
    
    LeaderboardEntry.mScrollingToExit = false
    
    LeaderboardEntry.mCourtesyTimer = 0.0
    
    LeaderboardEntry.mCurrentState = Constants.LeaderboardEntryStateEnter
    LeaderboardEntry.mEntrySpeed = 3000.0
    LeaderboardEntry.mEntryAmount = 0.0
end

function LeaderboardEntry:DetermineSize()
    local fDesiredImageSize = Constants.ValidDrawHeight / 10.0
    LeaderboardEntry.mCharScale = fDesiredImageSize / 128.0
    LeaderboardEntry.mCharWidth = fDesiredImageSize
    LeaderboardEntry.mCharSpacing = fDesiredImageSize * 1.5
    LeaderboardEntry.mCharHighlightScale = LeaderboardEntry.mCharScale * 1.5
    LeaderboardEntry.mCharHighlightWidth = LeaderboardEntry.mCharSpacing  
    LeaderboardEntry.mCharYPos = LeaderboardEntry.mCharWidth * -1.5
       
    LeaderboardEntry.mHintPropYOffsetScale = Constants.ValidDrawHeight * 0.08
    LeaderboardEntry.mHintPropY = LeaderboardEntry.mCharYPos
        
    LeaderboardEntry.mScrollYThreshold = Constants.ValidDrawHalfWidth 
    
    LeaderboardEntry.mScrollXRegisterHeight = Constants.ValidDrawHeight
    
    LeaderboardEntry.mPullDownStartAmount = LeaderboardEntry.mCharWidth * 0.2
    LeaderboardEntry.mPullDownActivateAmount = LeaderboardEntry.mCharWidth * 2.0  
    LeaderboardEntry.mPullDownMaxAmount = LeaderboardEntry.mPullDownActivateAmount
    LeaderboardEntry.mPullDownMaxXOffset = Constants.ValidDrawHalfWidth * 0.4  
    
    LeaderboardEntry.mXInputBounds = Constants.ValidDrawHalfWidth * 0.8
end

function LeaderboardEntry:UpdateSize()
    --update base letter scale
    LeaderboardEntry:DetermineSize()

    --update all props if they exist
    if LeaderboardEntry.mEnabled == true then     
        --update character props
        for i = 1, LeaderboardEntry.mNumChars do
            LeaderboardEntry.mChars[i].mProp:setScl( LeaderboardEntry.mCharScale, -LeaderboardEntry.mCharScale )        
        end
        LeaderboardEntry:PositionCharacters()
        
        --update hint prop
        LeaderboardEntry.mHintProp:setScl( LeaderboardEntry.mCharScale, -LeaderboardEntry.mCharScale )

        --update initials
        for i = 1, 3 do        
            LeaderboardEntry.mInitials[i].mProp:setScl( LeaderboardEntry.mCharScale, -LeaderboardEntry.mCharScale )        
        end
        LeaderboardEntry:PositionInitials()
    end
end

function LeaderboardEntry:Enable()    
    local textWalker = string.gmatch(LeaderboardEntry.mCharsString, ".")
    
    local i = 1 
    for i = 1, LeaderboardEntry.mNumChars do
        local fXLocation = Constants.ValidDrawHalfWidth + (LeaderboardEntry.mCharSpacing * i)
        local sChar = textWalker()
        
        LeaderboardEntry.mChars[i] = LeaderboardEntry:NewCharacter()   
        LeaderboardEntry.mChars[i].mProp:setDeck( RadFont.mFontDeck )
        LeaderboardEntry.mChars[i].mProp:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 0.0 )
        LeaderboardEntry.mChars[i].mProp:setScl( LeaderboardEntry.mCharScale, -LeaderboardEntry.mCharScale )
        LeaderboardEntry.mChars[i].mProp:setLoc( fXLocation, LeaderboardEntry.mCharYPos, 0.0 )
        LeaderboardEntry.mChars[i].mProp:setIndex( RadFont.mFontDeck.names[sChar] )
        LeaderboardEntry.mChars[i].mProp:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
        LeaderboardEntry.mChars[i].mXPos = fXLocation
        LeaderboardEntry.mChars[i].mChar = sChar
    
        Renderer.mUILayer:insertProp( LeaderboardEntry.mChars[i].mProp )
    end
    
    LeaderboardEntry.mCurrentState = Constants.LeaderboardEntryStateEnter
    LeaderboardEntry.mEntryAmount = ((LeaderboardEntry.mNumChars+1) * LeaderboardEntry.mCharSpacing) + Constants.ValidDrawHalfWidth
    
    LeaderboardEntry.mHintProp = MOAIProp.new()
    LeaderboardEntry.mHintProp:setDeck( RadFont.mFontDeck )
    LeaderboardEntry.mHintProp:setColor( Constants.SpecialColor[1], Constants.SpecialColor[2], Constants.SpecialColor[3], 0.0 )
    LeaderboardEntry.mHintProp:setScl( LeaderboardEntry.mCharScale, -LeaderboardEntry.mCharScale )
    LeaderboardEntry.mHintProp:setLoc( LeaderboardEntry.mCharWidth * -0.5, LeaderboardEntry.mHintPropY, 0.0 )
    LeaderboardEntry.mHintProp:setRot( 0.0, 0.0, 90.0 )
    LeaderboardEntry.mHintProp:setIndex( RadFont.mFontDeck.names[ "^" ] )
    LeaderboardEntry.mHintProp:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
    Renderer.mUILayer:insertProp( LeaderboardEntry.mHintProp )    
      
    for i = 1, 3 do        
        LeaderboardEntry.mInitials[i] = LeaderboardEntry:NewCharacter()    
        LeaderboardEntry.mInitials[i].mProp:setDeck( RadFont.mFontDeck )
        LeaderboardEntry.mInitials[i].mProp:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )
        LeaderboardEntry.mInitials[i].mProp:setScl( LeaderboardEntry.mCharScale, -LeaderboardEntry.mCharScale )
        LeaderboardEntry.mInitials[i].mProp:setIndex( RadFont.mFontDeck.names[ "_" ] )
        LeaderboardEntry.mInitials[i].mProp:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
        LeaderboardEntry.mInitials[i].mProp:setVisible( false )
        LeaderboardEntry.mInitials[i].mChar = "_"
    
        Renderer.mUILayer:insertProp( LeaderboardEntry.mInitials[i].mProp )    
    end
    LeaderboardEntry:PositionInitials()
        
    LeaderboardEntry.mInitialIndex = 1
    LeaderboardEntry.mScrollingToExit = false
    LeaderboardEntry.mScrollLockTimer = 0.0
    LeaderboardEntry.mEatKeyboardInput = true
    
    LeaderboardEntry.mAutoQuitTimer = LeaderboardEntry.mAutoQuitDuration
    
    LeaderboardEntry.mEnabled = true
end

function LeaderboardEntry:Disable()
    local i = 1
    for i = 1, LeaderboardEntry.mNumChars do
        Renderer.mUILayer:removeProp( LeaderboardEntry.mChars[i].mProp )
    end 

    for i = 1, 3 do
        Renderer.mUILayer:removeProp( LeaderboardEntry.mInitials[i].mProp )
    end 
    
    Renderer.mUILayer:removeProp( LeaderboardEntry.mHintProp ) 
    
    LeaderboardEntry.mEnabled = false
end

function LeaderboardEntry:Update( tGameInputs )
    if LeaderboardEntry.mCurrentState == Constants.LeaderboardEntryStateEnter then
        LeaderboardEntry:UpdateEnterState()
    elseif LeaderboardEntry.mCurrentState == Constants.LeaderboardEntryStateSelection then
        LeaderboardEntry:UpdateSelectionState(tGameInputs)
    elseif LeaderboardEntry.mCurrentState == Constants.LeaderboardEntryStateExit then
        LeaderboardEntry:UpdateExitState()
    end
end

function LeaderboardEntry:UpdateEnterState()
    local fMovementStep = LeaderboardEntry.mEntrySpeed * Constants.FrameStep
    LeaderboardEntry.mEntryAmount = LeaderboardEntry.mEntryAmount - fMovementStep
    
    LeaderboardEntry:ScrollChars(-fMovementStep)
    
    if LeaderboardEntry.mEntryAmount < 0.0 then
        LeaderboardEntry.mCurrentState = Constants.LeaderboardEntryStateSelection

        local i = 1
        for i = 1, 3 do        
            LeaderboardEntry.mInitials[i].mProp:setVisible( true )        
        end
    end
end

function LeaderboardEntry:UpdateSelectionState( tGameInputs )
    local fScrollAmount = 0.0
    local bShowPullDownHint = false
    local bShowPointToCharsHint = true
    
    --reset pull down flag to false
    LeaderboardEntry.mCanPullDown = false
    
    --validate touch is in bounds
    local bInputIsValid = tGameInputs[1].IsTouched()
    if bInputIsValid then
        if math.abs(tGameInputs[1].mX) > LeaderboardEntry.mXInputBounds then
            bInputIsValid = false
        end
    end
    
    --if we're in auto-move mode, don't allow input
    if LeaderboardEntry.mScrollingToExit == true then
        LeaderboardEntry:ScrollToExit()
    else
        --react to input
        if bInputIsValid then
            LeaderboardEntry.mCanPullDown = LeaderboardEntry.mScrollXRegistered
            
            --if we're currently pulling a letter down, monitor the amount we've pulled it down
            if LeaderboardEntry.mPullingDown then
                bShowPointToCharsHint = false
                local fDiffFromXAmount = math.abs(tGameInputs[1].mX - LeaderboardEntry.mPullDownRegisterX)
                LeaderboardEntry.mPullingDownAmount = (tGameInputs[1].mY - LeaderboardEntry.mPullDownRegisterY) * LeaderboardEntry.mPullDownMovementScale
                                
                if LeaderboardEntry.mPullingDownAmount > LeaderboardEntry.mPullDownActivateAmount then
                    LeaderboardEntry.mPullingDownAmount = math.min( LeaderboardEntry.mPullingDownAmount, LeaderboardEntry.mPullDownMaxAmount )
                    
                    --selection made!
                    LeaderboardEntry.mPullingDownAmount = 0.0
                    LeaderboardEntry.mPullDownRegisterY = LeaderboardEntry.mScrollYThreshold
                    LeaderboardEntry.mPullingDown = false  
                    
                    local iHighlightedIndex = LeaderboardEntry:GetHighlightedLetterIndex()
                    LeaderboardEntry:SelectLetter( iHighlightedIndex )                                    
                elseif LeaderboardEntry.mPullingDownAmount < 0.0 or fDiffFromXAmount > LeaderboardEntry.mPullDownMaxXOffset then
                    LeaderboardEntry.mScrollXVelocity = 0.0
                    LeaderboardEntry.mPullingDownAmount = 0.0
                    LeaderboardEntry.mPullDownRegisterY = LeaderboardEntry.mScrollYThreshold
                    LeaderboardEntry.mPullingDown = false
                else
                    bShowPullDownHint = true
                end
            elseif LeaderboardEntry.mScrollXRegistered then
                --scroll is registered, update text with movement
                bShowPullDownHint = true
                bShowPointToCharsHint = false

                local fScrollDiff = tGameInputs[1].mX - LeaderboardEntry.mScrollXLastPos
                fScrollAmount = fScrollDiff * LeaderboardEntry.mScrollXMovementScale               
                LeaderboardEntry.mScrollXVelocity = fScrollAmount
                
                --if our scroll lock is on, wipe any input we got
                if LeaderboardEntry.mScrollLockTimer > 0.0 then
                    fScrollAmount = 0.0
                    LeaderboardEntry.mScrollXVelocity = 0.0
                end
                
                local fDiffFromRegisteredX = math.abs(tGameInputs[1].mX - LeaderboardEntry.mPullDownRegisterX)
                local fDiffFromRegisteredY = tGameInputs[1].mY - LeaderboardEntry.mPullDownRegisterY
                if fScrollAmount < LeaderboardEntry.mPullDownMaxSpeed and fDiffFromRegisteredY > LeaderboardEntry.mPullDownStartAmount and fDiffFromRegisteredX < fDiffFromRegisteredY then
                    --we began pulling the letter!
                    LeaderboardEntry.mPullingDown = true
                    LeaderboardEntry.mPullDownRegisterX = tGameInputs[1].mX
                    LeaderboardEntry.mPullDownRegisterY = tGameInputs[1].mY
                else           
                    LeaderboardEntry.mPullDownRegisterY = math.min( LeaderboardEntry.mPullDownRegisterY, tGameInputs[1].mY )
                end  
            elseif tGameInputs[1].mY < LeaderboardEntry.mScrollXRegisterHeight then
                --first input, record the current x position
                LeaderboardEntry.mPullDownRegisterX = tGameInputs[1].mX
                LeaderboardEntry.mScrollXRegistered = true
            end
            
            LeaderboardEntry.mScrollXLastPos = tGameInputs[1].mX   
            LeaderboardEntry.mScrollYLastPos = tGameInputs[1].mY
        else
            --regulate swipe!
            if LeaderboardEntry.mScrollXRegistered then
                if LeaderboardEntry.mScrollXVelocity < -LeaderboardEntry.mScrollXSwipeMax then
                    LeaderboardEntry.mScrollXVelocity = math.max( LeaderboardEntry.mScrollXVelocity, -LeaderboardEntry.mScrollXSwipeMax )
                elseif LeaderboardEntry.mScrollXVelocity > LeaderboardEntry.mScrollXSwipeMax then
                    LeaderboardEntry.mScrollXVelocity = math.min( LeaderboardEntry.mScrollXVelocity, LeaderboardEntry.mScrollXSwipeMax )
                end
            else
                local fDecayStep = LeaderboardEntry.mScrollXVelocity * Constants.FrameStep * LeaderboardEntry.mScrollXVelocityDecay
                LeaderboardEntry.mScrollXVelocity = LeaderboardEntry.mScrollXVelocity - fDecayStep
                
                --swiping
                fScrollAmount = LeaderboardEntry.mScrollXVelocity
            end
            
            --no input, clear states
            LeaderboardEntry.mScrollXRegistered = false
            LeaderboardEntry.mScrollXLastPos = 0.0
            LeaderboardEntry.mScrollYLastPos = 0.0
            LeaderboardEntry.mPullingDownAmount = 0.0
            LeaderboardEntry.mPullDownRegisterY = LeaderboardEntry.mScrollYThreshold
            LeaderboardEntry.mPullingDown = false      
        end
        
        --regardless of scroll amount, scroll the chars to update attributes
        LeaderboardEntry:ScrollChars( fScrollAmount )            
    end
                    
    --update scroll lock
    LeaderboardEntry.mScrollLockTimer = math.max( LeaderboardEntry.mScrollLockTimer - Constants.FrameStep, 0.0 )
                        
    --update pull hint
    LeaderboardEntry.mHintPropYOffset = LeaderboardEntry.mHintPropYOffset + Constants.FrameStep 
    if LeaderboardEntry.mHintPropYOffset > 1.0 then
        LeaderboardEntry.mHintPropYOffset = LeaderboardEntry.mHintPropYOffset - 1.0
    end
    
    local fHintY = LeaderboardEntry.mHintPropY + (LeaderboardEntry.mHintPropYOffset * LeaderboardEntry.mHintPropYOffsetScale)
    LeaderboardEntry.mHintProp:setLoc( LeaderboardEntry.mCharWidth * -0.5, fHintY, 0.0 )
    
    --if we've pulled down a good amount, don't show the hint anymore
    if LeaderboardEntry.mPullingDownAmount > (LeaderboardEntry.mPullDownStartAmount * 2.0) then
        bShowPullDownHint = false
    end
    
    local fHintAlpha = 0.0
    if bShowPullDownHint == true then
        local fAlphaStep = Constants.FrameStep * LeaderboardEntry.mHintPropAlphaScale
        local fClampedScrollAmount = math.min( math.abs(fScrollAmount), LeaderboardEntry.mPullDownMaxSpeed )
        LeaderboardEntry.mHintPropAlphaDesired = 1.0 - (fClampedScrollAmount / LeaderboardEntry.mPullDownMaxSpeed) 
        
        if LeaderboardEntry.mHintPropAlpha < LeaderboardEntry.mHintPropAlphaDesired - fAlphaStep then
            LeaderboardEntry.mHintPropAlpha = LeaderboardEntry.mHintPropAlpha + fAlphaStep
        elseif LeaderboardEntry.mHintPropAlpha > LeaderboardEntry.mHintPropAlphaDesired + fAlphaStep then
            LeaderboardEntry.mHintPropAlpha = LeaderboardEntry.mHintPropAlpha - fAlphaStep
        else
            LeaderboardEntry.mHintPropAlpha = LeaderboardEntry.mHintPropAlphaDesired
        end
        
        fHintAlpha = LeaderboardEntry.mHintPropAlpha
    else
        LeaderboardEntry.mHintPropAlpha = 0.0
        LeaderboardEntry.mHintPropAlphaDesired = 0.0
    end
    LeaderboardEntry.mHintProp:setColor( Constants.SpecialColor[1], Constants.SpecialColor[2], Constants.SpecialColor[3], fHintAlpha )
    
    --update initials color
    local fInitialPulseStep = Constants.FrameStep * LeaderboardEntry.mInitialPulseScalar
    if LeaderboardEntry.mInitialPulseUp == true then
        LeaderboardEntry.mInitialPulse = LeaderboardEntry.mInitialPulse + fInitialPulseStep
        if LeaderboardEntry.mInitialPulse >= 1.0 then
            LeaderboardEntry.mInitialPulseUp = false
            LeaderboardEntry.mInitialPulse = 1.0
        end
    else
        LeaderboardEntry.mInitialPulse = LeaderboardEntry.mInitialPulse - fInitialPulseStep
        if LeaderboardEntry.mInitialPulse <= 0.0 then
            LeaderboardEntry.mInitialPulseUp = true
            LeaderboardEntry.mInitialPulse = 0.0
        end        
    end
    
    local i = 1
    for i = 1, 3 do        
        LeaderboardEntry.mInitials[i].mProp:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )   
    end 
    local fR = DFMath.lerp( Constants.Color2[1], Constants.SpecialColor[1], LeaderboardEntry.mInitialPulse )
    local fG = DFMath.lerp( Constants.Color2[2], Constants.SpecialColor[2], LeaderboardEntry.mInitialPulse )
    local fB = DFMath.lerp( Constants.Color2[3], Constants.SpecialColor[3], LeaderboardEntry.mInitialPulse )
    LeaderboardEntry.mInitials[ LeaderboardEntry.mInitialIndex ].mProp:setColor( fR, fG, fB, 1.0 )
    
    --if our highlight timer is set, highlight the previously set initial
    if LeaderboardEntry.mInitialHighlightTimer > 0.0 and LeaderboardEntry.mInitialHighlightIndex ~= -1 then
        local fRatio = LeaderboardEntry.mInitialHighlightTimer / LeaderboardEntry.mInitialHighlightDuration
        local fHighR = DFMath.lerp( Constants.Color2[1], Constants.SpecialColor[1], fRatio )
        local fHighG = DFMath.lerp( Constants.Color2[2], Constants.SpecialColor[2], fRatio )
        local fHighB = DFMath.lerp( Constants.Color2[3], Constants.SpecialColor[3], fRatio )
        LeaderboardEntry.mInitials[LeaderboardEntry.mInitialHighlightIndex].mProp:setColor( fHighR, fHighG, fHighB, 1.0 )
        LeaderboardEntry.mInitialHighlightTimer = LeaderboardEntry.mInitialHighlightTimer - Constants.FrameStep         
    end
    
    --update auto-quit
    if bInputIsValid then
        LeaderboardEntry.mAutoQuitTimer = LeaderboardEntry.mAutoQuitDuration
    else
        LeaderboardEntry.mAutoQuitTimer = LeaderboardEntry.mAutoQuitTimer - Constants.FrameStep 
        if LeaderboardEntry.mAutoQuitTimer <= 0.0 then
            LeaderboardEntry.mCurrentState = Constants.LeaderboardEntryStateExit
        end
    end
    
    --check for keyboard input
    LeaderboardEntry:CheckKeyboard()
end

function LeaderboardEntry:UpdateExitState()
    local fCourtesyWait = 0.5
    
    --update initials color
    local i = 1
    for i = 1, 3 do        
        LeaderboardEntry.mInitials[i].mProp:setColor( Constants.SpecialColor[1], Constants.SpecialColor[2], Constants.SpecialColor[3], 1.0 )
    end 
    
    --if our courtesy timer is set, we're just waiting to leave this state
    if LeaderboardEntry.mCourtesyTimer > 0.0 then
        LeaderboardEntry.mCourtesyTimer = LeaderboardEntry.mCourtesyTimer - Constants.FrameStep
            
        if LeaderboardEntry.mCourtesyTimer <= 0.0 then
            local sInitials = ""
            for i = 1, 3 do
                sInitials = sInitials .. LeaderboardEntry.mInitials[i].mChar
            end  
            
            --copy to temp backup
            Constants.PreviousGameInitials = sInitials
                
            --copy initial props in to leaderboard entry
            if Constants.LeaderboardIndex ~= -1 then    
                Constants.Leaderboards[Constants.LeaderboardIndex][1] = sInitials
                Leaderboard:SaveLeaders()
            end
            
            --tell cloud about our score
            Leaderboard:ReportPlayerScore()
            
            --aaaannd, we're out
            Constants.DesiredGameState = Constants.GameStateLeaderboardShowcase        
        end
    else    
        local fMovementStep = LeaderboardEntry.mEntrySpeed * Constants.FrameStep
        local fWidthDiff = LeaderboardEntry.mCharHighlightWidth - LeaderboardEntry.mCharWidth
        local bStillOnScreen = false
        
        local i = 1 
        for i = 1, LeaderboardEntry.mNumChars do
            --update position
            local x, y, z = LeaderboardEntry.mChars[i].mProp:getLoc()
            x = x - fMovementStep

            if x > -LeaderboardEntry.mCharSpacing and x < 0.0 then
                x = x - (fWidthDiff * 0.5)
                LeaderboardEntry.mChars[i].mProp:setLoc( x, y, z )
                LeaderboardEntry.mChars[i].mProp:setScl( LeaderboardEntry.mCharHighlightScale, -LeaderboardEntry.mCharHighlightScale )
                LeaderboardEntry.mChars[i].mProp:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )
            else
                local fAlpha = 1.0 - (math.min(math.abs(x) / Constants.ValidDrawHalfWidth, 1.0))
                LeaderboardEntry.mChars[i].mProp:setLoc( x, y, z )
                LeaderboardEntry.mChars[i].mProp:setScl( LeaderboardEntry.mCharScale, -LeaderboardEntry.mCharScale )
                    
                if LeaderboardEntry.mChars[i].mChar == '-' or LeaderboardEntry.mChars[i].mChar == ';' then
                    LeaderboardEntry.mChars[i].mProp:setColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], fAlpha )
                else
                    LeaderboardEntry.mChars[i].mProp:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], fAlpha )
                end                
            end
            
            if x > -Constants.ValidDrawHalfWidth then
                bStillOnScreen = true
            end
        end

        --if we don't have any left on screen, we're done scrolling
        if bStillOnScreen == false then
            LeaderboardEntry.mCourtesyTimer = fCourtesyWait
        end
    end
end

function LeaderboardEntry:CheckKeyboard()        
    if not LeaderboardEntry.mScrollingToExit then    
        local i = 1
        local keyboard = MOAIInputMgr.device.keyboard
        if keyboard then  
            local textWalker = string.gmatch(LeaderboardEntry.mCharsString, ".")
            for i = 1, LeaderboardEntry.mNumChars do
                local sChar = textWalker()
                local sCharUpper = string.upper(sChar)
                if keyboard:keyDown(sChar) or keyboard:keyDown(sCharUpper) then
                    --key was pressed, if we're not eating input, respond to it
                    if not LeaderboardEntry.mEatKeyboardInput then
                        --respond to input
                        local iCharIndex = LeaderboardEntry:GetLetterIndexFromChar(sChar)
                        LeaderboardEntry:SelectLetter( iCharIndex )
                        LeaderboardEntry.mEatKeyboardInput = true
                    end
                    
                    return
                end
            end
            
            --look for special keys            
            if keyboard:keyDown(8) or keyboard:keyDown(127) then
                --backspace or delete was hit
                local iBackspaceIndex = LeaderboardEntry:GetLetterIndexFromChar("-")            
                LeaderboardEntry:SelectLetter( iBackspaceIndex )
                LeaderboardEntry.mEatKeyboardInput = true
                return
            end
            
            --if we got here, there wasn't input, so reset our eat input flag
            LeaderboardEntry.mEatKeyboardInput = false
        end
    end
end

function LeaderboardEntry:HandleEsc()
    if LeaderboardEntry.mCurrentState == Constants.LeaderboardEntryStateSelection then
        local iCharIndex = LeaderboardEntry:GetHighlightedLetterIndex()
        if iCharIndex ~= -1 then
            local iPropIndex = LeaderboardEntry.mChars[iCharIndex].mProp:getIndex()
            local iExitIndex = RadFont.mFontDeck.names[ ";" ]
            
            if iPropIndex == iExitIndex then
                LeaderboardEntry.mCurrentState = Constants.LeaderboardEntryStateExit 
            else
                LeaderboardEntry.mScrollingToExit = true
            end
        end
    end
end

function LeaderboardEntry:SetScrollLockTimer(fDuration)
    LeaderboardEntry.mScrollLockTimer = fDuration

    --no input, clear scroll state
    LeaderboardEntry.mScrollXRegistered = false
    LeaderboardEntry.mScrollXLastPos = 0.0
    LeaderboardEntry.mScrollYLastPos = 0.0
end

function LeaderboardEntry:ScrollChars(fAmount)
    local fWrapBorderLeft = -Constants.ValidDrawHalfWidth - LeaderboardEntry.mCharSpacing
    local fWrapBorderRight = Constants.ValidDrawHalfWidth
    local fCharsXSpan = LeaderboardEntry.mNumChars * LeaderboardEntry.mCharSpacing
    local fWidthDiff = LeaderboardEntry.mCharHighlightWidth - LeaderboardEntry.mCharWidth
    local bScrollLeft = fAmount < 0.0
    
    local i = 1 
    for i = 1, LeaderboardEntry.mNumChars do
        --update position
        local x, y, z = LeaderboardEntry.mChars[i].mProp:getLoc()
        x = LeaderboardEntry.mChars[i].mXPos
        y = LeaderboardEntry.mCharYPos
        z = 0.0
        
        x = x + fAmount
        if bScrollLeft then
            if x < fWrapBorderLeft then
                x = x + fCharsXSpan
            end        
        else
            if x > fWrapBorderRight then
                x = x - fCharsXSpan
            end
        end
        
        LeaderboardEntry.mChars[i].mXPos = x
                              
        --set position and scale according to placement
        if x > -LeaderboardEntry.mCharSpacing and x < 0.0 then
            x = x - (fWidthDiff * 0.5)
            y = y + LeaderboardEntry.mPullingDownAmount
            z = 10.0
            LeaderboardEntry.mChars[i].mProp:setLoc( x, y, z )
            LeaderboardEntry.mChars[i].mProp:setScl( LeaderboardEntry.mCharHighlightScale, -LeaderboardEntry.mCharHighlightScale )
            
            if LeaderboardEntry.mCanPullDown then
                LeaderboardEntry.mChars[i].mProp:setColor( Constants.SpecialColor[1], Constants.SpecialColor[2], Constants.SpecialColor[3], 1.0 )
            elseif LeaderboardEntry.mChars[i].mChar == '-' or LeaderboardEntry.mChars[i].mChar == ';' then
                LeaderboardEntry.mChars[i].mProp:setColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1.0 )
            else
                LeaderboardEntry.mChars[i].mProp:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )
            end
        else
            local fAlpha = 1.0 - (math.min(math.abs(x) / Constants.ValidDrawHalfWidth, 1.0))
            LeaderboardEntry.mChars[i].mProp:setLoc( x, y, z )
            LeaderboardEntry.mChars[i].mProp:setScl( LeaderboardEntry.mCharScale, -LeaderboardEntry.mCharScale )
            
            if LeaderboardEntry.mChars[i].mChar == '-' or LeaderboardEntry.mChars[i].mChar == ';' then
                LeaderboardEntry.mChars[i].mProp:setColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], fAlpha )
            else
                LeaderboardEntry.mChars[i].mProp:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], fAlpha )
            end
        end
    end  
end

function LeaderboardEntry:SelectLetter( i )
    if i ~= -1 then
        local iSelectedIndex = LeaderboardEntry.mChars[i].mProp:getIndex()
        local iBackspaceIndex = RadFont.mFontDeck.names[ "-" ]
        local iExitIndex = RadFont.mFontDeck.names[ ";" ]
        local iUnderscoreIndex = RadFont.mFontDeck.names[ "_" ]
        
        --if we selected exit, we're done
        if iSelectedIndex == iExitIndex then
            LeaderboardEntry.mCurrentState = Constants.LeaderboardEntryStateExit 
            AudioManager:PlaySound( 'Radius/SFX/InputLetter' )
        --if we selected backspace.. back up
        elseif iSelectedIndex == iBackspaceIndex then
            LeaderboardEntry.mInitials[ LeaderboardEntry.mInitialIndex ].mProp:setIndex( iUnderscoreIndex )
            LeaderboardEntry.mInitials[ LeaderboardEntry.mInitialIndex ].mChar = "_"    
            
            LeaderboardEntry.mInitialIndex = LeaderboardEntry.mInitialIndex - 1
            if LeaderboardEntry.mInitialIndex <= 0 then
                LeaderboardEntry.mInitialIndex = 1
            end                
            
            LeaderboardEntry.mInitials[ LeaderboardEntry.mInitialIndex ].mProp:setIndex( iUnderscoreIndex )
            LeaderboardEntry.mInitials[ LeaderboardEntry.mInitialIndex ].mChar = "_"                
            
            AudioManager:PlaySound( 'Radius/SFX/DeleteLetter' )
        else
            --we're setting a letter, make it shine
            LeaderboardEntry.mInitialHighlightTimer = LeaderboardEntry.mInitialHighlightDuration 
            LeaderboardEntry.mInitialHighlightIndex = LeaderboardEntry.mInitialIndex  
            
            --set the letter we selected
            LeaderboardEntry.mInitials[ LeaderboardEntry.mInitialIndex ].mProp:setIndex( iSelectedIndex )
            LeaderboardEntry.mInitials[ LeaderboardEntry.mInitialIndex ].mChar = LeaderboardEntry.mChars[i].mChar
        
            LeaderboardEntry.mInitialIndex = LeaderboardEntry.mInitialIndex + 1
            if LeaderboardEntry.mInitialIndex > 3 then
                LeaderboardEntry.mInitialIndex = 3
                LeaderboardEntry.mScrollingToExit = true
            end
            
            LeaderboardEntry:SetScrollLockTimer( 0.5 )
            AudioManager:PlaySound( 'Radius/SFX/InputLetter' )
        end
    end  
end

function LeaderboardEntry:GetHighlightedLetterIndex()
    local i = 1 
    for i = 1, LeaderboardEntry.mNumChars do
        local x, y, z = LeaderboardEntry.mChars[i].mProp:getLoc()
        x = LeaderboardEntry.mChars[i].mXPos
        if x > -LeaderboardEntry.mCharSpacing and x < 0.0 then 
            return i
        end
    end      
    
    return -1
end

function LeaderboardEntry:GetLetterIndexFromChar( sChar )
    local i = 1 
    for i = 1, LeaderboardEntry.mNumChars do
        if LeaderboardEntry.mChars[i].mChar == sChar then
            return i
        end
    end      
    
    return -1
end

function LeaderboardEntry:ScrollToExit()
    LeaderboardEntry.mScrollXVelocity = 0.0
    
    local fMovementStep = LeaderboardEntry.mEntrySpeed * 0.033--Constants.FrameStep
    LeaderboardEntry.mEntryAmount = LeaderboardEntry.mEntryAmount - fMovementStep
    
    LeaderboardEntry:ScrollChars(-fMovementStep)    
    
    local i = 1 
    for i = 1, LeaderboardEntry.mNumChars do
        local x, y, z = LeaderboardEntry.mChars[i].mProp:getLoc()
        if x > -LeaderboardEntry.mCharSpacing and x < 0.0 then 
            local iSelectedIndex = LeaderboardEntry.mChars[i].mProp:getIndex()
            local iExitIndex = RadFont.mFontDeck.names[ ";" ]
            
            if iSelectedIndex == iExitIndex then
                LeaderboardEntry.mScrollingToExit = false
                LeaderboardEntry:SetScrollLockTimer( 1.0 )
            end
            
            break
        end
    end    
end

function LeaderboardEntry:PositionInitials()
    local fInitialX = LeaderboardEntry.mCharWidth * -1.5    
    for i = 1, 3 do        
        LeaderboardEntry.mInitials[i].mProp:setLoc( fInitialX, LeaderboardEntry.mCharWidth * 1.5, 0.0 )     
        fInitialX = fInitialX + LeaderboardEntry.mCharWidth
    end
end

function LeaderboardEntry:PositionCharacters()
    local i = 1 
    for i = 1, LeaderboardEntry.mNumChars do
        local fXLocation = -Constants.ValidDrawHalfWidth + (LeaderboardEntry.mCharSpacing * i)
        LeaderboardEntry.mChars[i].mProp:setLoc( fXLocation, LeaderboardEntry.mCharYPos, 0.0 )
        LeaderboardEntry.mChars[i].mXPos = fXLocation
    end
end

return LeaderboardEntry