local Constants = require('Constants')
local DFMath = require('DFCommon.Math')
local RadFont = require('RadFont')
local Renderer = require('Renderer')
local RadEffects = require('RadEffects')
local AudioManager = require('AudioManager')
local OpponentScore = {}

-- Text Line Object
function OpponentScore:NewScore()
    local ScoreApproach = {}
    
    function ScoreApproach:Init() 
        ScoreApproach.mEnabled = false
        ScoreApproach.mTextActive = false
        ScoreApproach.mScore = 0
        ScoreApproach.mInitials = nil
        ScoreApproach.mScoreText = nil
        ScoreApproach.mNumCharactersInScore = 0
        ScoreApproach.mInitialProps = {}
        ScoreApproach.mScoreProps = {}
        ScoreApproach.mCurrentZ = 0.0
    end
    
    function ScoreApproach:Enable( iScore, tInitials, tScoreText )
        ScoreApproach.mEnabled = true
        ScoreApproach.mTextActive = false
        ScoreApproach.mScore = iScore
        ScoreApproach.mInitials = tInitials
        ScoreApproach.mScoreText = tScoreText   
        ScoreApproach.mNumCharactersInScore = #tScoreText
    end
    
    function ScoreApproach:CreateText( tColor1, tColor2 )   
        ScoreApproach.mCurrentZ = OpponentScore.mStartZ
        
        local i = 1
        local initialsWalker = string.gmatch( ScoreApproach.mInitials, "." )
        local fXBegin = OpponentScore.mLetterWidth * -1.5
        local fZ = OpponentScore.mStartZ
        local fY = (OpponentScore.mCosAngle * fZ) + OpponentScore.mYOffset
        for i = 1, 3 do
            ScoreApproach.mInitialProps[i] = MOAIProp.new()
            ScoreApproach.mInitialProps[i].__typename = "ScoreApproachLetterProp"
            ScoreApproach.mInitialProps[i]:setDeck( RadFont.mFontDeck )        
            ScoreApproach.mInitialProps[i]:setScl( OpponentScore.mLetterScale, -OpponentScore.mLetterScale )
            ScoreApproach.mInitialProps[i]:setLoc( fXBegin, fY, fZ )
            ScoreApproach.mInitialProps[i]:setRot( OpponentScore.mScoreAngle, 0.0, 0.0 )
            ScoreApproach.mInitialProps[i]:setIndex( RadFont.mFontDeck.names[ initialsWalker() ] )
            ScoreApproach.mInitialProps[i]:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
            ScoreApproach.mInitialProps[i]:setColor( tColor1[1], tColor1[2], tColor1[3], 1.0 )
            Renderer.mGameLayer:insertProp( ScoreApproach.mInitialProps[i] )        
            
            fXBegin = fXBegin + OpponentScore.mLetterWidth
        end
        
        local scoreWalker = string.gmatch( ScoreApproach.mScoreText, "." )
        fXBegin = (ScoreApproach.mNumCharactersInScore * OpponentScore.mLetterWidth * -0.5)
        fZ = fZ + OpponentScore.mLetterWidth
        fY = (OpponentScore.mCosAngle * fZ) + OpponentScore.mYOffset
        for i = 1, ScoreApproach.mNumCharactersInScore do
            ScoreApproach.mScoreProps[i] = MOAIProp.new()
            ScoreApproach.mScoreProps[i].__typename = "ScoreApproachLetterProp"
            ScoreApproach.mScoreProps[i]:setDeck( RadFont.mFontDeck )        
            ScoreApproach.mScoreProps[i]:setScl( OpponentScore.mLetterScale, -OpponentScore.mLetterScale )
            ScoreApproach.mScoreProps[i]:setLoc( fXBegin, fY, fZ )
            ScoreApproach.mScoreProps[i]:setRot( OpponentScore.mScoreAngle, 0.0, 0.0 )
            ScoreApproach.mScoreProps[i]:setIndex( RadFont.mFontDeck.names[ scoreWalker() ] )
            ScoreApproach.mScoreProps[i]:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
            ScoreApproach.mScoreProps[i]:setColor( tColor2[1], tColor2[2], tColor2[3], 1.0 )
            Renderer.mGameLayer:insertProp( ScoreApproach.mScoreProps[i] )
            
            fXBegin = fXBegin + OpponentScore.mLetterWidth
        end   
        
        ScoreApproach.mTextActive = true
    end
          
    function ScoreApproach:Update( iCurrentScore, tColor1, tColor2 )
        --if our text isn't active, wait until it is
        if not ScoreApproach.mTextActive then
            if iCurrentScore > ScoreApproach.mScore - OpponentScore.mScoreRange then
                --show score zoomin in
                ScoreApproach:CreateText( tColor1, tColor2 )
            end
        else
            --update text position with current score
            local fLowestScore = ScoreApproach.mScore - OpponentScore.mScoreRange
            local fT = (iCurrentScore - fLowestScore) / OpponentScore.mScoreRange
            local fDesiredZ = DFMath.lerp( OpponentScore.mStartZ, OpponentScore.mEndZ, fT )
            local fCurrentToDesiredZ = fDesiredZ - ScoreApproach.mCurrentZ
            local fZMovementStep = OpponentScore.mMaxZMovement * Constants.FrameStep
            if fCurrentToDesiredZ < fZMovementStep then
                ScoreApproach.mCurrentZ = fDesiredZ
            else
                ScoreApproach.mCurrentZ = ScoreApproach.mCurrentZ + fZMovementStep
            end

            --if we're beyond z, paste it on the front
            local i = 1
            local fZFront = 20.0
            local fXBegin = OpponentScore.mLetterWidth * -1.5                
            if ScoreApproach.mCurrentZ > 0.0 then
                local fY = Constants.ValidDrawHalfHeight - (OpponentScore.mLetterWidth * 2.0)
                for i = 1, 3 do
                    ScoreApproach.mInitialProps[i]:setLoc( fXBegin, fY, fZFront )
                    ScoreApproach.mInitialProps[i]:setColor( tColor1[1], tColor1[2], tColor1[3], 1.0 ) 
                    ScoreApproach.mInitialProps[i]:setRot( 0.0, 0.0, 0.0 ) 
                    fXBegin = fXBegin + OpponentScore.mLetterWidth
                end            
                
                fXBegin = (ScoreApproach.mNumCharactersInScore * OpponentScore.mLetterWidth * -0.5)
                fY = fY + OpponentScore.mLetterWidth
                for i = 1, ScoreApproach.mNumCharactersInScore do
                    ScoreApproach.mScoreProps[i]:setLoc( fXBegin, fY, fZFront )
                    ScoreApproach.mScoreProps[i]:setColor( tColor2[1], tColor2[2], tColor2[3], 1.0 )  
                    ScoreApproach.mScoreProps[i]:setRot( 0.0, 0.0, 0.0 )
                    fXBegin = fXBegin + OpponentScore.mLetterWidth
                end              
            else
                local fAlpha = (ScoreApproach.mCurrentZ - OpponentScore.mStartZ) / (OpponentScore.mEndZ - OpponentScore.mStartZ)
                
                --move toward front
                local fZ = ScoreApproach.mCurrentZ
                local fY = (OpponentScore.mCosAngle * fZ) + OpponentScore.mYOffset
                for i = 1, 3 do
                    ScoreApproach.mInitialProps[i]:setLoc( fXBegin, fY, fZ )
                    ScoreApproach.mInitialProps[i]:setColor( tColor1[1], tColor1[2], tColor1[3], fAlpha )  
                    fXBegin = fXBegin + OpponentScore.mLetterWidth
                end            
                
                fXBegin = (ScoreApproach.mNumCharactersInScore * OpponentScore.mLetterWidth * -0.5)
                fZ = fZ + OpponentScore.mLetterWidth
                fY = (OpponentScore.mCosAngle * fZ) + OpponentScore.mYOffset
                for i = 1, ScoreApproach.mNumCharactersInScore do
                    ScoreApproach.mScoreProps[i]:setLoc( fXBegin, fY, fZ )
                    ScoreApproach.mScoreProps[i]:setColor( tColor2[1], tColor2[2], tColor2[3], fAlpha )  
                    fXBegin = fXBegin + OpponentScore.mLetterWidth
                end             
            end           
                        
            --check for score being past the camera
            if ScoreApproach.mCurrentZ > OpponentScore.mEndZ then    
                ScoreApproach:Disable( true )
            end
        end
    end
      
    function ScoreApproach:Disable( bShowFanfare )
        if bShowFanfare then
            local x, y, z
            x, y, z = ScoreApproach.mInitialProps[2]:getLoc()
            x = x + (OpponentScore.mLetterWidth * 0.5)
            RadEffects:SetEffectLayerToGameLayer()
            RadEffects:SetEffectTransform( x, y, z, 0.0, 0.0, 0.0 )
            RadEffects:CreateEffect( "Effects/Particles/Chord/ChargeCreation.particles", nil, nil, nil, Constants.ParticlePriorityHigh )
            RadEffects:CreateEffect( "Effects/Particles/Chord/ChargeFlash.particles", nil, nil, nil, Constants.ParticlePriorityHigh ) 
            RadEffects:CreateEffect( "Effects/Particles/Chord/ChargeEmbers.particles", nil, nil, nil, Constants.ParticlePriorityHigh )  
            AudioManager:PlaySound( 'Radius/SFX/Pro' )
        end
        
        if ScoreApproach.mTextActive then
            local i = 1
            for i = 1, 3 do
                Renderer.mGameLayer:removeProp( ScoreApproach.mInitialProps[i] )
            end
            ScoreApproach.mInitialProps = {}
            
            for i = 1, ScoreApproach.mNumCharactersInScore do
                Renderer.mGameLayer:removeProp( ScoreApproach.mScoreProps[i] )
            end
            ScoreApproach.mScoreProps = {}
        end

        ScoreApproach.mTextActive = false
        ScoreApproach.mEnabled = false
    end
    
    ScoreApproach:Init()
    return ScoreApproach
end

function OpponentScore:Init()
    OpponentScore.mScoreRange = 5000
    OpponentScore.mMinScoreSpacing = OpponentScore.mScoreRange * 0.5
    OpponentScore.mScoreAngle = 40.0
    OpponentScore.mSinAngle = math.sin( math.rad(OpponentScore.mScoreAngle) )
    OpponentScore.mCosAngle = math.cos( math.rad(OpponentScore.mScoreAngle) )
    
    OpponentScore.mNumScores = 10
    OpponentScore.mScoreLines = {}
    
    OpponentScore:DetermineSize()
        
    local i = 1
    for i = 1, OpponentScore.mNumScores do
        OpponentScore.mScoreLines[i] = OpponentScore:NewScore()
    end     
end

function OpponentScore:DetermineSize()
    local fDesiredLetterImageSize = Constants.ValidDrawHeight / 20.0
    OpponentScore.mLetterScale = fDesiredLetterImageSize / 128.0
    OpponentScore.mLetterWidth = fDesiredLetterImageSize
    
    OpponentScore.mYOffset = OpponentScore.mLetterWidth * 8.0
    OpponentScore.mStartZ = Renderer.mGameCameraZDistance * -2.0
    OpponentScore.mEndZ = Renderer.mGameCameraZDistance * 0.5
    OpponentScore.mMaxZMovement = (OpponentScore.mEndZ - OpponentScore.mStartZ) / 3.0
end

function OpponentScore:UpdateSize()
    OpponentScore:DetermineSize()
    
    --update size of any visible scores
    local i = 1
    for i = 1, OpponentScore.mNumScores do
        if OpponentScore.mScoreLines[i].mEnabled and OpponentScore.mScoreLines[i].mTextActive then
            local j = 1
            for j = 1, 3 do 
                OpponentScore.mScoreLines[i].mInitialProps[j]:setScl( OpponentScore.mLetterScale, -OpponentScore.mLetterScale )
            end
            for j = 1, OpponentScore.mScoreLines[i].mNumCharactersInScore do
                OpponentScore.mScoreLines[i].mScoreProps[j]:setScl( OpponentScore.mLetterScale, -OpponentScore.mLetterScale )
            end               
        end
    end    
end

local function UpdateOpponentScores( s1, s2, s3, s4, s5, s6, s7, s8, s9, s10 )
    --make sure we got some scores back
    local tScoreTable = { s1, s2, s3, s4, s5, s6, s7, s8, s9, s10 }
    if tScoreTable[1] ~= nil then
        local iPrevScore = -1
        local iScoreApproachIndex = 1
        
        local i = 1
        
        --add all the scores we got back
        for i = 1, 10 do
            if tScoreTable[i] ~= nil then
                local sInitials = DFRad:ValueToInitials( tScoreTable[i].context )
                local iScore = tScoreTable[i].value
                local sScore = tostring( iScore )
                
                --don't queue up scores that are too close in a row
                if iPrevScore < 0 or iScore < iPrevScore - OpponentScore.mMinScoreSpacing then  
                    OpponentScore.mScoreLines[iScoreApproachIndex]:Enable( iScore, sInitials, sScore )
                    iScoreApproachIndex = iScoreApproachIndex + 1
                    iPrevScore = iScore
                end
            end
        end
    
        --clear all the other score entries
        for i = iScoreApproachIndex, Constants.LeaderboardCount do
            OpponentScore.mScoreLines[i]:Disable( false )
        end
    end
end

function OpponentScore:InitForGame()
    --run through the scores from best to worst..
    local iPrevScore = -1
    local iScoreApproachIndex = 1
    
    local i = 1
    for i = 1, Constants.LeaderboardCount do
        --don't queue up scores that are too close in a row
        local iScore = tonumber( Constants.Leaderboards[i][2] )
        if iPrevScore < 0 or iScore < iPrevScore - OpponentScore.mMinScoreSpacing then  
            OpponentScore.mScoreLines[iScoreApproachIndex]:Enable( iScore, Constants.Leaderboards[i][1], Constants.Leaderboards[i][2] )
            iScoreApproachIndex = iScoreApproachIndex + 1
            iPrevScore = iScore
        end
    end
    
    --if we're on iOS, ping game center and try to get our buddy's scores to compete against
    if MOAIGameCenterIOS then
        MOAIGameCenterIOS.setGetScoresCallback( UpdateOpponentScores )
        MOAIGameCenterIOS.getScores( Constants.IOSLeaderboardID, GKLeaderboardPlayerScopeFriendsOnly )
    end
end

function OpponentScore:ClearScores()
    local i = 1
    for i = 1, OpponentScore.mNumScores do
        if OpponentScore.mScoreLines[i].mEnabled then
            OpponentScore.mScoreLines[i]:Disable( false )
        end
    end    
end

function OpponentScore:Update( iCurrentScore )
    local i = 1
    for i = 1, OpponentScore.mNumScores do
        if OpponentScore.mScoreLines[i].mEnabled then
            OpponentScore.mScoreLines[i]:Update( iCurrentScore, Constants.Color3, Constants.Color2 )
        end
    end 
end

return OpponentScore
