local Constants = require('Constants')
local TextManager = require('TextManager')
local AudioManager = require('AudioManager')
local HUD = require('HUD')
local PauseScreen = {}

function PauseScreen:Init()
    PauseScreen.mPauseTimer = 0.0
    PauseScreen.mPauseMusicBeginTime = 3.0
    PauseScreen.mPauseMusicFullVolumeTime = 8.0
    PauseScreen.mPauseMusicCue = nil
end

function PauseScreen:ShowPauseScreen()    
    PauseScreen.mPauseTimer = 0.0
    PauseScreen:ShowPauseText()
    PauseScreen:CleanUpMusicCue()
end

function PauseScreen:HidePauseScreen()
    TextManager:ClearAll()
    PauseScreen:CleanUpMusicCue()
end

function PauseScreen:ShowPauseText()
    TextManager:ClearAll()
    
    local fDesiredLetterSize = Constants.ValidDrawHeight / 10.0
    local fLetterScale = fDesiredLetterSize / 128.0
    local fLetterSize = 128.0 * fLetterScale 
    TextManager:SetTextAttributes( fLetterSize, fLetterScale, Constants.Color2, 0.5 )
    TextManager:SetFadeAttributes( 0.5, 15.0 )
    
    local fLineX = fLetterSize * -3.0
    local fLineY = fLetterSize * -0.5
    local tPausedLine = "paused"
    TextManager:AddLine( tPausedLine, fLineX, fLineY, 30.0 )    
end

function PauseScreen:CleanUpMusicCue()
    if PauseScreen.mPauseMusicCue ~= nil then
        AudioManager:StopSound( PauseScreen.mPauseMusicCue, true )
        PauseScreen.mPauseMusicCue = nil    
    end
end

function PauseScreen:UpdatePauseScreen( tGameInputs )
    PauseScreen.mPauseTimer = PauseScreen.mPauseTimer + Constants.FrameStep
    if PauseScreen.mPauseTimer > PauseScreen.mPauseMusicBeginTime and PauseScreen.mPauseMusicCue == nil then
        --begin pause screen music
        PauseScreen.mPauseMusicCue = AudioManager:PlaySound( 'Radius/Music/PauseLoop' )
    end
    
    --update pause music volume
    if PauseScreen.mPauseMusicCue ~= nil then
        local fVolumeRange = PauseScreen.mPauseMusicFullVolumeTime - PauseScreen.mPauseMusicBeginTime
        local fVolume = math.min( (PauseScreen.mPauseTimer - PauseScreen.mPauseMusicBeginTime) / fVolumeRange, 1.0 )
        PauseScreen.mPauseMusicCue:setVolume( fVolume )
    end
    
    if tGameInputs[1]:IsTouched() or tGameInputs[2]:IsTouched() then
        --if there's input coming in, make sure the paused text is showing
        TextManager:ClearAll()
        PauseScreen:ShowPauseText()
    end 
    
    if tGameInputs[1]:WasTouched() then
        if HUD:IsInEscBounds( tGameInputs[1].mX, tGameInputs[1].mY ) then
            return Constants.mPauseScreenReturnQuit    
        end
        
        if HUD:IsInNextBounds( tGameInputs[1].mX, tGameInputs[1].mY ) then
            return Constants.mPauseScreenReturnNext    
        end
    end
    
    return Constants.mPauseScreenReturnNone
end

function PauseScreen:UpdateSize()
    --if the player is messing with the screen, just jiggle the handle
    if Constants.mPaused then
        TextManager:ClearAll()
        PauseScreen:ShowPauseText()
    end
end

return PauseScreen