local Constants = require("Constants")
local Grid = require("Grid")
local Wind = require("Wind")
local PostDebug = require("postfx.PostDebug")
local Post = require("postfx.Post")
local Renderer = require("Renderer")
local ColorManager = require("ColorManager")
local RadEffects = require("RadEffects")
--require "socket"

local PerformanceTest = {}

function PerformanceTest.Start()
    -- uncomment this to force a result for the test
    PerformanceTest.OverrideQuality = Constants.VisualQualityLevelHigh

    PerformanceTest.FrameCount = 0
    PerformanceTest.StartTime = 0 --socket.gettime()*1000
    PerformanceTest.PreviousFrameTime = PerformanceTest.StartTime
    PerformanceTest.TotalFrameTime = 0
end

function PerformanceTest.Update()
    PerformanceTest.FrameCount = PerformanceTest.FrameCount + 1
    
    local currentTime = 10000000 --socket.gettime()*1000
    local elapsed = currentTime - PerformanceTest.PreviousFrameTime
    PerformanceTest.TotalFrameTime = PerformanceTest.TotalFrameTime + elapsed
    
    PerformanceTest.AveragePerformance = PerformanceTest.TotalFrameTime / PerformanceTest.FrameCount
    print("PerformanceTest.AveragePerformance: " .. PerformanceTest.FrameCount .. " = " .. elapsed .. "(" .. PerformanceTest.AveragePerformance .. " * " .. PerformanceTest.TotalFrameTime .. ")")
    
    if PerformanceTest.FrameCount == 10 then
        Grid:Enable(true)
        Wind:Enable(true)
        PostDebug:BasicComp()
    elseif PerformanceTest.FrameCount == 20 then
        for i=1,10 do
            RadEffects:SetEffectTransform( 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 )
            RadEffects:SetEffectLayerToGameLayer()
            RadEffects:CreateInfiniteDurationEffect( "Effects/Particles/Backgrounds/Magicalness.particles")
        end
        
    elseif PerformanceTest.FrameCount == 40 then
        PostDebug:ShowAccumWithEdges(true)
    elseif PerformanceTest.FrameCount == 50 then
        PostDebug:ShowWideAccumBlur()
    elseif PerformanceTest.FrameCount == 60 then
        PostDebug:ShowCrossBlur()
    elseif PerformanceTest.FrameCount == 70 then
        for i=1,100 do
            RadEffects:SetEffectTransform( 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 )
            RadEffects:SetEffectLayerToGameLayer()
            RadEffects:CreateInfiniteDurationEffect( "Effects/Particles/Backgrounds/Magicalness.particles")
        end
    
        local finalColorBuffer = Post:GetUnusedBuffer()
        local sceneBuffer = Post:GetUnusedBuffer()
        Post:GenerateBasicCompFilter(sceneBuffer)
        Post:EdgeFilter(sceneBuffer, finalColorBuffer, 1)
        Post.rScreen:setInBuffer(finalColorBuffer)
        
        RadEffects:SetEffectTransform( 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 )
        RadEffects:SetEffectLayerToGameLayer()
        RadEffects:CreateInfiniteDurationEffect( "Effects/Particles/Backgrounds/SwirlyStarsB.particles")
    elseif PerformanceTest.FrameCount == 80 then
        RadEffects:SetEffectTransform( 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 )
        RadEffects:SetEffectLayerToGameLayer() 
        RadEffects:CreateInfiniteDurationEffect( "Effects/Particles/Backgrounds/SwirlyStarsB.particles")
    
        local finalColorBuffer = Post:GetUnusedBuffer()   
        local accumBuffer = Post:GetUnusedBuffer()
        local scratchBuffer = Post:GetUnusedBuffer()
        local scratchBuffer2 = Post:GetUnusedBuffer()
        local remapBuffer = Post:GetUnusedBuffer()
        local sceneBuffer = Post:GetUnusedHalfBuffer()
        accumBuffer:setClearColor()
        sceneBuffer:setClearColor(0,0,0,0)
        
        local rTexture
        if gradToUse == 0 then 
            rTexture = Post.rGradientTexture
        elseif gradToUse == 1 then
            rTexture = Post.rGradientTextureC
        elseif gradToUse == 2 then 
            rTexture = Post.rGradientTextureB
        end

        Post:CombineFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, kBLEND_ALPHA, {1,1,1,1})
        Post:AccumFilter(sceneBuffer, accumBuffer, scratchBuffer, {.97,.97,.97,1}, {.99,.99})
        Post:BlurFilter(scratchBuffer, accumBuffer, .003, kBLEND_ALPHA, {1,1,1,1})
        Post:RemapFilter(accumBuffer, remapBuffer, rTexture, scrollRate, kBLEND_ALPHA, {1,1,1,1})
        Post:CombineFilter(remapBuffer, sceneBuffer, scratchBuffer2, kBLEND_ADD, {1,1,1,1})
        if bEdges then 
            Post:EdgeFilter(scratchBuffer2, finalColorBuffer, 1, kBLEND_ADD, {1,1,1,1})
            Post.rScreen:setInBuffer(finalColorBuffer)
        else
            Post.rScreen:setInBuffer(scratchBuffer2)
        end
    elseif PerformanceTest.FrameCount == 90 then
        for i=1,200 do
            RadEffects:SetEffectTransform( 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 )
            RadEffects:SetEffectLayerToGameLayer()
            RadEffects:CreateInfiniteDurationEffect( "Effects/Particles/Backgrounds/Magicalness.particles")
        end
    
        RadEffects:SetEffectTransform( 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 )
        RadEffects:SetEffectLayerToGameLayer() 
        RadEffects:CreateInfiniteDurationEffect( "Effects/Particles/Backgrounds/SwirlyStarsB.particles")
    
        ColorManager.BGColor            = {0.2,0,0}
        ColorManager:SetColor()
        
        local finalColorBuffer = Post:GetUnusedHalfBuffer()   
        local accumBuffer = Post:GetUnusedHalfBuffer()
        local scratchBuffer = Post:GetUnusedHalfBuffer()
        local sceneBuffer = Post:GetUnusedHalfBuffer()
        accumBuffer:setClearColor()
        
        Post:CombineFilter(Renderer.mGameBuffer, Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ALPHA, {1,1,1,1})
        Post:BlitFilter(Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ADD)
        Post:AccumFlowFilter(sceneBuffer, accumBuffer, scratchBuffer, Post.rFlowTexture, {.99,.94,.94,1}, {1,1}, {0,.0}, -.02)
        Post:BlurFilter(scratchBuffer, accumBuffer, .002, kBLEND_ALPHA, {1,1,1,1})
        Post:BlitFilter(scratchBuffer, finalColorBuffer)
        Post.rScreen:setInBuffer(finalColorBuffer)
    elseif PerformanceTest.FrameCount == 100 then
        local finalColorBuffer = Post:GetUnusedQuarterBuffer()
        local sceneBuffer = Post:GetUnusedQuarterBuffer()
        Post:GenerateBasicCompFilter(sceneBuffer)
        Post:EdgeFilter(sceneBuffer, finalColorBuffer, 1)
        Post.rScreen:setInBuffer(finalColorBuffer)
    end
    
    PerformanceTest.PreviousFrameTime = currentTime
    
    if PerformanceTest.FrameCount == 120 then
        local nVisQual = Constants.VisualQualityLevelLow
        local sVisQual = "Low"
        
        if PerformanceTest.AveragePerformance < 17.0 then
            nVisQual = Constants.VisualQualityLevelUltra
        elseif PerformanceTest.AveragePerformance < 24.0 then
            nVisQual = Constants.VisualQualityLevelHigh
        end
        
        if nil ~= PerformanceTest.OverrideQuality then
            nVisQual = PerformanceTest.OverrideQuality
        end
        
        if nVisQual == Constants.VisualQualityLevelHigh then
            sVisQual = "High"
        elseif nVisQual == Constants.VisualQualityLevelUltra then
            sVisQual = "Ultra"
        end
        
        Constants.DesiredVisualQuality = nVisQual
        
        if g_DebugDisplay then
            g_DebugDisplay:SetValue("VIZ", sVisQual)
        end
    
        Constants.DesiredGameState = Constants.GameStateTitle
    end
end

function PerformanceTest.Stop()
    
end

return PerformanceTest
