local ParticleSystem = require("ParticleSystem")
local Constants = require("Constants")
local RadEffects = {}

function RadEffects:Init(tRenderer)
    RadEffects.NewGoodExplosionFunction = RadEffects.NoEffect
    RadEffects.particleAccumLayer = nil
    RadEffects.tRenderer = tRenderer
    
    RadEffects.mPosX = 0.0
    RadEffects.mPosY = 0.0
    RadEffects.mPosZ = 0.0
    RadEffects.mRotX = 0.0
    RadEffects.mRotY = 0.0
    RadEffects.mRotZ = 0.0    
    RadEffects.mPropLayer = nil
    
    RadEffects.tInifiniteEffects = {}
end

function RadEffects.NoEffect()
end

function RadEffects.ClearInfiniteDurationEffects()
    if RadEffects.tInfiniteEffects then
        for idx,rEffect in ipairs(RadEffects.tInfiniteEffects) do
            if rEffect and rEffect.rParticleSystem then
                rEffect.rParticleSystem:die()
            end
        end
    end
    
    RadEffects.tInfiniteEffects = {}
end

function RadEffects.ClearAttachedEffects(sPropId)
    if RadEffects.tInfiniteEffects then
        for idx=#(RadEffects.tInfiniteEffects),1,-1 do
            local rEffect = RadEffects.tInfiniteEffects[idx]
            if not rEffect.rParentProp or sPropId == rEffect.sPropId then
                
                table.remove(RadEffects.tInfiniteEffects, idx)
                
                -- TODO: actually want it to just stop spawning new particles if possible
                rEffect.rParticleSystem:die()
            end
        end
    end
end

function RadEffects:SetGoodExplosionCallback(fnGoodExplosion)
    RadEffects.NewGoodExplosionFunction = fnGoodExplosion
end

function RadEffects:ClearEffectTransform()
    RadEffects.mPosX = 0.0
    RadEffects.mPosY = 0.0
    RadEffects.mPosZ = 0.0
    RadEffects.mRotX = 0.0
    RadEffects.mRotY = 0.0
    RadEffects.mRotZ = 0.0    
end

function RadEffects:SetEffectTransform( posX, posY, posZ, rotX, rotY, rotZ, scale )
    RadEffects.mPosX = posX
    RadEffects.mPosY = posY
    RadEffects.mPosZ = posZ
    RadEffects.mRotX = rotX
    RadEffects.mRotY = rotY
    RadEffects.mRotZ = rotZ   
    if not scale then scale = 1.0 end
    RadEffects.mScale = scale
end

function RadEffects:SetEffectLayerToGameLayer()
    RadEffects.mPropLayer = RadEffects.tRenderer.mGameLayer
end

function RadEffects:SetEffectLayerToParticleAccumLayer()
    RadEffects.mPropLayer = RadEffects.particleAccumLayer
    
    if Constants.VisualQuality == Constants.VisualQualityLevelLow then
        RadEffects.mPropLayer = RadEffects.tRenderer.mGameLayer
    end
end

function RadEffects:SetEffectLayer( rLayer )
    RadEffects.mPropLayer = rLayer
    if Constants.VisualQuality == Constants.VisualQualityLevelLow then
        RadEffects.mPropLayer = RadEffects.tRenderer.mGameLayer
    end
end

function RadEffects:CreateInfiniteDurationEffect( sPath, rTargetProp, tColor )
    local rEffectSystem = self:CreateEffect(sPath, rTargetProp, tColor, nil, Constants.ParticlePriorityHigh )
    
    table.insert(RadEffects.tInfiniteEffects, rEffectSystem)
    
    return rEffectSystem
end


function RadEffects:CreateAttachedInfiniteEffect(sPath, rParentProp, sPropId, rTargetProp, tColor)
    local posX = 0.0
    local posY = 0.0
    local posZ = 0.0
    
    self:SetEffectTransform( posX, posY, posZ, 0.0, 0.0, 0.0 )
    
    local rEffectSystem = self:CreateInfiniteDurationEffect(sPath, rTargetProp, tColor)

    local rProp = rEffectSystem:getProp()
    rProp:setParent(rParentProp)
    
    
    rEffectSystem.rParentProp = rParentProp
    rEffectSystem.sPropId = sPropId
    
    rParentProp:setVisible(true)
    
    return rEffectSystem
end

function RadEffects:CreateEffect( sPath, rTargetProp, tColor, rCompletedCallback, iPriority )
    local rEffectSystem = nil
    
    if Constants.UseParticleEffects then
        if RadEffects.mPropLayer ~= nil then
            rEffectSystem = ParticleSystem.new( nil, sPath )
            rEffectSystem:setPriority( iPriority )
            rEffectSystem:init()
            rEffectSystem.mCompleteCallback = rCompletedCallback
                        
            local pEffectProp = rEffectSystem:getProp()
            pEffectProp:setLoc( RadEffects.mPosX, RadEffects.mPosY, RadEffects.mPosZ )
            pEffectProp:setRot( RadEffects.mRotX, RadEffects.mRotY, RadEffects.mRotZ )

            if rTargetProp ~= nil then
                rEffectSystem:setTargetProp(rTargetProp)
            end
            
            
            local rMaterial = pEffectProp:getMaterial()
            
            if rMaterial then
                rMaterial:setShaderValue( "g_fSystemScale", MOAIMaterial.VALUETYPE_FLOAT, RadEffects.mScale )
                if tColor then
                    rMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, tColor )
                end
            end
            
            rEffectSystem:start()   
            rEffectSystem:addToSceneLayer( RadEffects.mPropLayer )         
        end
    end
    
    return rEffectSystem
end

function RadEffects.SpawnPurpleStars(posX, posY, posZ, rTargetProp, rCallback)
    local Explosion = {}
    
    function Explosion:Init()   
        RadEffects:SetEffectTransform( posX, posY, posZ, 0.0, 0.0, 0.0 )
        RadEffects:SetEffectLayer( RadEffects.particleAccumLayer )
        self.rStarEffect = RadEffects:CreateEffect( "Effects/Particles/Explosions/PurpleStars/StarsA.particles", nil, nil, nil, Constants.ParticlePriorityLow )

        RadEffects:SetEffectLayer( RadEffects.tRenderer.mGameLayer )        
        self.rRippleEffect = RadEffects:CreateEffect( "Effects/Particles/Explosions/PurpleStars/RingRippleB.particles", nil, nil, nil, Constants.ParticlePriorityLow )                 
        
        RadEffects:SetEffectLayer( RadEffects.tRenderer.mGameLayer )
        self.rTracerEffect = RadEffects:CreateEffect( "Effects/Particles/Explosions/BubbleChamber/SimpleTracers.particles", nil, nil, nil, Constants.ParticlePriorityLow )  
        
        if not rCallback then
            rCallback = RadEffects.BubbleChamberTargetCallback
        end
            
        if rTargetProp ~= nil then
            RadEffects:SetEffectLayer( RadEffects.tRenderer.mGameLayer )
            self.rSeekingEffect = RadEffects:CreateEffect( "Effects/Particles/Explosions/PurpleStars/SeekingStars.particles", rTargetProp, Constants.GoodEffectColor, rCallback, Constants.ParticlePriorityLow )                 
        end
    end
    
    Explosion:Init()
    return Explosion
end


function RadEffects.SpawnAccumTracers(posX, posY, posZ, rTargetProp)
    local Explosion = {}
    
    function Explosion:Init()
        RadEffects:SetEffectTransform( posX, posY, posZ, 0.0, 0.0, 0.0 )    
        RadEffects:SetEffectLayer( RadEffects.tRenderer.mGameLayer )
        RadEffects:CreateEffect( "Effects/Particles/Explosions/AccumTracers/RingRippleC.particles", nil, nil, nil, Constants.ParticlePriorityLow )                 
        
        if rTargetProp ~= nil then
            RadEffects:SetEffectLayer( RadEffects.particleAccumLayer )
            RadEffects:CreateEffect( "Effects/Particles/Explosions/AccumTracers/SeekingStars.particles", rTargetProp, nil, nil, Constants.ParticlePriorityLow )                 
        end
    end
    
    Explosion:Init()
    return Explosion
end

function RadEffects:BubbleChamberTargetCallback( tParticleEffect )
    local Explosion = {}
    if tParticleEffect ~= nil then
        if tParticleEffect.rTargetProp ~= nil then
            local fTargetX, fTargetY, fTargetZ = tParticleEffect.rTargetProp:getLoc()
            RadEffects:SetEffectTransform( fTargetX, fTargetY, fTargetZ, 0.0, 0.0, 0.0 )   
            RadEffects:SetEffectLayer( RadEffects.tRenderer.mGameLayer )
            local rEffect = RadEffects:CreateEffect( "Effects/Particles/Bursts/BlipFlare.particles", nil, nil, nil, Constants.ParticlePriorityLow )                 
            Explosion.rEffect = rEffect
            
            local rEffectProp = rEffect:getProp()
            local rEffectMaterial = rEffectProp:getMaterial()
            rEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, Constants.GoodEffectColor)
        end
    end
    return Explosion
end

function RadEffects.SpawnBubbleChamber(posX, posY, posZ, rTargetProp, rCallback)
    local Explosion = {}
    
    function Explosion:Init()
        RadEffects:SetEffectTransform( posX, posY, posZ, 0.0, 0.0, 0.0 )   
        RadEffects:SetEffectLayer( RadEffects.particleAccumLayer )
        self.rRippleEffect = RadEffects:CreateEffect( "Effects/Particles/Explosions/BubbleChamber/RingRippleC.particles", nil, Constants.GoodEffectColor, nil, Constants.ParticlePriorityLow )  
        self.rTracerEffect = RadEffects:CreateEffect( "Effects/Particles/Explosions/BubbleChamber/SimpleTracers.particles", nil, Constants.GoodEffectColor, nil, Constants.ParticlePriorityLow )  
      
        if rTargetProp ~= nil then
            RadEffects:SetEffectTransform( posX, posY, posZ, 0.0,-90.0, 0.0 )  
            RadEffects:SetEffectLayer( RadEffects.particleAccumLayer )
            
            if not rCallback then
                rCallback = RadEffects.BubbleChamberTargetCallback
            end
            
            self.rSeekingEffect = RadEffects:CreateEffect( "Effects/Particles/Explosions/BubbleChamber/SeekingStars.particles", rTargetProp, Constants.GoodEffectColor, rCallback, Constants.ParticlePriorityLow  )  
        end
        
    end
    
    Explosion:Init()
    return Explosion
end

function RadEffects.SpawnBubbleChamberAlphaBlend(posX, posY, posZ, rTargetProp, rCallback)
    local Explosion = {}
    
    function Explosion:Init()
        RadEffects:SetEffectTransform( posX, posY, posZ, 0.0, 0.0, 0.0 )   
        RadEffects:SetEffectLayer( RadEffects.particleAccumLayer )
        self.rRippleEffect = RadEffects:CreateEffect( "Effects/Particles/Explosions/BubbleChamber/RingRippleCAlphaBlend.particles", nil, Constants.GoodEffectColor, nil, Constants.ParticlePriorityLow )  
        self.rTracerEffect = RadEffects:CreateEffect( "Effects/Particles/Explosions/BubbleChamber/SimpleTracersAlphaBlend.particles", nil, Constants.GoodEffectColor, nil, Constants.ParticlePriorityLow )  
      
        if rTargetProp ~= nil then
            RadEffects:SetEffectTransform( posX, posY, posZ, 0.0,-90.0, 0.0 )  
            RadEffects:SetEffectLayer( RadEffects.particleAccumLayer )
            
            if not rCallback then
                rCallback = RadEffects.BubbleChamberTargetCallback
            end
            
            self.rSeekingEffect = RadEffects:CreateEffect( "Effects/Particles/Explosions/BubbleChamber/SeekingStarsAlphaBlend.particles", rTargetProp, Constants.GoodEffectColor, rCallback, Constants.ParticlePriorityLow  )  
        end
        
    end
    
    Explosion:Init()
    return Explosion
end

function RadEffects.SpawnBubbleChamberBuild(posX, posY, posZ, rTargetProp)
    local Explosion = {}
    
    function Explosion:Init()
        RadEffects:SetEffectTransform( posX, posY, posZ, 0.0, 0.0, 0.0 )   
        RadEffects:SetEffectLayer( RadEffects.tRenderer.mGameLayer )
        RadEffects:CreateEffect( "Effects/Particles/Explosions/BubbleChamber/RingRippleC.particles", nil, nil, nil, Constants.ParticlePriorityLow )   
        
        RadEffects:SetEffectLayer( RadEffects.particleAccumLayer )
        RadEffects:CreateEffect( "Effects/Particles/Explosions/BubbleChamber/SimpleTracers.particles", nil, nil, nil, Constants.ParticlePriorityLow )   
       
        if rTargetProp ~= nil then
            RadEffects:SetEffectTransform( posX, posY, posZ, 0.0,-90.0, 0.0 )  
            RadEffects:SetEffectLayer( RadEffects.particleAccumLayer )
            RadEffects:CreateEffect( "Effects/Particles/Explosions/BubbleChamber/SeekingStars.particles", rTargetProp, nil, RadEffects.BubbleChamberTargetCallback, Constants.ParticlePriorityLow )  
        end
    end
    
    Explosion:Init()
    return Explosion
end

function RadEffects:NewFingerEffect(sParticlePath, bUseParticleLayer)
    if Constants.UseParticleEffects then
        local rParticleEffect = ParticleSystem.new(nil, sParticlePath)
        rParticleEffect:init()
        rParticleEffect:start()
        if bUseParticleLayer then
            rParticleEffect:addToSceneLayer(RadEffects.particleAccumLayer)
        else
            rParticleEffect:addToSceneLayer(RadEffects.tRenderer.mGameLayer) 
        end
        return rParticleEffect
    end
    
    return nil
end

return RadEffects
