local Constants = require("Constants")
local Renderer = require('Renderer')
local RadShapes = {}


-------------------------------------------------------------
--[[

----------
--DRAW MODES
-----------
DrawModeCircle
DrawModeX
DrawModeInvertedQuad
DrawModeStar
DrawModeDiamond
DrawModeDoubleTriangle
DrawModeArc

-----------
--FUNCTIONS
-----------
SetNumRingSides
SetRadii
SetRingColor
SetDrawMode
SetArcRadians     -- only used with DrawModeArc
        
--]]
-------------------------------------------------------------

-- Rad Shape Object
function RadShapes:NewRadShape()
    local RadShape = {}
    
    function RadShape:Init()      
        RadShape.mUsed = false
        RadShape.mPropLayer = nil
    end
    
    function RadShape:CreateProp( tLayer )
        RadShape.mProp = ShapeProp.new()
        RadShape.mProp:setDeck( Renderer.mShapeDeck )
        RadShape.mProp:SetNumSides( 16 )
        RadShape.mProp:SetRadii( 0.0, 1.0 )  
        RadShape.mProp:SetDrawMode( ShapeProp.DrawModeArc )
        RadShape.mProp:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
        RadShape.mProp:setVisible( true )
        RadShape.mPropLayer = tLayer
        
        RadShape.mPropLayer:insertProp( RadShape.mProp )
        
        RadShape.mUsed = true
    end
    
    function RadShape:DestroyProp()
        if RadShape.mProp ~= nil then
            RadShape.mPropLayer:removeProp( RadShape.mProp ) 
        end
        RadShape.mProp = nil
        RadShape.mPropLayer = nil
        RadShape.mUsed = false
    end
    
    function RadShape:SetType( iType, iFidelity )
        RadShape.mProp:SetDrawMode( iType )
        RadShape.mProp:SetNumSides( iFidelity )
    end
    
    function RadShape:GetPosition()
        return RadShape.mProp:GetPosition()
    end
    
    function RadShape:SetPosition( x, y, z )
        local shapeLocX, shapeLocY = RadShape.mProp:getLoc()
        
        -- this is a minor hack so that we can take advantage of prop sorting and ALSO take advantage of
        --  props having depth. We have to do this to get certain batches to draw in front or behind
        --  other batches. NOTE: this also forces all shapes in a batch to draw at the same z, so
        --  hopefully we never care about that rule.
        z = z * 0.5
        
        RadShape.mProp:setLoc(shapeLocX, shapeLocY, z - 1)
        RadShape.mProp:SetPosition( x, y, z )
    end
  
    function RadShape:GetRotation()
        return RadShape.mProp:GetRotation()
    end
    
    function RadShape:SetRotation( x, y, z )
        RadShape.mProp:SetRotation( x, y, z )
    end
    
    function RadShape:SetColor( r, g, b, a )
        RadShape.mProp:SetColor( r, g, b, a )
    end

    function RadShape:SetRadii( fInner, fOuter )
        RadShape.mProp:SetRadii( fInner, fOuter )
    end

    function RadShape:SetArcRadians( fStart, fEnd )
        RadShape.mProp:SetArcRadians( fStart, fEnd )
    end
    
    function RadShape:SetBatchIndex( iShapeIndex )
        RadShape.mProp:SetBatchIndex( iShapeIndex )
    end
    
    function RadShape:SetNumSides( iNumSides )
        RadShape.mProp:SetNumSides( iNumSides )
    end
    
    RadShape:Init()
    return RadShape
end

function RadShapes:Init()
    RadShapes.mNumShapes = 250
    RadShapes.mShapes = {}
    local i = 1
    for i = 1, RadShapes.mNumShapes do
        RadShapes.mShapes[i] = RadShapes:NewRadShape()
    end      
    
    RadShapes.mPropLayer = Renderer.mBackgroundLayer
end

function RadShapes:SetPropLayer( tLayer )
    --sets layer the shapes will be put in 
    RadShapes.mPropLayer = tLayer
    if RadShapes.mPropLayer == nil then
        RadShapes.mPropLayer = Renderer.mBackgroundLayer
    end
end

function RadShapes:ClearShapes()
    local i = 1
    for i = 1, RadShapes.mNumShapes do
        if RadShapes.mShapes[i].mUsed then  
            RadShapes.mShapes[i]:DestroyProp()
        end
    end   
end

function RadShapes:CreateNewShape()
    local i = 1
    for i = 1, RadShapes.mNumShapes do
        if not RadShapes.mShapes[i].mUsed then  
            RadShapes.mShapes[i]:CreateProp( RadShapes.mPropLayer )
            return RadShapes.mShapes[i]
        end
    end     
    return nil
end

    
function RadShapes:CreateNewShapeBatch(iType, iNumShapes)
    local shape = self:CreateNewShape()
    
    if not iNumShapes or iNumShapes < 1 then
        iNumShapes = 1
    end
    
    shape.mProp:SetDrawMode( iType )
    shape.mProp:SetNumShapes( iNumShapes )
    
    return shape
end

return RadShapes
