local DFGraphics = require('DFCommon.Graphics')
local DFMath = require('DFCommon.Math')
local Constants = require('Constants')
local GameRing = require('GameRing')
local UI = require('UI')
local ChargeManager = require('ChargeManager')
local ColorManager = require('ColorManager')
local PatternManager = require('PatternManager')
local InputVisuals = require('InputVisuals')
local Score = require('Score')
local ChargeField = require('ChargeField')
local ParticleSystemManager = require('ParticleSystemManager')
local ParticleSystem = require('ParticleSystem')
local HUD = require('HUD')
local Renderer = {}

local tPreloadTextures = {
    "White",
    "Effects/Textures/Black",
    "Effects/Textures/Light/CircleFlare",
    "Effects/Textures/Light/CrossFlare",
    "DracogenScreen",
    "SplashScreen",
    "Images/BlockFont",
    "Effects/Textures/ScoreGlow",
    "Effects/Textures/SquareGlow",
}

local tPreloadParticles = {
    "Effects/Particles/Backgrounds/HealthGlowSpecial.particles",
    "Effects/Particles/Backgrounds/RingGlow.particles",
    "Effects/Particles/Backgrounds/Magicalness.particles",
    "Effects/Particles/Backgrounds/ScoreGlowSpecial.particles",
    "Effects/Particles/Backgrounds/SwirlyStars.particles",
    "Effects/Particles/Backgrounds/SwirlyStarsB.particles",
    "Effects/Particles/Bursts/BlipFlare.particles",
    "Effects/Particles/Bursts/HealthBits.particles",
    "Effects/Particles/Bursts/HealthRing.particles",
    "Effects/Particles/Bursts/HealthTracers.particles",
    "Effects/Particles/Bursts/ScratchDeath.particles",
    "Effects/Particles/Bursts/ScratchDeathHorizontalFlare.particles",
    "Effects/Particles/Bursts/ScratchHit.particles",
    "Effects/Particles/Bursts/ScratchHitFlare.particles",
    "Effects/Particles/Bursts/ScratchHitRing.particles",
    "Effects/Particles/Bursts/ScratchShards.particles",
    "Effects/Particles/Bursts/SmallFlare.particles",
    "Effects/Particles/Bursts/SmallFlare_Blue.particles",
    "Effects/Particles/Chord/ChargeCreation.particles",
    "Effects/Particles/Chord/ChargeEmbers.particles",
    "Effects/Particles/Chord/ChargeFlash.particles",
    "Effects/Particles/Chord/CreationBuild.particles",
    "Effects/Particles/Chord/CreationComplete.particles",
    "Effects/Particles/Chord/CreationEmbers.particles",
    "Effects/Particles/Chord/CreationFlash.particles",
    "Effects/Particles/Chord/CreationGlow.particles",
    "Effects/Particles/Chord/CreationSparkles.particles",
    "Effects/Particles/Explosions/AccumTracers/RingRippleC.particles",
    "Effects/Particles/Explosions/AccumTracers/SeekingStars.particles",
    "Effects/Particles/Explosions/AccumTracers/Tris.particles",
    "Effects/Particles/Explosions/BubbleChamber/RingRippleC.particles",
    "Effects/Particles/Explosions/BubbleChamber/SimpleTracers.particles",
    "Effects/Particles/Explosions/BubbleChamber/SeekingStars.particles",
    "Effects/Particles/Explosions/BubbleChamber/SeekingStarsBold.particles",
    "Effects/Particles/Explosions/Misc/RingRipple.particles",
    "Effects/Particles/Explosions/Misc/RingRippleB.particles",
    "Effects/Particles/Explosions/Misc/RingRippleC.particles",
    "Effects/Particles/Explosions/Misc/SeekingStars.particles",
    "Effects/Particles/Explosions/Misc/SharpStars.particles",
    "Effects/Particles/Explosions/Misc/StarsA.particles",
    "Effects/Particles/Explosions/Misc/StarsB.particles",
    "Effects/Particles/Explosions/Misc/Tris.particles",
    "Effects/Particles/Explosions/PurpleStars/RingRippleB.particles",
    "Effects/Particles/Explosions/PurpleStars/SeekingStars.particles",
    "Effects/Particles/Explosions/PurpleStars/StarsA.particles",
    "Effects/Particles/Fingers/BlackDots",
    "Effects/Particles/Fingers/BlackDotsLong",
    "Effects/Particles/Fingers/BlobDots.particles",
    "Effects/Particles/Fingers/BlobDotsFade.particles",
    "Effects/Particles/Fingers/BubbleTracers.particles",
    "Effects/Particles/Fingers/Dots.particles",
    "Effects/Particles/Fingers/Flames.particles",
    "Effects/Particles/Fingers/PurpleRings",
    "Effects/Particles/Fingers/SeekingStars.particles",
    "Effects/Particles/Fingers/SoftGlow.particles",
    "Effects/Particles/Fingers/SoftGlowGreen.particles",
    "Effects/Particles/Fingers/YellowDots.particles",
    "Effects/Particles/Fingers/YellowDotsMedium.particles",
    "Effects/Particles/Fingers/YellowDotsSlow.particles",
    "Effects/Particles/Fingers/YellowGlow.particles",
}

function Renderer:PreInit()
    --Renderer members
    Renderer.mGameViewport = nil
    Renderer.mUIViewport = nil
    Renderer.mGameCamera = nil
    
    Renderer.mGameLayer = nil
    Renderer.mPartition = nil
    Renderer.mGameDeck = nil
    Renderer.mGameProp = nil

    Renderer.mTestDeck = nil
    Renderer.mTestProp = nil
    
    Renderer.mDesiredScreenWidth = Constants.ValidDrawWidth
    Renderer.mDesiredScreenHeight = Constants.ValidDrawHeight
    Renderer.mTrueScreenWidth = Renderer.mDesiredScreenWidth
    Renderer.mTrueScreenHeight = Renderer.mDesiredScreenHeight  
    
    Renderer.mAspectRatio = Renderer.mDesiredScreenWidth / Renderer.mDesiredScreenHeight
    Renderer.mFarPlane = 100000
    
    --setup viewport
    local gameViewport, uiViewport = DFGraphics.createWindow( "dropchord", Renderer.mDesiredScreenWidth, Renderer.mDesiredScreenHeight, OnScreenResize )    
    Renderer.mGameViewport = gameViewport
    Renderer.mUIViewport = uiViewport
    
    Constants.NativeScreenWidth = Renderer.mGameViewport.sizeX
    Constants.NativeScreenHeight = Renderer.mGameViewport.sizeY
end

function Renderer:Init(onResizeCallback)
    
    --Renderer:Preload()

    print("Renderer:Init()")
    Renderer.onResizeCallback = onResizeCallback
    
    local rShaderOverride = DFGraphics.loadShader( "Shaders/builtinDeck2D.shd" )
  	MOAIShaderMgr.setShader( MOAIShaderMgr.DECK2D_SHADER, rShaderOverride )
  	
  	rShaderOverride = DFGraphics.loadShader( "Shaders/builtinDeck2DTexOnly.shd" )
  	MOAIShaderMgr.setShader( MOAIShaderMgr.DECK2D_TEX_ONLY_SHADER, rShaderOverride )
  	
  	rShaderOverride = DFGraphics.loadShader( "Shaders/builtinFont.shd" )
  	MOAIShaderMgr.setShader( MOAIShaderMgr.FONT_SHADER, rShaderOverride )
  	
  	rShaderOverride = DFGraphics.loadShader( "Shaders/builtinLine.shd" )
  	MOAIShaderMgr.setShader( MOAIShaderMgr.LINE_SHADER, rShaderOverride )
  	
  	rShaderOverride = DFGraphics.loadShader( "Shaders/builtinMeshDeck.shd" )
  	MOAIShaderMgr.setShader( MOAIShaderMgr.MESH_DECK_SHADER, rShaderOverride )
  	
  	rShaderOverride = DFGraphics.loadShader( "Shaders/builtinMesh.shd" )
  	MOAIShaderMgr.setShader( MOAIShaderMgr.MESH_SHADER, rShaderOverride )
  	
  	rShaderOverride = DFGraphics.loadShader( "Shaders/builtinSkinnedMesh.shd" )
  	MOAIShaderMgr.setShader( MOAIShaderMgr.SKINNED_MESH_SHADER, rShaderOverride )
    
    Renderer.mHalfSizeGameViewport = MOAIViewport.new()
    Renderer.mHalfSizeGameViewport:setSize( Renderer.mGameViewport.sizeX, Renderer.mGameViewport.sizeY )       
    Renderer.mQuarterSizeGameViewport = MOAIViewport.new()
    Renderer.mQuarterSizeGameViewport:setSize( Renderer.mGameViewport.sizeX, Renderer.mGameViewport.sizeY )    
    Renderer.mEighthSizeGameViewport = MOAIViewport.new()
    Renderer.mEighthSizeGameViewport:setSize( Renderer.mGameViewport.sizeX, Renderer.mGameViewport.sizeY )    

    Renderer.mBufferViewport = MOAIViewport.new()
    Renderer.mEighthSizeGameViewport:setSize( Renderer.mGameViewport.sizeX, Renderer.mGameViewport.sizeY ) 
    
	--setup camera    
	Renderer.mGameCamera = MOAICamera.new()
    Renderer.mGameCamera:setOrtho( false )
    Renderer.mGameCamera:setFieldOfView( 90.0 )
    Renderer.mGameCameraZDistance = Renderer.mGameCamera:getFocalLength(Renderer.mGameViewport.sizeX)    
	Renderer.mGameCamera:setLoc( 0, 0, Renderer.mGameCameraZDistance )
    Renderer.mGameCamera:setScl( 1.0, -1.0 )
    Renderer.mGameCamera:setFarPlane( Renderer.mFarPlane )
        
    --setup frame buffers
    Renderer.mGameBuffer = MOAIFrameBuffer.new ()
	Renderer.mGameBuffer:init ( Renderer.mGameViewport.sizeX, Renderer.mGameViewport.sizeY)
	Renderer.mGameBuffer:setClearColor ( 0, 0, 0, 0 )
    Renderer.mBackgroundBuffer = MOAIFrameBuffer.new ()
	Renderer.mBackgroundBuffer:init ( Renderer.mGameViewport.sizeX, Renderer.mGameViewport.sizeY)
	Renderer.mBackgroundBuffer:setClearColor ( 0, 0, 0, 1 )
    Renderer.mUIBuffer= MOAIFrameBuffer.new ()
	Renderer.mUIBuffer:init ( Renderer.mGameViewport.sizeX, Renderer.mGameViewport.sizeY)
	Renderer.mUIBuffer:setClearColor ( 0, 0, 0, 0 )
        
    --setup layers
    Renderer.mBackgroundLayer = MOAILayer2D.new()
    Renderer.mBackgroundLayer:setDebugName("Renderer.mBackgroundLayer")
    Renderer.mBackgroundLayer:setViewport( Renderer.mUIViewport )
    Renderer.mBackgroundLayer:setCamera( Renderer.mGameCamera )
    if Constants.UsePostEffects then
        Renderer.mBackgroundLayer:setFrameBuffer ( Renderer.mBackgroundBuffer )
    end
    
    Renderer.mBackgroundColorQuad = MOAIProp.new()
    local quadDeck = MOAIGfxQuad2D.new()
    local whiteTexture = DFGraphics.loadTexture( "White", true )
    quadDeck:setTexture( whiteTexture )
    quadDeck:setRect( -0.6, -0.5, 0.6, 0.5 )
    Renderer.mBackgroundColorQuad:setDeck(quadDeck)
    Renderer.mBackgroundLayer:insertProp(Renderer.mBackgroundColorQuad)
    Renderer:DrawBackground()
    
    MOAISim.pushRenderPass( Renderer.mBackgroundLayer )
    
    Renderer.mGameLayer = MOAILayer2D.new()
    Renderer.mGameLayer:setDebugName("Renderer.mGameLayer")
    Renderer.mGameLayer:setViewport( Renderer.mUIViewport )
    Renderer.mGameLayer:setCamera( Renderer.mGameCamera )
    Renderer.mGameLayer:setSortMode( MOAILayer.SORT_Z_ASCENDING )
    if Constants.UsePostEffects then
        Renderer.mGameLayer:setFrameBuffer ( Renderer.mGameBuffer )
    end
    MOAISim.pushRenderPass( Renderer.mGameLayer )
    
     --setup layers
    Renderer.mUILayer = MOAILayer2D.new()
    Renderer.mUILayer:setDebugName("Renderer.mUILayer")
    Renderer.mUILayer:setViewport( Renderer.mUIViewport )
    Renderer.mUILayer:setCamera( Renderer.mGameCamera )
    Renderer.mUILayer:setSortMode( MOAILayer.SORT_Z_ASCENDING )
    if Constants.UsePostEffects then
        Renderer.mUILayer:setFrameBuffer ( Renderer.mUIBuffer )
    end
    MOAISim.pushRenderPass( Renderer.mUILayer )
    
    --setup deck
    Renderer.mGameDeck = MOAIScriptDeck.new()
    Renderer.mGameDeck:setRect( 0, 0, 1, 1 )
    Renderer.mGameDeck:setDrawCallback( Renderer.Draw )
    
    Renderer.mBackgroundDeck = MOAIScriptDeck.new()
    Renderer.mBackgroundDeck:setRect( 0, 0, 1, 1 )
    Renderer.mBackgroundDeck:setDrawCallback( Renderer.DrawBackground )
                
    Renderer.mScoreDeck = MOAIScriptDeck.new()
    Renderer.mScoreDeck:setRect( 0, 0, 1, 1 )
    Renderer.mScoreDeck:setDrawCallback( Renderer.DrawScore )
    
    Renderer.mGameProp = MOAIProp2D.new()
    Renderer.mGameProp:setDeck( Renderer.mGameDeck )
    Renderer.mGameProp:setLoc( 0, 0, 0 )
    Renderer.mGameProp:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
        
    Renderer.mGameLayer:insertProp( Renderer.mGameProp )  

    Renderer.mBackgroundProp = MOAIProp2D.new()
    Renderer.mBackgroundProp:setDeck( Renderer.mBackgroundDeck )
    Renderer.mBackgroundProp:setLoc( 0, 0, 0 )
    Renderer.mBackgroundProp:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
        
    Renderer.mBackgroundLayer:insertProp( Renderer.mBackgroundProp )  
    
    Renderer.mUIProp = MOAIProp2D.new()
    Renderer.mUIProp:setDeck( Renderer.mScoreDeck )
    Renderer.mUIProp:setLoc( 0, 0, 0 )
    Renderer.mUIProp:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
        
    Renderer.mUILayer:insertProp( Renderer.mUIProp )  
    
    Renderer.mLetterboxLayer = nil
    
    if Constants.UsePostEffects then
        Renderer.mPostLayer = MOAILayer2D.new()
        Renderer.mPostLayer:setDebugName("Renderer.mPostLayer")
        Renderer.mPostLayer:setViewport ( Renderer.mGameViewport )
        Renderer.mPostOverlayLayer = MOAILayer2D.new()
        Renderer.mPostOverlayLayer:setDebugName("Renderer.mPostOverlayLayer")
        Renderer.mPostOverlayLayer:setViewport(Renderer.mGameViewport)
        Renderer.mLetterboxLayer = Renderer.mPostOverlayLayer
    else
        Renderer.mLetterboxLayer = MOAILayer2D.new()
        Renderer.mLetterboxLayer:setDebugName("Renderer.mLetterboxLayer")
        Renderer.mLetterboxLayer:setViewport( Renderer.mGameViewport )
        Renderer.mLetterboxLayer:setCamera( Renderer.mGameCamera )
        MOAISim.pushRenderPass( Renderer.mLetterboxLayer )
    end    

    --used by all shape props
    Renderer.mShapeDeck = ShapeDeck.new()
    
    --rad specific device setup
    MOAIGfxDevice.initDeviceForRad()
    
    --setup framerate
    MOAISim.setStep( Constants.InvFramerate )
    
    DFEffects.setCamera(Renderer.mGameCamera)
    
    MOAIGfxDevice.setClearColor( 0, 0, 0, 1)
    
    Renderer:ScaleScreen(Renderer.mGameViewport.sizeX, Renderer.mGameViewport.sizeY)
    
    print("Renderer:Init() complete")
end
      
function Renderer:Draw()
    if Constants.GameState == Constants.GameStateIntro                      then Renderer:DrawGameStateIntro()    
    elseif Constants.GameState == Constants.GameStateMenu                   then Renderer:DrawGameStateMenu()    
    elseif Constants.GameState == Constants.GameStateInGame                 then Renderer:DrawGameStateInGame()    
    elseif Constants.GameState == Constants.GameStateComplete               then Renderer:DrawGameStateComplete()     
    elseif Constants.GameState == Constants.GameStatePost                   then Renderer:DrawGameStatePost()     
    elseif Constants.GameState == Constants.GameStateLeaderboardShowcase    then Renderer:DrawGameStateLeaderboardShowcase()   
    elseif Constants.GameState == Constants.GameStateCredits                then Renderer:DrawGameStateCredits()     
    elseif Constants.GameState == Constants.GameStateHowToPlay              then Renderer:DrawGameStateHowToPlay()     
    end
    
    ParticleSystemManager.update()
    
    if g_DebugDisplay then
        g_DebugDisplay:Draw()
    end
end

function Renderer:DrawBackground()
     --set clear color does not seem to work on the framebuffer, using a big filled rectangle instead.
     Renderer.mBackgroundColorQuad:setScl(Constants.NativeScreenWidth, Constants.NativeScreenHeight)
     Renderer.mBackgroundColorQuad:setColor(Constants.BGColor[1], Constants.BGColor[2], Constants.BGColor[3], 1 ) 
end

function Renderer:DrawScore()
    if Constants.GameState == Constants.GameStateInGame or Constants.GameState == Constants.GameStateComplete then 
       Score:Draw()
    end
end

function Renderer:DrawGameStateIntro()
    ChargeManager:DrawBeamDisabled()    
    ChargeManager:DrawExtra()
    HUD:Draw()
end

function Renderer:DrawGameStateMenu()
    ChargeManager:DrawBeamDisabled()    
    ChargeManager:DrawExtra()
    UI:Draw()
    HUD:Draw()
end
    
function Renderer:DrawGameStateInGame()
    PatternManager:DrawSlices()       
    ChargeManager:DrawExtra()
    ChargeManager:DrawBeam()
    PatternManager:Draw()
    UI:Draw()
    HUD:Draw()
end

function Renderer:DrawGameStateComplete()
    PatternManager:DrawSlices()       
    ChargeManager:DrawExtra()
    ChargeManager:DrawBeam()
    PatternManager:Draw()
    UI:Draw()
    HUD:Draw()
end

function Renderer:DrawGameStatePost()
    PatternManager:DrawSlices()
    ChargeManager:DrawExtra()
    ChargeManager:DrawBeam()
    PatternManager:Draw()
end

function Renderer:DrawGameStateLeaderboardShowcase()
    ChargeManager:DrawBeamDisabled()
end

function Renderer:DrawGameStateCredits()
    ChargeManager:DrawBeamDisabled()
end

function Renderer:DrawGameStateHowToPlay()
    ChargeManager:DrawBeamDisabled()
end

function OnScreenResize(width, height)
    
    Renderer:ScaleScreen(width, height)

    if Renderer.onResizeCallback then
        Renderer.onResizeCallback(width, height)
    end
end

function Renderer:ScaleScreen(width, height)
    local validWidth = width
    local validHeight = height
    
    local aspect = width/height
    if aspect < (Constants.MinAspectRatio) then
        -- we need to clamp width here dude
        validWidth = width
        validHeight = width / Constants.MinAspectRatio
    end

    Constants.NativeScreenWidth = width
    Constants.NativeScreenHeight = height
    
    Constants.ValidDrawWidth = validWidth
    Constants.ValidDrawHeight = validHeight

    Constants.ValidDrawHalfWidth = Constants.ValidDrawWidth * .5
    Constants.ValidDrawHalfHeight = Constants.ValidDrawHeight * .5   
    
    Renderer.mGameCameraZDistance = Renderer.mGameCamera:getFocalLength(Constants.ValidDrawWidth)    
	Renderer.mGameCamera:setLoc( 0, 0, Renderer.mGameCameraZDistance )

    Renderer.mTrueScreenWidth = width
    Renderer.mTrueScreenHeight = height
    
    Renderer.mAspectRatio = Constants.ValidDrawWidth / Constants.ValidDrawHeight
    
    Renderer.mGameBuffer:init ( Renderer.mTrueScreenWidth, Renderer.mTrueScreenHeight)
    Renderer.mBackgroundBuffer:init ( Renderer.mTrueScreenWidth, Renderer.mTrueScreenHeight)
    Renderer.mUIBuffer:init ( Renderer.mTrueScreenWidth, Renderer.mTrueScreenHeight)
    
    Renderer.mGameViewport:setSize( Renderer.mTrueScreenWidth, Renderer.mTrueScreenHeight )
    Renderer.mGameViewport:setScale( Renderer.mTrueScreenWidth, Renderer.mTrueScreenHeight )
    Renderer.mGameViewport:setOffset(-1 + (1/1),-1 + (1/1))
    Renderer.mUIViewport:setSize( Renderer.mTrueScreenWidth, Renderer.mTrueScreenHeight )
    Renderer.mUIViewport:setScale( Renderer.mTrueScreenWidth, Renderer.mTrueScreenHeight )
    
    Constants.ScreenBufferWidth = math.min(width, Constants.ScreenBufferMaxWidth)
    Constants.ScreenBufferHeight = math.min(height, Constants.ScreenBufferMaxHeight)
    
    local bufScaleX = width / Constants.ScreenBufferWidth
    local bufScaleY = height / Constants.ScreenBufferHeight
    
    local bufScreenW = Renderer.mTrueScreenWidth * bufScaleX
    local bufScreenH = Renderer.mTrueScreenHeight * bufScaleY
    
    Renderer.mBufferViewport:setSize( Renderer.mTrueScreenWidth, Renderer.mTrueScreenHeight )
    Renderer.mBufferViewport:setScale(bufScreenW,bufScreenH)
    Renderer.mBufferViewport:setOffset(-1 + (1/bufScaleX),-1 + (1/bufScaleY))
    
    -- a little explanation of this, at least for half buffers.
    -- the point -1,0 will move the 0,0 of these shrunken viewports to the left side of the screenX
    -- the point -1 + (1/bufScaleX),0 will move it to the right side of the screen (that being our parent's transform)
    -- so the resulting transform will be an average of those two points. Continue down the scaled stack!
    -- NOTE: this would be so much easier if we just used matrices instead of viewports, but that's not really an option
    --        unless we just don't use MOAI rendering at all.
    
--[[
 ....    M,           M                 
..        ...M.......... ...............
.     M  MMMM  MM MMMMM ................
.   .. MM .M     . MMMMMMMMM............
.    M 8  ...............MMMM...........
.    .  .MMMMMMMMMMMM .....MM...........
.  M MMMMMMMMMMMMMMMMMMMM ..............
..MMMMMMMMMMMMMMMMMMMMMMMMMM............
.MMMMMMMMMMMM .MMMMMMMMMMMMMMM .........
.  M  MMMMMM ........MMMMMMMM. .........
..   ..MM.MMMM.....MMMMMMMMMM...........
..      MMMMMM=...MMMMMMMMMMM...........
...M.   M.MMM.MMMMMMMMMMMMMMM ..........
  .M......MMMMMMMMMMMMMMMM.MM           
..............:..MMMMMMM  D.M           
 ................MM ..MMMMMMM...........
  .......... ....7M..    7MM.           
. ..M....... .....M ....MMMM............
  ...D...............   MMMM            
  ...M......MMMMMMMM.   MMMM            
 .....M... M.......,M .MMMMM............
  ...~MM8 =....7 , ..  MMMM~MMMMMM8     
  .. MMMMMM...MMMMMM .MMMMMMMMM       M.
  .MMMMMMMMMM. MMMMMMMMMMMMMMMM?MM...MMM
 MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
 MMMMMMMM.MMMMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMM..MMMMMMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMM..MMMMMMMMMMMMMMMMMMMMMMMMMMM
]]--    
    
    -- seriously don't change this stuff unless you want to feel some pain
    
    Renderer.mHalfSizeGameViewport:setSize( Renderer.mTrueScreenWidth, Renderer.mTrueScreenHeight )
    Renderer.mHalfSizeGameViewport:setScale( bufScreenW * 2, bufScreenH * 2)
    Renderer.mHalfSizeGameViewport:setOffset((-1 -1 + (1/bufScaleX))/2,(-1 -1 + (1/bufScaleY))/2)    
    
    Renderer.mQuarterSizeGameViewport:setSize( Renderer.mTrueScreenWidth, Renderer.mTrueScreenHeight )
    Renderer.mQuarterSizeGameViewport:setScale( bufScreenW * 4, bufScreenH * 4)
    Renderer.mQuarterSizeGameViewport:setOffset((-1 + (-1 -1 + (1/bufScaleX))/2)/2,(-1 + (-1 -1 + (1/bufScaleY))/2)/2)
    
    Renderer.mEighthSizeGameViewport:setSize( Renderer.mTrueScreenWidth, Renderer.mTrueScreenHeight )
    Renderer.mEighthSizeGameViewport:setScale( bufScreenW * 8, bufScreenH * 8)
    Renderer.mEighthSizeGameViewport:setOffset((-1 + (-1 + (-1 -1 + (1/bufScaleX))/2)/2)/2,(-1 + (-1 + (-1 -1 + (1/bufScaleY))/2)/2)/2)   
    
    g_port = Renderer.mEighthSizeGameViewport
    g_bufScaleX = bufScaleX
end

function Renderer:VisualQualityChange()
    Constants.UpdateScreenBufferSize()
    Constants.UpdateFramerate()
    
    --[[
    if Constants.VisualQuality == Constants.VisualQualityLevelHigh then
        Renderer.mBackgroundLayer:insertProp( Renderer.mBackgroundProp )  
        Renderer.mGameLayer:insertProp( Renderer.mGameProp )  
        Renderer.mUILayer:insertProp( Renderer.mUIProp )  
    else
        Renderer.mGameLayer:insertProp( Renderer.mBackgroundProp )  
        Renderer.mGameLayer:insertProp( Renderer.mGameProp )  
        Renderer.mGameLayer:insertProp( Renderer.mUIProp )  
    end
    ]]--
    
    --setup framerate
    MOAISim.setStep( Constants.InvFramerate )
    
    Renderer:ScaleScreen(Renderer.mTrueScreenWidth, Renderer.mTrueScreenHeight)
end

function Renderer:Preload()
    Renderer.tPreloadedTextures = {}
    Renderer.tPreloadedParticles = {}

    for i,sTexturePath in ipairs(tPreloadTextures) do
        local rTexture = DFGraphics.loadTexture( sTexturePath, true )
        table.insert(Renderer.tPreloadedTextures, rTexture)
    end
    
    for i,sParticlePath in ipairs(tPreloadParticles) do
        local rPfx = ParticleSystem.new(nil, sParticlePath)
        rPfx:preload()
    end
    
    DFGraphics.blockOnAsyncTextureLoad()
end

function Renderer:GetScreenSize()
    if Renderer.mGameViewport then
        return Renderer.mTrueScreenWidth, Renderer.mTrueScreenHeight
    end

    return 1, 1
end

function Renderer:GetHalfScreenSize()
    local screenX, screenY = Renderer:GetScreenSize()
    screenX = screenX * 0.5
    screenY = screenY * 0.5
    return screenX, screenY
end

return Renderer
