local Constants = require('Constants')
local DFMath = require('DFCommon.Math')
local DFGraphics = require('DFCommon.Graphics')
local RadFont = require('RadFont')
local AudioManager = require('AudioManager')
local HUDGlow = require('HUDGlow')
local Achievement = require('DFCommon.Achievement')

local Score = {}

function Score:Init( tRenderer )  
    local i = 1

    --bleh.  this needs to be declared before we determine the size
    local sVerse = {"v","e","r","s","e",".","0","0","("}
    Score.mVerseTextCount = 9

    Score.mHealthStarCount = 6
    
    Score:DetermineSize()
    
    Score.mScore = 0
    Score.mTargetScore = 0
    Score.mScoreUpdateDuration = 0.2
    Score.mScoreUpdateTimer = Score.mScoreUpdateDuration
    Score.mScoreDirty = true
    
    Score.mLayer = tRenderer.mUILayer
    
    Score.mMultiplier = 1
    Score.mMultiplierDirty = true
    Score.mScoreDigits = 8
    
    Score.mQuadDeck = MOAIGfxQuad2D.new()
    local whiteTexture = DFGraphics.loadTexture( "White", true )
    Score.mQuadDeck:setTexture( whiteTexture )
    Score.mQuadDeck:setRect( 0, 0, 1, 1 )
    
    Score.mFullBackgroundProp = MOAIProp.new()
    Score.mFullBackgroundProp:setDeck(Score.mQuadDeck)
    Score.mLayer:insertProp(Score.mFullBackgroundProp)
    Score.mFullBackgroundProp:setVisible(false)
    
    Score.mHealthBarProp = MOAIProp.new()
    Score.mHealthBarProp:setDeck(Score.mQuadDeck)
    Score.mLayer:insertProp(Score.mHealthBarProp)
    Score.mHealthBarProp:setVisible(false)
    
    Score.mHUDPositionOffset = 0.0    
    Score.mHUDPositionSpeedComplete = Constants.ValidDrawHeight * 0.1
    local fScoreImageYBase = -Constants.ValidDrawHalfHeight + (Score.mDigitWidth * 1.5)
    
    Score.mScorePropsL = {}
    Score.mScorePropsR = {}
    for i = 1, Score.mScoreDigits do
        Score.mScorePropsL[i] = MOAIProp.new()
        Score.mScorePropsL[i]:setDeck( RadFont.mFontDeck )        
        Score.mScorePropsL[i]:setScl( Score.mDigitScale, -Score.mDigitScale )
        Score.mScorePropsL[i]:setIndex( RadFont.mFontDeck.names["0"] )
        Score.mScorePropsL[i]:setVisible( false )
        
        Score.mScorePropsR[i] = MOAIProp.new()
        Score.mScorePropsR[i]:setDeck( RadFont.mFontDeck )        
        Score.mScorePropsR[i]:setScl( -Score.mDigitScale, -Score.mDigitScale )
        Score.mScorePropsR[i]:setIndex( RadFont.mFontDeck.names["0"] )
        Score.mScorePropsR[i]:setVisible( false )
        
        Score.mLayer:insertProp( Score.mScorePropsL[i] )
        Score.mLayer:insertProp( Score.mScorePropsR[i] )    
    
        fScoreImageYBase = fScoreImageYBase + Score.mDigitWidth
    end
    
    Score.mMultiplierPropZ = 0.0
    Score.mMultiplierPropZSpeed = 100.0
        
    Score.mMultiplierPositionY = fScoreImageYBase
    Score.mMultiplierPropsL = {}
    Score.mMultiplierPropsR = {}
    for i = 1, 3 do
        Score.mMultiplierPropsL[i] = MOAIProp.new()
        Score.mMultiplierPropsL[i]:setDeck( RadFont.mFontDeck )   
        Score.mMultiplierPropsL[i]:setScl( Score.mDigitScale, -Score.mDigitScale )        
        Score.mMultiplierPropsL[i]:setIndex( RadFont.mFontDeck.names["k"] )
        Score.mMultiplierPropsL[i]:setVisible( false )

        Score.mMultiplierPropsR[i] = MOAIProp.new()
        Score.mMultiplierPropsR[i]:setDeck( RadFont.mFontDeck )   
        Score.mMultiplierPropsR[i]:setScl( -Score.mDigitScale, -Score.mDigitScale )        
        Score.mMultiplierPropsR[i]:setIndex( RadFont.mFontDeck.names["k"] )
        Score.mMultiplierPropsR[i]:setVisible( false )
        
        Score.mLayer:insertProp( Score.mMultiplierPropsL[i] )
        Score.mLayer:insertProp( Score.mMultiplierPropsR[i] )
    end
    Score:OffsetScoreAndMultiplierProps()

    Score.mTargetProps = {}
    for i = 1, 2 do
        Score.mTargetProps[i] = MOAIProp.new()
        Score.mTargetProps[i]:setLoc(0.0, 0.0, 0.0)
        Score.mLayer:insertProp( Score.mTargetProps[i] )
    end
    Score.mTargetPropIndex = 1
    
    Score.mVerseTextProps = {}
    for i = 1, Score.mVerseTextCount do
        Score.mVerseTextProps[i] = MOAIProp.new()
        Score.mVerseTextProps[i]:setDeck( RadFont.mFontDeck )        
        Score.mVerseTextProps[i]:setScl( Score.mVerseImageScale, -Score.mVerseImageScale )
        Score.mVerseTextProps[i]:setIndex( RadFont.mFontDeck.names[ sVerse[i] ] )
        Score.mVerseTextProps[i]:setVisible(false)
        Score.mVerseTextProps[i]:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
        
        Score.mLayer:insertProp( Score.mVerseTextProps[i] )        
    end
    Score:PositionVerseText()
    
    Score.mHealthStarProps = {}
    for i = 1, Score.mHealthStarCount do
        Score.mHealthStarProps[i] = ShapeProp.new()
        Score.mHealthStarProps[i]:setDeck( tRenderer.mShapeDeck )      
        Score.mHealthStarProps[i]:SetNumSides( 32 )  
        Score.mHealthStarProps[i]:SetRadii( Score.mHealthStarHeight * 0.5, Score.mHealthStarHeight )
        Score.mHealthStarProps[i]:SetDrawMode( ShapeProp.DrawModeStar )
        Score.mHealthStarProps[i]:setVisible( false )
        
        Score.mLayer:insertProp( Score.mHealthStarProps[i] )        
    end
    Score:PositionHealthStars()
        
    Score.mGoodHitAmount = 1000
    
    Score.mMultiplierFade = 0.0
    Score.mMultiplierFadeInc = 2.0

    Score.mMultiplierLost = false
    Score.mMultiplierLostTimer = 0.0
    
    Score.mHealth = 1.0
    Score.mHealthDraw = Score.mHealth
    Score.mHealthDrawInc = .25
  
    Score.mHealthAmountGood = 0.5
    Score.mHealthAmountBad = 0.2
  
    Score.mHealthWarning = false
    Score.mHealthFlashTimer = 0.0
    Score.mHealthFlashInterval = .25
    Score.mHealthFlashOn = false

    Score.mHealthFade = 0.0
    Score.mHealthFadeInc = 8.0
    Score.mHealthFadeGood = true
    
    Score.mNumBitsHit = 0
    Score.mNumBitsToLevelBuildState = 50
    Score.mBitCounterWidth = 0.0
    Score.mBitCounterFade = 0.0
    Score.mBitCounterFadeRate = 0.25
    
    Score.mGreatFanfareZPos = 10.0
    Score.mGreatFanfareTimer = 0.0
    Score.mGreatFanfareDuration = 0.0
    
    Score.mGreatFanfareDeck = DFGraphics.loadSpriteSheet( 'Images/misc', true )
    local iGreatFanfareSheetIndex = Score.mGreatFanfareDeck.names[ "chkchkchk" ]
    Score.mGreatFanfarePropWidth = Score.mGreatFanfareDeck.rects[iGreatFanfareSheetIndex].origWidth * Score.mDigitScale
    Score.mGreatFanfarePropHeight = Score.mGreatFanfareDeck.rects[iGreatFanfareSheetIndex].origHeight * Score.mDigitScale
    Score.mGreatFanfareProp = {}
    
    Score.mGreatFanfareProp = MOAIProp.new()
    Score.mGreatFanfareProp:setDeck( Score.mGreatFanfareDeck )
    Score.mGreatFanfareProp:setScl( Score.mDigitScale, -Score.mDigitScale )           
    Score.mGreatFanfareProp:setLoc( Score.mGreatFanfarePropWidth * -0.5, Score.mGreatFanfarePropHeight * 0.5, Score.mGreatFanfareZPos )
    Score.mGreatFanfareProp:setIndex( iGreatFanfareSheetIndex )
    Score.mGreatFanfareProp:setVisible(false)
    Score.mGreatFanfareProp:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
    
    Score.mTextTimer = 0.0
    Score.mTextIndex = 1
    Score.mLevelIntensity = 0.0
    Score.mScoreVisible = false
    
    Score.mDesiredRotation = 0.0
    Score.mCurrentRotation = 0.0
    Score.mRotationInc = 90.0
    
    Score.mLayer:insertProp( Score.mGreatFanfareProp )                   
end  

function Score:DetermineSize()
    local fDesiredImageSize = Constants.ValidDrawHeight / 10.0
    Score.mDigitScale = fDesiredImageSize / 128.0
    Score.mDigitWidth = fDesiredImageSize
    Score.mDigitPositionLx = -Constants.ValidDrawHalfWidth
    Score.mDigitPositionRx = Constants.ValidDrawHalfWidth

    local fBorderBuffer = Constants.ValidDrawHeight * 0.005
    Score.mHealthBorderDrawWidth = Score.mDigitWidth * 8.0
    Score.mHealthBorderDrawHeight = Score.mDigitWidth
    Score.mHealthBorderDrawX = Score.mHealthBorderDrawWidth * -0.5
    Score.mHealthBorderDrawY = -Constants.ValidDrawHalfHeight + fBorderBuffer
    
    local fHealthDrawHeightTotal = Score.mHealthBorderDrawHeight - (fBorderBuffer * 2.0) 
    Score.mHealthDrawWidth = Score.mHealthBorderDrawWidth - (fBorderBuffer * 2.0)
    Score.mHealthDrawHeight = fHealthDrawHeightTotal * 5.0 / 8.0
    Score.mHealthDrawX = Score.mHealthBorderDrawX + fBorderBuffer
    Score.mHealthDrawY = Score.mHealthBorderDrawY + fBorderBuffer   
    
    Score.mHealthBitHitY = Score.mHealthDrawY + Score.mHealthDrawHeight + fBorderBuffer
    Score.mHealthBitHitHeight = (Score.mHealthBorderDrawY + Score.mHealthBorderDrawHeight - fBorderBuffer) - Score.mHealthBitHitY    
    
    Score.mVerseAdditionalScale = 0.25
    Score.mVerseImageScale = Score.mDigitScale * Score.mVerseAdditionalScale    
    Score.mVerseLetterWidth = Score.mDigitWidth * Score.mVerseAdditionalScale
    Score.mVerseTextPositionX = (Score.mVerseTextCount * Score.mVerseLetterWidth) * -0.5

    Score.mHealthStarHeight = Score.mHealthDrawHeight * 0.5
    Score.mHealthTextPositionX = Score.mHealthDrawX + (Score.mHealthStarHeight * 1.05)
        
    Score.mHUDPositionSpeedComplete = Constants.ValidDrawHeight * 0.1
end
  
function Score:UpdateSize()
    Score:DetermineSize()
    
    local iGreatFanfareSheetIndex = Score.mGreatFanfareDeck.names[ "chkchkchk" ]
    Score.mGreatFanfareProp:setScl( Score.mDigitScale, -Score.mDigitScale )
    Score.mGreatFanfarePropWidth = Score.mGreatFanfareDeck.rects[iGreatFanfareSheetIndex].origWidth * Score.mDigitScale
    Score.mGreatFanfarePropHeight = Score.mGreatFanfareDeck.rects[iGreatFanfareSheetIndex].origHeight * Score.mDigitScale    
    
    for i = 1, Score.mScoreDigits do      
        Score.mScorePropsL[i]:setScl( Score.mDigitScale, -Score.mDigitScale )     
        Score.mScorePropsR[i]:setScl( -Score.mDigitScale, -Score.mDigitScale )
    end
    for i = 1, 3 do
        Score.mMultiplierPropsL[i]:setScl( Score.mDigitScale, -Score.mDigitScale )   
        Score.mMultiplierPropsR[i]:setScl( -Score.mDigitScale, -Score.mDigitScale ) 
    end
    for i = 1, Score.mVerseTextCount do
        Score.mVerseTextProps[i]:setScl( Score.mVerseImageScale, -Score.mVerseImageScale )
    end
    for i = 1, Score.mHealthStarCount do
        Score.mHealthStarProps[i]:SetRadii( Score.mHealthStarHeight * 0.5, Score.mHealthStarHeight )
    end
    
    Score:OffsetScoreAndMultiplierProps()
    Score:PositionVerseText()
    Score:PositionHealthStars()
    Score:SetBitCounterWidth()
end

function Score:SetBitCounterWidth()
    local fBitCounterArea = Score.mVerseTextPositionX - Score.mHealthDrawX
    Score.mBitCounterWidth = fBitCounterArea / ((Score.mNumBitsToLevelBuildState+1.0) * 2.0)
end

function Score:InitForIntro()
    Score.mMultiplierLostTimer = 0.0
    Score.mMultiplierLost = false
    Score.mMultiplier = 1     
    Score.mMultiplierDirty = true

    Score.mHUDPositionOffset = Constants.ValidDrawHeight * .5

    Score.mHealth = 1.0
    Score.mHealthDraw = Score.mHealth

    Score.mScore = 0
    Score.mTargetScore = 0
    Score.mScoreUpdateTimer = Score.mScoreUpdateDuration
    Score.mScoreDirty = true
    
    Score:HideFanfare()
    Score.mHealthWarning = false
    
    --move the score and multiplier elements off the screen
    Score:OffsetScoreAndMultiplierProps()
    Score:PositionVerseText()
    Score:PositionHealthStars()
    Score:SetScoreVisible(false)
        
    Score:SetFramesForScore()
    Score:SetFramesForMultiplier()    
end

function Score:InitForGame()
    Score.mMultiplier = 1     
    Score.mMultiplierDirty = true
    
    Score.mScore = 0
    Score.mTargetScore = 0    
    Score.mScoreDirty = true
    
    Score:SetScoreVisible(true)
    Score:SetNumBitsToBuildState()
    Score:UpdateVerseTextForGameState()    
end

function Score:SetScoreVisible(bVisible)
    local i = 1
    for i = 1, Score.mScoreDigits do
        Score.mScorePropsL[i]:setVisible( bVisible )
        Score.mScorePropsR[i]:setVisible( bVisible )
    end

    for i = 1, Score.mVerseTextCount do
        Score.mVerseTextProps[i]:setVisible( bVisible )
    end    
    
    HUDGlow:SetScoreVisible(bVisible)

    Score.mScoreVisible = bVisible

    --if we're turning on the score, we need to make sure the components are dressed
    if bVisible then
        Score.mMultiplierDirty = true
        Score:SetFramesForMultiplier()
        Score:UpdateHealthStars()
    else
        for i = 1, 3 do
            Score.mMultiplierPropsL[i]:setVisible( false )
            Score.mMultiplierPropsR[i]:setVisible( false )
        end    
        for i = 1, Score.mHealthStarCount do
            Score.mHealthStarProps[i]:setVisible( false )
        end                
    end
end

function Score:SetScoreRotation(x, fSpeed)
    Score.mDesiredRotation = x
    Score.mRotationInc = fSpeed    
end

function Score:InitForComplete()
    Score.mHealthWarning = false
    Score.mHealthFlashOn = false
    Score.mHealthFlashTimer = 0.0
end

function Score:InitForPost()
    Score.mHUDPositionOffset = Constants.ValidDrawHeight * .5
    Score:OffsetScoreAndMultiplierProps()

    local i = 1
    for i = 1, Score.mVerseTextCount do
        Score.mVerseTextProps[i]:setVisible( false )
    end  
    for i = 1, Score.mHealthStarCount do
        Score.mHealthStarProps[i]:setVisible( false )
    end      
    
    Score:HideFanfare()
    Score:SetScoreRotation( 0.0, 220.0 )
    Score.mCurrentRotation = Score.mDesiredRotation    
    for i = 1, Score.mScoreDigits do
        Score.mScorePropsL[i]:setRot( 0.0, 0.0, 0.0 )
        Score.mScorePropsR[i]:setRot( 0.0, 0.0, 0.0 )
    end    
    
    if Score.mBitQuadsLeft ~= nil and Score.mBitQuadsRight ~= nil then
        for i = 1, #Score.mBitQuadsLeft do
            Score.mBitQuadsLeft[i]:setVisible(false)
            Score.mBitQuadsRight[i]:setVisible(false)
        end
    end
    
    Score.mFullBackgroundProp:setVisible(false)
    Score.mHealthBarProp:setVisible(false)
end
 
function Score:OffsetScoreAndMultiplierProps()
    local fScoreImageYBase = -Constants.ValidDrawHalfHeight + (Score.mDigitWidth * 1.5)    
    local i = 1
    for i = 1, Score.mScoreDigits do
        Score.mScorePropsL[i]:setLoc( Score.mDigitPositionLx - Score.mHUDPositionOffset, fScoreImageYBase, 0.0 )
        Score.mScorePropsR[i]:setLoc( Score.mDigitPositionRx + Score.mHUDPositionOffset, fScoreImageYBase, 0.0 )        
        fScoreImageYBase = fScoreImageYBase + Score.mDigitWidth        
    end
    
    Score.mMultiplierPositionY = fScoreImageYBase
    for i = 1, 3 do
        Score.mMultiplierPropsL[i]:setLoc( Score.mDigitPositionLx + (Score.mDigitWidth * (i-1)) - Score.mHUDPositionOffset, Score.mMultiplierPositionY, Score.mMultiplierPropZ )
        Score.mMultiplierPropsR[i]:setLoc( Score.mDigitPositionRx - (Score.mDigitWidth * (i-1)) + Score.mHUDPositionOffset, Score.mMultiplierPositionY, Score.mMultiplierPropZ )       
    end
end

function Score:IsScoreShowingWarning()
    return Score.mHealthWarning and Score.mHealthFlashOn 
end 

function Score:HasMultiplier()
    return Score.mMultiplier > 1 and not Score.mMultiplierLost
end

function Score:Update( tChargeManager, fLevelIntensity )
    local i = 1
    
    --cache for local use
    Score.mLevelIntensity = fLevelIntensity
    
    Score:UpdateScoreValue()
    
    --multiplier pulse
    Score.mMultiplierFade = Score.mMultiplierFade - (Constants.FrameStep * Score.mMultiplierFadeInc)
    Score.mMultiplierFade = math.max( Score.mMultiplierFade, 0.0 )

    --multiplier lost
    if Score.mMultiplierLost then
        Score.mMultiplierLostTimer = Score.mMultiplierLostTimer - Constants.FrameStep
        if Score.mMultiplierLostTimer < 0.0 then
            Score.mMultiplierLostTimer = math.max( Score.mMultiplierLostTimer, 0.0 )
            Score.mMultiplierLost = false
            Score.mMultiplier = 1  
            Score.mMultiplierDirty = true
        end
    end
    
    Score:SetFramesForScore()
    Score:SetFramesForMultiplier()
    
    --score orientation
    local fRotDiff = Score.mCurrentRotation - Score.mDesiredRotation
    if math.abs(fRotDiff) > 0.0 then
        --only update the score orientations if we're not at the desired orient
        local fRotationStep = Score.mRotationInc * Constants.FrameStep
        if fRotDiff > fRotationStep then
            Score.mCurrentRotation = Score.mCurrentRotation - fRotationStep
        elseif fRotDiff < -fRotationStep then
            Score.mCurrentRotation = Score.mCurrentRotation + fRotationStep
        else
            Score.mCurrentRotation = Score.mDesiredRotation
        end
    
        for i = 1, Score.mScoreDigits do
            Score.mScorePropsL[i]:setRot( 0.0, Score.mCurrentRotation, 0.0 )
            Score.mScorePropsR[i]:setRot( 0.0, -Score.mCurrentRotation, 0.0 )
        end
    end
    
    --fanfare
    if Score.mGreatFanfareTimer > 0.0 then
        Score.mGreatFanfareTimer = Score.mGreatFanfareTimer - Constants.FrameStep
                  
        if Score.mGreatFanfareTimer < 0.0 then
            Score:HideFanfare()
        else
            local fPartialDuration = Score.mGreatFanfareDuration * 0.5
            local fEntranceRatio = math.max(Score.mGreatFanfareTimer-fPartialDuration, 0.0) / Score.mGreatFanfareDuration
            local fAlpha = 1.0 - fEntranceRatio
            Score.mGreatFanfareProp:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], fAlpha )
            
            local fZAddition = fEntranceRatio * 500.0
            Score.mGreatFanfareProp:setLoc( Score.mGreatFanfarePropWidth * -0.5, Score.mGreatFanfarePropHeight * 0.5, Score.mGreatFanfareZPos+fZAddition )            
        end
    end
    
    --hud movement
    if Score.mHUDPositionOffset > 0.0 then
        Score.mHUDPositionOffset = Score.mHUDPositionOffset - (Constants.FrameStep * 500.0)
        Score.mHUDPositionOffset = math.max( Score.mHUDPositionOffset, 0.0 )
        Score:OffsetScoreAndMultiplierProps()
        Score:PositionVerseText()
        Score:PositionHealthStars()
    end
                
    --draw health
    local fHealthStep = Score.mHealthDrawInc * Constants.FrameStep    
    if math.abs(Score.mHealth - Score.mHealthDraw) < fHealthStep then
        Score.mHealthDraw = Score.mHealth
    elseif Score.mHealth > Score.mHealthDraw then
        Score.mHealthDraw = Score.mHealthDraw + fHealthStep
    else 
        Score.mHealthDraw = Score.mHealthDraw - fHealthStep
    end
    
    --health flash
    Score.mHealthFlashTimer = Score.mHealthFlashTimer - Constants.FrameStep
    if Score.mHealthFlashTimer < 0.0 then
        Score.mHealthFlashTimer = Score.mHealthFlashInterval
        Score.mHealthFlashOn = not Score.mHealthFlashOn
    end
    
    --health pulse
    Score.mHealthFade = Score.mHealthFade - (Constants.FrameStep * Score.mHealthFadeInc)
    Score.mHealthFade = math.max( Score.mHealthFade, 0.0 ) 

    --health stars 
    Score:UpdateHealthStars()
    
    --tint all props
    for i = 1, Score.mScoreDigits do
        Score.mScorePropsL[i]:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )
        Score.mScorePropsR[i]:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )        
    end
            
    for i = 1, Score.mVerseTextCount do
        Score.mVerseTextProps[i]:setColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3]  )
    end    
    
    --multiplier
    local fMultiplierR, fMultiplierG, fMultiplierB    
    if Score.mMultiplierLost then
        fMultiplierR = DFMath.lerp( Constants.BGColor[1], Constants.BadEffectColor[1], Score.mMultiplierLostTimer )   
        fMultiplierG = DFMath.lerp( Constants.BGColor[2], Constants.BadEffectColor[2], Score.mMultiplierLostTimer )
        fMultiplierB = DFMath.lerp( Constants.BGColor[3], Constants.BadEffectColor[3], Score.mMultiplierLostTimer )
    else
        fMultiplierR = DFMath.lerp( Constants.Color3[1], Constants.SpecialColor[1], Score.mMultiplierFade )    
        fMultiplierG = DFMath.lerp( Constants.Color3[2], Constants.SpecialColor[2], Score.mMultiplierFade )
        fMultiplierB = DFMath.lerp( Constants.Color3[3], Constants.SpecialColor[3], Score.mMultiplierFade )      
    end
    
    --multiplier pos
    Score.mMultiplierPropZ = math.max( Score.mMultiplierPropZ - (Constants.FrameStep * Score.mMultiplierPropZSpeed), 0.0 )
    local fMultiplierHalfZ = Score.mMultiplierPropZ * 0.5
    
    local fMultiplierScale = 1.0
    if Score.mMultiplier > 2 then
        fMultiplierScale = DFMath.lerp( 1.0, 1.9, math.min(Score.mMultiplier/16.0, 1.0) )
    end
    local fMultiplierImageScale = fMultiplierScale * Score.mDigitScale
    local fMultiplierYShift = ((Score.mDigitWidth * fMultiplierScale) - Score.mDigitWidth) * 0.5
    
    local x, y, z
    for i = 2, 3 do
        x, y, z = Score.mMultiplierPropsL[i]:getLoc()
        x = (Score.mDigitPositionLx + Score.mDigitWidth + (Score.mDigitWidth * fMultiplierScale * (i-2))) - Score.mHUDPositionOffset
        y = Score.mMultiplierPositionY + fMultiplierYShift
        
        Score.mMultiplierPropsL[i]:setLoc( x+Score.mMultiplierPropZ, y-fMultiplierHalfZ, Score.mMultiplierPropZ )
        Score.mMultiplierPropsL[i]:setScl( fMultiplierImageScale, -fMultiplierImageScale )
        
        x, y, z = Score.mMultiplierPropsR[i]:getLoc()
        x = (Score.mDigitPositionRx - Score.mDigitWidth - (Score.mDigitWidth * fMultiplierScale * (i-2))) + Score.mHUDPositionOffset
        y = Score.mMultiplierPositionY + fMultiplierYShift
        
        Score.mMultiplierPropsR[i]:setLoc( x-Score.mMultiplierPropZ, y-fMultiplierHalfZ, Score.mMultiplierPropZ )      
        Score.mMultiplierPropsR[i]:setScl( -fMultiplierImageScale, -fMultiplierImageScale ) 
    end
    
    for i = 1, 3 do
        Score.mMultiplierPropsL[i]:setColor( fMultiplierR, fMultiplierG, fMultiplierB, 1.0 )
        Score.mMultiplierPropsR[i]:setColor( fMultiplierR, fMultiplierG, fMultiplierB, 1.0 )        
    end    
    
    --bit counter fade
    Score.mBitCounterFade = math.max( Score.mBitCounterFade - (Score.mBitCounterFadeRate * Constants.FrameStep), 0.0 )
    
    --bit counter props    
    local fBitXOffset = Score.mBitCounterWidth * ((Score.mNumBitsHit-1) * 2.0)
    local fBitHitY = Score.mHealthBitHitY - Score.mHUDPositionOffset
    Score.mTargetProps[1]:setLoc(Score.mHealthDrawX + fBitXOffset, fBitHitY, 0.0)
    Score.mTargetProps[2]:setLoc(Score.mHealthDrawX + Score.mHealthDrawWidth - fBitXOffset - Score.mBitCounterWidth, fBitHitY, 0.0) 
    
    --verse text props
    if Constants.FullMixMode or Constants.LevelState == Constants.LevelStateBuild then
        Score:UpdateBuildText()    
    elseif Constants.LevelState == Constants.LevelStateStart then   
        if Score.mNumBitsHit == Score.mNumBitsToLevelBuildState then
            local sComplete = {"c","o","m","p","l","e","t","e","!"}
            for i = 1, Score.mVerseTextCount do
                Score.mVerseTextProps[i]:setIndex( RadFont.mFontDeck.names[ sComplete[i] ] )        
            end
        else
            local fBitCounterPercent = Score.mNumBitsHit / Score.mNumBitsToLevelBuildState
            fBitCounterPercent = math.floor( fBitCounterPercent * 100.0 )
            local sCounterPercentString = tostring( fBitCounterPercent )
            local percentStringWalker = string.gmatch(sCounterPercentString, ".")
            
            local sFirstDigit = percentStringWalker()
            local sSecondDigit = percentStringWalker()
            if sSecondDigit == nil then
                --single digit
                Score.mVerseTextProps[ Score.mVerseTextCount-2 ]:setIndex( RadFont.mFontDeck.names["."] )
                Score.mVerseTextProps[ Score.mVerseTextCount-1 ]:setIndex( RadFont.mFontDeck.names[sFirstDigit] )
            else    
                Score.mVerseTextProps[ Score.mVerseTextCount-2 ]:setIndex( RadFont.mFontDeck.names[sFirstDigit] )
                Score.mVerseTextProps[ Score.mVerseTextCount-1 ]:setIndex( RadFont.mFontDeck.names[sSecondDigit] )
            end
        end
    end
end

function Score:UpdateVerseTextForGameState()
    if not Constants.FullMixMode then 
        local i = 1    
        if Constants.LevelState == Constants.LevelStateBonus then
            local sBonus = {"b",".","o",".","n",".","u",".","s"}
            for i = 1, Score.mVerseTextCount do
                Score.mVerseTextProps[i]:setIndex( RadFont.mFontDeck.names[ sBonus[i] ] )        
            end    
        elseif Constants.LevelState == Constants.LevelStateHealth then
            local sHealth = {".","h","e","a","l","t","h","!","."}
            for i = 1, Score.mVerseTextCount do
                Score.mVerseTextProps[i]:setIndex( RadFont.mFontDeck.names[ sHealth[i] ] )        
            end                
        elseif Constants.LevelState == Constants.LevelStateBuild then
            Score.mTextTimer = 0.0
            Score:UpdateBuildText()
        else
            local sVerse = {"v","e","r","s","e",".","0","0","("}
            for i = 1, Score.mVerseTextCount do
                Score.mVerseTextProps[i]:setIndex( RadFont.mFontDeck.names[ sVerse[i] ] )        
            end
            
            local fBitCounterPercent = Score.mNumBitsHit / Score.mNumBitsToLevelBuildState
            fBitCounterPercent = math.floor( fBitCounterPercent * 100.0 )
            local sCounterPercentString = tostring( fBitCounterPercent )
            local percentStringWalker = string.gmatch(sCounterPercentString, ".")
            
            local sFirstDigit = percentStringWalker()
            local sSecondDigit = percentStringWalker()
            if sSecondDigit == nil then
                --single digit
                Score.mVerseTextProps[ Score.mVerseTextCount-2 ]:setIndex( RadFont.mFontDeck.names["."] )
                Score.mVerseTextProps[ Score.mVerseTextCount-1 ]:setIndex( RadFont.mFontDeck.names[sFirstDigit] )
            else    
                Score.mVerseTextProps[ Score.mVerseTextCount-2 ]:setIndex( RadFont.mFontDeck.names[sFirstDigit] )
                Score.mVerseTextProps[ Score.mVerseTextCount-1 ]:setIndex( RadFont.mFontDeck.names[sSecondDigit] )
            end
        end
    end
end

function Score:UpdateBuildText()      
    Score.mTextTimer = Score.mTextTimer - Constants.FrameStep
    if Score.mTextTimer <= 0.0 then               
        Score.mTextIndex = Score.mTextIndex + 1
        if Score.mTextIndex > 1 then
            Score.mTextIndex = 0
        end        
        
        local sBuild = {}        
        if Score.mTextIndex == 0 then     sBuild = {"d","r","o","p",".",".",".",".","."} 
        elseif Score.mTextIndex == 1 then sBuild = {".",".",".",".","c","h","o","r","d"} 
        end
        
        if Score.mLevelIntensity < .2 then Score.mTextTimer = 2.0
        elseif Score.mLevelIntensity < .4 then Score.mTextTimer = 1.0
        elseif Score.mLevelIntensity < .6 then Score.mTextTimer = 0.5
        elseif Score.mLevelIntensity < .8 then Score.mTextTimer = 0.25
        else Score.mTextTimer = 0.125
        end
        
        local i = 0
        for i = 1, Score.mVerseTextCount do
            Score.mVerseTextProps[i]:setIndex( RadFont.mFontDeck.names[ sBuild[i] ] )        
        end    
    end
end

function Score:UpdateComplete(fLevelIntensity)
    Score.mHUDPositionOffset = Score.mHUDPositionOffset + (Score.mHUDPositionSpeedComplete * Constants.FrameStep)
    Score:OffsetScoreAndMultiplierProps()
    Score:PositionVerseText()
    Score:PositionHealthStars()
    Score:UpdateScoreValue()
    Score:HideFanfare()
    Score.mLevelIntensity = fLevelIntensity
end

function Score:UpdateHealthStars()
    local iNumProps = math.floor(Score.mHealthDraw - 0.001) + 1
    
    local i = 1
    for i = 1, Score.mHealthStarCount do
        if i < iNumProps then
            Score.mHealthStarProps[i]:setVisible( true )
            Score.mHealthStarProps[i]:SetColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1.0 )
        else
            Score.mHealthStarProps[i]:setVisible( false )
        end
    end 
end

function Score:UpdateScoreValue()
    Score.mScoreUpdateTimer = Score.mScoreUpdateTimer - Constants.FrameStep
    if Score.mScoreUpdateTimer < 0.0 then
        Score.mScore = Score.mTargetScore
        Score.mScoreUpdateTimer = Score.mScoreUpdateDuration
        Score.mScoreDirty = true        
    end
end

function Score:SetFramesForScore()
    if Score.mScoreDirty then
        --break score in to components
        local iCurrentIndex = Score.mScoreDigits
        local iTestScore = Score.mScore
        
        while iTestScore >= 1 do
            local iDeckIndex = math.floor(iTestScore % 10)
            local iPropIndex = RadFont.mFontDeck.names[ tostring(iDeckIndex) ]
            Score.mScorePropsL[iCurrentIndex]:setIndex( iPropIndex )
            Score.mScorePropsR[iCurrentIndex]:setIndex( iPropIndex )
            
            iCurrentIndex = iCurrentIndex - 1
            iTestScore = iTestScore / 10
        end
        
        local iZeroIndex = RadFont.mFontDeck.names["0"]
        local i = 1
        for i = 1, iCurrentIndex do
            Score.mScorePropsL[i]:setIndex( iZeroIndex )
            Score.mScorePropsR[i]:setIndex( iZeroIndex )
        end
        
        Score.mScoreDirty = false
    end
end
  
function Score:SetFramesForMultiplier()    
    if Score.mMultiplierDirty then
        --break multiplier in to components
        local iHighestMultiplierIndex = 1 
        if Score.mMultiplier > 1 then
            --set the 'x'
            local iXIndex = RadFont.mFontDeck.names["x"]
            Score.mMultiplierPropsL[1]:setIndex( iXIndex )
            Score.mMultiplierPropsR[1]:setIndex( iXIndex )        
        
            Score.mMultiplierPropsL[1]:setVisible(true)
            Score.mMultiplierPropsR[1]:setVisible(true)        
            
            if Score.mMultiplier > 9 then
                local iTestMultiplier = Score.mMultiplier
                local iDeckIndex = math.floor(iTestMultiplier % 10)
                local iPropIndex = RadFont.mFontDeck.names[ tostring(iDeckIndex) ]
                Score.mMultiplierPropsL[3]:setIndex( iPropIndex )
                Score.mMultiplierPropsR[3]:setIndex( iPropIndex )    
                Score.mMultiplierPropsL[3]:setVisible(true)
                Score.mMultiplierPropsR[3]:setVisible(true)      
              
                iTestMultiplier = iTestMultiplier / 10    
                iDeckIndex = math.floor(iTestMultiplier % 10)
                iPropIndex = RadFont.mFontDeck.names[ tostring(iDeckIndex) ]            
                Score.mMultiplierPropsL[2]:setIndex( iPropIndex )
                Score.mMultiplierPropsR[2]:setIndex( iPropIndex )      
                Score.mMultiplierPropsL[2]:setVisible(true)
                Score.mMultiplierPropsR[2]:setVisible(true) 
                
                iHighestMultiplierIndex = 4 
            else
                local iPropIndex = RadFont.mFontDeck.names[ tostring(Score.mMultiplier) ]
                Score.mMultiplierPropsL[2]:setIndex( iPropIndex )
                Score.mMultiplierPropsR[2]:setIndex( iPropIndex )      
                Score.mMultiplierPropsL[2]:setVisible(true)
                Score.mMultiplierPropsR[2]:setVisible(true)            
                
                iHighestMultiplierIndex = 3
            end
        end
        
        local i = 1
        for i = iHighestMultiplierIndex, 3 do
            Score.mMultiplierPropsL[i]:setVisible(false)
            Score.mMultiplierPropsR[i]:setVisible(false)
        end
        
        Score.mMultiplierDirty = false
    end
end

function Score:Draw()         
    --no render if we're hidden
    if not Score.mScoreVisible then
        if Score.mBitQuadsLeft ~= nil and Score.mBitQuadsRight ~= nil then
            for i = 1, #Score.mBitQuadsLeft do
                Score.mBitQuadsLeft[i]:setVisible(false)
                Score.mBitQuadsRight[i]:setVisible(false)
            end
        end
        Score.mFullBackgroundProp:setVisible(false)
        Score.mHealthBarProp:setVisible(false)
        
        return
    end
    
    --health bar color
    local fHealthR = 0.0
    local fHealthG = 0.0
    local fHealthB = 0.0    
    if Score.mHealthWarning and Score.mHealthFlashOn then
        fHealthR = Constants.BadEffectColor[1]
        fHealthG = Constants.BadEffectColor[2]
        fHealthB = Constants.BadEffectColor[3]
    else
        fHealthR = Constants.Color2[1] 
        fHealthG = Constants.Color2[2] 
        fHealthB = Constants.Color2[3]
    end
    
    if Score.mHealthFadeGood then
        fHealthR = DFMath.lerp( fHealthR, Constants.SpecialColor[1], Score.mHealthFade )    
        fHealthG = DFMath.lerp( fHealthG, Constants.SpecialColor[2], Score.mHealthFade )
        fHealthB = DFMath.lerp( fHealthB, Constants.SpecialColor[3], Score.mHealthFade )          
    else
        fHealthR = DFMath.lerp( fHealthR, Constants.BadEffectColor[1], Score.mHealthFade )    
        fHealthG = DFMath.lerp( fHealthG, Constants.BadEffectColor[2], Score.mHealthFade )
        fHealthB = DFMath.lerp( fHealthB, Constants.BadEffectColor[3], Score.mHealthFade )          
    end   
        
    --health bar
    local bDrawFullBackground = false
    local fHealthPercent = Score.mHealthDraw
    while fHealthPercent > 1.0 do
        fHealthPercent = fHealthPercent - 1.0
        bDrawFullBackground = true
    end
    local fHealthWidth = Score.mHealthDrawWidth * fHealthPercent
    local fHealthBarY = Score.mHealthDrawY - Score.mHUDPositionOffset
    
    if bDrawFullBackground then
        Score.mFullBackgroundProp:setVisible(true)
        Score.mFullBackgroundProp:setColor(Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1)
        Score.mFullBackgroundProp:setLoc(Score.mHealthDrawX, fHealthBarY)
        Score.mFullBackgroundProp:setScl(Score.mHealthDrawWidth, Score.mHealthDrawHeight)
    else
        Score.mFullBackgroundProp:setVisible(false)
    end
    
    Score.mHealthBarProp:setVisible(true)
    Score.mHealthBarProp:setColor(fHealthR, fHealthG, fHealthB, 1)
    Score.mHealthBarProp:setLoc(Score.mHealthDrawX, fHealthBarY)
    Score.mHealthBarProp:setScl(fHealthWidth, Score.mHealthDrawHeight)
    
    --num bits hit
    if not Constants.FullMixMode then
        local r = Constants.Color3[1]
        local g = Constants.Color3[2]
        local b = Constants.Color3[3]
        if Constants.LevelState == Constants.LevelStateBuild then
            --pulse bits
            r = DFMath.lerp( Constants.Color3[1], Constants.SpecialColor[1], Constants.Heartbeat )    
            g = DFMath.lerp( Constants.Color3[2], Constants.SpecialColor[2], Constants.Heartbeat )
            b = DFMath.lerp( Constants.Color3[3], Constants.SpecialColor[3], Constants.Heartbeat )            
        end
        
        local fBitHitY = Score.mHealthBitHitY - Score.mHUDPositionOffset
        
        if not Score.mBitQuadsLeft then
            Score.mBitQuadsLeft = {}
            Score.mBitQuadsRight = {}
            
            local maxBits = 100 --?
            
            -- we draw these bits twice (left and right side of hud)
            for i = 1, maxBits do
                local quad = MOAIProp.new()
                quad:setDeck(Score.mQuadDeck)
                quad:setVisible(false)
                
                Score.mLayer:insertProp(quad)
                table.insert(Score.mBitQuadsLeft, quad)                
            end
            for i = 1, maxBits do
                local quad = MOAIProp.new()
                quad:setDeck(Score.mQuadDeck)
                quad:setVisible(false)
                
                Score.mLayer:insertProp(quad)
                table.insert(Score.mBitQuadsRight, quad)                
            end

        end
        
        local numBitsHit = math.floor( math.max(Score.mNumBitsHit, 0.0) )
        
        local i = 1
        for i = 1, numBitsHit do
            Score.mBitQuadsLeft[i]:setVisible(true)
            Score.mBitQuadsRight[i]:setVisible(true)
            
            if i == Score.mNumBitsHit and Score.mNumBitsHit ~= Score.mNumBitsToLevelBuildState then
                r = DFMath.lerp( Constants.Color3[1], Constants.SpecialColor[1], Score.mBitCounterFade )    
                g = DFMath.lerp( Constants.Color3[2], Constants.SpecialColor[2], Score.mBitCounterFade )
                b = DFMath.lerp( Constants.Color3[3], Constants.SpecialColor[3], Score.mBitCounterFade )            
            end
            Score.mBitQuadsLeft[i]:setColor( r, g, b, 1.0 )
            Score.mBitQuadsRight[i]:setColor( r, g, b, 1.0 )
            
            local fBitXOffset = Score.mBitCounterWidth * ((i-1) * 2.0)
            local fLeftBit = Score.mHealthDrawX + fBitXOffset
            Score.mBitQuadsLeft[i]:setLoc(fLeftBit, fBitHitY)
            Score.mBitQuadsLeft[i]:setScl(Score.mBitCounterWidth, Score.mHealthBitHitHeight)
            
            local fRightBit = Score.mHealthDrawX + Score.mHealthDrawWidth - fBitXOffset - Score.mBitCounterWidth
            Score.mBitQuadsRight[i]:setLoc(fRightBit, fBitHitY)
            Score.mBitQuadsRight[i]:setScl(Score.mBitCounterWidth, Score.mHealthBitHitHeight)
        end
        for i = numBitsHit + 1, #Score.mBitQuadsLeft do
            Score.mBitQuadsLeft[i]:setVisible(false)
            Score.mBitQuadsRight[i]:setVisible(false)
        end
    end
    
    --border
    if Score.mHealthWarning and Score.mHealthFlashOn then
        MOAIGfxDevice.setPenColor( Constants.BadEffectColor[1], Constants.BadEffectColor[2], Constants.BadEffectColor[3] )
    else
        MOAIGfxDevice.setPenColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3] )
    end

    local fBorderY = Score.mHealthBorderDrawY - Score.mHUDPositionOffset
    MOAIDraw.drawRect( Score.mHealthBorderDrawX, fBorderY, Score.mHealthBorderDrawX + Score.mHealthBorderDrawWidth, fBorderY + Score.mHealthBorderDrawHeight )
end
    
function Score:PositionVerseText()
    local fVerseTextPositionY = Score.mHealthBitHitY - Score.mHUDPositionOffset + Score.mVerseLetterWidth
    local fXOffset = 0.0
    local i = 1
    for i = 1, Score.mVerseTextCount do
        Score.mVerseTextProps[i]:setLoc( Score.mVerseTextPositionX + fXOffset, fVerseTextPositionY, 0.0 )
        fXOffset = fXOffset + Score.mVerseLetterWidth
    end
end
    
function Score:PositionHealthStars()        
    local fHealthTextPositionY = Score.mHealthDrawY - Score.mHUDPositionOffset + (Score.mHealthStarHeight * 1.05)
    local fXOffset = 0.0
    local i = 1
    for i = 1, Score.mHealthStarCount do
        Score.mHealthStarProps[i]:setLoc( Score.mHealthTextPositionX + fXOffset, fHealthTextPositionY, 0.0 )
        fXOffset = fXOffset + (Score.mHealthStarHeight * 2.0)
    end   
end
    
function Score:HideFanfare()
    Score.mGreatFanfareTimer = 0.0
    Score.mGreatFanfareProp:setVisible(false)
end

function Score:ShowGreatFanfare(fDuration)
    Score.mGreatFanfareDuration = fDuration
    Score.mGreatFanfareTimer = Score.mGreatFanfareDuration
    Score.mGreatFanfareProp:setVisible(true)
    Score.mGreatFanfareProp:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3] )
end
  
function Score:GoodHit(iNumBitsHit)
    local lBig = 99999999
    
    local fScaledMultiplier = 1.0 + ((Score.mMultiplier-1.0) * 0.25)
    Score.mTargetScore = Score.mTargetScore + (fScaledMultiplier * Score.mGoodHitAmount)
    if Score.mTargetScore > lBig then
        Score.mTargetScore = Score.mTargetScore - lBig
    end
    
    --high score achievement
    if Score.mTargetScore > 10000000 then
        Achievement.updateAchievement( "DA_HighScore", 100 )
    end
    
    --if we're in the bonus round, don't increment our num bits hit when we get one
    if Constants.LevelState ~= Constants.LevelStateBonus and not Constants.FullMixMode then
        Score.mNumBitsHit = Score.mNumBitsHit + iNumBitsHit
        Score.mNumBitsHit = math.min( Score.mNumBitsHit, Score.mNumBitsToLevelBuildState )        
        Score.mBitCounterFade = 1.0
    end    
end
  
function Score:IncMultiplier()
    Score.mMultiplier = Score.mMultiplier + 1
    Score.mMultiplier = math.min( Score.mMultiplier, 99 )
    Score.mMultiplierDirty = true
    Score.mMultiplierFade = 1.0
    Score.mMultiplierPropZ = 50.0
    
    --high multiplier achievement
    if Score.mMultiplier == 99 then
        Achievement.updateAchievement( "DA_HighMultiplier", 100 )
    end
end
  
function Score:ClearMultiplier()
    if Score.mMultiplierLost == false then    
        Score.mMultiplierLost = true
        Score.mMultiplierLostTimer = 1.0
    end
end
  
function Score:AddHealthForLevelComplete( fPercent )
    Score.mHealth = Score.mHealth + (Score.mHealthAmountGood * fPercent)
  
    if Score.mHealthFadeGood == true or Score.mHealthFade <= 0.0 then
        Score.mHealthFade = 1.0
        Score.mHealthFadeInc = 2.0
        Score.mHealthFadeGood = true
    end
    
    Score:UpdateWarning()    
end
  
function Score:BadHit()
    local fAmount = Score.mHealthAmountBad
    if Constants.TouchBotEnabled then
        fAmount = fAmount * 0.05
    end
    
    if Constants.CurrentGameLevel == 1 then
        fAmount = fAmount * .25
    elseif Constants.CurrentGameLevel == 2 then
        fAmount = fAmount * .5        
    elseif Constants.CurrentGameLevel == 3 then
        fAmount = fAmount * .75
    end
    
    Score.mHealth = Score.mHealth - fAmount
    if Score.mHealth <= 0.0 then
        --waa waaa.. you lose
        Constants.DesiredGameState = Constants.GameStatePost
        AudioManager:PlaySound( 'Radius/SFX/ScratchHit' )
    end
    Score.mHealth = math.max(Score.mHealth, 0.0)  
    
    Score:SetHealthFadeBad()
    Score:UpdateWarning()
end
  
function Score:GetBitHitTargetProp()
    Score.mTargetPropIndex = Score.mTargetPropIndex + 1
    if Score.mTargetPropIndex > 2 then
        Score.mTargetPropIndex = 1
    end

    return Score.mTargetProps[Score.mTargetPropIndex]
end

function Score:SetHealthFadeBad()
    Score.mHealthFade = 1.0
    Score.mHealthFadeInc = 1.0
    Score.mHealthFadeGood = false    
end
  
function Score:UpdateWarning()
    local bShouldBeFlashing = Score.mHealth <= .3
    if bShouldBeFlashing and Score.mHealthWarning == false then
        --start flashing!
        Score.mHealthWarning = true
        Score.mHealthFlashTimer = 0.0
        Score.mHealthFlashInterval = .25
        Score.mHealthFlashOn = true   
    elseif bShouldBeFlashing == false and Score.mHealthWarning then
        --stop flashing!
        Score.mHealthWarning = false
    end
end
  
function Score:SetNumBitsToBuildState()
    if Constants.CurrentGameLevel == 1 then
        Score.mNumBitsToLevelBuildState = 35
    elseif Constants.CurrentGameLevel == Constants.PointHorizLevel then
        Score.mNumBitsToLevelBuildState = 20
    elseif Constants.CurrentGameLevel == Constants.HoldTutorialLevel then 
        Score.mNumBitsToLevelBuildState = 40
    elseif Constants.CurrentGameLevel == Constants.PointVertLevel then
        Score.mNumBitsToLevelBuildState = 30
    elseif Constants.CurrentGameLevel == Constants.PointBothLevel then
        Score.mNumBitsToLevelBuildState = 35
    elseif Constants.CurrentGameLevel == Constants.ThrowTutorialLevel then 
        Score.mNumBitsToLevelBuildState = 75
    elseif Constants.CurrentGameLevel == Constants.AllLevel then
        Score.mNumBitsToLevelBuildState = 65
    elseif Constants.CurrentGameLevel == Constants.NumGameLevels then
        Score.mNumBitsToLevelBuildState = 65        
    else
        Score.mNumBitsToLevelBuildState = 50
    end

    Score.mNumBitsHit = 0    
    Score:SetBitCounterWidth()
end
  
function Score:ForceBuildState()
   Score.mNumBitsHit = Score.mNumBitsToLevelBuildState    
end  

function Score:ShouldEnterBuildState()
    return Score.mNumBitsHit == Score.mNumBitsToLevelBuildState
end

function Score:SetNumBitsHitFromPercentage(fPercent)
    Score.mNumBitsHit = Score.mNumBitsToLevelBuildState * fPercent
end

return Score
