local DFFile = require("DFCommon.File")
local DFGraphics = require("DFCommon.Graphics")
local Constants = require("Constants")
local Renderer = require("Renderer")
local Post = require("postfx.Post")

local TitleScreen = {}

function TitleScreen:new(rLayer)
    local Ob = {}
        
    function Ob:OnScreenResize(width, height)
        local width = Constants.ValidDrawWidth
        local height = Constants.ValidDrawHeight
    
        local imgSize = 2048
        local rawImageW = 1920
        local rawImageH = 1080

        local imgScale = height / imgSize
        
        local imgW = (imgScale * imgSize) - 64
        local imgH = (imgScale * imgSize) - 64
        
        Ob.mDFBox:setScl(imgW*0.35, -imgH*0.35)
        Ob.mDracogenBox:setScl(imgW*0.35, -imgH*0.35)
    end
    
    function Ob:Reset()
        Ob.mTimeRemaining = Ob.mShowDuration
        Ob.mState = Ob.mStateDracogen
        
        if Constants.VisualQuality == Constants.VisualQualityLevelLow then
            Ob.mDracogenBox:setBlendMode(MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_COLOR)
            Ob.mDFBox:setBlendMode(MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_COLOR)
        else
            Ob.mDracogenBox:setBlendMode(MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA)
            Ob.mDFBox:setBlendMode(MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA)
        end
    end
    
    function Ob:RequestClear()
        if Ob.mTimeRemaining > Ob.mFadeTime then
            Ob.mFadeTime = Ob.mFadeTime * 0.3
            Ob.mTimeRemaining = Ob.mFadeTime 
        end
    end
    
    function Ob:Update()
        if Ob.mTimeRemaining > 0.0 then
            Ob.mTimeRemaining = Ob.mTimeRemaining - 0.033 --Constants.FrameStep

            --set color according to time
            local fAlpha = 1.0
            if Ob.mTimeRemaining > Ob.mShowDuration - Ob.mFadeTime then
                fAlpha = (Ob.mShowDuration - Ob.mTimeRemaining) / Ob.mFadeTime
            elseif Ob.mTimeRemaining < Ob.mFadeTime then
                fAlpha = Ob.mTimeRemaining / Ob.mFadeTime
            end
            
            if fAlpha > 0.999999 and Ob.mWaitingToLoad then
                Renderer:Preload()    
                Post.Preload()
    
                Ob.mWaitingToLoad = false
            end
            
            if Ob.mState == Ob.mStateDracogen then
                Ob.mDracogenBox:setColor(1.0, 1.0, 1.0, fAlpha)
            elseif Ob.mState == Ob.mStateDF then
                Ob.mDFBox:setColor(1.0, 1.0, 1.0, fAlpha)
            end

            --update state if we're out of time
            if Ob.mTimeRemaining <= 0.0 then
                Ob.mState = Ob.mState + 1
                Ob.mTimeRemaining = Ob.mShowDuration
                
                if Ob.mState == Ob.mStateDF then
                    Ob.mLayer:removeProp(Ob.mDracogenBox)
                    Ob.mLayer:insertProp(Ob.mDFBox)
                elseif Ob.mState == Ob.mStateDone then
                    Ob.mLayer:removeProp(Ob.mDFBox)
                    Constants.DesiredGameState = Constants.GameStatePreIntro
                end
            end
        end
    end
    
    function Ob:Show()
        Ob:Reset()
        Ob.mLayer:insertProp(Ob.mDracogenBox)        
    end
    
    local splashDeck = MOAIGfxQuad2D.new()
    local splashTexture = DFGraphics.loadTexture( "SplashScreen", true )
    splashTexture:setFilter(MOAITexture.GL_LINEAR, MOAITexture.GL_LINEAR)
    splashDeck:setTexture( splashTexture )
    splashDeck:setRect( -1, -1, 1, 1 )
    Ob.mDFBox = MOAIProp2D.new()
    Ob.mDFBox:setDeck( splashDeck )
    Ob.mDFBox:setLoc(0, 0, 1 )
    Ob.mDFBox:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
    Ob.mDFBox:setColor(1.0, 1.0, 1.0, 0.0)

    local dracogenDeck = MOAIGfxQuad2D.new()
    local dracogenTexture = DFGraphics.loadTexture( "DracogenScreen", true )
    dracogenTexture:setFilter(MOAITexture.GL_LINEAR, MOAITexture.GL_LINEAR)
    dracogenDeck:setTexture( dracogenTexture )
    dracogenDeck:setRect( -1, -1, 1, 1 )
    Ob.mDracogenBox = MOAIProp2D.new()
    Ob.mDracogenBox:setDeck( dracogenDeck )
    Ob.mDracogenBox:setLoc(0, 0, 1 )
    Ob.mDracogenBox:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
    Ob.mDracogenBox:setColor(1.0, 1.0, 1.0, 0.0)
    
    Ob.mWaitingToLoad = true
    
    Ob.mLayer = rLayer
    
    Ob.mTimeRemaining = 0.0
    Ob.mShowDuration = 3.5
    Ob.mFadeTime = 1.25
    
    Ob.mStateDracogen = 0
    Ob.mStateDF = 1
    Ob.mStateDone = 2
    Ob.mState = Ob.mStateDracogen
    
    Ob:Reset()
    
    return Ob
end

return TitleScreen