local Constants = require('Constants')
local DFGraphics = require('DFCommon.Graphics')
local DFMath = require('DFCommon.Math')
local RadFont = require('RadFont')
local RadEffects = require('RadEffects')
local AudioManager = require('AudioManager')
local ColorManager = require('ColorManager')
local GameRing = require('GameRing')
local UI = {}

function UI:Init( tRenderer )
    UI.mPropZ = 2.0
    
    UI.mNewAbilityState = Constants.UINewAbilityStateNone
    UI.mNewAbilityTimer = 0.0
    UI.mNewAbilityDuration = 3.0

    UI.mPreIntroTimer = 0.0
    UI.mPreIntroHideTitle = 0
    UI.mPreIntroFadeIn = 0
    UI.mPreIntroOffsetScale = 400.0

    UI.mIntroTimer = 0.0
    UI.mIntroExplodeDuration = 0.5
    
    UI.mMenuTimer = 0.0
    UI.mMenuSelectionSound = nil
    
    UI.mCompleteTimer = 0.0
    UI.mCompleteWhiteFadeDuration = 4.0
    UI.mCompleteTitleFadeInMark = 6.0
    UI.mCompleteTitleFadeDuration = 4.0
    UI.mCompleteDuration = 15.0
    UI.mIntensityDuration = 4.0
    
    UI.mPostState = 0
    UI.mPostInterval = 0.5
    UI.mPostTimer = UI.mPostInterval
    UI.mPostStateAllowBackToIntro = 5
    UI.mBigTitleMovementScale = 500.0
        
    local fDesiredLetterImageSize = Constants.ValidDrawHeight / 10.0
    UI.mImageScale = fDesiredLetterImageSize / 128.0
    UI.mLetterWidth = fDesiredLetterImageSize
    UI.mScoreDigits = 8
    UI.mScoreYOffsetPercent = 0.0
    UI.mScoreYOffset = 0.0
    UI.mScoreYOffsetDesired = 0.0
    UI.mScoreYMovementSpeed = 400.0
    
    UI.mScoreProps = {}
    for i = 1, UI.mScoreDigits do
        UI.mScoreProps[i] = MOAIProp.new()
        UI.mScoreProps[i]:setDeck( RadFont.mFontDeck )        
        UI.mScoreProps[i]:setScl( UI.mImageScale, -UI.mImageScale )
        UI.mScoreProps[i]:setIndex( RadFont.mFontDeck.names["0"] )
        UI.mScoreProps[i]:setVisible(false)
        
        tRenderer.mUILayer:insertProp( UI.mScoreProps[i] )        
    end
    
    --"final score" has 10 symbols.. we fake the space
    local sFinalScore = {"f","i","n","a","l","s","c","o","r","e"}
    UI.mFinalScoreTextCount = #sFinalScore
    UI.mFinalScoreTextProps = {}
    for i = 1, UI.mFinalScoreTextCount do
        UI.mFinalScoreTextProps[i] = MOAIProp.new()
        UI.mFinalScoreTextProps[i]:setDeck( RadFont.mFontDeck )        
        UI.mFinalScoreTextProps[i]:setScl( UI.mImageScale, -UI.mImageScale )
        UI.mFinalScoreTextProps[i]:setIndex( RadFont.mFontDeck.names[ sFinalScore[i] ] )
        UI.mFinalScoreTextProps[i]:setVisible(false)
        
        tRenderer.mUILayer:insertProp( UI.mFinalScoreTextProps[i] )        
    end
    UI:PositionFinalScoreText()
    
    local sTitle = {"d","r","o","p","c","h","o","r","d"}
    UI.mTitleTextCount = #sTitle
    UI.mTitleImageAdditionalScale = 0.7
    UI.mTitleImageScale = UI.mImageScale * UI.mTitleImageAdditionalScale
    local fTitleLetterWidth = UI.mLetterWidth * UI.mTitleImageAdditionalScale
    local fTitleTextPositionX = (((UI.mTitleTextCount-5) * fTitleLetterWidth) * -0.5) + (Constants.ValidDrawHeight * .25)
    local fTitleTextPositionY = Constants.ValidDrawHeight * .25
    UI.mTitleTextProps = {}
    for i = 1, #sTitle do
        UI.mTitleTextProps[i] = MOAIProp.new()
        UI.mTitleTextProps[i]:setDeck( RadFont.mFontDeck )        
        UI.mTitleTextProps[i]:setScl( UI.mTitleImageScale, -UI.mTitleImageScale )
        UI.mTitleTextProps[i]:setLoc( 600 + fTitleTextPositionX + ((i-1) * fTitleLetterWidth), fTitleTextPositionY, UI.mPropZ )
        UI.mTitleTextProps[i]:setIndex( RadFont.mFontDeck.names[ sTitle[i] ] )
        UI.mTitleTextProps[i]:setVisible(false)
        UI.mTitleTextProps[i]:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
        
        tRenderer.mUILayer:insertProp( UI.mTitleTextProps[i] )        
    end
    UI:PositionTitleText(0.0, false)
        
    UI.mMenuAdditionalScale = 0.65    
    UI.mMenuTextCount = 0
    UI.mMenuTextProps = {}
    
    UI.mBigTitleDeck = DFGraphics.loadSpriteSheet( 'Images/misc', true )
    local iBigTitleSheetIndex = UI.mBigTitleDeck.names[ "title" ]
    UI.mBigTitleScale = UI.mImageScale * 4.0
    UI.mBigTitlePropWidth = UI.mBigTitleDeck.rects[iBigTitleSheetIndex].origWidth * UI.mBigTitleScale
    UI.mBigTitlePropHeight = UI.mBigTitleDeck.rects[iBigTitleSheetIndex].origHeight * UI.mBigTitleScale
    UI.mBigTitlePropCountX = math.floor(Constants.ValidDrawWidth / UI.mBigTitlePropWidth) + 2
    UI.mBigTitlePropCountY = ( math.floor((Constants.ValidDrawHalfHeight-UI.mLetterWidth) / UI.mBigTitlePropHeight) + 1 ) * 2.0
    UI.mBigTitleProps = {}
    
    local x = 1
    local y = 1
    local fBigTitleTextPositionX = -Constants.ValidDrawHalfWidth
    for y = 1, UI.mBigTitlePropCountY do
        UI.mBigTitleProps[y] = {}
            
        for x = 1, UI.mBigTitlePropCountX do
            --i'm indexing y,x so I can walk them vertically later
            UI.mBigTitleProps[y][x] = MOAIProp.new()
            UI.mBigTitleProps[y][x]:setDeck( UI.mBigTitleDeck )
            UI.mBigTitleProps[y][x]:setScl( UI.mBigTitleScale, -UI.mBigTitleScale )
            UI.mBigTitleProps[y][x]:setIndex(iBigTitleSheetIndex)
            UI.mBigTitleProps[y][x]:setVisible(false)
            
            tRenderer.mUILayer:insertProp( UI.mBigTitleProps[y][x] )                
        end
    end
    UI:PositionBigTitleText()
    
    UI.mClockDeck = DFGraphics.loadSpriteSheet( 'Images/misc', true )
    UI.mClockIndex = UI.mClockDeck.names[ "clock" ]
    UI.mClockScale = UI.mImageScale * 0.5
    UI.mClockPropWidth = UI.mClockDeck.rects[UI.mClockIndex].origWidth * UI.mClockScale
    UI.mClockPropHeight = UI.mClockDeck.rects[UI.mClockIndex].origHeight * UI.mClockScale
    
    UI.mClockProp = MOAIProp.new()
    UI.mClockProp:setDeck( UI.mClockDeck )
    UI.mClockProp:setScl( UI.mClockScale, -UI.mClockScale )
    UI.mClockProp:setIndex( UI.mClockIndex )
    UI.mClockProp:setVisible( false )
    UI.mClockProp:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
    tRenderer.mUILayer:insertProp( UI.mClockProp )
    UI:PositionClock()
    
    UI.mClockAlpha = 0.0
    UI.mClockAlphaDesired = 0.0
    UI.mClockAlphaScalar = 2.0
    UI.mClockColor = {0.0, 0.0, 0.0, 0.0}
    UI.mClockBarWidth = UI.mClockPropWidth * 7.5
    UI.mClockPercent = 0.0
        
    UI.mUseWhiteOverlay = false
    UI.mWhiteOverlayAlpha = 0.0
    UI.mWhiteOverlayInc = 255.0  
    
    --just to init vars
    UI:InitPreIntro(0.0, 1.0)
    UI:InitPost(0)
end

function UI:UpdateSize()
    local i = 1
    local x = 1
    local y = 1

    --update base letter scale and readjust position
    local fDesiredLetterImageSize = Constants.ValidDrawHeight / 10.0
    UI.mImageScale = fDesiredLetterImageSize / 128.0
    UI.mLetterWidth = fDesiredLetterImageSize
    UI.mScoreYOffset = UI.mScoreYOffsetPercent * UI.mLetterWidth
    UI.mScoreYOffsetDesired = UI.mScoreYOffset
    
    for i = 1, UI.mScoreDigits do       
        UI.mScoreProps[i]:setScl( UI.mImageScale, -UI.mImageScale )
    end
    for i = 1, UI.mFinalScoreTextCount do
        UI.mFinalScoreTextProps[i]:setScl( UI.mImageScale, -UI.mImageScale )    
    end
    UI:PositionFinalScoreText()   
    
    --title image
    UI.mTitleImageScale = UI.mImageScale * UI.mTitleImageAdditionalScale
    for i = 1, #UI.mTitleTextProps do
        UI.mTitleTextProps[i]:setScl( UI.mTitleImageScale, -UI.mTitleImageScale )
    end
    UI:PositionTitleText(0.0)   
        
    --big title
    UI.mBigTitleScale = UI.mImageScale * 2.0
    for y = 1, UI.mBigTitlePropCountY do            
        for x = 1, UI.mBigTitlePropCountX do
            UI.mBigTitleProps[y][x]:setScl( UI.mBigTitleScale, -UI.mBigTitleScale )
        end
    end
    UI:PositionBigTitleText()
    
    --menu
    UI.mMenuImageScale = UI.mTitleImageScale * UI.mMenuAdditionalScale
    for i = 1, UI.mMenuTextCount do 
        UI.mMenuTextProps[i]:setScl( UI.mMenuImageScale, -UI.mMenuImageScale )    
    end    
    UI:PositionMenuText() 
 
    --clock
    UI.mClockScale = UI.mImageScale * 0.5
    UI.mClockPropWidth = UI.mClockDeck.rects[UI.mClockIndex].origWidth * UI.mClockScale
    UI.mClockPropHeight = UI.mClockDeck.rects[UI.mClockIndex].origHeight * UI.mClockScale 
    UI.mClockBarWidth = UI.mClockPropWidth * 7.5
    UI.mClockProp:setScl( UI.mClockScale, -UI.mClockScale )   
    UI:PositionClock()
end

function UI:InitPreIntro(fHideTitleDuration, fTotalDuration)
    UI.mPreIntroTimer = 0
    UI.mPreIntroHideTitle = fHideTitleDuration
    UI.mPreIntroFadeIn = fTotalDuration

    UI.mNewAbilityState = Constants.UINewAbilityStateNone
    UI.mNewAbilityTimer = 0.0
    
    --make sure props are turned off
    local i = 1
    local x = 1
    local y = 1
    for i = 1, UI.mFinalScoreTextCount do
        UI.mFinalScoreTextProps[i]:setVisible(false)    
    end    
    for i = 1, UI.mScoreDigits do
        UI.mScoreProps[i]:setVisible(false)
    end          
    for i = 1, #UI.mTitleTextProps do
        UI.mTitleTextProps[i]:setVisible(false)   
    end  
    for y = 1, #UI.mBigTitleProps do
        local fNumColumnsOfBigTitles = #UI.mBigTitleProps[y]        
        for x = 1, fNumColumnsOfBigTitles do      
            UI.mBigTitleProps[y][x]:setVisible(false)
        end
    end 
    
    UI.mClockProp:setVisible(false)
    UI.mClockAlpha = 0.0
    UI.mClockAlphaDesired = 0.0
    
    UI:SetMenuColor(0.0)
end

function UI:InitIntro()
    UI.mIntroTimer = 0.0      
    AudioManager:PlaySound( 'Radius/SFX/DiscordLogoOut' )
end

function UI:InitMenu()
    UI.mMenuTimer = 0.0
    UI.mMenuSelectionSound = AudioManager:PlaySound( 'Radius/SFX/HoldandPlay' )
end

function UI:ExitMenu()
    UI:SetMenuColor(0.0)
    AudioManager:StopSound( UI.mMenuSelectionSound, true )
end

function UI:InitInGame()
    local i = 1
    for i = 1, #UI.mTitleTextProps do
        UI.mTitleTextProps[i]:setVisible(false)   
    end       
   
    UI.mClockProp:setVisible(true)
    UI.mClockAlpha = 0.0
    UI.mClockAlphaDesired = 0.0
    
    UI.mCompleteTimer = 0.0
    
    UI:SetMenuColor(0.0)
end
    
function UI:InitPost(iScore)
    UI.mPostState = 0
    UI.mPostTimer = UI.mPostInterval    
    UI.mScoreYOffsetPercent = 0.0
    UI.mScoreYOffset = 0.0
    
    --clear score
    for i = 1, UI.mScoreDigits do
        UI.mScoreProps[i]:setIndex( RadFont.mFontDeck.names["0"] )
    end 

    --set score
    local iCurrentIndex = UI.mScoreDigits
    local iTestScore = iScore
    while iTestScore >= 1 do
        local iDeckIndex = math.floor(iTestScore % 10)
        local iPropIndex = RadFont.mFontDeck.names[ tostring(iDeckIndex) ]
        UI.mScoreProps[iCurrentIndex]:setIndex( iPropIndex )
        
        iCurrentIndex = iCurrentIndex - 1
        iTestScore = iTestScore / 10
    end    
    
    UI.mClockProp:setVisible(false)
    UI.mClockAlpha = 0.0
    UI.mClockAlphaDesired = 0.0
        
    UI:PositionFinalScoreText()
end
  
function UI:InitLeaderboardEntry()
    local i = 1
    local x = 1
    local y = 1
    for i = 1, UI.mFinalScoreTextCount do
        UI.mFinalScoreTextProps[i]:setVisible(false)    
    end 
    
    for y = 1, #UI.mBigTitleProps do
        local fNumColumnsOfBigTitles = #UI.mBigTitleProps[y]        
        for x = 1, fNumColumnsOfBigTitles do      
            UI.mBigTitleProps[y][x]:setVisible(false)
        end
    end        

    UI.mScoreYOffset = 0.0
    UI.mScoreYOffsetPercent = 1.5
    UI.mScoreYOffsetDesired = UI.mScoreYOffsetPercent * UI.mLetterWidth
end
  
function UI:SetClockAlphaDesired( fAlpha )
    UI.mClockAlphaDesired = fAlpha
end
   
function UI:ShowNewAbility( iType, tTextManager )
    tTextManager.ClearAll()
    
    UI.mNewAbilityState = iType
    UI.mNewAbilityTimer = UI.mNewAbilityDuration
     
    local fLetterSize = UI:SetAttributesForText( tTextManager, Constants.Color2, 0.5 )
    local fLineX = fLetterSize * -2.0
    local fLineY = -fLetterSize
    local tBeamSpin = "spin"
    tTextManager:AddLine( tBeamSpin, fLineX, fLineY, UI.mNewAbilityDuration )      

    fLineX = fLetterSize * -3.0
    fLineY = fLineY + fLetterSize
    local tUnlock = "ready!"
    tTextManager:AddLine( tUnlock, fLineX, fLineY, UI.mNewAbilityDuration ) 
end

function UI:IsInAbilityTutorial()
    return UI.mNewAbilityState ~= Constants.UINewAbilityStateNone
end

function UI:IsAbleToSkipPreIntro()
    return UI.mPreIntroTimer >= UI.mPreIntroFadeIn
end

function UI:UpdatePreIntro( tGameInputs )
    UI.mPreIntroTimer = UI.mPreIntroTimer + Constants.FrameStep
        
    local fTint = 0.0
    if UI.mPreIntroTimer < UI.mPreIntroHideTitle then
        fTint = 0.0
    elseif UI.mPreIntroTimer < UI.mPreIntroFadeIn then   
        local fTestTimer = UI.mPreIntroTimer - UI.mPreIntroHideTitle
        local fCompleteRatio = fTestTimer / (UI.mPreIntroFadeIn - UI.mPreIntroHideTitle)
        
        fTint = fCompleteRatio
        UI:PositionTitleText(1.0-fCompleteRatio, true)        
    else
        fTint = 1.0
        UI:PositionTitleText(0.0, true)
        
        --if we're totally faded in, check for input
        if UI:CheckForInput(tGameInputs) then
            Constants.DesiredGameState = Constants.GameStateIntro
        end
    end
    
    local i = 1
    for i = 1, #UI.mTitleTextProps do           
        UI.mTitleTextProps[i]:setColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], fTint )
    end     
end
    
function UI:UpdateIntro( tChargeManager, tTextManager )
    UI.mIntroTimer = UI.mIntroTimer + Constants.FrameStep
    
    local i = 1
    if UI.mIntroTimer < UI.mIntroExplodeDuration then
        local fExplodeRatio = UI.mIntroTimer / UI.mIntroExplodeDuration
        for i = 1, #UI.mTitleTextProps do           
            UI.mTitleTextProps[i]:setColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0-fExplodeRatio )
        end                
        UI:PositionTitleText( fExplodeRatio, true )
    else
        for i = 1, #UI.mTitleTextProps do           
            UI.mTitleTextProps[i]:setVisible( false )
        end        
    end
end
    
function UI:UpdateMenu( fPercentSelected )
    UI.mWhiteOverlayAlpha = UI.mWhiteOverlayAlpha - (UI.mWhiteOverlayInc * Constants.FrameStep)
    UI.mUseWhiteOverlay = (UI.mWhiteOverlayAlpha > 0.0)
    
    UI.mMenuTimer = math.min( UI.mMenuTimer + Constants.FrameStep, 1.0 )
    UI:SetMenuColor( UI.mMenuTimer * fPercentSelected )
    
    if UI.mMenuSelectionSound ~= nil then
        UI.mMenuSelectionSound:setParameter( 'holdandplay', 1.0 - fPercentSelected )
    end
end
    
function UI:UpdateInGame( tGameInputs, tChargeManager, tTextManager )
    --if we're showing a new ability, update our timer and state
    if UI.mNewAbilityState ~= Constants.UINewAbilityStateNone then
        UI.mNewAbilityTimer = UI.mNewAbilityTimer - Constants.FrameStep
        
        if UI.mNewAbilityTimer <= 0.0 then
            tTextManager.ClearAll()
            UI.mNewAbilityState = Constants.UINewAbilityStateNone
        end
    end
    
    --if we're showing the throw tutorial and the player figured it out, we're done with the tutorial
    if UI.mShowThrowTutorial == true and tChargeManager:AreAnyChargesThrown() then
        UI.mShowThrowTutorial = false  
    end
    
    UI.mWhiteOverlayAlpha = UI.mWhiteOverlayAlpha - (UI.mWhiteOverlayInc * Constants.FrameStep)
    UI.mUseWhiteOverlay = (UI.mWhiteOverlayAlpha > 0.0)    
    
    UI:UpdateClock()
end

function UI:UpdateComplete( tPost, tPostLevel )
    UI.mCompleteTimer = UI.mCompleteTimer + Constants.FrameStep
        
    --set title    
    local fFadeAmount = UI.mCompleteTimer - UI.mCompleteTitleFadeInMark
    if fFadeAmount >= 0.0 then
        UI:PositionTitleText(0.0, true)

        local fAlpha = math.min( fFadeAmount / UI.mCompleteTitleFadeDuration, 1.0 )
        local i = 1
        for i = 1, #UI.mTitleTextProps do           
            UI.mTitleTextProps[i]:setColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], fAlpha )
        end       
    end
    
    --set pulse
    local fPulseAmount = math.min( UI.mCompleteTimer / (UI.mCompleteTitleFadeInMark+UI.mCompleteTitleFadeDuration), 1.0 )
    ColorManager:SetLineModeComplete(fPulseAmount)
    
    --exit after duration
    if UI.mCompleteTimer > UI.mCompleteDuration then
        Constants.DesiredGameState = Constants.GameStatePost
        UI:PositionTitleText(0.0, false)
        tPost:SetUpdateCallback( tPostLevel.UpdatePostBonus )
    end
    
    return 1.0 - math.min( UI.mCompleteTimer/UI.mIntensityDuration, 1.0 ) 
end
    
function UI:UpdatePost( tGameInputs )
    UI.mPostTimer = UI.mPostTimer - Constants.FrameStep
    if UI.mPostTimer <= 0.0 then
        UI.mPostState = UI.mPostState + 1
        UI.mPostTimer = UI.mPostInterval    
        
        local i = 1
        if UI.mPostState == 1 then
            AudioManager:PlaySound( 'Radius/SFX/GameOver' )
            local iHalfFinalScoreTextCount = UI.mFinalScoreTextCount/2
            for i = 1, UI.mFinalScoreTextCount/2 do
                UI.mFinalScoreTextProps[i]:setVisible(true)    
                UI.mFinalScoreTextProps[i]:setColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
            end          
        elseif UI.mPostState == 2 then
            AudioManager:PlaySound( 'Radius/SFX/GameOver' )      
            local iHalfFinalScoreTextCount = UI.mFinalScoreTextCount/2
            for i = iHalfFinalScoreTextCount, UI.mFinalScoreTextCount do
                UI.mFinalScoreTextProps[i]:setVisible(true)    
                UI.mFinalScoreTextProps[i]:setColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
            end    
        elseif UI.mPostState == 3 then
            AudioManager:PlaySound( 'Radius/SFX/GameOverBig' )      
            for i = 1, UI.mScoreDigits do
                UI.mScoreProps[i]:setVisible(true)
                UI.mScoreProps[i]:setColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
            end        
        elseif UI.mPostState == 4 then
            local x = 1
            local y = 1
            for y = 1, #UI.mBigTitleProps do
                local fNumColumnsOfBigTitles = #UI.mBigTitleProps[y]        
                for x = 1, fNumColumnsOfBigTitles do      
                    UI.mBigTitleProps[y][x]:setVisible(true)
                    UI.mBigTitleProps[y][x]:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )
                end
            end        
            AudioManager:PlayGameEndAudio()
        end
    end
        
    UI:UpdateBigTitleTextPosition()
    
    if UI.mPostState > UI.mPostStateAllowBackToIntro then
        if UI:CheckForInput(tGameInputs) then
            --leaderboards are only for full mix mode!
            if Constants.FullMixMode then
                --if we were good enough to get on the leaderboard, go to the entry screen
                if Constants.LeaderboardIndex ~= -1 then
                    Constants.DesiredGameState = Constants.GameStateLeaderboardEntry
                else
                    --if we're on iOS, we always want to enter our initials (if we're connected to gamecenter)
                    if MOAIGameCenterIOS then
                        local playerAlias = MOAIGameCenterIOS.getPlayerAlias()
                        if playerAlias == nil or playerAlias == "" then
                            Constants.DesiredGameState = Constants.GameStateLeaderboardShowcase
                        else
                            Constants.DesiredGameState = Constants.GameStateLeaderboardEntry
                        end
                    else
                        Constants.DesiredGameState = Constants.GameStateLeaderboardShowcase
                    end
                end
            else
                --if we're not in full mix mode, just go back to the menu
                Constants.DesiredGameState = Constants.GameStatePreIntro
            end
        end
    end
end

function UI:EscPressedDuringPost()
    if UI.mPostState > UI.mPostStateAllowBackToIntro then
        if Constants.LeaderboardIndex ~= -1 then
            Constants.DesiredGameState = Constants.GameStateLeaderboardEntry
        else
            Constants.DesiredGameState = Constants.GameStateLeaderboardShowcase
        end
    end
end

function UI:UpdateClock()
    --clock color
    UI.mClockPercent = GameRing.mTimerRatio
    
    local t = 1.0 
    if UI.mClockPercent > 0.0 then
        t = UI.mClockPercent
    end
    local a = ((1.0-t) * 0.5) + 0.5
    
    UI.mClockColor[1] = DFMath.lerp( Constants.BadEffectColor[1], Constants.Color1[1], t )
    UI.mClockColor[2] = DFMath.lerp( Constants.BadEffectColor[2], Constants.Color1[2], t )
    UI.mClockColor[3] = DFMath.lerp( Constants.BadEffectColor[3], Constants.Color1[3], t )
    
    --update clock alpha
    local fAlphaStep = UI.mClockAlphaScalar * Constants.FrameStep
    local fAlphaDiff = math.abs(UI.mClockAlphaDesired - UI.mClockAlpha)
    if fAlphaDiff < Constants.FrameStep then
        UI.mClockAlpha = UI.mClockAlphaDesired
    elseif UI.mClockAlpha < UI.mClockAlphaDesired then
        UI.mClockAlpha = UI.mClockAlpha + Constants.FrameStep
    else
        UI.mClockAlpha = UI.mClockAlpha - Constants.FrameStep
    end 
    UI.mClockColor[4] = UI.mClockAlpha * a
        
    UI.mClockProp:setColor( UI.mClockColor[1], UI.mClockColor[2], UI.mClockColor[3], UI.mClockColor[4] )
end

function UI:UpdateLeaderboardEntry()
    local fMovementStep = UI.mScoreYMovementSpeed * Constants.FrameStep
    if UI.mScoreYOffset < UI.mScoreYOffsetDesired - fMovementStep then
        UI.mScoreYOffset = UI.mScoreYOffset + fMovementStep
    elseif UI.mScoreYOffset > UI.mScoreYOffsetDesired + fMovementStep then
        UI.mScoreYOffset = UI.mScoreYOffset - fMovementStep
    else
        UI.mScoreYOffset = UI.mScoreYOffsetDesired
    end
    
    UI:PositionFinalScoreText() 
end  
  
function UI:UpdateBigTitleTextPosition()    
    local x = 1
    local y = 1
    local fMovementStep = Constants.FrameStep * UI.mBigTitleMovementScale
    local fBigTitleTextPositionX = -Constants.ValidDrawHalfWidth
    local nNumRowsOfBigTitles = #UI.mBigTitleProps
    for y = 1, nNumRowsOfBigTitles do
        local fNumColumnsOfBigTitles = #UI.mBigTitleProps[y]
        local bStepDirection = y == 2 or y == 4
        
        for x = 1, fNumColumnsOfBigTitles do      
            local fX, fY, fZ = UI.mBigTitleProps[y][x]:getLoc()
            
            if bStepDirection then
                fX = fX + fMovementStep
                if fX > Constants.ValidDrawHalfWidth then
                    fX = fX - (fNumColumnsOfBigTitles * UI.mBigTitlePropWidth)                
                end
            else
                fX = fX - fMovementStep 
                if fX < (-Constants.ValidDrawHalfWidth - UI.mBigTitlePropWidth) then
                    fX = fX + (fNumColumnsOfBigTitles * UI.mBigTitlePropWidth)
                end                
            end
            
            UI.mBigTitleProps[y][x]:setLoc( fX, fY, fZ ) 
        end
    end  
end
  
function UI:PositionTitleText(fOffsetPercent, bVisible)
    local fTitleLetterWidth = UI.mLetterWidth * UI.mTitleImageAdditionalScale
    local fTitleTextPositionX = (((UI.mTitleTextCount-2) * fTitleLetterWidth) * -0.5) + (Constants.ValidDrawHeight * .25)
    local fTitleTextPositionY = Constants.ValidDrawHeight * .25
    
    local fXOffsetInterval = UI.mPreIntroOffsetScale * fOffsetPercent
    local i = 1
    local iHalfTextCount = (UI.mTitleTextCount / 2)
    for i = 1, #UI.mTitleTextProps do
        local fXOffset = 0.0
        if i <= iHalfTextCount then
            fXOffset = (iHalfTextCount-i) * -fXOffsetInterval
        else
            fXOffset = (i-iHalfTextCount) * fXOffsetInterval
        end
        
        UI.mTitleTextProps[i]:setLoc( fTitleTextPositionX + ((i-1) * fTitleLetterWidth) + fXOffset, fTitleTextPositionY, UI.mPropZ )
        if bVisible ~= nil then
            UI.mTitleTextProps[i]:setVisible(bVisible)
        end
    end
end

function UI:PositionFinalScoreText()   
    local fScorePositionX = (UI.mScoreDigits * UI.mLetterWidth) * -0.5
    for i = 1, UI.mScoreDigits do
        UI.mScoreProps[i]:setLoc( fScorePositionX + ((i-1) * UI.mLetterWidth), UI.mLetterWidth + UI.mScoreYOffset, UI.mPropZ )
    end
    
    --"final score" has 10 symbols.. we fake the space
    local fScoreTextPositionX = ((UI.mFinalScoreTextCount+1) * UI.mLetterWidth) * -0.5
    for i = 1, UI.mFinalScoreTextCount do
        local fAdditionalX = 0.0
        if i > 5 then
            fAdditionalX = UI.mLetterWidth
        end
        UI.mFinalScoreTextProps[i]:setLoc( fScoreTextPositionX + ((i-1) * UI.mLetterWidth) + fAdditionalX, 0.0, UI.mPropZ )
    end
end

function UI:PositionBigTitleText()
    local iBigTitleSheetIndex = UI.mBigTitleDeck.names[ "title" ]
    UI.mBigTitleScale = UI.mImageScale * 4.0
    UI.mBigTitlePropWidth = UI.mBigTitleDeck.rects[iBigTitleSheetIndex].origWidth * UI.mBigTitleScale
    UI.mBigTitlePropHeight = UI.mBigTitleDeck.rects[iBigTitleSheetIndex].origHeight * UI.mBigTitleScale
    
    local x = 1
    local y = 1
    local fBigTitleTextPositionX = -Constants.ValidDrawHalfWidth
    for y = 1, UI.mBigTitlePropCountY do            
        for x = 1, UI.mBigTitlePropCountX do
            UI.mBigTitleProps[y][x]:setScl( UI.mBigTitleScale, -UI.mBigTitleScale )
            
            local fYPos = 0.0
            if y <= (UI.mBigTitlePropCountY / 2) then
                fYPos = -UI.mLetterWidth - (UI.mBigTitlePropHeight * (y-1))
            else
                local fAdditionalY = (UI.mBigTitlePropHeight * (y - (UI.mBigTitlePropCountY/2) - 1))
                fYPos = (UI.mLetterWidth * 3.0) + fAdditionalY
            end
            
            UI.mBigTitleProps[y][x]:setLoc( fBigTitleTextPositionX + ((x-1)*UI.mBigTitlePropWidth), fYPos, UI.mPropZ )          
        end
    end
end

function UI:PositionClock()
    UI.mClockProp:setLoc( UI.mClockPropWidth * -0.5, Constants.ValidDrawHalfHeight - (UI.mClockPropHeight*3.0), UI.mPropZ )
end

function UI:CreateMenuText( tRenderer, sText )
    UI:DestroyMenuText( tRenderer )
    
    local textWalker = string.gmatch(sText, ".")
    
    UI.mMenuTextCount = #sText
    UI.mMenuImageScale = UI.mTitleImageScale * UI.mMenuAdditionalScale
    UI.mMenuTextProps = {}
    for i = 1, UI.mMenuTextCount do
        UI.mMenuTextProps[i] = MOAIProp.new()
        UI.mMenuTextProps[i]:setDeck( RadFont.mFontDeck )        
        UI.mMenuTextProps[i]:setScl( UI.mMenuImageScale, -UI.mMenuImageScale )
        UI.mMenuTextProps[i]:setIndex( RadFont.mFontDeck.names[ textWalker() ] )
        UI.mMenuTextProps[i]:setVisible(false)
        UI.mMenuTextProps[i]:setBlendMode( MOAIProp2D.GL_SRC_ALPHA, MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA )
        
        tRenderer.mUILayer:insertProp( UI.mMenuTextProps[i] )        
    end
    UI:PositionMenuText()    
end

function UI:DestroyMenuText( tRenderer )
    if UI.mMenuTextCount > 0 then
        local i = 1
        for i = 1, UI.mMenuTextCount do        
            tRenderer.mUILayer:removeProp( UI.mMenuTextProps[i] )        
        end
        
        UI.mMenuTextProps = {}
        UI.mMenuTextCount = 0
    end
end

function UI:PositionMenuText()    
    local fMenuLetterWidth = UI.mLetterWidth * UI.mTitleImageAdditionalScale * UI.mMenuAdditionalScale
    local fMenuTextPositionX = (Constants.ValidDrawHeight * .3) - fMenuLetterWidth
    local fMenuTextPositionY = (Constants.ValidDrawHeight * .2) + fMenuLetterWidth
    for i = 1, UI.mMenuTextCount do
        UI.mMenuTextProps[i]:setLoc( fMenuTextPositionX, fMenuTextPositionY, UI.mPropZ )
        fMenuTextPositionX = fMenuTextPositionX + fMenuLetterWidth
    end
end

function UI:SetMenuColor(fAlpha)
    local bVisible = fAlpha > 0.0
    local i = 1
    for i = 1, UI.mMenuTextCount do
        UI.mMenuTextProps[i]:setVisible( bVisible )
        UI.mMenuTextProps[i]:setColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], fAlpha )
    end
end

function UI:SetTitleForIntro(fNoInputDuration)
    if UI.mIntroTimer > UI.mIntroExplodeDuration then
        local fAlpha = 0.0
        local fFadeStart = 6.0
        local fFadeDuration = 3.0
        if fNoInputDuration > fFadeStart then
            fAlpha = math.min( 1.0, (fNoInputDuration - fFadeStart) / fFadeDuration )
        end
        
        UI:PositionTitleText(0.0, true)
        
        local i = 1
        for i = 1, #UI.mTitleTextProps do
            UI.mTitleTextProps[i]:setColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], fAlpha )
        end        
    end
end

function UI:SetTitleVisible(bVisible)
    local i = 1
    for i = 1, #UI.mTitleTextProps do
        UI.mTitleTextProps[i]:setVisible(bVisible)
    end
end

function UI:SetAttributesForText( tTextManager, tColor, fFadeIn )
    local fDesiredLetterSize = Constants.ValidDrawHeight / 10.0
    local fLetterScale = fDesiredLetterSize / 128.0
    local fLetterSize = 128.0 * fLetterScale 
    tTextManager:SetTextAttributes( fLetterSize, fLetterScale, tColor )
    tTextManager:SetFadeAttributes( fFadeIn, 0.5 )
    tTextManager:SetZoomAttributes( 500.0, 10.0, 0.25 )
    return fLetterSize
end

function UI:CheckForInput( tGameInputs )
    return tGameInputs[1].IsTouched() or tGameInputs[2].IsTouched()
end
  
function UI:FlashWhite(fFadeMultiplier)
    UI.mWhiteOverlayAlpha = 1.0
    UI.mWhiteOverlayInc = fFadeMultiplier        
end

function UI:Draw()
    if UI.mClockAlpha > 0.0 then
        MOAIGfxDevice.setPenColor( UI.mClockColor[1], UI.mClockColor[2], UI.mClockColor[3], UI.mClockColor[4] )    
        local fBarWidth = UI.mClockPercent * UI.mClockBarWidth
        local fClockLeft = UI.mClockPropWidth * -0.5
        local fClockRight = UI.mClockPropWidth * 0.5
        local fBarHeight = DFMath.lerp( UI.mClockPropHeight*0.5, UI.mClockPropHeight*0.125, UI.mClockPercent )
        local fClockBottom = Constants.ValidDrawHalfHeight - (UI.mClockPropHeight*3.0)
        local fClockMidY = fClockBottom - (UI.mClockPropWidth * 0.5)
        
        MOAIDraw.fillRect( fClockLeft - fBarWidth, fClockMidY - fBarHeight, fClockLeft, fClockMidY + fBarHeight )
        MOAIDraw.fillRect( fClockRight, fClockMidY - fBarHeight, fClockRight + fBarWidth, fClockMidY + fBarHeight )
    end
        
    if UI.mUseWhiteOverlay then
        MOAIGfxDevice.setPenColor( 1.0, 1.0, 1.0, UI.mWhiteOverlayAlpha )
        MOAIDraw.fillRect( -Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight )
    end
end
  
return UI