local Constants = require('Constants')
local DFMath = require('DFCommon.Math')
local Wind = {}

function Wind:Init( tRenderer )
    Wind.mWindDeck = WindDeck.new()

    Wind:VisualQualityChange()
    Wind.mWindDeck:SetPointWidth( 4.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 200.0, 500.0, 0.5 )
    Wind.mWindDeck:SetFrequency( 500.0 )
    Wind.mWindProp = MOAIProp2D.new()
    Wind.mWindProp:setDeck( Wind.mWindDeck )
        
    tRenderer.mGameLayer:insertProp( Wind.mWindProp )
    
    Wind:Enable( false )
end
  
function Wind:UpdateSize()
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, -Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], tMin[3], tMax[1], tMax[2], tMax[3] )    
end

function Wind:VisualQualityChange()
    if Constants.VisualQuality == Constants.VisualQualityLevelUltra then
        Wind.mWindDeck:SetNumPoints(1000);
    else
        Wind.mWindDeck:SetNumPoints(100);
    end
    
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, -Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], tMin[3], tMax[1], tMax[2], tMax[3] )
    Wind.mWindDeck:SetColor( Constants.BGColor[1]-0.05, Constants.BGColor[2]-0.05, Constants.BGColor[3]-0.05, 1.0 )

    Wind:UpdateSize()
end

function Wind:Enable(bEnable)
    Wind.mEnabled = bEnable
    Wind.mWindProp:setVisible(bEnable)
end

function Wind:SetColor(r, g, b, a)
    Wind.mWindDeck:SetColor( r, g, b, a )
end

function Wind:SetLayer( tLayer )
    tLayer:insertProp( Wind.mWindProp )
end

function Wind:Update()
    if Wind.mEnabled == true then
        Wind.mWindDeck:Update( Constants.FrameStep )
    end
end
  
return Wind