
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local ChargeField = require('ChargeField')
local Wind = require('Wind')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local Post = require('postfx.Post')
local ParticleSystem = require("ParticleSystem")
local PostDebug = {}



-----------------------------------------------------------------------------
-- Show Debug Buffers
-----------------------------------------------------------------------------

function PostDebug:ShowGame()

    print("Show Background")
    Post:ClearFilters()
    Post.rScreen:setInBuffer(Renderer.mGameBuffer)
end

function PostDebug:ShowUI()

    print("Show Background")
    Post:ClearFilters()
    Post.rScreen:setInBuffer(Renderer.mUIBuffer)
end

function PostDebug:ShowBackground()

    print("Show Background")
    Post:ClearFilters()
    Post.rScreen:setInBuffer(Renderer.mBackgroundBuffer)
end

function PostDebug:BasicComp()
    print("Show Basic Comp")
    Post:ClearFilters()

    local finalColorBuffer = Post:GetUnusedBuffer()

    Post:GenerateBasicCompFilter(finalColorBuffer)
    Post.rScreen:setInBuffer(finalColorBuffer)
    
end

function PostDebug:ShowFXAA()
    print("Show FXAA")
    
    Post:ClearFilters()
    
    local finalColorBuffer = Post:GetUnusedBuffer()
    Post:GenerateBasicCompFilter(finalColorBuffer)
    
    local aaBuffer = Post:GetUnusedBuffer()
    Post:FXAAFilter(finalColorBuffer, aaBuffer)
    
    Post.rScreen:setInBuffer(finalColorBuffer)
    
end

function PostDebug:ShowBlur()
    print("Show Blur")
    Post:ClearFilters()

    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    Post:GenerateBasicCompFilter(sceneBuffer)
    Post:SuperBlurFilter(sceneBuffer, finalColorBuffer, 0.002, kBLEND_ADD)    
    Post:BlitFilter(Renderer.mUIBuffer, finalColorBuffer, kBLEND_ALPHA)
    Post:BlitFilter(Renderer.mGameBuffer, finalColorBuffer, kBLEND_ALPHA)
    Post.rScreen:setInBuffer(finalColorBuffer)
end


function PostDebug:ShowEnergyRipple()
    print("Show Energy Ripple")
    Post:ClearFilters()

    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    Post:GenerateBasicCompFilter(sceneBuffer)
    Post:BlitFilter(sceneBuffer, finalColorBuffer, kBLEND_ADD, {1,1,1,1})    
    Post:EnergyRippleFilter(sceneBuffer, finalColorBuffer, 1.0, kBLEND_ADD, {1,1,1,1})    
    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostDebug:ShowTVNoise()
    print("Show TV Noise")
    Post:ClearFilters()

    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    Post:GenerateBasicCompFilter(sceneBuffer)
    Post:BlitFilter(sceneBuffer, finalColorBuffer, kBLEND_ADD, {1,1,1,1})    
    Post:TVNoiseFilter(sceneBuffer, finalColorBuffer, 50.0, kBLEND_ADD, {1,1,1,1})    
    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostDebug:ShowDirectionalBlur()
    print("Show Directional Blur")
    Post:ClearFilters()

    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    Post:GenerateBasicCompFilter(sceneBuffer)
    Post:DirectionalBlurFilter(sceneBuffer, finalColorBuffer, {.01, .01}, kBLEND_ADD, {1,1,1,1})
    Post:DirectionalBlurFilter(sceneBuffer, finalColorBuffer, {.01, -.01}, kBLEND_ADD, {1,1,1,1})
    Post:BlitFilter(sceneBuffer, finalColorBuffer, kBLEND_ADD)
    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostDebug:ShowCrossBlur()
    print("Show Cross Blur")
    Post:ClearFilters()
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager:SetColor()
    
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local particleBuffer = Post:GetUnusedBuffer()
    particleBuffer:setClearColor()
    local rPfx = ParticleSystem.new(nil, "Effects/Particles/Backgrounds/SwirlyStarsB.particles")
    rPfx:init()
    rPfx:start()
	local pfxProp = rPfx:getProp()
    pfxProp:setLoc(0, 0, 0)
    
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(particleBuffer)
	particleLayer:insertProp(pfxProp)

    --dim particle accumulation buffer
    Post:BlitFilter(Post.rBlackTexture, particleBuffer, kBLEND_ALPHA, {0,0,0,.07})
    
    Post:GenerateBasicCompFilter(sceneBuffer)
    Post:BlurFilter(sceneBuffer, scratchBuffer, .003, kBLEND_ADD, {2,2,2,1})    
    Post:CrossBlurFilter(scratchBuffer, scratchQuarterBuffer, .008, kBLEND_ADD, {.5,.5,.5,1})    
    Post:BlurFilter(scratchQuarterBuffer, finalColorBuffer, .003, kBLEND_ADD, {1,1,1,1})    
    Post:BlitFilter(Renderer.mUIBuffer, finalColorBuffer, kBLEND_ALPHA)
    Post:BlitFilter(Renderer.mGameBuffer, finalColorBuffer, kBLEND_ALPHA)
    Post:BlitFilter(scratchBuffer, finalColorBuffer, kBLEND_ADD, {2,2,2,1})  
        
    Post:BlitFilter(particleBuffer, finalColorBuffer, kBLEND_ADD)
    
    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostDebug:ShowEdges()
    print("Show Edges")
    Post:ClearFilters()

    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    Post:GenerateBasicCompFilter(sceneBuffer)
    Post:EdgeFilter(sceneBuffer, finalColorBuffer, 1)
    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostDebug:ShowAccum()
    print("Show Accum")
    Post:ClearFilters()

    local finalColorBuffer = Post:GetUnusedBuffer()   
    local accumBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    
    accumBuffer:setClearColor()
    
    Post:AccumFilter(Renderer.mGameBuffer, accumBuffer, scratchBuffer, {.95,.94,.9,1})
    Post:BlitFilter(scratchBuffer, accumBuffer)
    Post:BlitFilter(scratchBuffer, finalColorBuffer)
    Post.rScreen:setInBuffer(finalColorBuffer)
end
--[[
function PostDebug:ShowAccumFlow()
    print("Show Accum Flow")
    Post:ClearFilters()
    
        local finalColorBuffer = Post:GetUnusedBuffer()   
    local accumBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    
    accumBuffer:setClearColor()
    
    Post:AccumFilter(Renderer.mGameBuffer, accumBuffer, scratchBuffer, {.95,.94,.9,1})
    Post:BlitFilter(scratchBuffer, accumBuffer)
    Post:BlitFilter(scratchBuffer, finalColorBuffer)
    Post.rScreen:setInBuffer(finalColorBuffer)
end
]]--
function PostDebug:ShowBlurredAccum()
    print("Show Accum with Blur")
    Post:ClearFilters()

    ColorManager.BGColor            = {0,0,0}
    ColorManager:SetColor()
    
    local finalColorBuffer = Post:GetUnusedBuffer()   
    local accumBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    Post:CombineFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, kBLEND_ALPHA, {1,1,1,1})
    Post:BlitFilter(Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ADD)
    Post:AccumFilter(sceneBuffer, accumBuffer, scratchBuffer, {.95,.94,.9,1})
    Post:BlurFilter(scratchBuffer, accumBuffer, .005, kBLEND_ALPHA, {.9,.7,.7,1})
    Post:BlitFilter(scratchBuffer, finalColorBuffer)
    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostDebug:ShowAccumWithEdges(bBlurFirst)
    print("Show Accum with Edges")
    Post:ClearFilters()

    local finalColorBuffer = Post:GetUnusedBuffer()   
    local accumBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    Post:CombineFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, kBLEND_Alpha, {1,1,1,1})
    Post:AccumFilter(sceneBuffer, accumBuffer, scratchBuffer, {.99,.94,.9,1})

    if bBlurFirst then
        Post:BlurFilter(scratchBuffer, accumBuffer, .006, kBLEND_ALPHA, {1,1,1,1})
        Post:EdgeFilter(scratchBuffer, finalColorBuffer, 1)
    else
        Post:BlitFilter(scratchBuffer, accumBuffer)
        Post:BlurFilter(accumBuffer, scratchBuffer, .006, kBLEND_ALPHA, {1,1,1,1})
        Post:EdgeFilter(scratchBuffer, finalColorBuffer, 1)
        Post:EdgeFilter(sceneBuffer, finalColorBuffer, 1, kBLEND_ADD)
    end
    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostDebug:ShowWideAccumBlur()
    print("Show wide accum blur")
    Post:ClearFilters()

    local finalColorBuffer = Post:GetUnusedBuffer()   
    local accumBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    Post:CombineFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, kBLEND_ALPHA, {1,1,1,1})
    Post:AccumFilter(sceneBuffer, accumBuffer, scratchBuffer, {.99,.94,.9,1})
    Post:BlurFilter(scratchBuffer, accumBuffer, .006, kBLEND_ALPHA, {1,1,1,1})
    Post:CombineFilter(sceneBuffer, accumBuffer, finalColorBuffer, kBLEND_ADD, {1,1,1,1})
    Post.rScreen:setInBuffer(finalColorBuffer)
end


function PostDebug:ShowAccumFlow()
    print("Show Accum Flow")
    Post:ClearFilters()

    ColorManager.BGColor            = {0,0,0}
    ColorManager:SetColor()
    
    local finalColorBuffer = Post:GetUnusedBuffer()   
    local accumBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    Post:CombineFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, kBLEND_ALPHA, {1,1,1,1})
    Post:BlitFilter(Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ADD)
    Post:AccumFlowFilter(sceneBuffer, accumBuffer, scratchBuffer, Post.rFlowTexture, {.99,.94,.94,1}, {1,1}, {0,.0}, -.02)
    Post:BlurFilter(scratchBuffer, accumBuffer, .002, kBLEND_ALPHA, {1,1,1,1})
    Post:BlitFilter(scratchBuffer, finalColorBuffer)
    Post.rScreen:setInBuffer(finalColorBuffer)
end


function PostDebug:ShowAccumFlowWithEdges(bBlurFirst)
    print("Show Accum Flow With Edges")
    Post:ClearFilters()

    local finalColorBuffer = Post:GetUnusedBuffer()   
    local accumBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchBuffer2 = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    Post:CombineFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, kBLEND_Alpha, {1,1,1,1})
    
    if bBlurFirst then
        Post:AccumFlowFilter(sceneBuffer, accumBuffer, scratchBuffer, Post.rFlowTexture, {.99,.94,.94,1}, {.98,.98}, {0,.0}, -.05)
        Post:BlurFilter(scratchBuffer, accumBuffer, .002, kBLEND_ALPHA, {1,1,1,1})
        Post:EdgeFilter(scratchBuffer, finalColorBuffer, 1, kBLEND_ADD, {10,10,10,1})
        Post:EdgeFilter(sceneBuffer, finalColorBuffer, 1, kBLEND_ADD, {2,.8,.2,1})
    else
        Post:AccumFlowFilter(sceneBuffer, accumBuffer, scratchBuffer, Post.rFlowTexture, {.99,.94,.94,1}, {.99,1.0}, {0,.0}, -.05)
        Post:BlitFilter(scratchBuffer, accumBuffer)
        Post:BlurFilter(accumBuffer, scratchBuffer2, .00, kBLEND_ALPHA, {1,1,1,1})
        Post:EdgeFilter(scratchBuffer2, finalColorBuffer, 1, kBLEND_ADD, {1,1,1,1})
        Post:EdgeFilter(sceneBuffer, finalColorBuffer, 1, kBLEND_ADD, {2,.8,.2,1})
    end
    
    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostDebug:ShowBlurredRemap(scrollRate, gradToUse, bEdges)
    print("Show blurred remap")
    Post:ClearFilters()

    local finalColorBuffer = Post:GetUnusedBuffer()   
    local accumBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchBuffer2 = Post:GetUnusedBuffer()
    local remapBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    sceneBuffer:setClearColor(0,0,0,0)
    
    local rTexture
    if gradToUse == 0 then 
        rTexture = Post.rGradientTexture
    elseif gradToUse == 1 then
        rTexture = Post.rGradientTextureC
    elseif gradToUse == 2 then 
        rTexture = Post.rGradientTextureB
    end

    Post:CombineFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, kBLEND_ALPHA, {1,1,1,1})
    Post:AccumFilter(sceneBuffer, accumBuffer, scratchBuffer, {.97,.97,.97,1}, {.99,.99})
    Post:BlurFilter(scratchBuffer, accumBuffer, .003, kBLEND_ALPHA, {1,1,1,1})
    Post:RemapFilter(accumBuffer, remapBuffer, rTexture, scrollRate, kBLEND_ALPHA, {1,1,1,1})
    Post:CombineFilter(remapBuffer, sceneBuffer, scratchBuffer2, kBLEND_ADD, {1,1,1,1})
    if bEdges then 
        Post:EdgeFilter(scratchBuffer2, finalColorBuffer, 1, kBLEND_ADD, {1,1,1,1})
        Post.rScreen:setInBuffer(finalColorBuffer)
    else
        Post.rScreen:setInBuffer(scratchBuffer2)
    end
end

return PostDebug