local Constants = require('Constants')

--ultra quality post fx (PC quality)
local PostStateIntro = require('postfx.PostStateIntro')
local PostStateGameOver = require('postfx.PostStateGameOver')
local PostState_Level1 = require('postfx.PostState_Level1')
local PostState_Level2 = require('postfx.PostState_Level2')
local PostState_Level3 = require('postfx.PostState_Level3')
local PostState_Level4 = require('postfx.PostState_Level4')
local PostState_Level5 = require('postfx.PostState_Level5')
local PostState_Level6 = require('postfx.PostState_Level6')
local PostState_Level7 = require('postfx.PostState_Level7')
local PostState_Level8 = require('postfx.PostState_Level8')
local PostState_Level9 = require('postfx.PostState_Level9')
local PostState_Level10 = require('postfx.PostState_Level10')
local PostState_Leaderboards= require('postfx.PostState_Leaderboards')

-- high quality post fx
local PostStateHigh_Intro = require('postfx.PostStateIntro')
local PostStateHigh_GameOver = require('postfx.PostStateGameOver')
local PostStateHigh_Level1 = require('postfx.PostState_Level1')
local PostStateHigh_Level2 = require('postfx.PostState_Level2')
local PostStateHigh_Level3 = require('postfx.PostState_Level3')
local PostStateHigh_Level4 = require('postfx.PostState_Level4')
local PostStateHigh_Level5 = require('postfx.PostState_Level5')
local PostStateHigh_Level6 = require('postfx.PostState_Level6')
local PostStateHigh_Level7 = require('postfx.PostState_Level7')
local PostStateHigh_Level8 = require('postfx.PostState_Level8')
local PostStateHigh_Level9 = require('postfx.PostState_Level9')
local PostStateHigh_Level10 = require('postfx.PostState_Level10')
local PostStateHigh_Leaderboards= require('postfx.PostState_Leaderboards')

-- low quality post fx
local PostStateLow_Intro = require('postfx.PostStateLow_Intro')
local PostStateLow_GameOver = require('postfx.PostStateLow_GameOver')
local PostStateLow_Level1 = require('postfx.PostStateLow_Level1')
local PostStateLow_Level2 = require('postfx.PostStateLow_Level2')
local PostStateLow_Level3 = require('postfx.PostStateLow_Level3')
local PostStateLow_Level4 = require('postfx.PostStateLow_Level4')
local PostStateLow_Level5 = require('postfx.PostStateLow_Level5')
local PostStateLow_Level6 = require('postfx.PostStateLow_Level6')
local PostStateLow_Level7 = require('postfx.PostStateLow_Level7')
local PostStateLow_Level8 = require('postfx.PostStateLow_Level8')
local PostStateLow_Level9 = require('postfx.PostStateLow_Level9')
local PostStateLow_Level10 = require('postfx.PostStateLow_Level10')
local PostStateLow_Leaderboards= require('postfx.PostStateLow_Leaderboards')

local PostDebug = require('postfx.PostDebug')
local PostManager = {}

-- we numbered the visual styles before we built the levels and music, so they need to get remapped a little bit
local tLevelToStyleMap = 
{
    level1 = "level1",
    level2 = "level2",
    level3 = "level5",
    level4 = "level7",
    level5 = "level8",
    level6 = "level6",
    level7 = "level4",
    level8 = "level3",
    level9 = "level9",
    level10 = "level10",
}

function PostManager:Init()
    local i = 1
    PostManager.mNumFullMixIndexes = 4
    PostManager.mFullMixIndexes = {}
    for i = 1, PostManager.mNumFullMixIndexes do
        PostManager.mFullMixIndexes[i] = -1
    end
    
    PostManager.mPostStatesUltra = {
        intro = PostStateIntro,
        gameover = PostStateGameOver,
        leaderboards = PostState_Leaderboards,
        level1  = PostState_Level1,
        level2  = PostState_Level2,
        level3  = PostState_Level3,
        level4  = PostState_Level4,
        level5  = PostState_Level5,
        level6  = PostState_Level6,
        level7  = PostState_Level7,
        level8  = PostState_Level8,
        level9  = PostState_Level9,
        level10 = PostState_Level10,
    }
    PostManager.mPostStatesHigh = {
        intro = PostStateHigh_Intro,
        gameover = PostStateHigh_GameOver,
        leaderboards = PostStateHigh_Leaderboards,
        level1  = PostStateHigh_Level1,
        level2  = PostStateHigh_Level2,
        level3  = PostStateHigh_Level3,
        level4  = PostStateHigh_Level4,
        level5  = PostStateHigh_Level5,
        level6  = PostStateHigh_Level6,
        level7  = PostStateHigh_Level7,
        level8  = PostStateHigh_Level8,
        level9  = PostStateHigh_Level9,
        level10 = PostStateHigh_Level10,
    }    
    PostManager.mPostStatesLow = {
        intro = PostStateLow_Intro,
        gameover = PostStateLow_GameOver,
        leaderboards = PostStateLow_Leaderboards,
        level1  = PostStateLow_Level1,
        level2  = PostStateLow_Level2,
        level3  = PostStateLow_Level3,
        level4  = PostStateLow_Level4,
        level5  = PostStateLow_Level5,
        level6  = PostStateLow_Level6,
        level7  = PostStateLow_Level7,
        level8  = PostStateLow_Level8,
        level9  = PostStateLow_Level9,
        level10 = PostStateLow_Level10,
    }
end

function PostManager:RandomizeFullMixIndex()
    local i = 1
    local iIndex = -1
    while true do
        --don't use post level 1 cause the bonus is too crazy
        iIndex = math.random(2.0, 10.0)   

        local bUsedPreviously = false
        for i = 1, PostManager.mNumFullMixIndexes do
            if PostManager.mFullMixIndexes[i] == iIndex then
                bUsedPreviously = true
                break
            end
        end        
        
        if not bUsedPreviously then
            break
        end
    end 
 
    --push back indexes and set this one as our first
    i = PostManager.mNumFullMixIndexes
    while i > 1 do
        PostManager.mFullMixIndexes[i] = PostManager.mFullMixIndexes[i-1]
        i = i - 1
    end 
    PostManager.mFullMixIndexes[1] = iIndex    
end

function PostManager:SetStateIntro()
    print ("CurrentGameLevel: " .. Constants.CurrentGameLevel)
    if Constants.UsePostEffects then
        local sStyleKey = "intro"
        
        if Constants.OverrideVisualStyle then
            sStyleKey = Constants.OverrideVisualStyle
        end
        
        if Constants.VisualQuality == Constants.VisualQualityLevelUltra then
            PostManager.mPostStatesUltra[sStyleKey]:Start()
        elseif Constants.VisualQuality == Constants.VisualQualityLevelHigh then
            PostManager.mPostStatesHigh[sStyleKey]:Start()
        else
            PostManager.mPostStatesLow[sStyleKey]:Start()
        end
    end
end

function PostManager:SetStateGameOver()
    print ("CurrentGameLevel: " .. Constants.CurrentGameLevel)
    if Constants.UsePostEffects then
        local sStyleKey = "leaderboards"
        
        if Constants.OverrideVisualStyle then
            sStyleKey = Constants.OverrideVisualStyle
        end
        
        if Constants.VisualQuality == Constants.VisualQualityLevelUltra then
            PostManager.mPostStatesUltra[sStyleKey]:Start()
        elseif Constants.VisualQuality == Constants.VisualQualityLevelHigh then
            PostManager.mPostStatesHigh[sStyleKey]:Start()
        else
            PostManager.mPostStatesLow[sStyleKey]:Start()
        end
    end
end

function PostManager:SetStateCredits()
    print ("CurrentGameLevel: " .. Constants.CurrentGameLevel)
    if Constants.UsePostEffects then
        local sStyleKey = "leaderboards"
        
        if Constants.OverrideVisualStyle then
            sStyleKey = Constants.OverrideVisualStyle
        end
        
        if Constants.VisualQuality == Constants.VisualQualityLevelUltra then
            PostManager.mPostStatesUltra[sStyleKey]:Start()
        elseif Constants.VisualQuality == Constants.VisualQualityLevelHigh then
            PostManager.mPostStatesHigh[sStyleKey]:Start()
        else
            PostManager.mPostStatesLow[sStyleKey]:Start()
        end
    end
end

function PostManager:SetStateLeaderboardShowcase()
    print ("CurrentGameLevel: " .. Constants.CurrentGameLevel)
    if Constants.UsePostEffects then
        local sStyleKey = "leaderboards"
        
        if Constants.OverrideVisualStyle then
            sStyleKey = Constants.OverrideVisualStyle
        end
        
        if Constants.VisualQuality == Constants.VisualQualityLevelUltra then
            PostManager.mPostStatesUltra[sStyleKey]:Start()
        elseif Constants.VisualQuality == Constants.VisualQualityLevelHigh then
            PostManager.mPostStatesHigh[sStyleKey]:Start()
        else
            PostManager.mPostStatesLow[sStyleKey]:Start()
        end
    end
end

function PostManager:SetLevelStateStart()
    print ("CurrentGameLevel: " .. Constants.CurrentGameLevel)
    if Constants.UsePostEffects then
        local iTypeIndex = Constants.CurrentGameLevel
        if Constants.FullMixMode then
            iTypeIndex = PostManager.mFullMixIndexes[1]
        end
        
        local sStyleKey = tLevelToStyleMap["level" .. iTypeIndex]

        if Constants.OverrideVisualStyle then
            sStyleKey = Constants.OverrideVisualStyle
        end
    
        if Constants.VisualQuality == Constants.VisualQualityLevelUltra then
            PostManager.mPostStatesUltra[sStyleKey]:Start()
        elseif Constants.VisualQuality == Constants.VisualQualityLevelHigh then
            PostManager.mPostStatesHigh[sStyleKey]:Start()
        else
            PostManager.mPostStatesLow[sStyleKey]:Start()
        end
    end
end

function PostManager:SetLevelStateBuild()
    if Constants.UsePostEffects then
        local iTypeIndex = Constants.CurrentGameLevel
        if Constants.FullMixMode then
            iTypeIndex = PostManager.mFullMixIndexes[1]
        end
        
        local sStyleKey = tLevelToStyleMap["level" .. iTypeIndex]
    
        if Constants.OverrideVisualStyle then
            sStyleKey = Constants.OverrideVisualStyle
        end

        if Constants.VisualQuality == Constants.VisualQualityLevelUltra then
            PostManager.mPostStatesUltra[sStyleKey]:Build()
        elseif Constants.VisualQuality == Constants.VisualQualityLevelHigh then
            PostManager.mPostStatesHigh[sStyleKey]:Build()
        else
            PostManager.mPostStatesLow[sStyleKey]:Build()
        end
    end
end

function PostManager:SetLevelStateBonus()
    if Constants.UsePostEffects then
        local iTypeIndex = Constants.CurrentGameLevel
        if Constants.FullMixMode then
            iTypeIndex = PostManager.mFullMixIndexes[1]
        end
        
        local sStyleKey = tLevelToStyleMap["level" .. iTypeIndex]
    
        if Constants.OverrideVisualStyle then
            sStyleKey = Constants.OverrideVisualStyle
        end
    
        if Constants.VisualQuality == Constants.VisualQualityLevelUltra then
            PostManager.mPostStatesUltra[sStyleKey]:Bonus()
        elseif Constants.VisualQuality == Constants.VisualQualityLevelHigh then
            PostManager.mPostStatesHigh[sStyleKey]:Bonus()
        else
            PostManager.mPostStatesLow[sStyleKey]:Bonus()
        end
    end
end

return PostManager
