local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')

local PostStateGameOver = {}

function PostStateGameOver:Start()
    print("PostStateGameOver: Start")
    Post:Reset()

    Post:SetUpdateCallback(PostStateGameOver.Update)
    
    -----------------------------------------------
    -- Setup Colors 
    -----------------------------------------------      
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.75, .34, .9}
    ColorManager.Color2             = {.1, .0, .28} --score
    ColorManager.Color3             = {.75, .34, .9} --multiplier
    ColorManager:SetColor()
    
    ChargeField:Enable(true)
    ChargeField:SetColor(.1,.1,.1,1)
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 3.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 0.0, 4.0 )
    ChargeField.mChargeFieldDeck:SetActivatedDistance( 60.0 )
    
    local finalColorBuffer = Post:GetUnusedBuffer()   
    local accumBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchBuffer2 = Post:GetUnusedBuffer()
    local scratchBuffer3 = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local sceneBufferWithBackground = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()

    Post:BlitFilter(Renderer.mGameBuffer,sceneBuffer, kBLEND_ADD, {1,1,1,1})
    local uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer,sceneBuffer, kBLEND_ADD, {1,1,1,1})
    Post:CombineFilter(Renderer.mBackgroundBuffer, sceneBuffer, sceneBufferWithBackground, kBLEND_ADD, {1,1,1,1})
    
    local flowFilter = Post:AccumFlowFilter(sceneBufferWithBackground, accumBuffer, scratchBuffer, Post.rFlowTexture, {.95,.95,.95,1}, {1.02,1.02}, {0,0}, .01) --.05)
    flowFilter:setShaderValue("g_fFlowPulseFreq", MOAIMaterial.VALUETYPE_FLOAT, 0)
    Post:BlurFilter(scratchBuffer, accumBuffer, .0000)
    Post:BlurFilter(accumBuffer, scratchBuffer2, .0, kBLEND_ALPHA, {1,1,1,1})
    Post:EdgeFilter(scratchBuffer2, finalColorBuffer, 1, kBLEND_ADD, {.2,.7,.2,1})
    Post:EdgeFilter(sceneBuffer, finalColorBuffer, 1, kBLEND_ADD, {2,.8,.2,1})
    Post:BlurFilter(sceneBuffer, finalColorBuffer, .005, kBLEND_ADD, {1,.8,2,1})
    Post:BlurFilter(sceneBuffer, finalColorBuffer, .01, kBLEND_ADD, {.3,.8,1,1})
    
    Post.rScreen:setInBuffer(finalColorBuffer)
end

return PostStateGameOver