local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local RadEffects = require('RadEffects')
local ChargeManager = require('ChargeManager')

local PostStateIntro = {}

local flowIntensity = nil
local flowFilter = nil
local uiBlitFilter = nil
local uiBlitFilter2 = nil
local warpFilter = nil
local totalTime = nil
local directionalBlurFilter = nil
local audioAverage = 0.0

function PostStateIntro:Start()
    print("PostStateIntro: Start")
    Post:Reset()
    
    totalTime = 0.0 
    flowIntensity = 0.0
    
    Post:SetUpdateCallback(PostStateIntro.IntroUpdate) 
    
    -----------------------------------------------
    -- Setup Colors 
    -----------------------------------------------      
    ColorManager.BGColor            = {0.2, 0.2, 0.0}
    ColorManager.Color1             = {1.0, 0.5, 0.1}
    ColorManager.Color2             = {0.8, 0.3, 0.0} 
    ColorManager.Color3             = {0.0, 0.05, 0.3} 
    ColorManager.SpecialColor       = {0.8, 0.3, 0.0} 
    ColorManager:SetColor()
    
    Grid:SetColor(.1,.1,.1, 1)
    Grid:Enable(true)
    Grid.mGridDeck:SetPointWidth( 2 )
    Grid.mGridDeck:SetSpringK( 1.0 )
    Grid.mGridDeck:SetSpringDampen( 0.02 )
    
    local finalColorBuffer = Post:GetUnusedBuffer()   
    local accumBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchBuffer2 = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedHalfBuffer()
    local scratchEightBuffer2 = Post:GetUnusedEighthBuffer()
    local blurSourceBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    uiBlitFilter = Post:Level0AFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer)
    
    Post:BlitFilter(scratchBuffer, accumBuffer)
    warpFilter = Post:WarpFilter(Renderer.mBackgroundBuffer, accumBuffer, 0, kBLEND_ADD, {1,1,1,1})
    
    flowFilter = Post:AccumFlowFilter(sceneBuffer, accumBuffer, scratchBuffer, Post.rFlowTexture, {.95,.95,.95,1}, {.99,1.0}, {0,.0}, flowIntensity) --.05)
    if Constants.VisualQuality == Constants.VisualQualityLevelUltra then
        Post:BlurFilter(sceneBuffer, blurSourceBuffer, .003, kBLEND_ADD, {1,.8,.2,1})  
    else
        Post:BlitFilter(sceneBuffer, blurSourceBuffer)
    end
    Post:BlurFilter(blurSourceBuffer, scratchQuarterBuffer, .01, kBLEND_ADD, {1,1,1,1}) 
    Post:Level0BFilter(accumBuffer, scratchQuarterBuffer, sceneBuffer, finalColorBuffer)

    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostStateIntro.IntroUpdate()
    flowIntensity = flowIntensity + Constants.FrameStep*.01
    totalTime = totalTime + Constants.FrameStep
    flowIntensity = DFMath.pin(flowIntensity, 0.0, .025)
    local blitIntensity = .5 + (math.sin(totalTime*10) + 1) * .2
    
    audioAverage = DFMath.lerp( 1.0 * Constants.tAudioSpectrum[3], audioAverage, 0)    

    uiBlitFilter:setShaderValue("g_vColor", MOAIMaterial.VALUETYPE_VEC4, {blitIntensity,blitIntensity,blitIntensity,1})
    if uiBlitFilter2 then
        uiBlitFilter2:setShaderValue("g_vColor", MOAIMaterial.VALUETYPE_VEC4, {blitIntensity,blitIntensity,blitIntensity,1})
    end
    local blurIntensity =  DFMath.pin((.05 - totalTime*.01), 0.0, 1.0)
       
    -- this math is weird because of the ranges the different constants animate through
    local warpIntensity = 1.0
    warpIntensity = DFMath.pin(warpIntensity, 0.0, 25.0)
    if warpFilter then
        warpFilter:setShaderValue( "g_fWarpIntensity", MOAIMaterial.VALUETYPE_FLOAT, warpIntensity)
    end
    
    if flowFilter then flowFilter:setShaderValue( "g_fFlowIntensity", MOAIMaterial.VALUETYPE_FLOAT, flowIntensity * DFMath.pin(warpIntensity,0.0, 1.0)) end
    
    --print ("Warp Intensity: " .. warpIntensity)
end

return PostStateIntro
