local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local GlowRing = require('GlowRing')
local Grid = require('Grid')
local HUDGlow = require('HUDGlow')
local AudioBars = require('AudioBars')
local Wind = require('Wind')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local ParticleSystem = require("ParticleSystem")
local RadEffects = require('RadEffects')
local InputVisuals = require('InputVisuals')
local Score = require('Score')
local PostState_Level1= {}

local totalTime = 0.0

function PostState_Level1:Reset()
    totalTime = 0.0
    audioSmoothed = 0.0
    bonusIntensityRamp = 0.0
end



------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- SSSSSSSS  TTTTTTTT  AAAAAAAA  RRRRRR    TTTTTTTT
-- SS           TT     AA    AA  RR    RR     TT
-- SSSSSSSS     TT     AAAAAAAA  RRRRRR       TT
--       SS     TT     AA    AA  RR   RR      TT
-- SSSSSSSS     TT     AA    AA  RR    RR     TT
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level1:Start()
    print("PostState_Level1: Start")
    Post:Reset()
    PostState_Level1:Reset()
    
    Post:SetUpdateCallback(PostState_Level1.UpdateStart)   
    
    ColorManager.BGColor            = {0.0,.00,.00}
    ColorManager.Color1             = {.9, .5, .1}
    ColorManager.Color2             = {.9, .25, .05} --score
    ColorManager.Color3             = {.4, .11, .6} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Grid:SetColor(.02,.01,.03, 1)
    Grid:Enable(true)
    Grid.mGridDeck:SetPointWidth( 2 )
    Grid.mGridDeck:SetSpringK( 1.0 )
    Grid.mGridDeck:SetSpringDampen( 0.008 )
    
    RadEffects:SetGoodExplosionCallback(RadEffects.SpawnBubbleChamber)
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(1, .4, 0, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(.75, .2, .1, 1)
    HUDGlow:EnableBumpin(true)
  
    Post:EnableFullscreenEffects(false)
end

function PostState_Level1.UpdateStart()
    local level_intensity = Score.mLevelIntensity
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    local gridColor = level_intensity*.02
    Grid:SetColor(.02 + gridColor*.5, .01 + gridColor,.03 + gridColor*2, 1)
    Grid.mGridDeck:SetPointWidth( 100 *  audioSmoothed + 1)
    Grid.mGridDeck:SetSpringK( .5 + level_intensity*.5  )
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB   UU    UU  II  LL       DDDDD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBBB  UU    UU  II  LL       DD   DD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBB   UUUUUUUU  II  LLLLLLl  DDDDD
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level1:Build()
    print("PostState_Level1: Build")
    Post:Reset()
    PostState_Level1:Reset()

    Post:SetUpdateCallback(PostState_Level1.UpdateBuild)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.9, .5, .1}
    ColorManager.Color2             = {.9, .25, .05} --score
    ColorManager.Color3             = {.4, .11, .6} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Grid:SetColor(.4,.2,.65, 1)
    Grid:Enable(true)
    Grid.mGridDeck:SetPointWidth( 1.0 )
    Grid.mGridDeck:SetSpringK( 10.0 )
    Grid.mGridDeck:SetSpringDampen( 0.125 )

    RadEffects:SetEffectTransform( 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 )
    RadEffects:CreateInfiniteDurationEffect( "Effects/Particles/Backgrounds/Squares.particles", nil, {.2, .2, .7, 1} )
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(1, .4, 0, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(.75, .2, .1, 1)
    HUDGlow:EnableBumpin(true, 1.5)
    
    RadEffects:SetGoodExplosionCallback(RadEffects.SpawnBubbleChamberBuild)
    
    Post:EnableFullscreenEffects(false)
end 

function PostState_Level1.UpdateBuild()
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)   
    local level_intensity = Score.mLevelIntensity
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB    OOOOOO  NN    NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NNNN  NN  UU    UU  SS      
-- BBBBBBBB  OO    OO NN NN NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NN  NNNN  UU    UU        SS
-- BBBBBBB    OOOOOO  NN   NNN  UUUUUUUU  SSSSSSSS
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level1:Bonus()
    print("PostState_Level1: Bonus")
    Post:Reset()
    PostState_Level1:Reset()
    
    bonusIntensityRamp = 2.0
    
    Post:SetUpdateCallback(PostState_Level1.UpdateBonus)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.9, .5, .1}
    ColorManager.Color2             = {.9, .25, .05} --score
    ColorManager.Color3             = {.2,.6,.65} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Grid:SetColor(.2,.6,.65, 1)
    Grid:Enable(true)
    Grid.mGridDeck:SetPointWidth( 1.0 )
    Grid.mGridDeck:SetSpringK( 1.0 )
    Grid.mGridDeck:SetSpringDampen( 0.002 )
   
    RadEffects:SetEffectTransform( 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 )
    RadEffects:CreateInfiniteDurationEffect( "Effects/Particles/Backgrounds/Squares.particles", nil, {.2, .6, .65, 1} )
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(1, .4, 0, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(.75, .2, .1, 1)
    HUDGlow:EnableBumpin(true, 2.0)

    RadEffects:SetGoodExplosionCallback(RadEffects.SpawnBubbleChamber)
    
    Post:EnableFullscreenEffects(false)
end

function PostState_Level1.UpdateBonus()
    
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    totalTime = totalTime + Constants.FrameStep
        
    bonusIntensityRamp = bonusIntensityRamp - .03
    bonusIntensityRamp = DFMath.pin(bonusIntensityRamp,0,1)
end


    
return PostState_Level1
