local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local GlowRing = require('GlowRing')
local HUDGlow = require('HUDGlow')
local Wind = require('Wind')
local Score = require('Score')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local RadEffects = require('RadEffects')
local InputVisuals = require('InputVisuals')
local PostState_Level2= {}

local goodEffectColor = {0.5,0.1,0.5,1}
local totalTime = nil
local audioSmoothed = 0.0

function PostState_Level2:Reset()
    totalTime = 0.0
    audioSmoothed = 0.0
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- SSSSSSSS  TTTTTTTT  AAAAAAAA  RRRRRR    TTTTTTTT
-- SS           TT     AA    AA  RR    RR     TT
-- SSSSSSSS     TT     AAAAAAAA  RRRRRR       TT
--       SS     TT     AA    AA  RR   RR      TT
-- SSSSSSSS     TT     AA    AA  RR    RR     TT
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level2:Start()
    print("PostState_Level2: Start")
    Post:Reset()
    PostState_Level2:Reset()
 
    Post:SetUpdateCallback(PostState_Level2.UpdateStart)   

    -----------------------------------------------
    -- Setup Colors 
    -----------------------------------------------      
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.75, .34, .9}
    ColorManager.Color2             = {.2, .2, .48} --score
    ColorManager.Color3             = {.25, .11, .3} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    ChargeField:Enable(false)
    ChargeField:SetColor(.1,.1,.1,1)
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 3.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 0.0, 4.0 )
    ChargeField.mChargeFieldDeck:SetActivatedDistance( 60.0 )
    
    RadEffects:SetGoodExplosionCallback(self.SpawnPurpleStars)
    
    Post:EnableFullscreenEffects(false)
end

function PostState_Level2.UpdateStart()
    local level_intensity = Score.mLevelIntensity
    
    totalTime = totalTime + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    if level_intensity > .5 then
         ChargeField:Enable(true)
    end
end
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB   UU    UU  II  LL       DDDDD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBBB  UU    UU  II  LL       DD   DD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBB   UUUUUUUU  II  LLLLLLl  DDDDD
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------


function PostState_Level2:Build()
    print("PostState_Level2: Build")
    Post:Reset()
    PostState_Level2:Reset()

    totalTime = 0.0 
    Post:SetUpdateCallback(PostState_Level2.UpdateBuild)
    
    -----------------------------------------------
    -- Setup Colors 
    -----------------------------------------------      
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.75, .34, .9}
    ColorManager.Color2             = {.2, .2, .58} --score
    ColorManager.Color3             = {.25, .11, .3} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(.2,.15,.25, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 1.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 200.0, .2) 
    Wind.mWindDeck:SetFrequency( 10.0 )
    
    ChargeField:Enable(true)
    ChargeField:SetColor(.1,.1,.1,1)
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 3.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 0.0, 4.0 )
    ChargeField.mChargeFieldDeck:SetActivatedDistance( 60.0 )
    
    RadEffects:SetGoodExplosionCallback(self.SpawnPurpleStars)
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(.55, .2, .7, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(.2, .2, .7, 1)
    HUDGlow:EnableBumpin(true, 1.5)
    
    Post:EnableFullscreenEffects(false)
end


function PostState_Level2.UpdateBuild()
    totalTime = totalTime + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    Wind.mWindDeck:SetPointWidth( Constants.Heartbeat * 6.0 + 4.0)
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB    OOOOOO  NN    NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NNNN  NN  UU    UU  SS      
-- BBBBBBBB  OO    OO NN NN NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NN  NNNN  UU    UU        SS
-- BBBBBBB    OOOOOO  NN   NNN  UUUUUUUU  SSSSSSSS
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level2:Bonus()
    print("PostState_Level2: Bonus")
    Post:Reset()
    PostState_Level2:Reset()

    totalTime = 0.0 
    Post:SetUpdateCallback(PostState_Level2.UpdateBonus)
    
    -----------------------------------------------
    -- Setup Colors 
    -----------------------------------------------      
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.75, .34, .9}
    ColorManager.Color2             = {.3,0,.9} --score
    ColorManager.Color3             = {.25, .11, .3} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(.2,.15,.25, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 1.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 200.0, 1) 
    Wind.mWindDeck:SetFrequency( 10.0 )
     
    RadEffects:SetGoodExplosionCallback(self.SpawnPurpleStars)
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(.55, .2, .7, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(.2, .2, .7, 1)
    HUDGlow:EnableBumpin(true, 2.0)
    
    Post:EnableFullscreenEffects(false)
end


function PostState_Level2.UpdateBonus()
    totalTime = totalTime + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)

    Wind.mWindDeck:SetPointWidth( Constants.Heartbeat * 6.0 + 4.0)    
end


function PostState_Level2.SpawnPurpleStars(posX, posY, posZ, rTargetProp)
    local rExplosion = RadEffects.SpawnPurpleStars(posX, posY, posZ, rTargetProp, PostState_Level2.BubbleChamberTargetCallback)
    
    local rStarEffect = rExplosion.rStarEffect
    local rStarEffectProp = rStarEffect:getProp()
    local rStarEffectMaterial = rStarEffectProp:getMaterial()
    rStarEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rTracerEffect = rExplosion.rTracerEffect
    local rTracerEffectProp = rTracerEffect:getProp()
    local rTracerEffectMaterial = rTracerEffectProp:getMaterial()
    rTracerEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rRippleEffect = rExplosion.rRippleEffect
    local rRippleEffectProp = rRippleEffect:getProp()
    local rRippleEffectMaterial = rRippleEffectProp:getMaterial()
    rRippleEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rSeekingEffect = rExplosion.rSeekingEffect
    local rSeekingEffectProp = rSeekingEffect:getProp()
    local rSeekingEffectMaterial = rSeekingEffectProp:getMaterial()
    rSeekingEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)

    return rExplosion
end


function PostState_Level2.BubbleChamberTargetCallback( tParticleEffect )
    local Explosion = {}
    if tParticleEffect ~= nil then
        if tParticleEffect.rTargetProp ~= nil then
            local fTargetX, fTargetY, fTargetZ = tParticleEffect.rTargetProp:getLoc()
            RadEffects:SetEffectTransform( fTargetX, fTargetY, fTargetZ, 0.0, 0.0, 0.0 )   
            RadEffects:SetEffectLayer( RadEffects.tRenderer.mGameLayer )
            local rEffect = RadEffects:CreateEffect( "Effects/Particles/Bursts/BlipFlare.particles", nil, nil, nil, Constants.ParticlePriorityLow )                 
            Explosion.rEffect = rEffect
            
            local rEffectProp = rEffect:getProp()
            local rEffectMaterial = rEffectProp:getMaterial()
            rEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
        end
    end
    
    return Explosion
end


return PostState_Level2
