local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local GlowRing = require('GlowRing')
local HUDGlow = require('HUDGlow')
local Wind = require('Wind')
local Score = require('Score')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local ParticleSystem = require("ParticleSystem")
local RadEffects = require('RadEffects')
local PostState_Level3 = {}

local goodEffectColor = {0.3,0.2,0,1}
local totalTime = 0.0
local audioSmoothed = 0.0

function PostState_Level3:Reset()
    -- hmmmm
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- SSSSSSSS  TTTTTTTT  AAAAAAAA  RRRRRR    TTTTTTTT
-- SS           TT     AA    AA  RR    RR     TT
-- SSSSSSSS     TT     AAAAAAAA  RRRRRR       TT
--       SS     TT     AA    AA  RR   RR      TT
-- SSSSSSSS     TT     AA    AA  RR    RR     TT
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level3:Start()
    print("PostState_Level3: Start")
    Post:Reset()
    PostState_Level3:Reset()
    
    Post:SetUpdateCallback(PostState_Level3.UpdateStart)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.55, .54, .1}
    ColorManager.Color2             = {.3, .1, .1} --score
    ColorManager.Color3             = {.75, .34, .2} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
        
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    
    --blurs
    HUDGlow:Enable(true)
    HUDGlow:SetColor(.5, .1, .1, 1)
    HUDGlow:EnableBumpin(true, 1.0)
    
    Post:EnableFullscreenEffects(false)
end


function PostState_Level3.UpdateStart()
    local level_intensity = Score.mLevelIntensity
    totalTime = totalTime  + Constants.FrameStep   
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB   UU    UU  II  LL       DDDDD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBBB  UU    UU  II  LL       DD   DD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBB   UUUUUUUU  II  LLLLLLl  DDDDD
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level3:Build()
    print("PostState_Level3: Build")
    Post:Reset()
    PostState_Level3:Reset()
    
    Post:SetUpdateCallback(PostState_Level3.UpdateBuild)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.85, .54, .1}
    ColorManager.Color2             = {.55, .37, .1} --score
    ColorManager.Color3             = {.55, .24, .09} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(.2,.15,.25, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 2.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 400.0, .05) 
    Wind.mWindDeck:SetFrequency( 100.0 )
            
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(.8, .2, .1, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(.8, .2, .1, 1)
    HUDGlow:EnableBumpin(true, 1.5)
    
    Post:EnableFullscreenEffects(false)
end

function PostState_Level3.UpdateBuild()
    local level_intensity = Score.mLevelIntensity
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB    OOOOOO  NN    NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NNNN  NN  UU    UU  SS      
-- BBBBBBBB  OO    OO NN NN NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NN  NNNN  UU    UU        SS
-- BBBBBBB    OOOOOO  NN   NNN  UUUUUUUU  SSSSSSSS
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------


function PostState_Level3:Bonus()
    print("PostState_Level3: Bonus")
    Post:Reset()
    PostState_Level3:Reset()
    
    Post:SetUpdateCallback(PostState_Level3.UpdateBonus)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.85, .54, .1}
    ColorManager.Color2             = {.8, .64, .1} --score
    ColorManager.Color3             = {.75, .34, .13} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(.2,.15,.25, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 2.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 2000.0, .05) 
    Wind.mWindDeck:SetFrequency( 300.0 )
        
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(.85, .2, .1, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(.8, .2, .1, 1)
    HUDGlow:EnableBumpin(true, 2.0)

    --sin wave
    RadEffects:SetEffectTransform( 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 )
    RadEffects:CreateInfiniteDurationEffect( "Effects/Particles/Backgrounds/Waves.particles", nil, {0.3, 0.1, 1, 1} )
    
    Post:EnableFullscreenEffects(false)
end


function PostState_Level3.UpdateBonus()
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)    
end

function PostState_Level3.SpawnBubbleChamber(posX, posY, posZ, rTargetProp)
    local rExplosion = RadEffects.SpawnBubbleChamber(posX, posY, posZ, rTargetProp, PostState_Level3.BubbleChamberTargetCallback)
    
    local rTracerEffect = rExplosion.rTracerEffect
    if rTracerEffect then
        local rTracerEffectProp = rTracerEffect:getProp()
        local rTracerEffectMaterial = rTracerEffectProp:getMaterial()
        rTracerEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    end
    local rRippleEffect = rExplosion.rRippleEffect
    if rRippleEffect then
        local rRippleEffectProp = rRippleEffect:getProp()
        local rRippleEffectMaterial = rRippleEffectProp:getMaterial()
        rRippleEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    end
    local rSeekingEffect = rExplosion.rSeekingEffect
    if rSeekingEffect then
        local rSeekingEffectProp = rSeekingEffect:getProp()
        local rSeekingEffectMaterial = rSeekingEffectProp:getMaterial()
        rSeekingEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    end
    
    return rExplosion
end

function PostState_Level3.BubbleChamberTargetCallback( tParticleEffect )
    local Explosion = {}
    if tParticleEffect ~= nil then
        if tParticleEffect.rTargetProp ~= nil then
            local fTargetX, fTargetY, fTargetZ = tParticleEffect.rTargetProp:getLoc()
            RadEffects:SetEffectTransform( fTargetX, fTargetY, fTargetZ, 0.0, 0.0, 0.0 )   
            RadEffects:SetEffectLayer( RadEffects.tRenderer.mGameLayer )
            local rEffect = RadEffects:CreateEffect( "Effects/Particles/Bursts/BlipFlare.particles", nil, nil, nil, Constants.ParticlePriorityLow )                 
            Explosion.rEffect = rEffect
            
            local rEffectProp = rEffect:getProp()
            local rEffectMaterial = rEffectProp:getMaterial()
            rEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
        end
    end
    
    return Explosion
end

return PostState_Level3