local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local GlowRing = require('GlowRing')
local HUDGlow = require('HUDGlow')
local Wind = require('Wind')
local RadShapes = require('RadShapes')
local Score = require('Score')
local AudioRing = require('AudioRing')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local ParticleSystem = require("ParticleSystem")
local InputVisuals = require('InputVisuals')
local RadEffects = require('RadEffects')
local PostState_Level4 = {}

local goodEffectColor = {0.7,0.1,0.7,1}
local totalTime = 0.0
local audioSmoothed = 0.0
local audioBarOffset = 0.0

function PostState_Level4:Reset()    
    -- hmmm
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- SSSSSSSS  TTTTTTTT  AAAAAAAA  RRRRRR    TTTTTTTT
-- SS           TT     AA    AA  RR    RR     TT
-- SSSSSSSS     TT     AAAAAAAA  RRRRRR       TT
--       SS     TT     AA    AA  RR   RR      TT
-- SSSSSSSS     TT     AA    AA  RR    RR     TT
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level4:Start()
    print("PostState_Level4: Start")
    Post:Reset()
    PostState_Level4:Reset()
            
    Post:SetUpdateCallback(PostState_Level4.Update)   
  
    ColorManager.BGColor            =  {.9,.8, .01}
    ColorManager.Color1             =  {.1,.1,.3}
    ColorManager.Color2             =  {.3,.1,.4}
    ColorManager.Color3             =  {.1, .5, .3} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
            
    Wind:Enable(true)
    Wind:SetColor(.1,.1,.3, 1)
    Wind:SetLayer(Renderer.mGameLayer)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 2.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 400.0, .2) 
    Wind.mWindDeck:SetFrequency( 300.0 )
    
    AudioRing:SetScale(.1)
    AudioRing:SetMaxScale(1000.0)
    AudioRing:SetColor(.3,.1,.5,1)
    AudioRing.rLayer = Renderer.mGameLayer-- particleLayer --
    AudioRing:Enable(true)
    
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    
    Post:EnableFullscreenEffects(false)
    
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/PurpleRings.particles", false)
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(.2, .4, 0, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(.2, .4, 0, 1)
    HUDGlow:EnableBumpin(true, 1.0)
    
end

function PostState_Level4.Update()
    totalTime = totalTime  + Constants.FrameStep
    
    local randomfloat = math.random(-100,100)
    local xParam = math.cos(randomfloat) * 350
    local yParam = math.sin(randomfloat) * 350
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    Wind.mWindDeck:SetPointWidth( 3.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 300.0, .3) 

    audioBarOffset = math.sin(totalTime*.2) --DFMath.pin(totalTime*.01, 0, .25);
    AudioRing:SetRingOffset(audioBarOffset)
    AudioRing:SetScale(Constants.Heartbeat*.1)  
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB   UU    UU  II  LL       DDDDD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBBB  UU    UU  II  LL       DD   DD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBB   UUUUUUUU  II  LLLLLLl  DDDDD
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level4:Build()
    print("PostState_Level4: Build")
     Post:Reset()
    PostState_Level4:Reset()
            
    Post:SetUpdateCallback(PostState_Level4.UpdateBuild)   
  
    ColorManager.BGColor            =  {.9,.8, .01}
    ColorManager.Color1             =  {.1,.1,.3}
    ColorManager.Color2             =  {.3,.1,.4}
    ColorManager.Color3             =  {.1, .5, .3} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
            
    Wind:Enable(true)
    Wind:SetColor(.1,.1,.3, 1)
    Wind:SetLayer(Renderer.mGameLayer)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 2.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 400.0, .2) 
    Wind.mWindDeck:SetFrequency( 300.0 )
    
    AudioRing:SetMaxScale(1000.0)
    AudioRing:SetScale(.3)
    AudioRing:SetColor(.3,.1,.5,1)
    AudioRing.rLayer = Renderer.mGameLayer-- particleLayer --
    AudioRing:Enable(true)

    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(.2, .4, 0, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(.2, .4, 0, 1)
    HUDGlow:EnableBumpin(true, 1.5)
    
    Post:EnableFullscreenEffects(false)
    
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/PurpleRings.particles", false)
    -----------------------------------------------
    -- Setup Props
    -----------------------------------------------  
    local fRR = Constants.GameRingRadius
    PostState_Level4.mNumArcShapes = 0

    RadShapes:SetPropLayer( Renderer.mGameLayer )
    PostState_Level4.mArcShapes = {}  
    for i = 1, PostState_Level4.mNumArcShapes do
        PostState_Level4.mArcShapes[i] = RadShapes:CreateNewShape() 
        PostState_Level4.mArcShapes[i]:SetType( ShapeProp.DrawModeArc, 64 )
        PostState_Level4.mArcShapes[i]:SetColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
        if i == 1 then
            PostState_Level4.mArcShapes[i]:SetPosition( -Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, -15.0 )
        end
        if i == 2 then
            PostState_Level4.mArcShapes[i]:SetPosition( Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, -15.0 )
        end
        if i == 3 then
            PostState_Level4.mArcShapes[i]:SetPosition( -Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, -15.0 )
        end
        if i == 4 then
            PostState_Level4.mArcShapes[i]:SetPosition( Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, -15.0 )
        end
        PostState_Level4.mArcShapes[i]:SetArcRadians( 0, math.pi * 2)
        PostState_Level4.mArcShapes[i]:SetRadii( fRR * 1.2, fRR * 1.3 )
    end 
end


function PostState_Level4.UpdateBuild()
    local level_intensity = Score.mLevelIntensity
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    local randomfloat = math.random(-100,100)
    local xParam = math.cos(randomfloat) * 350
    local yParam = math.sin(randomfloat) * 350
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    Wind.mWindDeck:SetPointWidth( 6.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 800.0, .5) 
    
    audioBarOffset = (totalTime*(.2 + level_intensity)) --DFMath.pin(totalTime*.01, 0, .25);
    AudioRing:SetRingOffset(audioBarOffset)
    AudioRing:SetScale(Constants.Heartbeat*.5 + level_intensity)
    
    local fRR = Constants.GameRingRadius
    local intensityMirrored = level_intensity
    if intensityMirrored > 0.5 then
        intensityMirrored = 1-intensityMirrored
    end
    intensityMirrored = intensityMirrored * 2
    for i = 1, PostState_Level4.mNumArcShapes do
        PostState_Level4.mArcShapes[i]:SetRadii( ((fRR * 0.5) + (fRR * 0.5)) * intensityMirrored, ((fRR * 0.5) + (fRR * 0.6)) * intensityMirrored)
    end 
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB    OOOOOO  NN    NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NNNN  NN  UU    UU  SS      
-- BBBBBBBB  OO    OO NN NN NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NN  NNNN  UU    UU        SS
-- BBBBBBB    OOOOOO  NN   NNN  UUUUUUUU  SSSSSSSS
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level4:Bonus()
    print("PostState_Level4: Bonus")
     Post:Reset()
    PostState_Level4:Reset()
            
    Post:SetUpdateCallback(PostState_Level4.UpdateBonus)   
  
    ColorManager.BGColor            =  {.9,.8, .01}
    ColorManager.Color1             =  {.1,.1,.3}
    ColorManager.Color2             =  {.3,.1,.4}
    ColorManager.Color3             =  {.1, .5, .3} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1}     
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(.1,.1,.3, 1)
    Wind:SetLayer(Renderer.mGameLayer)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 2.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 400.0, .2) 
    Wind.mWindDeck:SetFrequency( 300.0 )
    
    AudioRing:SetMaxScale(1000.0)
    AudioRing:SetScale(.3)
    AudioRing:SetColor(.3,.1,.5,1)
    AudioRing.rLayer = Renderer.mGameLayer-- particleLayer --
    AudioRing:Enable(true)
    
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    
    Post:EnableFullscreenEffects(false)
    
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    
    RadEffects:SetEffectLayerToParticleAccumLayer()
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/PurpleRings.particles", false)
        
    RadEffects:SetEffectTransform( 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 )
    RadEffects:CreateInfiniteDurationEffect( "Effects/Particles/Backgrounds/Magicalness.particles", nil, nil )
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(.2, .4, 0, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(.2, .4, 0, 1)
    HUDGlow:EnableBumpin(true, 2.0)
    
    -----------------------------------------------
    -- Setup Props
    -----------------------------------------------  
    local fRR = Constants.GameRingRadius
    PostState_Level4.mNumArcShapes = 0

    RadShapes:SetPropLayer( Renderer.mGameLayer )
    PostState_Level4.mArcShapes = {}  
    for i = 1, PostState_Level4.mNumArcShapes do
        PostState_Level4.mArcShapes[i] = RadShapes:CreateNewShape() 
        PostState_Level4.mArcShapes[i]:SetType( ShapeProp.DrawModeArc, 64 )
        PostState_Level4.mArcShapes[i]:SetColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
        if i == 1 then
            PostState_Level4.mArcShapes[i]:SetPosition( -Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, -15.0 )
        end
        if i == 2 then
            PostState_Level4.mArcShapes[i]:SetPosition( Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, -15.0 )
        end
        if i == 3 then
            PostState_Level4.mArcShapes[i]:SetPosition( -Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, -15.0 )
        end
        if i == 4 then
            PostState_Level4.mArcShapes[i]:SetPosition( Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, -15.0 )
        end
        PostState_Level4.mArcShapes[i]:SetArcRadians( 0, math.pi * 2)
        PostState_Level4.mArcShapes[i]:SetRadii( fRR * 1.2, fRR * 1.3 )
    end  
end

 
function PostState_Level4.UpdateBonus()
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    local randomfloat = math.random(-100,100)
    local xParam = math.cos(randomfloat) * 350
    local yParam = math.sin(randomfloat) * 350
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    Wind.mWindDeck:SetPointWidth( 6.0 + Constants.Heartbeat*8)
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 800.0, .5) 
    
    audioBarOffset = .25
    AudioRing:SetRingOffset(audioBarOffset)
    AudioRing:SetScale(Constants.Heartbeat*3)
    
    local fRR = Constants.GameRingRadius
    local powTime = math.pow(1-Constants.Heartbeat,3)
    for i = 1, PostState_Level4.mNumArcShapes do
        PostState_Level4.mArcShapes[i]:SetRadii( (fRR * 0.5) + (fRR * powTime), (fRR * 0.5) + (fRR * 1.2 * powTime))
    end  
end


function PostState_Level4.SpawnBubbleChamber(posX, posY, posZ, rTargetProp)
    local rExplosion = RadEffects.SpawnBubbleChamber(posX, posY, posZ, rTargetProp, PostState_Level4.BubbleChamberTargetCallback)
    
    local rTracerEffect = rExplosion.rTracerEffect
    local rTracerEffectProp = rTracerEffect:getProp()
    local rTracerEffectMaterial = rTracerEffectProp:getMaterial()
    rTracerEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rRippleEffect = rExplosion.rRippleEffect
    local rRippleEffectProp = rRippleEffect:getProp()
    local rRippleEffectMaterial = rRippleEffectProp:getMaterial()
    rRippleEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rSeekingEffect = rExplosion.rSeekingEffect
    local rSeekingEffectProp = rSeekingEffect:getProp()
    local rSeekingEffectMaterial = rSeekingEffectProp:getMaterial()
    rSeekingEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    return rExplosion
end

function PostState_Level4.BubbleChamberTargetCallback( tParticleEffect )
    local Explosion = {}
    if tParticleEffect ~= nil then
        if tParticleEffect.rTargetProp ~= nil then
            local fTargetX, fTargetY, fTargetZ = tParticleEffect.rTargetProp:getLoc()
            RadEffects:SetEffectTransform( fTargetX, fTargetY, fTargetZ, 0.0, 0.0, 0.0 )   
            RadEffects:SetEffectLayer( RadEffects.tRenderer.mGameLayer )
            local rEffect = RadEffects:CreateEffect( "Effects/Particles/Bursts/BlipFlare.particles", nil, nil, nil, Constants.ParticlePriorityLow )                 
            Explosion.rEffect = rEffect
            
            local rEffectProp = rEffect:getProp()
            local rEffectMaterial = rEffectProp:getMaterial()
            rEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
        end
    end
    
    return Explosion
end

return PostState_Level4
