local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local GlowRing = require('GlowRing')
local HUDGlow = require('HUDGlow')
local Wind = require('Wind')
local Score = require('Score')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local ParticleSystem = require("ParticleSystem")
local RadEffects = require('RadEffects')
local InputVisuals = require('InputVisuals')
local RadShapes = require('RadShapes')
local PostState_Level7 = {}

local goodEffectColor = {0.2,0.3,1,1}
local totalTime = 0.0
local audioSmoothed = 0.0
local arcSize = 0.0


function PostState_Level7:Reset()
    -- ehhhhh
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- SSSSSSSS  TTTTTTTT  AAAAAAAA  RRRRRR    TTTTTTTT
-- SS           TT     AA    AA  RR    RR     TT
-- SSSSSSSS     TT     AAAAAAAA  RRRRRR       TT
--       SS     TT     AA    AA  RR   RR      TT
-- SSSSSSSS     TT     AA    AA  RR    RR     TT
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level7:Start()
    print("PostState_Level7: Start")
    Post:Reset()
    PostState_Level7:Reset()
    
    Post:SetUpdateCallback(PostState_Level7.UpdateStart)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {0, .3, 1}
    ColorManager.Color2             = {.1, .5, 1} --score
    ColorManager.Color3             = {.4, .1, .7} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()

    Wind:Enable(true)
    Wind:SetColor(.1,.5,1, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 0.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 400.0, .2) 
    Wind.mWindDeck:SetFrequency( 300.0 )
    
    ChargeField:Enable(true)
    ChargeField:SetColor( Constants.Color1[1]*0.5, Constants.Color1[2]*0.5, Constants.Color1[3]*0.5, 1.0 )
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 3.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 1.0, 4.0 )

    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)

    Post:EnableFullscreenEffects(false)
    --InputVisuals:SetFingerEffects("Effects/Particles/Fingers/SoftGlow.particles", false)
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(0, .3, 1, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(0, .6, 1, 1)    
    HUDGlow:EnableBumpin(true, 1.0)
    
    PostState_Level7.ChargeFieldOffset = Constants.ValidDrawWidth * 0.02
end

function PostState_Level7.UpdateStart()
    totalTime = totalTime  + Constants.FrameStep
    
    local randomfloat = math.random(-100,100)
    local xParam = math.cos(randomfloat) * 350
    local yParam = math.sin(randomfloat) * 350
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    Wind.mWindDeck:SetPointWidth( Constants.Heartbeat * 6.0 )
    
    local xOffset = math.cos(totalTime) * PostState_Level7.ChargeFieldOffset
    local yOffset = math.sin(totalTime) * PostState_Level7.ChargeFieldOffset
    ChargeField.mChargeFieldProp:setLoc( xOffset, yOffset, 0.0 )    
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB   UU    UU  II  LL       DDDDD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBBB  UU    UU  II  LL       DD   DD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBB   UUUUUUUU  II  LLLLLLl  DDDDD
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level7:Build()
    print("PostState_Level7: Build")
    Post:Reset()
    PostState_Level7:Reset()
    
    Post:SetUpdateCallback(PostState_Level7.UpdateBuild)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {0, .3, 1}
    ColorManager.Color2             = {.1, .5, 1} --score
    ColorManager.Color3             = {.4, .1, .7} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(.1,.5,1, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 0.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 400.0, .2) 
    Wind.mWindDeck:SetFrequency( 300.0 )
    
    ChargeField:Enable(true)
    ChargeField:SetColor( Constants.Color1[1]*0.5, Constants.Color1[2]*0.5, Constants.Color1[3]*0.5, 1.0 )
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 3.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 1.0, 4.0 )
        
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)

    Post:EnableFullscreenEffects(false)
    --InputVisuals:SetFingerEffects("Effects/Particles/Fingers/SoftGlow.particles", false)
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(0, .3, 1, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(0, .6, 1, 1)    
    HUDGlow:EnableBumpin(true, 1.5)

    -----------------------------------------------
    -- Setup Props
    -----------------------------------------------  
    local fRR = Constants.GameRingRadius
    PostState_Level7.mDiamondMoveSpeed = 100.0
    PostState_Level7.mNumArcShapes = 8

    RadShapes:SetPropLayer( Renderer.mGameLayer )
    arcSize = math.pi/PostState_Level7.mNumArcShapes
    PostState_Level7.mArcShapes = {}  
    for i = 1, PostState_Level7.mNumArcShapes do
        PostState_Level7.mArcShapes[i] = RadShapes:CreateNewShape() 
        PostState_Level7.mArcShapes[i]:SetType( ShapeProp.DrawModeArc, 4 )
        PostState_Level7.mArcShapes[i]:SetColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
        PostState_Level7.mArcShapes[i]:SetPosition( 0.0, 0.0, -15.0 )
        PostState_Level7.mArcShapes[i]:SetArcRadians( 2 * i * arcSize + arcSize*0.5, 2 * i * arcSize + arcSize + arcSize*0.5)
        PostState_Level7.mArcShapes[i]:SetRadii( fRR, fRR * 1.1 )
    end  
end


function PostState_Level7.UpdateBuild()
    local level_intensity = Score.mLevelIntensity
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)

    local randomfloat = math.random(-100,100)
    local xParam = math.cos(randomfloat) * 350
    local yParam = math.sin(randomfloat) * 350
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    Wind.mWindDeck:SetPointWidth( 3.0 + Constants.Heartbeat * 6.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 700.0, .3) 
    
    local fRR = Constants.GameRingRadius
    local intensityMirrored = level_intensity
    if intensityMirrored > 0.5 then
        intensityMirrored = 1-intensityMirrored
    end
    for i = 1, PostState_Level7.mNumArcShapes do
        --PostState_Level7.mArcShapes[i]:SetArcRadians( 2*i*arcSize + arcSize*0.5, 2*i*arcSize + arcSize + arcSize*0.5)
        PostState_Level7.mArcShapes[i]:SetRadii( fRR *1.02, fRR * (1.02 + intensityMirrored))
    end  
    
    local xOffset = math.cos(totalTime) * PostState_Level7.ChargeFieldOffset
    local yOffset = math.sin(totalTime) * PostState_Level7.ChargeFieldOffset
    ChargeField.mChargeFieldProp:setLoc( xOffset, yOffset, 0.0 )     
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB    OOOOOO  NN    NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NNNN  NN  UU    UU  SS      
-- BBBBBBBB  OO    OO NN NN NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NN  NNNN  UU    UU        SS
-- BBBBBBB    OOOOOO  NN   NNN  UUUUUUUU  SSSSSSSS
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level7:Bonus()
    print("PostState_Level7: Bonus")
    Post:Reset()
    PostState_Level7:Reset()
    
    Post:SetUpdateCallback(PostState_Level7.UpdateBonus)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {0, .3, 1}
    ColorManager.Color2             = {.1, .5, 1} --score
    ColorManager.Color3             = {.4, .1, .7} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(.1,.5,1, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 0.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 400.0, .2) 
    Wind.mWindDeck:SetFrequency( 300.0 )
    
    ChargeField:Enable(true)
    ChargeField:SetColor( Constants.Color1[1]*0.5, Constants.Color1[2]*0.5, Constants.Color1[3]*0.5, 1.0 )
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 3.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 1.0, 4.0 )

    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    
    Post:EnableFullscreenEffects(false)
    
    --InputVisuals:SetFingerEffects("Effects/Particles/Fingers/SoftGlow.particles", false)
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(0, .3, 1, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(0, .6, 1, 1)    
    HUDGlow:EnableBumpin(true, 2.0)
    
    -----------------------------------------------
    -- Setup Props
    -----------------------------------------------  
    local fRR = Constants.GameRingRadius
    PostState_Level7.mDiamondMoveSpeed = 100.0
    PostState_Level7.mNumArcShapes = 8

    RadShapes:SetPropLayer( Renderer.mGameLayer )
    arcSize = math.pi/PostState_Level7.mNumArcShapes
    PostState_Level7.mArcShapes = {}  
    for i = 1, PostState_Level7.mNumArcShapes do
        PostState_Level7.mArcShapes[i] = RadShapes:CreateNewShape() 
        PostState_Level7.mArcShapes[i]:SetType( ShapeProp.DrawModeArc, 4 )
        PostState_Level7.mArcShapes[i]:SetColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
        PostState_Level7.mArcShapes[i]:SetPosition( 0.0, 0.0, -15.0 )
        PostState_Level7.mArcShapes[i]:SetArcRadians( 2 * i * arcSize + arcSize*0.5, 2 * i * arcSize + arcSize + arcSize*0.5)
        PostState_Level7.mArcShapes[i]:SetRadii( fRR * 1.2, fRR * 1.3 )
    end  
end


function PostState_Level7.UpdateBonus()
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    local randomfloat = math.random(-100,100)
    local xParam = math.cos(randomfloat) * 350
    local yParam = math.sin(randomfloat) * 350
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    Wind.mWindDeck:SetPointWidth( 6.0 + Constants.Heartbeat * 12.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 1000.0, .4) 
                
    local powTime = math.pow(Constants.Heartbeat,3)
    
    local fRR = Constants.GameRingRadius
    local arcOffset = powTime * 0.15
    for i = 1, PostState_Level7.mNumArcShapes do
        PostState_Level7.mArcShapes[i]:SetArcRadians( 2*i*arcSize + arcSize*0.5 + arcOffset, 2*i*arcSize + arcSize + arcSize*0.5 - arcOffset)
        PostState_Level7.mArcShapes[i]:SetRadii( fRR *1.02, fRR * (1.1 +powTime * 0.8))
    end  
    
    local xOffset = math.cos(totalTime) * PostState_Level7.ChargeFieldOffset
    local yOffset = math.sin(totalTime) * PostState_Level7.ChargeFieldOffset
    ChargeField.mChargeFieldProp:setLoc( xOffset, yOffset, 0.0 )       
end

function PostState_Level7.SpawnBubbleChamber(posX, posY, posZ, rTargetProp)
    local rExplosion = RadEffects.SpawnBubbleChamber(posX, posY, posZ, rTargetProp)
    
    local rTracerEffect = rExplosion.rTracerEffect
    local rTracerEffectProp = rTracerEffect:getProp()
    local rTracerEffectMaterial = rTracerEffectProp:getMaterial()
    rTracerEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rRippleEffect = rExplosion.rRippleEffect
    local rRippleEffectProp = rRippleEffect:getProp()
    local rRippleEffectMaterial = rRippleEffectProp:getMaterial()
    rRippleEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rSeekingEffect = rExplosion.rSeekingEffect
    local rSeekingEffectProp = rSeekingEffect:getProp()
    local rSeekingEffectMaterial = rSeekingEffectProp:getMaterial()
    rSeekingEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    return rExplosion
end

return PostState_Level7