local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local GlowRing = require('GlowRing')
local HUDGlow = require('HUDGlow')
local Wind = require('Wind')
local Score = require('Score')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local RadEffects = require('RadEffects')
local InputVisuals = require('InputVisuals')
local RadShapes = require('RadShapes')
local PostState_Level9= {}

local totalTime = nil
local beatCounter = 0.0
local lastHeartbeat = 0.0
local currentHeartbeatDelta = 0.0
local audioSmoothed = 0.0
local goodEffectColor = {0.1,0.9,0.2,1}

function PostState_Level9:Reset()
    totalTime = 0.0
    audioSmoothed = 0.0
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- SSSSSSSS  TTTTTTTT  AAAAAAAA  RRRRRR    TTTTTTTT
-- SS           TT     AA    AA  RR    RR     TT
-- SSSSSSSS     TT     AAAAAAAA  RRRRRR       TT
--       SS     TT     AA    AA  RR   RR      TT
-- SSSSSSSS     TT     AA    AA  RR    RR     TT
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level9:Start()
    print("PostState_Level9: Start")
    Post:Reset()
    PostState_Level9:Reset()
 
    Post:SetUpdateCallback(PostState_Level9.UpdateStart)   

    -----------------------------------------------
    -- Setup Colors 
    -----------------------------------------------      
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {0, 0.5, 0}
    ColorManager.Color2             = {0, 0.4, 0.3} --score
    ColorManager.Color3             = {0, 0.3, 0} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(0,0.6,0.6, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 5.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 100.0, .1) 
    Wind.mWindDeck:SetFrequency( 20.0 )
    
    RadEffects:SetGoodExplosionCallback(self.SpawnPurpleStars)
    
    RadEffects:SetEffectLayerToParticleAccumLayer()
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/SoftGlowGreen.particles", false)
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(0, .5, 0, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(0, .6, .2, 1)
    HUDGlow:EnableBumpin(true, 1.0)

    Post:EnableFullscreenEffects(false)
    
    -----------------------------------------------
    -- Setup Props
    -----------------------------------------------  
    local fRR = Constants.GameRingRadius

    PostState_Level9.mNumArcShapes = 36
    RadShapes:SetPropLayer( Renderer.mGameLayer )
    local arcSize = 2*math.pi/PostState_Level9.mNumArcShapes
    
    PostState_Level9.mArcShapeBatch = RadShapes:CreateNewShapeBatch(ShapeProp.DrawModeArc, PostState_Level9.mNumArcShapes) 
    
    for i = 1, PostState_Level9.mNumArcShapes do
        PostState_Level9.mArcShapeBatch:SetBatchIndex(i)
        PostState_Level9.mArcShapeBatch:SetType( ShapeProp.DrawModeArc, 1 )
        PostState_Level9.mArcShapeBatch:SetNumSides(1)
        PostState_Level9.mArcShapeBatch:SetColor(Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
        PostState_Level9.mArcShapeBatch:SetPosition( 0.0, 0.0, -15.0 )
        PostState_Level9.mArcShapeBatch:SetArcRadians( i * arcSize, i * arcSize + arcSize-0.1)
        local randomLength = math.sin(i*4)*0.5+0.5
        PostState_Level9.mArcShapeBatch:SetRadii( fRR, fRR * (1.2 +randomLength*0.4))
    end
end

function PostState_Level9.UpdateStart()
    totalTime = totalTime + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
      
    local powTime = math.pow(Constants.Heartbeat,3)
        
    local fRR = Constants.GameRingRadius * 1.02
    
    for i = 1, PostState_Level9.mNumArcShapes do
        local randomLength = math.sin(i*0.5+totalTime)*0.5+0.4
        PostState_Level9.mArcShapeBatch:SetBatchIndex(i)
        PostState_Level9.mArcShapeBatch:SetRadii( fRR, fRR * (1.1 + randomLength * 0.2))
        local colorValue1 = DFMath.lerp(Constants.Color1[1], Constants.Color2[1]*2, randomLength)
        local colorValue2 = DFMath.lerp(Constants.Color1[2], Constants.Color2[2]*2, randomLength)
        local colorValue3 = DFMath.lerp(Constants.Color1[3], Constants.Color2[3]*2, randomLength)
        PostState_Level9.mArcShapeBatch:SetColor( colorValue1, colorValue2, colorValue3, 1.0 )
    end  
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB   UU    UU  II  LL       DDDDD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBBB  UU    UU  II  LL       DD   DD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBB   UUUUUUUU  II  LLLLLLl  DDDDD
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------


function PostState_Level9:Build()
    print("PostState_Level9: Build")
    Post:Reset()
    PostState_Level9:Reset()

    totalTime = 0.0 
    Post:SetUpdateCallback(PostState_Level9.UpdateBuild)
    
    -----------------------------------------------
    -- Setup Colors 
    -----------------------------------------------      
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {0, 0.5, 0}
    ColorManager.Color2             = {0, 0.4, 0.3} --score
    ColorManager.Color3             = {0, 0.3, 0} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(0,0.6,0.6, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 5.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 300.0, .15) 
    Wind.mWindDeck:SetFrequency( 50.0 )
      
    RadEffects:SetGoodExplosionCallback(self.SpawnPurpleStars)
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/SoftGlowGreen.particles", false)
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(0, .5, 0, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(0, .6, .2, 1)
    HUDGlow:EnableBumpin(true, 1.5)
    
    Post:EnableFullscreenEffects(false)
    
    -----------------------------------------------
    -- Setup Props
    -----------------------------------------------  
    local fRR = Constants.GameRingRadius

    PostState_Level9.mNumArcShapes = 36
    RadShapes:SetPropLayer( Renderer.mGameLayer )
    local arcSize = 2*math.pi/PostState_Level9.mNumArcShapes

    PostState_Level9.mArcShapeBatch = RadShapes:CreateNewShapeBatch(ShapeProp.DrawModeArc, PostState_Level9.mNumArcShapes) 

    for i = 1, PostState_Level9.mNumArcShapes do
        PostState_Level9.mArcShapeBatch:SetBatchIndex(i)
        PostState_Level9.mArcShapeBatch:SetType( ShapeProp.DrawModeArc, 1 )
        PostState_Level9.mArcShapeBatch:SetNumSides( 1 )
        PostState_Level9.mArcShapeBatch:SetColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
        PostState_Level9.mArcShapeBatch:SetPosition( 0.0, 0.0, -15.0 )
        PostState_Level9.mArcShapeBatch:SetArcRadians( i * arcSize, i * arcSize + arcSize-0.1)
        local randomLength = math.sin(i*4)*0.5+0.5
        PostState_Level9.mArcShapeBatch:SetRadii( fRR, fRR * (1.2 +randomLength*0.4))
    end
end


function PostState_Level9.UpdateBuild()
    local level_intensity = Score.mLevelIntensity
    totalTime = totalTime + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
        
    local fRR = Constants.GameRingRadius * 1.02
    local curvedIntensity = math.pow(level_intensity,4)
    
    for i = 1, PostState_Level9.mNumArcShapes do
        PostState_Level9.mArcShapeBatch:SetBatchIndex(i)
        local randomLength = math.sin(i*0.5+(totalTime*(1+level_intensity*5)))*0.5+0.5
        PostState_Level9.mArcShapeBatch:SetRadii( fRR, fRR * (1.1 + randomLength * DFMath.lerp(0.2, 0.6, level_intensity) - (audioSmoothed * 5 * level_intensity)))
        local colorValue1 = DFMath.lerp(Constants.Color1[1], Constants.Color2[1]*2, randomLength)
        local colorValue2 = DFMath.lerp(Constants.Color1[2], Constants.Color2[2]*2, randomLength)
        local colorValue3 = DFMath.lerp(Constants.Color1[3], Constants.Color2[3]*2, randomLength)
        PostState_Level9.mArcShapeBatch:SetColor( colorValue1, colorValue2, colorValue3, 1.0 )
    end 
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB    OOOOOO  NN    NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NNNN  NN  UU    UU  SS      
-- BBBBBBBB  OO    OO NN NN NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NN  NNNN  UU    UU        SS
-- BBBBBBB    OOOOOO  NN   NNN  UUUUUUUU  SSSSSSSS
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level9:Bonus()
    print("PostState_Level9: Bonus")
    Post:Reset()
    PostState_Level9:Reset()

    totalTime = 0.0 
    Post:SetUpdateCallback(PostState_Level9.UpdateBonus)
    
    -----------------------------------------------
    -- Setup Colors 
    -----------------------------------------------      
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {0, 0.5, 0}
    ColorManager.Color2             = {0, 0.4, 0.3} --score
    ColorManager.Color3             = {0, 0.3, 0} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(0,0.6,0.6, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetPointWidth( 5.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 600.0, .2) 
    Wind.mWindDeck:SetFrequency( 50.0 )
    
    RadEffects:SetGoodExplosionCallback(self.SpawnPurpleStars)
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/SoftGlowGreen.particles", false)
    
    --blurs
    GlowRing:Enable(true)
    GlowRing:SetColor(0, .5, 0, 1)
    HUDGlow:Enable(true)
    HUDGlow:SetColor(0, .6, .2, 1)
    HUDGlow:EnableBumpin(true, 2.0)
    
    Post:EnableFullscreenEffects(false)
    
    -----------------------------------------------
    -- Setup Props
    -----------------------------------------------  
    local fRR = Constants.GameRingRadius

    PostState_Level9.mNumArcShapes = 36
    RadShapes:SetPropLayer( Renderer.mGameLayer )
    local arcSize = 2*math.pi/PostState_Level9.mNumArcShapes
    
    PostState_Level9.mArcShapeBatch = RadShapes:CreateNewShapeBatch(ShapeProp.DrawModeArc, PostState_Level9.mNumArcShapes)

    for i = 1, PostState_Level9.mNumArcShapes do
        PostState_Level9.mArcShapeBatch:SetBatchIndex(i)
        PostState_Level9.mArcShapeBatch:SetType( ShapeProp.DrawModeArc, 1 )
        PostState_Level9.mArcShapeBatch:SetNumSides( 1 )
        PostState_Level9.mArcShapeBatch:SetColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
        PostState_Level9.mArcShapeBatch:SetPosition( 0.0, 0.0, -15.0 )
        PostState_Level9.mArcShapeBatch:SetArcRadians( i * arcSize, i * arcSize + arcSize-0.1)
        local randomLength = math.sin(i*4)*0.5+0.5
        PostState_Level9.mArcShapeBatch:SetRadii( fRR, fRR * (1.2 +randomLength*0.4))
    end
end


function PostState_Level9.UpdateBonus()
    totalTime = totalTime + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)

    local randomfloat = math.random(-100,100)
    local xParam = math.cos(randomfloat) * 350
    local yParam = math.sin(randomfloat) * 350
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    
    local powTime = math.pow(Constants.Heartbeat,3)
    
    currentHeartbeatDelta = Constants.Heartbeat - lastHeartbeat
    if currentHeartbeatDelta < 0 then -- HEART JUST SKIPPED A BEAT, LOL (increment this every time a new beat hits)
        beatCounter = beatCounter + 0.5
    end
    lastHeartbeat = Constants.Heartbeat 
    
    local fRR = Constants.GameRingRadius * 1.02
    
    for i = 1, PostState_Level9.mNumArcShapes do
        PostState_Level9.mArcShapeBatch:SetBatchIndex(i)
        local randomLength = math.sin(i*0.5+(totalTime*6))*0.5+0.5
        PostState_Level9.mArcShapeBatch:SetRadii( fRR, fRR * (1.1 + randomLength * 1.0 * (1-powTime)))
        local colorValue1 = DFMath.lerp(Constants.Color1[1], Constants.Color2[1]*2, randomLength)
        local colorValue2 = DFMath.lerp(Constants.Color1[2], Constants.Color2[2]*2, randomLength)
        local colorValue3 = DFMath.lerp(Constants.Color1[3], Constants.Color2[3]*2, randomLength)
        PostState_Level9.mArcShapeBatch:SetColor( colorValue1, colorValue2, colorValue3, 1.0 )
    end        
end

function PostState_Level9.SpawnPurpleStars(posX, posY, posZ, rTargetProp)
    local rExplosion = RadEffects.SpawnPurpleStars(posX, posY, posZ, rTargetProp, PostState_Level9.BubbleChamberTargetCallback)
    
    local rStarEffect = rExplosion.rStarEffect
    local rStarEffectProp = rStarEffect:getProp()
    local rStarEffectMaterial = rStarEffectProp:getMaterial()
    rStarEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rTracerEffect = rExplosion.rTracerEffect
    local rTracerEffectProp = rTracerEffect:getProp()
    local rTracerEffectMaterial = rTracerEffectProp:getMaterial()
    rTracerEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rRippleEffect = rExplosion.rRippleEffect
    local rRippleEffectProp = rRippleEffect:getProp()
    local rRippleEffectMaterial = rRippleEffectProp:getMaterial()
    rRippleEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rSeekingEffect = rExplosion.rSeekingEffect
    local rSeekingEffectProp = rSeekingEffect:getProp()
    local rSeekingEffectMaterial = rSeekingEffectProp:getMaterial()
    rSeekingEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)

    return rExplosion
end


function PostState_Level9.BubbleChamberTargetCallback( tParticleEffect )
    local Explosion = {}
    if tParticleEffect ~= nil then
        if tParticleEffect.rTargetProp ~= nil then
            local fTargetX, fTargetY, fTargetZ = tParticleEffect.rTargetProp:getLoc()
            RadEffects:SetEffectTransform( fTargetX, fTargetY, fTargetZ, 0.0, 0.0, 0.0 )   
            RadEffects:SetEffectLayer( RadEffects.tRenderer.mGameLayer )
            local rEffect = RadEffects:CreateEffect( "Effects/Particles/Bursts/BlipFlare.particles", nil, nil, nil, Constants.ParticlePriorityLow )                 
            Explosion.rEffect = rEffect
            
            local rEffectProp = rEffect:getProp()
            local rEffectMaterial = rEffectProp:getMaterial()
            rEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
        end
    end
    
    return Explosion
end

return PostState_Level9
