local Post = require('postfx.Post')
local Renderer = require('Renderer')
local Constants = require('Constants')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local RadShapes = require('RadShapes')

local PostStateCircles = {}

function PostStateCircles:Start()
    print("PostStateCircles: Start")
    local x = 1
    local y = 1
    
    Post:Reset()
        
    Post:SetUpdateCallback(PostStateCircles.ShapesUpdate) 
    
    -----------------------------------------------
    -- Setup Colors 
    -----------------------------------------------      
    ColorManager.BGColor            = {0, 0.5, 1}
    ColorManager.Color1             = {1, 0, 0}
    ColorManager.Color2             = {0, 0, 1}         --score
    ColorManager.Color3             = {0.5, 0.1, 0.5}   --multiplier
    ColorManager.SpecialColor       = {1, 1, 1}
    ColorManager:SetColor()
    
    local finalColorBuffer = Post:GetUnusedBuffer()
    Post:GenerateBasicCompFilter(finalColorBuffer)
    Post.rScreen:setInBuffer(finalColorBuffer)
    
    -----------------------------------------------
    -- Setup Props
    -----------------------------------------------  
    --circles    
    RadShapes:SetPropLayer( Renderer.mBackgrounLayer )
    
    PostStateCircles.mCircleRadius = Constants.GameRingRadius * 0.25
    PostStateCircles.mCircleDiam = PostStateCircles.mCircleRadius * 2.0
    
    PostStateCircles.mNumCirclesX = math.floor( Constants.ValidDrawWidth / PostStateCircles.mCircleDiam) + 3
    PostStateCircles.mNumCirclesY = math.floor( Constants.ValidDrawHeight / PostStateCircles.mCircleDiam) + 3
    
    PostStateCircles.mNumCirclesInLayer = PostStateCircles.mNumCirclesX * PostStateCircles.mNumCirclesY
    local xBasePos = -((PostStateCircles.mNumCirclesX * PostStateCircles.mCircleDiam) * 0.5)
    local yBasePos = -((PostStateCircles.mNumCirclesY * PostStateCircles.mCircleDiam) * 0.5)
            
    PostStateCircles.mBackCircles = {}
    for x = 0, PostStateCircles.mNumCirclesX-1 do
        for y = 0, PostStateCircles.mNumCirclesY-1 do
            local xPos = xBasePos + (x * PostStateCircles.mCircleDiam)
            local yPos = yBasePos + (y * PostStateCircles.mCircleDiam)
            
            local i = ((y * PostStateCircles.mNumCirclesX) + x) + 1
            PostStateCircles.mBackCircles[i] = RadShapes:CreateNewShape() 
            PostStateCircles.mBackCircles[i]:SetType( ShapeProp.DrawModeCircle, 16 )
            PostStateCircles.mBackCircles[i]:SetColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )
            PostStateCircles.mBackCircles[i]:SetPosition( xPos, yPos, -20.0 )
            PostStateCircles.mBackCircles[i]:SetRadii( PostStateCircles.mCircleRadius-10.0, PostStateCircles.mCircleRadius )
        end     
    end     
    
    PostStateCircles.mFrontCircles = {}
    for x = 0, PostStateCircles.mNumCirclesX-1 do
        for y = 0, PostStateCircles.mNumCirclesY-1 do            
            local xPos = xBasePos + (x * PostStateCircles.mCircleDiam)
            local yPos = yBasePos + (y * PostStateCircles.mCircleDiam)
            
            local i = ((y * PostStateCircles.mNumCirclesX) + x) + 1
            PostStateCircles.mFrontCircles[i] = RadShapes:CreateNewShape() 
            PostStateCircles.mFrontCircles[i]:SetType( ShapeProp.DrawModeCircle, 16 )
            PostStateCircles.mFrontCircles[i]:SetColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1.0 )
            PostStateCircles.mFrontCircles[i]:SetPosition( xPos, yPos, -20.0 )
            PostStateCircles.mFrontCircles[i]:SetRadii( PostStateCircles.mCircleRadius-10.0, PostStateCircles.mCircleRadius )
        end     
    end    
    
    --movement variables
    PostStateCircles.mMoveTimer = 0.0
    PostStateCircles.mMoveDuration = Constants.BeatDuration * 2.0
    
    PostStateCircles.mAllowMovement = true
    PostStateCircles.mMoveStateNone = 0
    PostStateCircles.mMoveStateMoving = 1
    PostStateCircles.mMoveState = PostStateCircles.mMoveStateNone
    PostStateCircles.mMoveIndex = 0
    PostStateCircles.mMoveXTarget = 0.0
    PostStateCircles.mMoveYTarget = 0.0
    
    PostStateCircles.mCircleWidth = PostStateCircles.mCircleRadius * 0.5
    PostStateCircles.mCircleGrowAmount = PostStateCircles.mCircleWidth * 0.75
    PostStateCircles.mGrowTimer = 0.0
    PostStateCircles.mGrowScalar = 2.0
end

function PostStateCircles:UpdateMovementState()
    PostStateCircles.mMoveTimer = PostStateCircles.mMoveTimer - Constants.FrameStep
    
    if PostStateCircles.mMoveTimer <= 0.0 then
        if PostStateCircles.mMoveState == PostStateCircles.mMoveStateNone and PostStateCircles.mAllowMovement then
            if PostStateCircles.mMoveIndex == 0 then
                PostStateCircles.mMoveXTarget = 0.0
                PostStateCircles.mMoveYTarget = -PostStateCircles.mCircleDiam
            elseif PostStateCircles.mMoveIndex == 1 then
                PostStateCircles.mMoveXTarget = -PostStateCircles.mCircleDiam
                PostStateCircles.mMoveYTarget = -PostStateCircles.mCircleDiam            
            elseif PostStateCircles.mMoveIndex == 2 then
                PostStateCircles.mMoveXTarget = -PostStateCircles.mCircleDiam     
                PostStateCircles.mMoveYTarget = 0.0
            elseif PostStateCircles.mMoveIndex == 3 then
                PostStateCircles.mMoveXTarget = -PostStateCircles.mCircleDiam
                PostStateCircles.mMoveYTarget = PostStateCircles.mCircleDiam            
            elseif PostStateCircles.mMoveIndex == 4 then
                PostStateCircles.mMoveXTarget = 0.0
                PostStateCircles.mMoveYTarget = PostStateCircles.mCircleDiam
            elseif PostStateCircles.mMoveIndex == 5 then
                PostStateCircles.mMoveXTarget = PostStateCircles.mCircleDiam
                PostStateCircles.mMoveYTarget = PostStateCircles.mCircleDiam            
            elseif PostStateCircles.mMoveIndex == 6 then
                PostStateCircles.mMoveXTarget = PostStateCircles.mCircleDiam
                PostStateCircles.mMoveYTarget = 0.0                
            elseif PostStateCircles.mMoveIndex == 7 then
                PostStateCircles.mMoveXTarget = PostStateCircles.mCircleDiam
                PostStateCircles.mMoveYTarget = -PostStateCircles.mCircleDiam                
            end
            
            PostStateCircles.mMoveState = PostStateCircles.mMoveStateMoving
        else
            PostStateCircles.mMoveState = PostStateCircles.mMoveStateNone
            PostStateCircles.mMoveXTarget = 0.0
            PostStateCircles.mMoveYTarget = 0.0
                
            PostStateCircles.mMoveIndex = PostStateCircles.mMoveIndex + 1
            if PostStateCircles.mMoveIndex > 7 then
                PostStateCircles.mMoveIndex = 0
            end
        end      
        
        PostStateCircles.mMoveTimer = PostStateCircles.mMoveTimer + PostStateCircles.mMoveDuration
    end    
end

function PostStateCircles.ShapesUpdate()
    local x = 0
    local y = 0
    
    PostStateCircles:UpdateMovementState()
    
    PostStateCircles.mGrowTimer = PostStateCircles.mGrowTimer + (Constants.FrameStep * PostStateCircles.mGrowScalar)
    
    local fTimerRatio = PostStateCircles.mMoveTimer / PostStateCircles.mMoveDuration
    local fXOffset = DFMath.lerp( 0.0, PostStateCircles.mMoveXTarget, fTimerRatio )
    local fYOffset = DFMath.lerp( 0.0, PostStateCircles.mMoveYTarget, fTimerRatio )
    
    local xBasePos = -((PostStateCircles.mNumCirclesX * PostStateCircles.mCircleDiam) * 0.5)
    local yBasePos = -((PostStateCircles.mNumCirclesY * PostStateCircles.mCircleDiam) * 0.5) 
    
    local fSinTimer = math.sin( PostStateCircles.mGrowTimer )
    local fNormalizedTimer = (fSinTimer*0.5) + 0.5

    local fGrowAmount = DFMath.lerp( -PostStateCircles.mCircleGrowAmount, PostStateCircles.mCircleGrowAmount, fNormalizedTimer )
    local fBigRadius = PostStateCircles.mCircleRadius + fGrowAmount
    local fSmallRadius = PostStateCircles.mCircleRadius - fGrowAmount
    
    for x = 0, PostStateCircles.mNumCirclesX-1 do
        for y = 0, PostStateCircles.mNumCirclesY-1 do            
            local xPos = xBasePos + (x * PostStateCircles.mCircleDiam) + fXOffset
            local yPos = yBasePos + (y * PostStateCircles.mCircleDiam) + fYOffset
            
            local i = ((y * PostStateCircles.mNumCirclesX) + x) + 1
            PostStateCircles.mFrontCircles[i]:SetColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1.0 )
            PostStateCircles.mFrontCircles[i]:SetPosition( xPos, yPos, -20.0 )
            
            local fDrawRadius = fBigRadius
            if i % 2 == 0 then
                fDrawRadius = fSmallRadius
            end
            
            PostStateCircles.mFrontCircles[i]:SetRadii( fDrawRadius-PostStateCircles.mCircleWidth, fDrawRadius )
            PostStateCircles.mBackCircles[i]:SetRadii( fDrawRadius-PostStateCircles.mCircleWidth, fDrawRadius )
        end     
    end 
end

return PostStateCircles