local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local Wind = require('Wind')
local Score = require('Score')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local ParticleSystem = require("ParticleSystem")
local RadEffects = require('RadEffects')
local PostState_Leaderboards= {}

local crossBlurFilter= nil
local noiseFilter = nil
local noiseFilter2 = nil
local energyFilter = nil
local uiBlitFilter = nil
local sceneOverlayFilter = nil
local gameBlitFilter = nil
local totalTime = 0.0
local audioSmoothed = 0.0

function PostState_Leaderboards:Reset()
     crossBlurFilter= nil
     noiseFilter = nil
     noiseFilter2 = nil
     energyFilter = nil
     uiBlitFilter = nil
     sceneOverlayFilter = nil
     gameBlitFilter = nil
end

function PostState_Leaderboards.Update()
    local level_intensity = .5

    totalTime = totalTime + Constants.FrameStep
    local fTimeMax = 3.0
    if totalTime > fTimeMax then
        totalTime = totalTime - fTimeMax
    end
    
    if crossBlurFilter then 
        crossBlurFilter:setShaderValue( "g_fBlurWidth", MOAIMaterial.VALUETYPE_FLOAT, Constants.Heartbeat * .005 + .003)
    end
    
    if noiseFilter then
        noiseFilter:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, totalTime * 1)
    end
        
    if noiseFilter2 then
        noiseFilter2:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, totalTime * 1.3)
    end
    
    if sceneOverlayFilter then
        sceneOverlayFilter:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4,  {.05 * level_intensity,.0 * level_intensity,.1*level_intensity,1})
    end
end

function PostState_Leaderboards:Start()
    print("PostState_Leaderboards: Start")
    Post:Reset()
    PostState_Leaderboards:Reset()
    
    Post:SetUpdateCallback(PostState_Leaderboards.Update)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.25, .24, .7}
    ColorManager.Color2             = {.35, .2, .05} --score
    ColorManager.Color3             = {.0, .34, .2} --multiplier
    ColorManager:SetColor()
    
    ChargeField:Enable(false)
    ChargeField:SetColor(.1,.2,.1,1)
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 3.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 0.0, 5.0 )
    ChargeField.mChargeFieldDeck:SetActivatedDistance( 120.0 )
        
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local lastFrameGridBuffer = Post:GetUnusedBuffer()
    local lastFrameGridBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
   
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    lastFrameGridBuffer:setClearColor()
   
    Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchBuffer, {.95,.92,.85,1}, {.99,.99}, {0.0,0.0})
    Post:BlurFilter(scratchBuffer, accumBuffer, .002) -- save off the acumm buffer

    sceneOverlayFilter = Post:BlitFilter(sceneBuffer, accumBuffer, kBLEND_ADD, {.05,.1,.2,1}, {.8,.8}, {0,0})    
    --noiseFilter = Post:TVNoiseFilter(sceneBuffer, accumBuffer, 0.0, kBLEND_ALPHA, {.3,.0,.4,.02})    
    
    --Post:CombineFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer)
    --Post:BlitFilter(scratchBuffer, sceneBuffer, kBLEND_ADD, {1,1,1,1})
    Post:LeaderboardAFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, scratchBuffer, sceneBuffer)
    
    --Post:BlurFilter(sceneBuffer, finalColorBuffer, .002, kBLEND_ADD, {.2,.2,2,1})    
    --Post:BlitFilter(scratchBuffer, finalColorBuffer, kBLEND_ADD, {1,1,1,1})  
    --Post:BlitFilter(Renderer.mUIBuffer, finalColorBuffer, kBLEND_ADD)
    --Post:EdgeFilter(Renderer.mUIBuffer, finalColorBuffer, 1, kBLEND_ADD, {1,1,1,1})  
    --Post:BlurFilter(accumBuffer, finalColorBuffer, .004, kBLEND_ADD, {0,1,3,1})    
    --noiseFilter2 = Post:TVNoiseFilter(sceneBuffer, finalColorBuffer, 0.0, kBLEND_ADD, {.03,.02,.05,1})    
    noiseFilter2 = Post:LeaderboardBFilter(sceneBuffer, scratchBuffer, Renderer.mUIBuffer, accumBuffer, finalColorBuffer)

    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
    
    RadEffects:SetGoodExplosionCallback(RadEffects.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer
    
    Post.rScreen:setInBuffer(finalColorBuffer)
end


return PostState_Leaderboards