local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local AudioBars = require('AudioBars')
local Wind = require('Wind')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local ParticleSystem = require("ParticleSystem")
local RadEffects = require('RadEffects')
local InputVisuals = require('InputVisuals')
local Score = require('Score')
local PostState_Level1= {}

local accumFilter = nil
local warpFilter = nil
local differenceFilter = nil
local blurFilter = nil
local directionalBlurFilter1 = nil
local audioSmoothed = 0.0
local bonusIntensityRamp = 0.0
local gameBlitFilter = nil
local uiBlitFilter = nil
local totalTime = 0.0
local crossBlurFilter = nil

function PostState_Level1:Reset()
    accumFilter = nil
    warpFilter = nil
    differenceFilter = nil
    blurFilter = nil
    directionalBlurFilter1 = nil
    bonusIntensityRamp = 0.0
    gameBlitFilter = nil
    uiBlitFilter = nil
    totalTime = 0.0
    crossBlurFilter = nil
end



------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- SSSSSSSS  TTTTTTTT  AAAAAAAA  RRRRRR    TTTTTTTT
-- SS           TT     AA    AA  RR    RR     TT
-- SSSSSSSS     TT     AAAAAAAA  RRRRRR       TT
--       SS     TT     AA    AA  RR   RR      TT
-- SSSSSSSS     TT     AA    AA  RR    RR     TT
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level1:Start()
    print("PostState_Level1: Start")
    Post:Reset()
    PostState_Level1:Reset()
    
    Post:SetUpdateCallback(PostState_Level1.UpdateStart)   
    
    ColorManager.BGColor            = {0.0,.00,.00}
    ColorManager.Color1             = {.8, .3, .1}
    ColorManager.Color2             = {.75, .2, .1} --score
    ColorManager.Color3             = {.1, .11, .2} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Grid:SetColor(.02,.01,.03, 1)
    Grid:Enable(true)
    Grid.mGridDeck:SetPointWidth( 2 )
    Grid.mGridDeck:SetSpringK( 1.0 )
    Grid.mGridDeck:SetSpringDampen( 0.008 )
    
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local blurSourceBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local scratchEightBuffer2 = Post:GetUnusedEighthBuffer()
    local particleBuffer = Post:GetUnusedBuffer()
    
    particleBuffer:setClearColor()
    
    --Post:AccumFilter(Post.rBlackTexture, particleBuffer, scratchBuffer, {.85,.90,.92,1}, {1.00,1.00}, {0.0,0.0}) 
    --warpFilter = Post:WarpFilter(Renderer.mBackgroundBuffer, scratchBuffer, .65, kBLEND_ADD, {1,1,1,1})  
    --uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlitFilter(Renderer.mGameBuffer, sceneBuffer, kBLEND_ADD)   
    --Post:WarpFilter(scratchBuffer, particleBuffer, .05, kBLEND_ALPHA, {1,1,1,1}, 1)    
    --Post:BlurFilter(sceneBuffer, blurSourceBuffer, .005, kBLEND_ADD, {2,2,2,1}) 
    --Post:BlitFilter(particleBuffer, blurSourceBuffer, kBLEND_ADD, {1,1,1,1})     
    --blurFilter = Post:BlurFilter(blurSourceBuffer, scratchQuarterBuffer, .001, kBLEND_ADD, {1,1,1,1})
    --Post:DirectionalBlurFilter(scratchQuarterBuffer, scratchEightBuffer2, {.01, .0}, kBLEND_ADD, {1,1,1,1})
    --directionalBlurFilter1 = Post:DirectionalBlurFilter(scratchQuarterBuffer, finalColorBuffer, {.01, .0}, kBLEND_ADD, {.4,.4,.4,1})
    --Post:BlitFilter(Renderer.mBackgroundBuffer, finalColorBuffer, kBLEND_ADD, {1,1,1,1})
    --Post:BlitFilter(sceneBuffer, finalColorBuffer, kBLEND_ADD)
    --Post:EdgeFilter(sceneBuffer, finalColorBuffer, 1, kBLEND_ADD, {2,2,2,2})
    --Post:BlitFilter(particleBuffer, finalColorBuffer, kBLEND_ADD)   
       
    warpFilter = Post:Level1AFilter(Post.rBlackTexture, particleBuffer, Renderer.mBackgroundBuffer, scratchBuffer)
    uiBlitFilter = Post:Level1BFilter(Renderer.mUIBuffer, Renderer.mGameBuffer, sceneBuffer)
    Post:WarpFilter(scratchBuffer, particleBuffer, .05, kBLEND_ALPHA, {1,1,1,1}, 1) 
    Post:Level1CFilter(sceneBuffer, particleBuffer, blurSourceBuffer)   
    blurFilter = Post:BlurFilter(blurSourceBuffer, scratchQuarterBuffer, .001, kBLEND_ADD, {1,1,1,1})
    directionalBlurFilter1 = Post:Level1DFilter(scratchQuarterBuffer, Renderer.mBackgroundBuffer, sceneBuffer, particleBuffer, finalColorBuffer)
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(particleBuffer)
    
    RadEffects:SetGoodExplosionCallback(RadEffects.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer
  
    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostState_Level1.UpdateStart()
    local level_intensity = Score.mLevelIntensity
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    if blurFilter then
        blurFilter:setShaderValue("g_fBlurWidth", MOAIMaterial.VALUETYPE_FLOAT, audioSmoothed*.1)
    end  
    
    if directionalBlurFilter1 then
        directionalBlurFilter1:setShaderValue("g_vBlurDirection", MOAIMaterial.VALUETYPE_VEC2, {audioSmoothed,0})
        directionalBlurFilter1:setShaderValue("g_vColor", MOAIMaterial.VALUETYPE_VEC4, {.4 + audioSmoothed*10,.4 + audioSmoothed*10,.4,1})
    end
    
    if warpFilter then
        warpFilter:setShaderValue( "g_fWarpIntensity", MOAIMaterial.VALUETYPE_FLOAT,  level_intensity * (1.5 - 40*audioSmoothed) + 0.65 )
    end
    
    local gridColor = level_intensity*.02
    Grid:SetColor(.02 + gridColor*.5, .01 + gridColor,.03 + gridColor*2, 1)
    Grid.mGridDeck:SetPointWidth( 100 *  audioSmoothed + 1)
    Grid.mGridDeck:SetSpringK( .5 + level_intensity*.5  )
    
     if uiBlitFilter then
        uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1, 1 + audioSmoothed*1.5 * level_intensity})
    end
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB   UU    UU  II  LL       DDDDD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBBB  UU    UU  II  LL       DD   DD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBB   UUUUUUUU  II  LLLLLLl  DDDDD
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level1:Build()
    print("PostState_Level1: Build")
    Post:Reset()
    PostState_Level1:Reset()

    Post:SetUpdateCallback(PostState_Level1.UpdateBuild)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {1, .54, .3}
    ColorManager.Color2             = {.6, .2, .1} --score
    ColorManager.Color3             = {.1, .11, .2} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Grid:SetColor(.4,.2,.65, 1)
    Grid:Enable(true)
    Grid.mGridDeck:SetPointWidth( 1.0 )
    Grid.mGridDeck:SetSpringK( 10.0 )
    Grid.mGridDeck:SetSpringDampen( 0.125 )
        
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedHalfBuffer()
    local scratchBuffer2 = Post:GetUnusedQuarterBuffer() 
    local differenceBuffer = Post:GetUnusedBuffer()
    local lastFrameGridBuffer = Post:GetUnusedBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    lastFrameGridBuffer:setClearColor()

    RadEffects:SetEffectTransform( 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 )
    RadEffects:SetEffectLayerToParticleAccumLayer() 
    RadEffects:CreateInfiniteDurationEffect( "Effects/Particles/Backgrounds/SwirlyStarsB.particles")
   
    --Post:BlitFilter(Renderer.mBackgroundBuffer, lastFrameGridBuffer)
    --Post:DifferenceFilter(Renderer.mBackgroundBuffer, lastFrameGridBuffer, differenceBuffer)  
    --Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchBuffer, {.8,.85,.9,1}, {1,1}, {0.0,-.002})   
    --warpFilter = Post:WarpFilter(scratchBuffer, accumBuffer, .05, kBLEND_ALPHA, {1,1,1,1})      
    --Post:BlitFilter(differenceBuffer, accumBuffer, kBLEND_ADD, {1,1,1,1})        
    --uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlitFilter(Renderer.mGameBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlitFilter(scratchBuffer, sceneBuffer, kBLEND_ADD, {1,1,1,1}) 
    --Post:BlurFilter(sceneBuffer, scratchBuffer2, 0.002, kBLEND_ADD, {2,2,2,1})     
    --crossBlurFilter = Post:CrossBlurFilter(scratchBuffer2, finalColorBuffer, .008, kBLEND_ADD, {.2,.15,.1,1})       
    --Post:BlitFilter(sceneBuffer, finalColorBuffer, kBLEND_ADD, {1,1,1,1})  
    --Post:BlurFilter(accumBuffer, finalColorBuffer, .01, kBLEND_ADD, {1,1,1,1})    
    

    Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchBuffer, {.8,.85,.9,1}, {1,1}, {0.0,-.002})   
    warpFilter = Post:WarpFilter(scratchBuffer, accumBuffer, .05, kBLEND_ALPHA, {1,1,1,1})      
    uiBlitFilter = Post:Level1EFilter(Renderer.mUIBuffer, Renderer.mGameBuffer, scratchBuffer, sceneBuffer)   
    Post:BlurFilter(sceneBuffer, scratchBuffer2, 0.002, kBLEND_ADD, {2,2,2,1})     
    crossBlurFilter = Post:Level1FFilter(scratchBuffer2, sceneBuffer, accumBuffer, finalColorBuffer)
    
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
    
    RadEffects:SetGoodExplosionCallback(RadEffects.SpawnBubbleChamberBuild)
    RadEffects.particleAccumLayer = particleLayer
    
    Post.rScreen:setInBuffer(finalColorBuffer) 
end 

function PostState_Level1.UpdateBuild()
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)   
    local level_intensity = Score.mLevelIntensity
    
    if warpFilter then
        warpFilter:setShaderValue( "g_fWarpIntensity", MOAIMaterial.VALUETYPE_FLOAT,  level_intensity*.1 + .05)
    end

    if crossBlurFilter then
        crossBlurFilter:setShaderValue( "g_fBlurWidth", MOAIMaterial.VALUETYPE_FLOAT, audioSmoothed * .75 + level_intensity*.001)
    end
    
    if uiBlitFilter then
        uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1, 1 + audioSmoothed*3})
    end
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB    OOOOOO  NN    NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NNNN  NN  UU    UU  SS      
-- BBBBBBBB  OO    OO NN NN NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NN  NNNN  UU    UU        SS
-- BBBBBBB    OOOOOO  NN   NNN  UUUUUUUU  SSSSSSSS
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level1:Bonus()
    print("PostState_Level1: Bonus")
    Post:Reset()
    PostState_Level1:Reset()
    
    bonusIntensityRamp = 2.0
    
    Post:SetUpdateCallback(PostState_Level1.UpdateBonus)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {1, .54, .3}
    ColorManager.Color2             = {.6, .2, .1} --score
    ColorManager.Color3             = {.1, .11, .2} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Grid:SetColor(.2,.6,.65, 1)
    Grid:Enable(true)
    Grid.mGridDeck:SetPointWidth( 1.0 )
    Grid.mGridDeck:SetSpringK( 1.0 )
    Grid.mGridDeck:SetSpringDampen( 0.002 )
        
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local differenceBuffer = Post:GetUnusedBuffer()
    local lastFrameGridBuffer = Post:GetUnusedBuffer()
    local remappedFinalColorBuffer = Post:GetUnusedBuffer()
   
    --local particleBuffer = Post:GetUnusedBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    lastFrameGridBuffer:setClearColor()

    RadEffects:SetEffectTransform( 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 )
    RadEffects:SetEffectLayerToParticleAccumLayer()
    RadEffects:CreateInfiniteDurationEffect( "Effects/Particles/Backgrounds/SwirlyStarsB.particles")
    
    --Post:BlitFilter(Renderer.mBackgroundBuffer, lastFrameGridBuffer)
    --Post:DifferenceFilter(Renderer.mBackgroundBuffer, lastFrameGridBuffer, differenceBuffer)  
    --Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchBuffer, {.9,.9,.9,1}, {1,1}, {0.0, -.002})
    --Post:BlitFilter(scratchBuffer, accumBuffer)
    --Post:BlitFilter(differenceBuffer, accumBuffer, kBLEND_ADD, {1,1,1,1})    
    --gameBlitFilter = Post:BlitFilter(Renderer.mGameBuffer, sceneBuffer)
    --uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer, sceneBuffer)   
    --Post:BlitFilter(scratchBuffer, sceneBuffer, kBLEND_ADD, {0,.3,1,1})
    --Post:BlurFilter(sceneBuffer, scratchBuffer, .003, kBLEND_ADD, {2,2,2,1})   
    --crossBlurFilter = Post:CrossBlurFilter(scratchBuffer, finalColorBuffer, .008, kBLEND_ADD, {.2,.15,.1,1})        
    --Post:BlitFilter(scratchBuffer, finalColorBuffer, kBLEND_ADD, {1,1,1,1})     
    --warpFilter = Post:WarpFilter(accumBuffer, finalColorBuffer, .8, kBLEND_ADD, {1,1,1,1})          
    --Post:BlitFilter(sceneBuffer, finalColorBuffer, kBLEND_ADD)   
    --Post:RemapFilter(finalColorBuffer, remappedFinalColorBuffer, Post.rGradientTextureC, 0, kBLEND_ALPHA, {1,1,1,1})
    
    Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchBuffer, {.9,.9,.9,1}, {1,1}, {0.0, -.002})
    Post:BlitFilter(scratchBuffer, accumBuffer)
    uiBlitFilter = Post:Level1GFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, scratchBuffer, sceneBuffer)
    Post:BlurFilter(sceneBuffer, scratchBuffer, .003, kBLEND_ADD, {2,2,2,1})   
    crossBlurFilter = Post:Level1HFilter(scratchBuffer, accumBuffer, sceneBuffer, finalColorBuffer) 
    Post:RemapFilter(finalColorBuffer, remappedFinalColorBuffer, Post.rGradientTextureC, 0, kBLEND_ALPHA, {1,1,1,1})
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
    
    RadEffects:SetGoodExplosionCallback(RadEffects.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer
    
    Post.rScreen:setInBuffer(remappedFinalColorBuffer)
end

function PostState_Level1.UpdateBonus()
    
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    totalTime = totalTime + Constants.FrameStep
        
    bonusIntensityRamp = bonusIntensityRamp - .03
    bonusIntensityRamp = DFMath.pin(bonusIntensityRamp,0,1)
    if warpFilter then
        warpFilter:setShaderValue( "g_fWarpIntensity", MOAIMaterial.VALUETYPE_FLOAT,  audioSmoothed*.05 + bonusIntensityRamp + .8 )
    end  
    
    if crossBlurFilter then
        crossBlurFilter:setShaderValue( "g_fBlurWidth", MOAIMaterial.VALUETYPE_FLOAT, bonusIntensityRamp*.01 )
        crossBlurFilter:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, {bonusIntensityRamp*2, bonusIntensityRamp*2, bonusIntensityRamp*2, 1} )
        crossBlurFilter:setShaderValue( "g_fWarpIntensity", MOAIMaterial.VALUETYPE_FLOAT,  audioSmoothed*.05 + bonusIntensityRamp + .8 )
    end
    
    local blitIntensity = .7 + (math.sin(totalTime*30) + 1) * .3
    uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1, 1 + audioSmoothed*10})
    uiBlitFilter:setShaderValue("g_vColor", MOAIMaterial.VALUETYPE_VEC4, {blitIntensity,blitIntensity,blitIntensity,blitIntensity})
    uiBlitFilter:setShaderValue( "g_vScale2", MOAIMaterial.VALUETYPE_VEC2, {1 - audioSmoothed*10, 1 - audioSmoothed*10})
    end


    
return PostState_Level1
