local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local Wind = require('Wind')
local Score = require('Score')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local ParticleSystem = require("ParticleSystem")
local RadEffects = require('RadEffects')
local InputVisuals = require('InputVisuals')
local RadShapes = require('RadShapes')
local PostState_Level10 = {}

local accumFilter = nil
local uiBlitFilter = nil
local goodEffectColor = {.25, .25, 1}
local totalTime = 0.0
local audioSmoothed = 0.0

function PostState_Level10:Reset()
     accumFilter = nil
     uiBlitFilter = nil
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- SSSSSSSS  TTTTTTTT  AAAAAAAA  RRRRRR    TTTTTTTT
-- SS           TT     AA    AA  RR    RR     TT
-- SSSSSSSS     TT     AAAAAAAA  RRRRRR       TT
--       SS     TT     AA    AA  RR   RR      TT
-- SSSSSSSS     TT     AA    AA  RR    RR     TT
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level10:Start()
    print("PostState_Level10: Start")
    Post:Reset()
    PostState_Level10:Reset()
    
    Post:SetUpdateCallback(PostState_Level10.UpdateStart)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {0.5,0.5,1}
    ColorManager.Color2             = {0.5,0.5,1} --score
    ColorManager.Color3             = {.1, .1, .4} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
 
    Wind:Enable(true)
    Wind:SetColor(1,1,1, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMax[2], -10, tMax[1], tMax[2], -10 )
    Wind.mWindDeck:SetPointWidth( 8.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 500.0, 200.0, .2) 
    Wind.mWindDeck:SetFrequency( 10.0 )
      
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local lastFrameGridBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()

    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(sceneBuffer)
    
    --Post:BlitFilter(Renderer.mGameBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlitFilter(Renderer.mUIBuffer, sceneBuffer, kBLEND_ADD)    
    --Post:BlurFilter(sceneBuffer, scratchBuffer, .005, kBLEND_ADD, {0.7,0.1,0.5,1})  
    --Post:DirectionalBlurFilter(scratchBuffer, lastFrameGridBuffer, {.01,0}, kBLEND_ADD, {0.2,0.2,0.2,1})     
    --Post:BlitFilter(lastFrameGridBuffer, accumBuffer, kBLEND_ADD)
    --Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {0,0,0,1}, {.97,.97}, {0.0,0.0})
    --Post:CombineFilter(lastFrameGridBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )    
    --Post:EdgeFilter(sceneBuffer, finalColorBuffer, 1, kBLEND_ADD, {1,0.7,1,1})
    --Post:BlitFilter(lastFrameGridBuffer, finalColorBuffer, kBLEND_ADD)
    --Post:BlitFilter(scratchQuarterBuffer, finalColorBuffer, kBLEND_ADD)
    
    Post:Level7Filter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, {1,1}, {1,1}) 
    Post:BlurFilter(sceneBuffer, lastFrameGridBuffer, .005, kBLEND_ADD, {0.14,0.02,0.1,1})  
    Post:Level7BFilter(lastFrameGridBuffer, accumBuffer, scratchQuarterBuffer, {0,0,0,1}, {.97,.97}, 1)
    Post:CombineFilter(lastFrameGridBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )  
    Post:Level10Filter(sceneBuffer, lastFrameGridBuffer, scratchQuarterBuffer, finalColorBuffer)

    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber2)
    RadEffects.particleAccumLayer = particleLayer
    
    --RadEffects:SetEffectLayerToParticleAccumLayer()
    --RadEffects:SetEffectLayerToGameLayer()
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/BlobDots.particles", false)

    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostState_Level10.UpdateStart()
    local randomfloat = math.random(-100,100)
    local xParam = math.cos(randomfloat) * 350
    local yParam = math.sin(randomfloat) * 350
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB   UU    UU  II  LL       DDDDD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBBB  UU    UU  II  LL       DD   DD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBB   UUUUUUUU  II  LLLLLLl  DDDDD
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level10:Build()
    print("PostState_Level10: Build")
    Post:Reset()
    PostState_Level10:Reset()
    
    Post:SetUpdateCallback(PostState_Level10.UpdateBuild)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {0.5,0.5,1}
    ColorManager.Color2             = {0.5,0.5,1} --score
    ColorManager.Color3             = {.1, .1, 4} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(1,1,1, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 8.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 500.0, 300.0, .25) 
    Wind.mWindDeck:SetFrequency( 10.0 )
    
    ChargeField:Enable(false)
    ChargeField:SetColor(.1,.2,.1,1)
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 3.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 0.0, 5.0 )
    ChargeField.mChargeFieldDeck:SetActivatedDistance( 120.0 )
        
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local lastFrameGridBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(sceneBuffer)
    
    --Post:BlitFilter(Renderer.mGameBuffer, sceneBuffer, kBLEND_ADD)
    --uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer, sceneBuffer, kBLEND_ADD)   
    --Post:BlurFilter(sceneBuffer, scratchBuffer, .005, kBLEND_ADD, {0.7,0.1,0.5,1})  
    --Post:DirectionalBlurFilter(scratchBuffer, lastFrameGridBuffer, {.01,0}, kBLEND_ADD, {0.2,0.2,0.2,1})      
    --Post:BlitFilter(lastFrameGridBuffer, accumBuffer, kBLEND_ADD)
    --accumFilter = Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.93,.93,.93,1}, {.97,.97}, {0.0,0.0})
    --Post:CombineFilter(lastFrameGridBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    --Post:EdgeFilter(sceneBuffer, finalColorBuffer, 1, kBLEND_ADD, {1,0.7,1,1})
    --Post:BlitFilter(lastFrameGridBuffer, finalColorBuffer, kBLEND_ADD)
    --Post:BlitFilter(scratchQuarterBuffer, finalColorBuffer, kBLEND_ADD)
    
    gameBlitFilter = Post:Level7Filter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, {1,1}, {1,1}) 
    Post:BlurFilter(sceneBuffer, lastFrameGridBuffer, .005, kBLEND_ADD, {0.14,0.02,0.1,1})  
    accumFilter = Post:Level7BFilter(lastFrameGridBuffer, accumBuffer, scratchQuarterBuffer, {.93,.93,.93,1}, {.97,.97}, 1)
    Post:CombineFilter(lastFrameGridBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )  
    Post:Level10Filter(sceneBuffer, lastFrameGridBuffer, scratchQuarterBuffer, finalColorBuffer)
    
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber2)
    RadEffects.particleAccumLayer = particleLayer
    
    --RadEffects:SetEffectLayerToParticleAccumLayer()
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/BlobDots.particles", false)

    Post.rScreen:setInBuffer(finalColorBuffer)
end


function PostState_Level10.UpdateBuild()
    local level_intensity = Score.mLevelIntensity

    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)

    local randomfloat = math.random(-100,100)
    local xParam = math.cos(randomfloat) * 350
    local yParam = math.sin(randomfloat) * 350
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    Wind.mWindDeck:SetFrequency( DFMath.lerp(10.0, 100.0, level_intensity) )
    
    if uiBlitFilter then   
        uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 + audioSmoothed*5*math.pow(level_intensity,4), 1 + audioSmoothed*5*math.pow(level_intensity,4)})
    end
    if gameBlitFilter then   
        gameBlitFilter:setShaderValue( "g_vScale2", MOAIMaterial.VALUETYPE_VEC2, {1 + audioSmoothed*5*math.pow(level_intensity,4), 1 + audioSmoothed*5*math.pow(level_intensity,4)})
    end
    
    if accumFilter then
        level_intensity = math.pow(level_intensity, 0.25)
        local colorChannel = DFMath.lerp(0, 0.93, level_intensity)
        accumFilter:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, {colorChannel,colorChannel,colorChannel,1})
    end
        
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB    OOOOOO  NN    NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NNNN  NN  UU    UU  SS      
-- BBBBBBBB  OO    OO NN NN NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NN  NNNN  UU    UU        SS
-- BBBBBBB    OOOOOO  NN   NNN  UUUUUUUU  SSSSSSSS
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level10:Bonus()
    print("PostState_Level10: Bonus")
    Post:Reset()
    PostState_Level10:Reset()
    
    Post:SetUpdateCallback(PostState_Level10.UpdateBonus)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {1,1,1}
    ColorManager.Color2             = {0.85,0.85,1} --score
    ColorManager.Color3             = {.3, .3, 1} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(1,1,1, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 10.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 500.0, 600.0, .3) 
    Wind.mWindDeck:SetFrequency( 100.0 )
        
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer2 = Post:GetUnusedBuffer()
    local lastFrameGridBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(sceneBuffer)
    
    --Post:BlitFilter(Renderer.mGameBuffer, sceneBuffer, kBLEND_ADD)
    --uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer, sceneBuffer, kBLEND_ADD) 
    --Post:BlurFilter(sceneBuffer, lastFrameGridBuffer, .005, kBLEND_ADD, {0.14,0.02,0.1,1})  
    --Post:DirectionalBlurFilter(scratchBuffer2, lastFrameGridBuffer, {.01,0}, kBLEND_ADD, {0.2,0.2,0.2,1}) 
    --Post:BlitFilter(lastFrameGridBuffer, accumBuffer, kBLEND_ADD)
    --accumFilter = Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.93,.93,.93,1}, {.97,.97}, {0.0,0.0}) 
    --Post:CombineFilter(lastFrameGridBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )  
    --Post:EdgeFilter(sceneBuffer, finalColorBuffer, 1, kBLEND_ADD, {1,0.7,1,1})
    --Post:BlitFilter(lastFrameGridBuffer, finalColorBuffer, kBLEND_ADD)
    --Post:BlitFilter(scratchQuarterBuffer, finalColorBuffer, kBLEND_ADD)
    
    gameBlitFilter = Post:Level7Filter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, {1,1}, {1,1}) 
    Post:BlurFilter(sceneBuffer, lastFrameGridBuffer, .005, kBLEND_ADD, {0.14,0.02,0.1,1})  
    accumFilter = Post:Level7BFilter(lastFrameGridBuffer, accumBuffer, scratchQuarterBuffer, {.93,.93,.93,1}, {.97,.97}, 1)
    Post:CombineFilter(lastFrameGridBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )  
    Post:Level10Filter(sceneBuffer, lastFrameGridBuffer, scratchQuarterBuffer, finalColorBuffer)
    
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber2)
    RadEffects.particleAccumLayer = particleLayer
    
    --RadEffects:SetEffectLayerToParticleAccumLayer()
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/BlobDots.particles", false)

    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostState_Level10.UpdateBonus()
    local level_intensity = Score.mLevelIntensity
    
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    local randomfloat = math.random(-100,100)
    local xParam = math.cos(randomfloat) * 350
    local yParam = math.sin(randomfloat) * 350
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    Wind.mWindDeck:SetFrequency( DFMath.lerp(0, 100.0, level_intensity ))
    
    local inversePowTime = math.pow(Constants.Heartbeat,3) 
    if uiBlitFilter then   
        uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 + inversePowTime*0.1, 1 + inversePowTime*0.1})
    end
    if gameBlitFilter then   
        gameBlitFilter:setShaderValue( "g_vScale2", MOAIMaterial.VALUETYPE_VEC2, {1 + inversePowTime*0.1, 1 + inversePowTime*0.1})
    end
    
    if accumFilter then
        local colorChannel = DFMath.lerp(0, 0.93, math.pow(level_intensity, 0.15))
        accumFilter:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, {colorChannel,colorChannel,colorChannel,1})
    end       
end

function PostState_Level10.UpdatePostBonus()
    local level_intensity = Score.mLevelIntensity
    
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    local randomfloat = math.random(-100,100)
    local xParam = math.cos(randomfloat) * 350
    local yParam = math.sin(randomfloat) * 350
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    Wind.mWindDeck:SetFrequency( DFMath.lerp(0, 100.0, level_intensity ))
        
    if accumFilter then
        local colorChannel = DFMath.lerp(0, 0.93, math.pow(level_intensity, 0.15))
        accumFilter:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, {colorChannel,colorChannel,colorChannel,1})
    end       
end

function PostState_Level10.SpawnBubbleChamber(posX, posY, posZ, rTargetProp)
    local rExplosion = RadEffects.SpawnBubbleChamber(posX, posY, posZ, rTargetProp)
    
    local rTracerEffect = rExplosion.rTracerEffect
    local rTracerEffectProp = rTracerEffect:getProp()
    local rTracerEffectMaterial = rTracerEffectProp:getMaterial()
    rTracerEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rRippleEffect = rExplosion.rRippleEffect
    local rRippleEffectProp = rRippleEffect:getProp()
    local rRippleEffectMaterial = rRippleEffectProp:getMaterial()
    rRippleEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rSeekingEffect = rExplosion.rSeekingEffect
    local rSeekingEffectProp = rSeekingEffect:getProp()
    local rSeekingEffectMaterial = rSeekingEffectProp:getMaterial()
    rSeekingEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    return rExplosion
end

function PostState_Level10.SpawnBubbleChamber2(posX, posY, posZ, rTargetProp, rCallback)
    local Explosion = {}
    
    function Explosion:Init()
        RadEffects:SetEffectTransform( posX, posY, posZ, 0.0, 0.0, 0.0 )   
        RadEffects:SetEffectLayer( RadEffects.particleAccumLayer )
        self.rRippleEffect = RadEffects:CreateEffect( "Effects/Particles/Explosions/BubbleChamber/RingRippleC.particles", nil, Constants.GoodEffectColor, nil, Constants.ParticlePriorityLow )  
        self.rTracerEffect = RadEffects:CreateEffect( "Effects/Particles/Explosions/BubbleChamber/SimpleTracers.particles", nil, Constants.GoodEffectColor, nil, Constants.ParticlePriorityLow )  
      
        if rTargetProp ~= nil then
            RadEffects:SetEffectTransform( posX, posY, posZ, 0.0,-90.0, 0.0 )  
            RadEffects:SetEffectLayer( RadEffects.particleAccumLayer )
            
            if not rCallback then
                rCallback = RadEffects.BubbleChamberTargetCallback
            end
            
            self.rSeekingEffect = RadEffects:CreateEffect( "Effects/Particles/Explosions/BubbleChamber/SeekingStarsBold.particles", rTargetProp, Constants.GoodEffectColor, rCallback, Constants.ParticlePriorityLow )  
        end    
    end
    
    Explosion:Init()
    return Explosion
end

return PostState_Level10