local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local Wind = require('Wind')
local Score = require('Score')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local RadEffects = require('RadEffects')
local InputVisuals = require('InputVisuals')
local PostState_Level2= {}

local flowFilter = nil
local uiFilterMultiply = nil
local uiBlitFilter = nil
local gameBlitFilter = nil
local goodEffectColor = {0.5,0.1,0.5,1}
local accumBlitFilter  = nil
local remapFilter = nil
local finalFilter = nil
local totalTime = nil
local audioSmoothed = 0.0

function PostState_Level2:Reset()
    flowFilter = nil
    uiBlitFilter = nil
    gameBlitFilter = nil
    accumBlitFilter  = nil
    uiFilterMultiply = nil
    remapFilter = nil
    finalFilter = nil
    totalTime = nil
    audioSmoothed = 0.0
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- SSSSSSSS  TTTTTTTT  AAAAAAAA  RRRRRR    TTTTTTTT
-- SS           TT     AA    AA  RR    RR     TT
-- SSSSSSSS     TT     AAAAAAAA  RRRRRR       TT
--       SS     TT     AA    AA  RR   RR      TT
-- SSSSSSSS     TT     AA    AA  RR    RR     TT
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level2:Start()
    print("PostState_Level2: Start")
    Post:Reset()
    PostState_Level2:Reset()

    totalTime = 0.0 
    Post:SetUpdateCallback(PostState_Level2.UpdateStart)   

    -----------------------------------------------
    -- Setup Colors 
    -----------------------------------------------      
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.75, .34, .9}
    ColorManager.Color2             = {.2, .2, .48} --score
    ColorManager.Color3             = {.25, .11, .3} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    ChargeField:Enable(false)
    ChargeField:SetColor(.1,.1,.1,1)
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 3.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 0.0, 4.0 )
    ChargeField.mChargeFieldDeck:SetActivatedDistance( 60.0 )
    
    local finalColorBuffer = Post:GetUnusedBuffer()   
    local accumBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchBuffer2 = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    uiBlitFilter = Post:Level2AFilter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer)
    flowFilter = Post:AccumFlowFilter(sceneBuffer, accumBuffer, scratchBuffer, Post.rFlowTexture, {.99,.99,.97,1}, {1,1}, {0,.0}, .01) --.05)
    flowFilter:setShaderValue("g_fFlowPulseFreq", MOAIMaterial.VALUETYPE_FLOAT, 0)   
    Post:BlitFilter(scratchBuffer, accumBuffer)
    accumBlitFilter = Post:Level2BFilter(accumBuffer,Renderer.mUIBuffer, sceneBuffer, finalColorBuffer)

    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
    
    RadEffects:SetGoodExplosionCallback(self.SpawnPurpleStars)
    RadEffects.particleAccumLayer = particleLayer
    
    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostState_Level2.UpdateStart()
    local level_intensity = Score.mLevelIntensity + 0.1
    
    totalTime = totalTime + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    local blitIntensity = .9 + (math.sin(totalTime*20) + 1) * .2
    uiBlitFilter:setShaderValue("g_vColor", MOAIMaterial.VALUETYPE_VEC4, {blitIntensity,blitIntensity,blitIntensity,blitIntensity})
    uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1, 1 + audioSmoothed*1})
    if uiFilterMultiply then
        uiFilterMultiply:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1, 1 + audioSmoothed*1})
    end
    
    flowFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {.995 + audioSmoothed*2, .995 + audioSmoothed*2})
    accumBlitFilter:setShaderValue("g_vColor", MOAIMaterial.VALUETYPE_VEC4, {.2*level_intensity + .1,4*level_intensity + .2,4*level_intensity+ 1,1})
    accumBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1, 1 + audioSmoothed*1})
    
    if level_intensity > .5 then
         ChargeField:Enable(true)
    end
end
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB   UU    UU  II  LL       DDDDD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBBB  UU    UU  II  LL       DD   DD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBB   UUUUUUUU  II  LLLLLLl  DDDDD
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------


function PostState_Level2:Build()
    print("PostState_Level2: Build")
    Post:Reset()
    PostState_Level2:Reset()

    totalTime = 0.0 
    Post:SetUpdateCallback(PostState_Level2.UpdateBuild)
    
    -----------------------------------------------
    -- Setup Colors 
    -----------------------------------------------      
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.75, .34, .9}
    ColorManager.Color2             = {.2, .2, .58} --score
    ColorManager.Color3             = {.25, .11, .3} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(.2,.15,.25, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 1.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 200.0, .2) 
    Wind.mWindDeck:SetFrequency( 100.0 )
    
    ChargeField:Enable(true)
    ChargeField:SetColor(.1,.1,.1,1)
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 3.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 0.0, 4.0 )
    ChargeField.mChargeFieldDeck:SetActivatedDistance( 60.0 )
    
    local finalColorBuffer = Post:GetUnusedBuffer()   
    local accumBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchBuffer2 = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local sceneBufferWithBackground = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    --Post:BlitFilter(Renderer.mGameBuffer,sceneBuffer, kBLEND_ADD, {1,1,1,1})
    --uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer,sceneBuffer, kBLEND_ADD, {1,1,1,1})
    --Post:CombineFilter(Renderer.mBackgroundBuffer, sceneBuffer, sceneBufferWithBackground, kBLEND_ADD, {1,1,1,1})    
    --flowFilter = Post:AccumFlowFilter(sceneBuffer, accumBuffer, scratchBuffer, Post.rFlowTexture, {.99,.99,.97,1}, {.99,.99}, {0,.0}, .00) --.05)
    --flowFilter:setShaderValue("g_fFlowPulseFreq", MOAIMaterial.VALUETYPE_FLOAT, 0)  
    --Post:BlurFilter(scratchBuffer, accumBuffer, .0000)
    --remapFilter = Post:RemapFilter(accumBuffer, scratchBuffer2, Post.rGradientTextureE, 2, kBLEND_ALPHA, {1,1,1,1})  
    --Post:EdgeFilter(scratchBuffer2, finalColorBuffer, 1, kBLEND_ADD, {0,.7,1,1})
    --Post:EdgeFilter(sceneBuffer, finalColorBuffer, 1, kBLEND_ADD, {2,.8,.2,1})
    --Post:BlurFilter(sceneBuffer, finalColorBuffer, .005, kBLEND_ADD, {1,.8,2,1})

    uiBlitFilter = Post:Level2AFilter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer)  
    flowFilter = Post:AccumFlowFilter(sceneBuffer, accumBuffer, scratchBuffer, Post.rFlowTexture, {.99,.99,.97,1}, {.99,.99}, {0,.0}, .00) --.05)
    flowFilter:setShaderValue("g_fFlowPulseFreq", MOAIMaterial.VALUETYPE_FLOAT, 0)   
    Post:BlurFilter(scratchBuffer, accumBuffer, .0000)  
    remapFilter = Post:RemapFilter(accumBuffer, scratchBuffer2, Post.rGradientTextureE, 2, kBLEND_ALPHA, {1,1,1,1})  
    finalFilter = Post:Level2CFilter(scratchBuffer2, sceneBuffer, finalColorBuffer, .005, {0,.7,1,1}, {2,.8,.2,1}, {1,.8,2,1}, {0,0,0,0})
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
    
    RadEffects:SetGoodExplosionCallback(self.SpawnPurpleStars)
    RadEffects.particleAccumLayer = particleLayer
    
    Post.rScreen:setInBuffer(finalColorBuffer)
end


function PostState_Level2.UpdateBuild()
    totalTime = totalTime + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    Wind.mWindDeck:SetPointWidth( Constants.Heartbeat * 2.0 )
    local blitIntensity = .7 + (math.sin(totalTime*30) + 1) * .3
    uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1, 1 + audioSmoothed*4})
    uiBlitFilter:setShaderValue("g_vColor", MOAIMaterial.VALUETYPE_VEC4, {blitIntensity,blitIntensity,blitIntensity,blitIntensity})
    
    if remapFilter then
        remapFilter:setShaderValue( "g_fScrollRate", MOAIMaterial.VALUETYPE_FLOAT, Score.mLevelIntensity * .2)
        remapFilter:setColor(Score.mLevelIntensity,Score.mLevelIntensity,Score.mLevelIntensity,1)
    end
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB    OOOOOO  NN    NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NNNN  NN  UU    UU  SS      
-- BBBBBBBB  OO    OO NN NN NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NN  NNNN  UU    UU        SS
-- BBBBBBB    OOOOOO  NN   NNN  UUUUUUUU  SSSSSSSS
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level2:Bonus()
    print("PostState_Level2: Bonus")
    Post:Reset()
    PostState_Level2:Reset()

    totalTime = 0.0 
    Post:SetUpdateCallback(PostState_Level2.UpdateBonus)
    
    -----------------------------------------------
    -- Setup Colors 
    -----------------------------------------------      
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.75, .34, .9}
    ColorManager.Color2             = {.3,0,.9} --score
    ColorManager.Color3             = {.25, .11, .3} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(.2,.15,.25, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 1.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 200.0, 1) 
    Wind.mWindDeck:SetFrequency( 100.0 )
     
    local finalColorBuffer = Post:GetUnusedBuffer()   
    local accumBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchBuffer2 = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local sceneBufferWithBackground = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
     
    --gameBlitFilter = Post:BlitFilter(Renderer.mGameBuffer,sceneBuffer, kBLEND_ADD, {1,1,1,1})
    --uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer,sceneBuffer, kBLEND_ADD, {1,1,1,1})
    --Post:CombineFilter(Renderer.mBackgroundBuffer, sceneBuffer, sceneBufferWithBackground, kBLEND_ADD, {1,1,1,1})  
    --flowFilter = Post:AccumFlowFilter(sceneBufferWithBackground, accumBuffer, scratchBuffer, Post.rFlowTexture, {.85,1,.99,1}, {.98,.98}, {0,.0}, .0) --.05)
    --flowFilter:setShaderValue("g_fFlowPulseFreq", MOAIMaterial.VALUETYPE_FLOAT, 0) 
    --Post:BlurFilter(scratchBuffer, accumBuffer, .0000)
    --Post:RemapFilter(accumBuffer, scratchBuffer2, Post.rGradientTextureF, 0, kBLEND_ALPHA, {1,1,1,1})
    --Post:EdgeFilter(scratchBuffer2, finalColorBuffer, 1, kBLEND_ADD, {.2,5,5,1})
    --Post:EdgeFilter(sceneBufferWithBackground, finalColorBuffer, 1, kBLEND_ADD, {3,.8,10,1})
    --Post:BlurFilter(sceneBufferWithBackground, finalColorBuffer, .004, kBLEND_ADD, {4,4,4,1})
    --Post:BlurFilter(sceneBufferWithBackground, finalColorBuffer, .006, kBLEND_ADD, {4,4,4,1})
    
    
    uiBlitFilter = Post:Level2AFilter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBufferWithBackground)      
    flowFilter = Post:AccumFlowFilter(sceneBufferWithBackground, accumBuffer, scratchBuffer, Post.rFlowTexture, {.85,1,.99,1}, {.98,.98}, {0,.0}, .0) --.05)
    flowFilter:setShaderValue("g_fFlowPulseFreq", MOAIMaterial.VALUETYPE_FLOAT, 0)   
    Post:BlurFilter(scratchBuffer, accumBuffer, .0000)
    Post:RemapFilter(accumBuffer, scratchBuffer2, Post.rGradientTextureF, 0, kBLEND_ALPHA, {1,1,1,1})
    finalFilter = Post:Level2CFilter(scratchBuffer2, sceneBufferWithBackground, finalColorBuffer, .004, {.2,5,5,1}, {3,.8,10,1}, {1,1,1,1}, {1,1,1,1})

    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
    
    RadEffects:SetGoodExplosionCallback(self.SpawnPurpleStars)
    RadEffects.particleAccumLayer = particleLayer

    Post.rScreen:setInBuffer(finalColorBuffer)
end


function PostState_Level2.UpdateBonus()
    totalTime = totalTime + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)

    Wind.mWindDeck:SetPointWidth( Constants.Heartbeat * 2.0 )
    
    local blitIntensity = .7 + (math.sin(totalTime*30) + 1) * .3
    uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1, 1 + audioSmoothed*10})
    uiBlitFilter:setShaderValue("g_vColor", MOAIMaterial.VALUETYPE_VEC4, {blitIntensity,blitIntensity,blitIntensity,blitIntensity})
    
    if gameBlitFilter then
        gameBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 - audioSmoothed*10, 1 - audioSmoothed*10})
    end
        
    flowFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1.01 - audioSmoothed*5, 1.01 - audioSmoothed*5})
end


function PostState_Level2.SpawnPurpleStars(posX, posY, posZ, rTargetProp)
    local rExplosion = RadEffects.SpawnPurpleStars(posX, posY, posZ, rTargetProp, PostState_Level2.BubbleChamberTargetCallback)
    
    local rStarEffect = rExplosion.rStarEffect
    local rStarEffectProp = rStarEffect:getProp()
    local rStarEffectMaterial = rStarEffectProp:getMaterial()
    rStarEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rTracerEffect = rExplosion.rTracerEffect
    local rTracerEffectProp = rTracerEffect:getProp()
    local rTracerEffectMaterial = rTracerEffectProp:getMaterial()
    rTracerEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rRippleEffect = rExplosion.rRippleEffect
    local rRippleEffectProp = rRippleEffect:getProp()
    local rRippleEffectMaterial = rRippleEffectProp:getMaterial()
    rRippleEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rSeekingEffect = rExplosion.rSeekingEffect
    local rSeekingEffectProp = rSeekingEffect:getProp()
    local rSeekingEffectMaterial = rSeekingEffectProp:getMaterial()
    rSeekingEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)

    return rExplosion
end


function PostState_Level2.BubbleChamberTargetCallback( tParticleEffect )
    local Explosion = {}
    if tParticleEffect ~= nil then
        if tParticleEffect.rTargetProp ~= nil then
            local fTargetX, fTargetY, fTargetZ = tParticleEffect.rTargetProp:getLoc()
            RadEffects:SetEffectTransform( fTargetX, fTargetY, fTargetZ, 0.0, 0.0, 0.0 )   
            RadEffects:SetEffectLayer( RadEffects.tRenderer.mGameLayer )
            local rEffect = RadEffects:CreateEffect( "Effects/Particles/Bursts/BlipFlare.particles", nil, nil, nil, Constants.ParticlePriorityLow )                 
            Explosion.rEffect = rEffect
            
            local rEffectProp = rEffect:getProp()
            local rEffectMaterial = rEffectProp:getMaterial()
            rEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
        end
    end
    
    return Explosion
end


return PostState_Level2
