local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local Wind = require('Wind')
local Score = require('Score')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local ParticleSystem = require("ParticleSystem")
local RadEffects = require('RadEffects')
local PostState_Level3 = {}

local noiseFilter = nil
local noiseFilter2 = nil
local energyFilter = nil
local energyFilter2 = nil
local uiBlitFilter = nil
local sceneOverlayFilter = nil
local goodEffectColor = {0.3,0.2,0,1}
local gameBlitFilter = nil
local totalTime = 0.0
local audioSmoothed = 0.0

function PostState_Level3:Reset()
     noiseFilter = nil
     noiseFilter2 = nil
     energyFilter = nil
     energyFilter2 = nil
     uiBlitFilter = nil
     sceneOverlayFilter = nil
     gameBlitFilter = nil
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- SSSSSSSS  TTTTTTTT  AAAAAAAA  RRRRRR    TTTTTTTT
-- SS           TT     AA    AA  RR    RR     TT
-- SSSSSSSS     TT     AAAAAAAA  RRRRRR       TT
--       SS     TT     AA    AA  RR   RR      TT
-- SSSSSSSS     TT     AA    AA  RR    RR     TT
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level3:Start()
    print("PostState_Level3: Start")
    Post:Reset()
    PostState_Level3:Reset()
    
    Post:SetUpdateCallback(PostState_Level3.UpdateStart)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.55, .54, .1}
    ColorManager.Color2             = {.3, .1, .1} --score
    ColorManager.Color3             = {.75, .34, .2} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
        
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchBuffer, {.9,.9,.9,1}, {.99,.99}, {0.0,0.0})
    energyFilter = Post:Level3AFilter(scratchBuffer, sceneBuffer, Renderer.mBackgroundBuffer, accumBuffer, {.05,.0,.1,1}, {.5,.5})
    gameBlitFilter = Post:Level3BFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, scratchBuffer, sceneBuffer)
    noiseFilter2 = Post:Level3CFilter(sceneBuffer, scratchBuffer, accumBuffer, finalColorBuffer, {1,1,1,1}, {.1,1,.25,1}, {0,0,0,0}) 
  
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
    
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer
    
    Post.rScreen:setInBuffer(finalColorBuffer)
end


function PostState_Level3.UpdateStart()
    local level_intensity = Score.mLevelIntensity
    totalTime = totalTime  + Constants.FrameStep
    
    if noiseFilter then
        noiseFilter:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, totalTime * 1)
    end
        
    if noiseFilter2 then
        noiseFilter2:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, totalTime * 1.3)
    end
    
    if sceneOverlayFilter then
        sceneOverlayFilter:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4,  {.05 * level_intensity,.0 * level_intensity,.1*level_intensity,1})
    end
    
    if energyFilter then
        local energyIntensity = .1 * (level_intensity + 0.25)
        energyFilter:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4,  {.5 * energyIntensity,0.0, energyIntensity,1})
    end
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB   UU    UU  II  LL       DDDDD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBBB  UU    UU  II  LL       DD   DD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBB   UUUUUUUU  II  LLLLLLl  DDDDD
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level3:Build()
    print("PostState_Level3: Build")
    Post:Reset()
    PostState_Level3:Reset()
    
    Post:SetUpdateCallback(PostState_Level3.UpdateBuild)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.85, .54, .1}
    ColorManager.Color2             = {.55, .37, .1} --score
    ColorManager.Color3             = {.55, .24, .09} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(.2,.15,.25, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 2.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 400.0, .05) 
    Wind.mWindDeck:SetFrequency( 100.0 )
            
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
 
    Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchBuffer, {.98,.98,.98,1}, {.99,.99}, {0.0,0.0})
    energyFilter = Post:Level3AFilter(scratchBuffer, sceneBuffer, Renderer.mBackgroundBuffer, accumBuffer, {.04,.0,.1,1}, {.7,.7})
    gameBlitFilter = Post:Level3BFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, scratchBuffer, sceneBuffer)
    noiseFilter2 = Post:Level3CFilter(sceneBuffer, scratchBuffer, accumBuffer, finalColorBuffer, {.5,.5,.5,1}, {.1,.1,.1,1}, {.0,.1,.3,.02}) 

    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
    
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer
    
    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostState_Level3.UpdateBuild()
    local level_intensity = Score.mLevelIntensity
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    if noiseFilter2 then
        noiseFilter2:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, totalTime * 1.3)
        noiseFilter2:setShaderValue( "g_fTime2", MOAIMaterial.VALUETYPE_FLOAT, audioSmoothed*620 + totalTime*5.2)
        noiseFilter2:setShaderValue( "g_vResolution", MOAIMaterial.VALUETYPE_VEC2, {audioSmoothed*audioSmoothed*4000*level_intensity + audioSmoothed*50,1})
    end
    
    if energyFilter then
        energyFilter:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, audioSmoothed*620 + totalTime*5.2)
        energyFilter:setShaderValue( "g_vResolution", MOAIMaterial.VALUETYPE_VEC2, {audioSmoothed*audioSmoothed*4000*level_intensity + audioSmoothed*50,1})
    end
    if energyFilter2 then
        energyFilter2:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, audioSmoothed*620 + totalTime*5.2)
        energyFilter2:setShaderValue( "g_vResolution", MOAIMaterial.VALUETYPE_VEC2, {audioSmoothed*audioSmoothed*4000*level_intensity + audioSmoothed*50,1})
    end
    
    if uiBlitFilter then
        uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1, 1 + audioSmoothed*7*level_intensity})
    end
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB    OOOOOO  NN    NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NNNN  NN  UU    UU  SS      
-- BBBBBBBB  OO    OO NN NN NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NN  NNNN  UU    UU        SS
-- BBBBBBB    OOOOOO  NN   NNN  UUUUUUUU  SSSSSSSS
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------


function PostState_Level3:Bonus()
    print("PostState_Level3: Bonus")
    Post:Reset()
    PostState_Level3:Reset()
    
    Post:SetUpdateCallback(PostState_Level3.UpdateBonus)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.85, .54, .1}
    ColorManager.Color2             = {.8, .64, .1} --score
    ColorManager.Color3             = {.75, .34, .13} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(.2,.15,.25, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 2.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 2000.0, .05) 
    Wind.mWindDeck:SetFrequency( 300.0 )
        
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
   
    --Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchBuffer, {.97,.97,.97,1}, {.99,.99}, {0.0,0.0})
    --Post:BlurFilter(scratchBuffer, accumBuffer, .002) -- save off the acumm buffer
    --Post:BlitFilter(sceneBuffer, accumBuffer, kBLEND_ADD, {.02,.0,.05,1}, {.7,.7}, {0,0})    
    --Post:BlitFilter(Renderer.mBackgroundBuffer, accumBuffer, kBLEND_ADD)
    --energyFilter = Post:EnergyRippleFilter(sceneBuffer, accumBuffer, 0.0, kBLEND_ALPHA, {.1,.01,.05,.02})   
    --gameBlitFilter = Post:BlitFilter(Renderer.mGameBuffer, sceneBuffer, kBLEND_ADD)
    --uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer, sceneBuffer, kBLEND_ADD)   
    --Post:BlitFilter(scratchBuffer, sceneBuffer, kBLEND_ADD, {.1,.1,.1,1})   
    --Post:BlurFilter(sceneBuffer, finalColorBuffer, .003, kBLEND_ADD, {.5,.5,.5,1})        
    --Post:BlitFilter(scratchBuffer, finalColorBuffer, kBLEND_ADD, {.5,.5,.5,1})  
    --Post:BlitFilter(sceneBuffer, finalColorBuffer, kBLEND_ADD)
    --Post:BlitFilter(accumBuffer, finalColorBuffer, kBLEND_ADD, {.5,.5,.5,1})    
    --Post:BlurFilter(accumBuffer, finalColorBuffer, .008, kBLEND_ADD, {.1,.1,.1,1})        
    --energyFilter2 = Post:EnergyRippleFilter(sceneBuffer, finalColorBuffer, 0.0, kBLEND_ADD, {.0,.1,.5,.02})    
    --noiseFilter2 = Post:TVNoiseFilter(sceneBuffer, finalColorBuffer, 0.0, kBLEND_ADD, {.03,.02,.1,1})    
 
    
    Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchBuffer, {.97,.97,.97,1}, {.99,.99}, {0.0,0.0})
    energyFilter = Post:Level3AFilter(scratchBuffer, sceneBuffer, Renderer.mBackgroundBuffer, accumBuffer, {.04,.0,.1,1}, {.7,.7})
    gameBlitFilter = Post:Level3BFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, scratchBuffer, sceneBuffer)
    noiseFilter2 = Post:Level3CFilter(sceneBuffer, scratchBuffer, accumBuffer, finalColorBuffer, {.5,.5,.5,1}, {.1,.1,.1,1}, {.0,.1,.3,.02})

    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
    
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer
    
    Post.rScreen:setInBuffer(finalColorBuffer)
end


function PostState_Level3.UpdateBonus()
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
 
    if noiseFilter2 then
        noiseFilter2:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, totalTime * 3.3)
        noiseFilter2:setShaderValue( "g_fTime2", MOAIMaterial.VALUETYPE_FLOAT, audioSmoothed*620 + totalTime*8.2)
        noiseFilter2:setShaderValue( "g_vResolution", MOAIMaterial.VALUETYPE_VEC2, {audioSmoothed*audioSmoothed*4000 + audioSmoothed*50,1})
    end
    
    if energyFilter then
        energyFilter:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, audioSmoothed*620 + totalTime*40)
        energyFilter:setShaderValue( "g_vResolution", MOAIMaterial.VALUETYPE_VEC2, {audioSmoothed*audioSmoothed*120000,1})
    end
    if energyFilter2 then
        energyFilter2:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, audioSmoothed*620 + totalTime*40)
        energyFilter2:setShaderValue( "g_vResolution", MOAIMaterial.VALUETYPE_VEC2, {audioSmoothed*audioSmoothed*120000,1})
    end
    
    local powTime = math.pow(Constants.Heartbeat,3) 
    if uiBlitFilter then
        uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1, 1 + audioSmoothed*10})
    end
    gameBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 - powTime*0.15, 1 - powTime*0.15})
    gameBlitFilter:setShaderValue( "g_vScale2", MOAIMaterial.VALUETYPE_VEC2, {1, 1 + audioSmoothed*10})
    
end

function PostState_Level3.SpawnBubbleChamber(posX, posY, posZ, rTargetProp)
    local rExplosion = RadEffects.SpawnBubbleChamber(posX, posY, posZ, rTargetProp, PostState_Level3.BubbleChamberTargetCallback)
    
    local rTracerEffect = rExplosion.rTracerEffect
    local rTracerEffectProp = rTracerEffect:getProp()
    local rTracerEffectMaterial = rTracerEffectProp:getMaterial()
    rTracerEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rRippleEffect = rExplosion.rRippleEffect
    local rRippleEffectProp = rRippleEffect:getProp()
    local rRippleEffectMaterial = rRippleEffectProp:getMaterial()
    rRippleEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rSeekingEffect = rExplosion.rSeekingEffect
    local rSeekingEffectProp = rSeekingEffect:getProp()
    local rSeekingEffectMaterial = rSeekingEffectProp:getMaterial()
    rSeekingEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    return rExplosion
end

function PostState_Level3.BubbleChamberTargetCallback( tParticleEffect )
    local Explosion = {}
    if tParticleEffect ~= nil then
        if tParticleEffect.rTargetProp ~= nil then
            local fTargetX, fTargetY, fTargetZ = tParticleEffect.rTargetProp:getLoc()
            RadEffects:SetEffectTransform( fTargetX, fTargetY, fTargetZ, 0.0, 0.0, 0.0 )   
            RadEffects:SetEffectLayer( RadEffects.tRenderer.mGameLayer )
            local rEffect = RadEffects:CreateEffect( "Effects/Particles/Bursts/BlipFlare.particles", nil, nil, nil, Constants.ParticlePriorityLow )                 
            Explosion.rEffect = rEffect
            
            local rEffectProp = rEffect:getProp()
            local rEffectMaterial = rEffectProp:getMaterial()
            rEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
        end
    end
    
    return Explosion
end

return PostState_Level3