local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local Wind = require('Wind')
local Score = require('Score')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local ParticleSystem = require("ParticleSystem")
local InputVisuals = require('InputVisuals')
local RadEffects = require('RadEffects')
local PostState_Level5 = {}

local noiseFilter = nil
local uiBlitFilter = nil
local uiEdgeFilter = nil
local goodEffectColor = {0.25,0.75,0,1}
local gameBlitFilter = nil
local totalTime = 0.0
local audioSmoothed = 0.0
local flipHack = 1

function PostState_Level5:Reset()
     noiseFilter = nil
     uiBlitFilter = nil
     uiEdgeFilter = nil
     gameBlitFilter = nil
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- SSSSSSSS  TTTTTTTT  AAAAAAAA  RRRRRR    TTTTTTTT
-- SS           TT     AA    AA  RR    RR     TT
-- SSSSSSSS     TT     AAAAAAAA  RRRRRR       TT
--       SS     TT     AA    AA  RR   RR      TT
-- SSSSSSSS     TT     AA    AA  RR    RR     TT
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level5:Start()
    print("PostState_Level5: Start")
    Post:Reset()
    PostState_Level5:Reset()
    
    Post:SetUpdateCallback(PostState_Level5.UpdateStart)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.1, .5, .1}
    ColorManager.Color2             = {.25, .75, 0} --score
    ColorManager.Color3             = {0, .1, 0} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(1,1,.1, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 0.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 400.0, .2) 
    Wind.mWindDeck:SetFrequency( 80.0 )
    
    ChargeField:Enable(true)
    ChargeField:SetColor( Constants.Color1[1]*0.5, Constants.Color1[2]*0.5, Constants.Color1[3]*0.5, 1.0 )
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 3.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 1.0, 1.0 )
    ChargeField.mChargeFieldDeck:SetActivatedDistance( 500.0 )
    
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
    
    --Post:CombineFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer)
    --Post:BlitFilter(Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlurFilter(sceneBuffer, finalColorBuffer, .003, kBLEND_ADD, {1,1,1,1})   
    --Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.95,.95,.85,1}, {.99,.99}, {0.0,0.0})
    --Post:CombineFilter(finalColorBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    --Post:CombineFilter(sceneBuffer, accumBuffer, finalColorBuffer, kBLEND_ALPHA, {1,1,1,1}, {1,1}, {0,0}, {2,1,1,1}, {1,1}, {0,0} )
    
    gameBlitFilter = Post:Level7Filter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, {1,1}, {1,1})      
    Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.95,.95,.85,1}, {.99,.99}, {0.0,0.0})  
    Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    Post:Level5Filter(sceneBuffer, accumBuffer, Renderer.mUIBuffer, finalColorBuffer, {0,0,0,1})
    
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer
    
    RadEffects:SetEffectLayerToParticleAccumLayer()
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/YellowDotsSlow.particles", false)

    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostState_Level5.UpdateStart()
    local level_intensity = Score.mLevelIntensity
    totalTime = totalTime  + Constants.FrameStep

    Wind.mWindDeck:SetPointWidth( Constants.Heartbeat * 3.0 )
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB   UU    UU  II  LL       DDDDD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBBB  UU    UU  II  LL       DD   DD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBB   UUUUUUUU  II  LLLLLLl  DDDDD
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level5:Build()
    print("PostState_Level5: Build")
    Post:Reset()
    PostState_Level5:Reset()
    
    Post:SetUpdateCallback(PostState_Level5.UpdateBuild)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.2, .95, .25}
    ColorManager.Color2             = {.25, .75, 0} --score
    ColorManager.Color3             = {0, .1, 0} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(1,1,.1, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( 0, 350, -200, 0, 350, -200 )
    Wind.mWindDeck:SetPointWidth( 15.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 700.0, 0.2) 
    Wind.mWindDeck:SetFrequency( 500.0 )
    
    ChargeField:Enable(true)
    ChargeField:SetColor( Constants.Color1[1]*0.5, Constants.Color1[2]*0.5, Constants.Color1[3]*0.5, 1.0 )
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 1.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 1.0, 1.0 )
    ChargeField.mChargeFieldDeck:SetActivatedDistance( 500.0 )
        
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
      
    --Post:BlitFilter(Renderer.mGameBuffer, sceneBuffer, kBLEND_ADD)
    --uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlitFilter(Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlurFilter(sceneBuffer, finalColorBuffer, .003, kBLEND_ADD, {2,2,2,1})   
    --Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.95,.95,.85,1}, {.99,.99}, {0.0,0.0})
    --Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    --Post:CombineFilter(sceneBuffer, accumBuffer, finalColorBuffer, kBLEND_ALPHA, {1,1,1,1}, {1,1}, {0,0}, {2,1,1,1}, {1,1}, {0,0} )
    --noiseFilter = Post:TVNoiseFilter(scratchBuffer, finalColorBuffer, 0.0, kBLEND_ADD, {.01,.05,.02,1})  
    
    gameBlitFilter = Post:Level7Filter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, {1,1}, {1,1})      
    Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.95,.95,.85,1}, {.99,.99}, {0.0,0.0})  
    Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    noiseFilter = Post:Level5Filter(sceneBuffer, accumBuffer, Renderer.mUIBuffer, finalColorBuffer, {.01,.05,.02,1})
    
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer
    
    RadEffects:SetEffectLayerToParticleAccumLayer()
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/YellowDotsMedium.particles", false)

    Post.rScreen:setInBuffer(finalColorBuffer)
end


function PostState_Level5.UpdateBuild()
    local level_intensity = Score.mLevelIntensity

    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)

    local xParam = math.cos(totalTime*2) * 350
    local yParam = math.sin(totalTime*2) * 350
    xParam = xParam * flipHack
    yParam = yParam * flipHack
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    flipHack = flipHack * -1
    Wind.mWindDeck:SetPointWidth( Constants.Heartbeat * 20.0 )
        
    if noiseFilter then
        noiseFilter:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, totalTime * 100.3)
        noiseFilter:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, {.01*level_intensity,.05*level_intensity,.02*level_intensity,1})
        noiseFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1, 1 + audioSmoothed*10*level_intensity})
    end
        
    if uiBlitFilter then
        uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1, 1 + audioSmoothed*10*level_intensity})
    end
    
    if gameBlitFilter then
        gameBlitFilter:setShaderValue( "g_vScale2", MOAIMaterial.VALUETYPE_VEC2, {1, 1 + audioSmoothed*10*level_intensity})
    end
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB    OOOOOO  NN    NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NNNN  NN  UU    UU  SS      
-- BBBBBBBB  OO    OO NN NN NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NN  NNNN  UU    UU        SS
-- BBBBBBB    OOOOOO  NN   NNN  UUUUUUUU  SSSSSSSS
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level5:Bonus()
    print("PostState_Level5: Bonus")
    Post:Reset()
    PostState_Level5:Reset()
    
    Post:SetUpdateCallback(PostState_Level5.UpdateBonus)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.2, .95, .25}
    ColorManager.Color2             = {.25, .75, 0} --score
    ColorManager.Color3             = {0, .1, 0} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(1,1,.1, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 0.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 700.0, .3) 
    Wind.mWindDeck:SetFrequency( 500.0 )
    
    ChargeField:Enable(true)
    ChargeField:SetColor( Constants.Color1[1]*0.5, Constants.Color1[2]*0.5, Constants.Color1[3]*0.5, 1.0 )
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 1.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 1.0, 1.0 )
    ChargeField.mChargeFieldDeck:SetActivatedDistance( 500.0 )
        
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)

    --gameBlitFilter = Post:BlitFilter(Renderer.mGameBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlitFilter(Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ADD)
    --uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer, sceneBuffer, kBLEND_ADD) 
    --Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.95,.95,.85,1}, {.99,.99}, {0.0,0.0})  
    --Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    --Post:BlurFilter(sceneBuffer, finalColorBuffer, .003, kBLEND_ADD, {2,2,2,1})  
    --Post:CombineFilter(sceneBuffer, accumBuffer, finalColorBuffer, kBLEND_ALPHA, {1,1,1,1}, {1,1}, {0,0}, {2,1,1,1}, {1,1}, {0,0} )
    --noiseFilter = Post:TVNoiseFilter(scratchBuffer, finalColorBuffer, 0.0, kBLEND_ADD, {.01,.05,.02,1})  
    --uiEdgeFilter = Post:EdgeFilter(Renderer.mUIBuffer, finalColorBuffer, 1, kBLEND_ADD, {2,2,2,2})
    
    gameBlitFilter = Post:Level7Filter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, {1,1}, {1,1})      
    Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.95,.95,.85,1}, {.99,.99}, {0.0,0.0})  
    Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    noiseFilter = Post:Level5Filter(sceneBuffer, accumBuffer, Renderer.mUIBuffer, finalColorBuffer, {.01,.05,.02,1})
    
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer
    
    RadEffects:SetEffectLayerToParticleAccumLayer()
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/YellowDots.particles", false)
    
    Post.rScreen:setInBuffer(finalColorBuffer)
end


function PostState_Level5.UpdateBonus()
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    local xParam = math.cos(totalTime*2) * 350
    local yParam = math.sin(totalTime*2) * 350
    xParam = xParam * flipHack
    yParam = yParam * flipHack
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    flipHack = flipHack * -1
    Wind.mWindDeck:SetPointWidth( 20.0 + Constants.Heartbeat * 10)
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 700.0, .3) 
         
    local powTime = math.pow(Constants.Heartbeat,3) 
    if gameBlitFilter then   
        gameBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 - powTime*0.1, 1 - powTime*0.1})
        gameBlitFilter:setShaderValue( "g_vScale2", MOAIMaterial.VALUETYPE_VEC2, {1 + powTime*0.1, 1 + audioSmoothed*10})
    end
    
    if uiBlitFilter then
        uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 + powTime*0.1, 1 + audioSmoothed*10})
        uiEdgeFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 + powTime*0.1, 1 + audioSmoothed*10})
    end
    
    if noiseFilter then
        noiseFilter:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, totalTime * 100.3)
        noiseFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 + powTime*0.1, 1 + audioSmoothed*10})
    end
end

function PostState_Level5.SpawnBubbleChamber(posX, posY, posZ, rTargetProp)
    local rExplosion = RadEffects.SpawnBubbleChamber(posX, posY, posZ, rTargetProp, PostState_Level5.BubbleChamberTargetCallback)
    
    local rTracerEffect = rExplosion.rTracerEffect
    local rTracerEffectProp = rTracerEffect:getProp()
    local rTracerEffectMaterial = rTracerEffectProp:getMaterial()
    rTracerEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rRippleEffect = rExplosion.rRippleEffect
    local rRippleEffectProp = rRippleEffect:getProp()
    local rRippleEffectMaterial = rRippleEffectProp:getMaterial()
    rRippleEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rSeekingEffect = rExplosion.rSeekingEffect
    local rSeekingEffectProp = rSeekingEffect:getProp()
    local rSeekingEffectMaterial = rSeekingEffectProp:getMaterial()
    rSeekingEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    return rExplosion
end

function PostState_Level5.BubbleChamberTargetCallback( tParticleEffect )
    local Explosion = {}
    if tParticleEffect ~= nil then
        if tParticleEffect.rTargetProp ~= nil then
            local fTargetX, fTargetY, fTargetZ = tParticleEffect.rTargetProp:getLoc()
            RadEffects:SetEffectTransform( fTargetX, fTargetY, fTargetZ, 0.0, 0.0, 0.0 )   
            RadEffects:SetEffectLayer( RadEffects.tRenderer.mGameLayer )
            local rEffect = RadEffects:CreateEffect( "Effects/Particles/Bursts/BlipFlare.particles", nil, nil, nil, Constants.ParticlePriorityLow )                 
            Explosion.rEffect = rEffect
            
            local rEffectProp = rEffect:getProp()
            local rEffectMaterial = rEffectProp:getMaterial()
            rEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
        end
    end
    
    return Explosion
end

return PostState_Level5