
local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local Wind = require('Wind')
local Score = require('Score')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local ParticleSystem = require("ParticleSystem")
local RadEffects = require('RadEffects')
local RadShapes = require('RadShapes')
local InputVisuals = require('InputVisuals')
local PostState_Level6 = {}

local uiBlitFilter = nil
local uiEdgeFilter = nil
local accumFilter = nil
local goodEffectColor = {0.2,0.2,0.2,1}
local gameBlitFilter = nil
local totalTime = 0.0
local audioSmoothed = 0.0
local flipHack = 1
local lastHeartbeat = 0.0
local currentHeartbeatDelta = 0.0

function PostState_Level6:Reset()
     uiBlitFilter = nil
     uiEdgeFilter = nil
     accumFilter = nil
     gameBlitFilter = nil
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- SSSSSSSS  TTTTTTTT  AAAAAAAA  RRRRRR    TTTTTTTT
-- SS           TT     AA    AA  RR    RR     TT
-- SSSSSSSS     TT     AAAAAAAA  RRRRRR       TT
--       SS     TT     AA    AA  RR   RR      TT
-- SSSSSSSS     TT     AA    AA  RR    RR     TT
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level6:Start()
    print("PostState_Level6: Start")
    Post:Reset()
    PostState_Level6:Reset()
    
    Post:SetUpdateCallback(PostState_Level6.UpdateStart)   
    
    ColorManager.BGColor            = {0, 0, 0}
    ColorManager.Color1             = {.6, .6, .6}
    ColorManager.Color2             = {.5, .5, .5} --score
    ColorManager.Color3             = {.2, .2, .2} --multiplier
    ColorManager.BadEffectColor     = {0, 1, 1, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetLayer(Renderer.mBackgroundLayer)
    Wind:SetColor(.1,.1,.1, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], 100, tMax[1], tMax[2], 100 )
    Wind.mWindDeck:SetPointWidth( 200.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, -1000.0, 400.0, .2) 
    Wind.mWindDeck:SetFrequency( 50.0 )
           
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer2 = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
  
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
    
    --Post:BlitFilter(Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlitFilter(Renderer.mGameBuffer, sceneBuffer, kBLEND_ALPHA)
    --Post:BlitFilter(Renderer.mUIBuffer, sceneBuffer, kBLEND_ALPHA,{1,1,1,1}, {1,1}, {0.0,0.0})
    --Post:EdgeFilter(Renderer.mUIBuffer, sceneBuffer, 1, kBLEND_ADD, {3,3,3,3})
    --Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.95,.95,.95,1}, {1.01,1.01}, {0.0,0.0})
    --Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    --Post:CombineFilter(sceneBuffer, accumBuffer, finalColorBuffer, kBLEND_ALPHA, {1,1,1,1}, {1,1}, {0,0}, {1,1,1,1}, {1,1}, {0,0} )
    --Post:InvertFilter(finalColorBuffer, scratchBuffer2, kBLEND_ALPHA)
    
    Post:Level6AFilter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, {1,1}, {1,1})
    accumFilter = Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.95,.95,.95,1}, {1.01,1.01}, {0.0,0.0})    
    Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    Post:Level6BFilter(sceneBuffer, accumBuffer, scratchBuffer2)
  
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer
    
    Post.rScreen:setInBuffer(scratchBuffer2)   

    RadEffects:SetEffectLayer( RadEffects.particleAccumLayer )
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/BlackDots.particles", true)
    
    RadShapes:SetPropLayer( Renderer.mBackgroundLayer )  
    
    --diamond location and size
    local fDiamondWidth = Constants.GameRingRadius * 0.15
    PostState_Level6.mNumDiamondShapes = 10
    PostState_Level6.mDiamondMoveSpeed = 100.0
    PostState_Level6.mDiamondRadii = {fDiamondWidth * 0.75, fDiamondWidth}
    
    -- 2x diamond shapes, some left, some right
    PostState_Level6.mDiamondShapeBatch = RadShapes:CreateNewShapeBatch(ShapeProp.DrawModeDiamond, PostState_Level6.mNumDiamondShapes * 2) 
    
    -- right shapes
    for i = 1, PostState_Level6.mNumDiamondShapes do
        PostState_Level6.mDiamondShapeBatch:SetBatchIndex(i)
        PostState_Level6.mDiamondShapeBatch:SetType( ShapeProp.DrawModeDiamond, 1 )
        PostState_Level6.mDiamondShapeBatch:SetColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1.0 )
        local xPos = math.random( 0.0, Constants.ValidDrawHalfWidth )
        local yMax = Constants.ValidDrawHalfHeight*0.3
        local yPos = math.random( -yMax, yMax )
        PostState_Level6.mDiamondShapeBatch:SetPosition( xPos, yPos, -20.0 )
        PostState_Level6.mDiamondShapeBatch:SetRadii( PostState_Level6.mDiamondRadii[1], PostState_Level6.mDiamondRadii[2] )
    end
    
    -- left shapes
    for i = PostState_Level6.mNumDiamondShapes, 2 * PostState_Level6.mNumDiamondShapes do
        PostState_Level6.mDiamondShapeBatch:SetBatchIndex(i)
        PostState_Level6.mDiamondShapeBatch:SetType( ShapeProp.DrawModeDiamond, 1 )
        PostState_Level6.mDiamondShapeBatch:SetColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1.0 )
        local xPos = math.random( -Constants.ValidDrawHalfWidth, 0.0 )
        local yMax = Constants.ValidDrawHalfHeight*0.3
        local yPos = math.random( -yMax, yMax )
        PostState_Level6.mDiamondShapeBatch:SetPosition( xPos, yPos, -20.0 )
        PostState_Level6.mDiamondShapeBatch:SetRadii( PostState_Level6.mDiamondRadii[1], PostState_Level6.mDiamondRadii[2] )
    end  

end

function PostState_Level6.UpdateStart()
    totalTime = totalTime  + Constants.FrameStep
    
    local yParam = Constants.ValidDrawHalfHeight * flipHack
    Wind.mWindDeck:SetDimensions( -Constants.ValidDrawHalfWidth * 2, yParam, 100, Constants.ValidDrawHalfWidth * 2, yParam, 100 )
    flipHack = flipHack * -1 
    
    local powTime = 1-math.pow(Constants.Heartbeat,3)  
    
    -- right shapes
    for i = 1, PostState_Level6.mNumDiamondShapes do   
        PostState_Level6.mDiamondShapeBatch:SetBatchIndex(i)
        local x, y, z = PostState_Level6.mDiamondShapeBatch:GetPosition()
        x = x - (PostState_Level6.mDiamondMoveSpeed * Constants.FrameStep) - i*0.2
        if x < 0 then
            local yMax = Constants.ValidDrawHalfHeight*0.3
            y = math.random( -yMax, yMax )
            x = Constants.ValidDrawHalfWidth
        end
        local diamondScale = x*2/Constants.ValidDrawHalfWidth
        if diamondScale > 1 then
            diamondScale = 1
        end
        PostState_Level6.mDiamondShapeBatch:SetPosition( x, y, z )
        PostState_Level6.mDiamondShapeBatch:SetRadii( PostState_Level6.mDiamondRadii[1] * diamondScale * powTime, PostState_Level6.mDiamondRadii[2] * diamondScale )     
    end 
    
    -- left shapes
    for i = PostState_Level6.mNumDiamondShapes, 2 * PostState_Level6.mNumDiamondShapes do   
        PostState_Level6.mDiamondShapeBatch:SetBatchIndex(i)
        local x, y, z = PostState_Level6.mDiamondShapeBatch:GetPosition()
        x = x + (PostState_Level6.mDiamondMoveSpeed * Constants.FrameStep) + i*0.2
        if x > 0 then
            local yMax = Constants.ValidDrawHalfHeight*0.3
            y = math.random( -yMax, yMax )
            x = -Constants.ValidDrawHalfWidth - 64
        end
        local diamondScale = -x*2/Constants.ValidDrawHalfWidth
        if diamondScale > 1 then
            diamondScale = 1
        end
        PostState_Level6.mDiamondShapeBatch:SetPosition( x, y, z ) 
        PostState_Level6.mDiamondShapeBatch:SetRadii( PostState_Level6.mDiamondRadii[1] * diamondScale * powTime, PostState_Level6.mDiamondRadii[2] * diamondScale )    
    end 
    
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB   UU    UU  II  LL       DDDDD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBBB  UU    UU  II  LL       DD   DD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBB   UUUUUUUU  II  LLLLLLl  DDDDD
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level6:Build()
    print("PostState_Level6: Build")
    Post:Reset()
    PostState_Level6:Reset()
    
    Post:SetUpdateCallback(PostState_Level6.UpdateBuild)   
    
    ColorManager.BGColor            = {0, 0, 0}
    ColorManager.Color1             = {.6, .6, .6}
    ColorManager.Color2             = {.5, .5, .5} --score
    ColorManager.Color3             = {.2, .2, .2} --multiplier
    ColorManager.BadEffectColor     = {0, 1, 1, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetLayer(Renderer.mBackgroundLayer)
    Wind:SetColor(.1,.1,.1, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], 100, tMax[1], tMax[2], 100 )
    Wind.mWindDeck:SetPointWidth( 200.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, -1000.0, 400.0, .2) 
    Wind.mWindDeck:SetFrequency( 50.0 )
       
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer2 = Post:GetUnusedBuffer()
    local lastFrameGridBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    lastFrameGridBuffer:setClearColor()
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
    
    --Post:BlitFilter(Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlitFilter(Renderer.mGameBuffer, sceneBuffer, kBLEND_ALPHA)
    --uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer, sceneBuffer, kBLEND_ALPHA,{1,1,1,1}, {1,1}, {0.0,0.0})
    --uiEdgeFilter = Post:EdgeFilter(Renderer.mUIBuffer, sceneBuffer, 1, kBLEND_ADD, {3,3,3,3})
    --Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.95,.95,.95,1}, {1.01,1.01}, {0.0,0.0})
    --Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    --Post:CombineFilter(sceneBuffer, accumBuffer, finalColorBuffer, kBLEND_ALPHA, {1,1,1,1}, {1,1}, {0,0}, {1,1,1,1}, {1,1}, {0,0} ) 
    --Post:InvertFilter(finalColorBuffer, scratchBuffer2, kBLEND_ALPHA)
    --Post:BlitFilter(lastFrameGridBuffer, finalColorBuffer, kBLEND_MULTIPLY)
    
    gameBlitFilter = Post:Level6AFilter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, {1,1}, {1,1})
    accumFilter = Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.95,.95,.95,1}, {1.01,1.01}, {0.0,0.0})    
    Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    Post:Level6BFilter(sceneBuffer, accumBuffer, scratchBuffer2)
    
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer

    Post.rScreen:setInBuffer(scratchBuffer2)
    
    RadEffects:SetEffectLayer( RadEffects.particleAccumLayer )
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/BlackDotsLong.particles", true)
    
    RadShapes:SetPropLayer( Renderer.mBackgroundLayer )
    
    --diamond location and size
    local fDiamondWidth = Constants.GameRingRadius * 0.15
    PostState_Level6.mNumDiamondShapes = 10
    PostState_Level6.mDiamondMoveSpeed = 200.0
    PostState_Level6.mDiamondRadii = {fDiamondWidth * 0.75, fDiamondWidth}
    
    -- 2x diamond shapes, some left, some right
    PostState_Level6.mDiamondShapeBatch = RadShapes:CreateNewShapeBatch(ShapeProp.DrawModeDiamond, PostState_Level6.mNumDiamondShapes * 2) 
    
    -- right shapes
    for i = 1, PostState_Level6.mNumDiamondShapes do
        PostState_Level6.mDiamondShapeBatch:SetBatchIndex(i)
        PostState_Level6.mDiamondShapeBatch:SetType( ShapeProp.DrawModeDiamond, 1 )
        PostState_Level6.mDiamondShapeBatch:SetColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1.0 )
        local xPos = math.random( 0.0, Constants.ValidDrawHalfWidth )
        local yMax = Constants.ValidDrawHalfHeight*0.3
        local yPos = math.random( -yMax, yMax )
        PostState_Level6.mDiamondShapeBatch:SetPosition( xPos, yPos, -20.0 )
        PostState_Level6.mDiamondShapeBatch:SetRadii( PostState_Level6.mDiamondRadii[1], PostState_Level6.mDiamondRadii[2] )
    end
    
    -- left index
    for i = PostState_Level6.mNumDiamondShapes, 2 * PostState_Level6.mNumDiamondShapes do
        PostState_Level6.mDiamondShapeBatch:SetBatchIndex(i)
        PostState_Level6.mDiamondShapeBatch:SetType( ShapeProp.DrawModeDiamond, 1 )
        PostState_Level6.mDiamondShapeBatch:SetColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1.0 )
        local xPos = math.random( -Constants.ValidDrawHalfWidth, 0.0 )
        local yMax = Constants.ValidDrawHalfHeight*0.3
        local yPos = math.random( -yMax, yMax )
        PostState_Level6.mDiamondShapeBatch:SetPosition( xPos, yPos, -20.0 )
        PostState_Level6.mDiamondShapeBatch:SetRadii( PostState_Level6.mDiamondRadii[1], PostState_Level6.mDiamondRadii[2] )
    end  

end

function PostState_Level6.UpdateBuild()
    local level_intensity = Score.mLevelIntensity
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)

    local yParam = Constants.ValidDrawHalfHeight * flipHack
    Wind.mWindDeck:SetDimensions( -Constants.ValidDrawHalfWidth * 2, yParam, 100, Constants.ValidDrawHalfWidth * 2, yParam, 100 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, -1000.0, 800.0, .3) 
    Wind.mWindDeck:SetFrequency( 100.0 )
    Wind.mWindDeck:SetPointWidth( 140.0 + Constants.Heartbeat * 20)
    flipHack = flipHack * -1
      
    local powTime = math.pow(Constants.Heartbeat,4)  
    if uiBlitFilter then      
        uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 + powTime*0.05, 1 + powTime*0.05})
        uiEdgeFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 + powTime*0.05, 1 + powTime*0.05})
    end
    
    if gameBlitFilter then   
        gameBlitFilter:setShaderValue( "g_vScale2", MOAIMaterial.VALUETYPE_VEC2, {1 + powTime*0.05, 1 + powTime*0.05})
    end
    
    local powTime = 1-math.pow(Constants.Heartbeat,3)  
    local moveSpeed = DFMath.lerp(100, 400, level_intensity)
    local powRotation = math.pow(level_intensity, 6)
    
    -- right shapes
    for i = 1, PostState_Level6.mNumDiamondShapes do       
        PostState_Level6.mDiamondShapeBatch:SetBatchIndex(i)
        local x, y, z = PostState_Level6.mDiamondShapeBatch:GetPosition()
        x = x - (moveSpeed * Constants.FrameStep) - i*0.2
        if x < 0 then
            local yMax = Constants.ValidDrawHalfHeight*0.3
            y = math.random( -yMax, yMax )
            x = Constants.ValidDrawHalfWidth
        end
        local diamondScale = x*2/Constants.ValidDrawHalfWidth
        if diamondScale > 1 then
            diamondScale = 1
        end
        PostState_Level6.mDiamondShapeBatch:SetPosition( x, y, z )
        PostState_Level6.mDiamondShapeBatch:SetRotation( 0, 0, -totalTime * (80 + i*5) *powRotation )
        PostState_Level6.mDiamondShapeBatch:SetRadii( PostState_Level6.mDiamondRadii[1] * diamondScale * powTime, PostState_Level6.mDiamondRadii[2] * diamondScale )     
    end 
    
    -- left shapes
    for i = PostState_Level6.mNumDiamondShapes, PostState_Level6.mNumDiamondShapes * 2 do       
        PostState_Level6.mDiamondShapeBatch:SetBatchIndex(i)
        local x, y, z = PostState_Level6.mDiamondShapeBatch:GetPosition()
        x = x + (moveSpeed * Constants.FrameStep) + i*0.2
        if x > 0 then
            local yMax = Constants.ValidDrawHalfHeight*0.3
            y = math.random( -yMax, yMax )
            x = -Constants.ValidDrawHalfWidth - 64
        end
        local diamondScale = -x*2/Constants.ValidDrawHalfWidth
        if diamondScale > 1 then
            diamondScale = 1
        end
        PostState_Level6.mDiamondShapeBatch:SetPosition( x, y, z ) 
        PostState_Level6.mDiamondShapeBatch:SetRotation( 0, 0, totalTime * (80 + i*5)*powRotation)
        PostState_Level6.mDiamondShapeBatch:SetRadii( PostState_Level6.mDiamondRadii[1] * diamondScale * powTime, PostState_Level6.mDiamondRadii[2] * diamondScale )    
    end 
    
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB    OOOOOO  NN    NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NNNN  NN  UU    UU  SS      
-- BBBBBBBB  OO    OO NN NN NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NN  NNNN  UU    UU        SS
-- BBBBBBB    OOOOOO  NN   NNN  UUUUUUUU  SSSSSSSS
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level6:Bonus()
    print("PostState_Level6: Bonus")
    Post:Reset()
    PostState_Level6:Reset()
    
    Post:SetUpdateCallback(PostState_Level6.UpdateBonus)   
    
    ColorManager.BGColor            = {0, 0, 0}
    ColorManager.Color1             = {.6, .6, .6}
    ColorManager.Color2             = {.5, .5, .5} --score
    ColorManager.Color3             = {.2, .2, .2} --multiplier
    ColorManager.BadEffectColor     = {0, 1, 1, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetLayer(Renderer.mBackgroundLayer)
    Wind:SetColor(.1,.1,.1, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], 100, tMax[1], tMax[2], 100 )
    Wind.mWindDeck:SetPointWidth( 200.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, -1000.0, 400.0, .2) 
    Wind.mWindDeck:SetFrequency( 50.0 )
        
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer2 = Post:GetUnusedBuffer()
    local lastFrameGridBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    lastFrameGridBuffer:setClearColor()
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)

    --Post:BlitFilter(Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ADD)
    --gameBlitFilter = Post:BlitFilter(Renderer.mGameBuffer, sceneBuffer, kBLEND_ALPHA)   
    --uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer, sceneBuffer, kBLEND_ALPHA,{1,1,1,1}, {1,1}, {0.0,0.0})
    --uiEdgeFilter = Post:EdgeFilter(Renderer.mUIBuffer, sceneBuffer, 1, kBLEND_ADD, {3,3,3,3})
    --accumFilter = Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.95,.95,.95,1}, {1.01,1.01}, {0.0,0.0})    
    --Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )   
    --Post:CombineFilter(sceneBuffer, accumBuffer, finalColorBuffer, kBLEND_ALPHA, {1,1,1,1}, {1,1}, {0,0}, {1,1,1,1}, {1,1}, {0,0} )  
    --Post:InvertFilter(finalColorBuffer, scratchBuffer2, kBLEND_ALPHA)
    --Post:BlitFilter(lastFrameGridBuffer, finalColorBuffer, kBLEND_MULTIPLY)  

    gameBlitFilter = Post:Level6AFilter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, {1,1}, {1,1})
    accumFilter = Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.95,.95,.95,1}, {1.01,1.01}, {0.0,0.0})    
    Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    Post:Level6BFilter(sceneBuffer, accumBuffer, scratchBuffer2)

    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer

    Post.rScreen:setInBuffer(scratchBuffer2)
    
    RadEffects:SetEffectLayer( RadEffects.particleAccumLayer )
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/BlackDotsLong.particles", true)
    
    -----------------------------------------------
    -- Setup Props
    -----------------------------------------------  
    local fRR = Constants.GameRingRadius
    PostState_Level6.mNumArcShapes = 8

    RadShapes:SetPropLayer( Renderer.mBackgroundLayer )
    PostState_Level6.mArcShape = RadShapes:CreateNewShape() 
    PostState_Level6.mArcShape:SetType( ShapeProp.DrawModeArc, 64 )
    PostState_Level6.mArcShape:SetColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
    PostState_Level6.mArcShape:SetPosition( 0.0, 0.0, -15.0 )
    PostState_Level6.mArcShape:SetArcRadians( 0, 2 * math.pi)
    PostState_Level6.mArcShape:SetRadii( fRR, fRR)
    
    --diamond location and size
    local fDiamondWidth = Constants.GameRingRadius * 0.15
    PostState_Level6.mNumDiamondShapes = 10
    PostState_Level6.mDiamondMoveSpeed = 400.0
    PostState_Level6.mDiamondRadii = {fDiamondWidth * 0.75, fDiamondWidth}
    
    -- 2x diamond shapes, some left, some right
    PostState_Level6.mDiamondShapeBatch = RadShapes:CreateNewShapeBatch(ShapeProp.DrawModeDiamond, PostState_Level6.mNumDiamondShapes * 2) 
    
    -- right shapes
    for i = 1, PostState_Level6.mNumDiamondShapes do
        PostState_Level6.mDiamondShapeBatch:SetBatchIndex(i)
        PostState_Level6.mDiamondShapeBatch:SetType( ShapeProp.DrawModeDiamond, 1 )
        PostState_Level6.mDiamondShapeBatch:SetColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1.0 )
        local xPos = math.random( 0.0, Constants.ValidDrawHalfWidth )
        local yMax = Constants.ValidDrawHalfHeight*0.3
        local yPos = math.random( -yMax, yMax )
        PostState_Level6.mDiamondShapeBatch:SetPosition( xPos, yPos, -20.0 )
        PostState_Level6.mDiamondShapeBatch:SetRadii( PostState_Level6.mDiamondRadii[1], PostState_Level6.mDiamondRadii[2] )
    end
    
    -- left shapes
    for i = PostState_Level6.mNumDiamondShapes, PostState_Level6.mNumDiamondShapes * 2 do
        PostState_Level6.mDiamondShapeBatch:SetBatchIndex(i)
        PostState_Level6.mDiamondShapeBatch:SetType( ShapeProp.DrawModeDiamond, 1 )
        PostState_Level6.mDiamondShapeBatch:SetColor( Constants.Color3[1], Constants.Color3[2], Constants.Color3[3], 1.0 )
        local xPos = math.random( -Constants.ValidDrawHalfWidth, 0.0 )
        local yMax = Constants.ValidDrawHalfHeight*0.3
        local yPos = math.random( -yMax, yMax )
        PostState_Level6.mDiamondShapeBatch:SetPosition( xPos, yPos, -20.0 )
        PostState_Level6.mDiamondShapeBatch:SetRadii( PostState_Level6.mDiamondRadii[1], PostState_Level6.mDiamondRadii[2] )
    end  

end

function PostState_Level6.UpdateBonus() 
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    local powTime = math.pow(Constants.Heartbeat,6)
    local redColorValue = 0.98 - powTime * 0.89
    local greenColorValue = 0.98 - powTime * 0.64
    local blueColorValue = 0.98 - powTime * 0.48
    
    local yParam = Constants.ValidDrawHalfHeight * flipHack
    Wind.mWindDeck:SetDimensions( -Constants.ValidDrawHalfWidth * 2, yParam, 100, Constants.ValidDrawHalfWidth * 2, yParam, 100 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, -1000.0, 1500.0, .4) 
    Wind.mWindDeck:SetFrequency( 100.0 )
    Wind.mWindDeck:SetPointWidth( 60.0 + Constants.Heartbeat * 40)
    flipHack = flipHack * -1
 
    local powTime = math.pow(Constants.Heartbeat,3)
    if uiBlitFilter then   
        uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 + powTime*0.1, 1 + powTime*0.1})
        uiEdgeFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 + powTime*0.1, 1 + powTime*0.1})
    end
    
    if gameBlitFilter then   
        gameBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 - powTime*0.1, 1 - powTime*0.1})
        gameBlitFilter:setShaderValue( "g_vScale2", MOAIMaterial.VALUETYPE_VEC2, {1 + powTime*0.1, 1 + powTime*0.1})
    end
    
    if accumFilter then
        accumFilter:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, {redColorValue, greenColorValue, blueColorValue, 1})
    end
    
    local fRR = Constants.GameRingRadius
    currentHeartbeatDelta = Constants.Heartbeat - lastHeartbeat
    local radiusScale = 1-Constants.Heartbeat
    if currentHeartbeatDelta < 0 then -- Heartbeat is increasing
        PostState_Level6.mArcShape:SetRadii( fRR*radiusScale*2 + fRR, fRR*2*(radiusScale+.05) + fRR)
        PostState_Level6.mArcShape:SetColor( .09, .34, .5, 1.0 )
    else
        PostState_Level6.mArcShape:SetRadii( fRR*10, fRR*10)
    end
    lastHeartbeat = Constants.Heartbeat 
    
    
    local powTime2 = 1-powTime  
    local moveSpeed = PostState_Level6.mDiamondMoveSpeed*powTime*3
    -- right shapes
    for i = 1, PostState_Level6.mNumDiamondShapes do
        PostState_Level6.mDiamondShapeBatch:SetBatchIndex(i)
        local x, y, z = PostState_Level6.mDiamondShapeBatch:GetPosition()
        x = x - (moveSpeed * Constants.FrameStep) - i*0.2
        if x < 0 then
            local yMax = Constants.ValidDrawHalfHeight*0.3
            y = math.random( -yMax, yMax )
            x = Constants.ValidDrawHalfWidth
        end
        local diamondScale = x*2/Constants.ValidDrawHalfWidth
        if diamondScale > 1 then
            diamondScale = 1
        end
        PostState_Level6.mDiamondShapeBatch:SetPosition( x, y, z )
        local diamondColor = {0,0,0}
        diamondColor[1] = DFMath.lerp(0, .045, diamondScale) * powTime2
        diamondColor[2] = DFMath.lerp(0, .17, diamondScale) * powTime2
        diamondColor[3] = DFMath.lerp(0, .25, diamondScale) * powTime2
        PostState_Level6.mDiamondShapeBatch:SetColor( diamondColor[1], diamondColor[2], diamondColor[3], 1.0 )
        PostState_Level6.mDiamondShapeBatch:SetRadii( PostState_Level6.mDiamondRadii[1] * diamondScale * powTime2, (PostState_Level6.mDiamondRadii[2]+PostState_Level6.mDiamondRadii[2]*powTime2) * diamondScale )     
    end 
    
    -- left shapes
    for i = PostState_Level6.mNumDiamondShapes, PostState_Level6.mNumDiamondShapes * 2 do       
        PostState_Level6.mDiamondShapeBatch:SetBatchIndex(i)
        local x, y, z = PostState_Level6.mDiamondShapeBatch:GetPosition()
        x = x + (moveSpeed * Constants.FrameStep) + i*0.2
        if x > 0 then
            local yMax = Constants.ValidDrawHalfHeight*0.3
            y = math.random( -yMax, yMax )
            x = -Constants.ValidDrawHalfWidth - 90
        end
        local diamondScale = -x*2/Constants.ValidDrawHalfWidth
        if diamondScale > 1 then
            diamondScale = 1
        end
        PostState_Level6.mDiamondShapeBatch:SetPosition( x, y, z ) 
        local diamondColor = {0,0,0}
        diamondColor[1] = DFMath.lerp(0, .045, diamondScale) * powTime2
        diamondColor[2] = DFMath.lerp(0, .17, diamondScale) * powTime2
        diamondColor[3] = DFMath.lerp(0, .25, diamondScale) * powTime2
        PostState_Level6.mDiamondShapeBatch:SetColor( diamondColor[1], diamondColor[2], diamondColor[3], 1.0 )
        PostState_Level6.mDiamondShapeBatch:SetRadii( PostState_Level6.mDiamondRadii[1] * diamondScale * powTime2, (PostState_Level6.mDiamondRadii[2]+PostState_Level6.mDiamondRadii[2]*powTime2) * diamondScale )    
    end  
end

function PostState_Level6.SpawnBubbleChamber(posX, posY, posZ, rTargetProp)
    local rExplosion = RadEffects.SpawnBubbleChamber(posX, posY, posZ, rTargetProp, PostState_Level6.BubbleChamberTargetCallback)
    
    local rTracerEffect = rExplosion.rTracerEffect
    local rTracerEffectProp = rTracerEffect:getProp()
    local rTracerEffectMaterial = rTracerEffectProp:getMaterial()
    rTracerEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rRippleEffect = rExplosion.rRippleEffect
    local rRippleEffectProp = rRippleEffect:getProp()
    local rRippleEffectMaterial = rRippleEffectProp:getMaterial()
    rRippleEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rSeekingEffect = rExplosion.rSeekingEffect
    local rSeekingEffectProp = rSeekingEffect:getProp()
    local rSeekingEffectMaterial = rSeekingEffectProp:getMaterial()
    rSeekingEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    return rExplosion
end

function PostState_Level6.BubbleChamberTargetCallback( tParticleEffect )
    local Explosion = {}
    if tParticleEffect ~= nil then
        if tParticleEffect.rTargetProp ~= nil then
            local fTargetX, fTargetY, fTargetZ = tParticleEffect.rTargetProp:getLoc()
            RadEffects:SetEffectTransform( fTargetX, fTargetY, fTargetZ, 0.0, 0.0, 0.0 )   
            RadEffects:SetEffectLayer( RadEffects.particleAccumLayer )
            local rEffect = RadEffects:CreateEffect( "Effects/Particles/Bursts/BlipFlare.particles", nil, nil, nil, Constants.ParticlePriorityLow )                 
            Explosion.rEffect = rEffect
            
            local rEffectProp = rEffect:getProp()
            local rEffectMaterial = rEffectProp:getMaterial()
            rEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
        end
    end
    
    return Explosion
end

return PostState_Level6