local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local Wind = require('Wind')
local Score = require('Score')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local ParticleSystem = require("ParticleSystem")
local RadEffects = require('RadEffects')
local InputVisuals = require('InputVisuals')
local RadShapes = require('RadShapes')
local PostState_Level7 = {}

local noiseFilter = nil
local uiBlitFilter = nil
local edgeFilter = nil
local goodEffectColor = {0.2,0.3,1,1}
local gameBlitFilter = nil
local totalTime = 0.0
local audioSmoothed = 0.0
local arcSize = 0.0


function PostState_Level7:Reset()
     noiseFilter = nil
     uiBlitFilter = nil
     edgeFilter = nil
     gameBlitFilter = nil
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- SSSSSSSS  TTTTTTTT  AAAAAAAA  RRRRRR    TTTTTTTT
-- SS           TT     AA    AA  RR    RR     TT
-- SSSSSSSS     TT     AAAAAAAA  RRRRRR       TT
--       SS     TT     AA    AA  RR   RR      TT
-- SSSSSSSS     TT     AA    AA  RR    RR     TT
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level7:Start()
    print("PostState_Level7: Start")
    Post:Reset()
    PostState_Level7:Reset()
    
    Post:SetUpdateCallback(PostState_Level7.UpdateStart)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {0, .3, 1}
    ColorManager.Color2             = {.1, .5, 1} --score
    ColorManager.Color3             = {.1, .1, .15} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()

    Wind:Enable(true)
    Wind:SetColor(.1,.5,1, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 0.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 400.0, .2) 
    Wind.mWindDeck:SetFrequency( 300.0 )
    
    ChargeField:Enable(true)
    ChargeField:SetColor( Constants.Color1[1]*0.5, Constants.Color1[2]*0.5, Constants.Color1[3]*0.5, 1.0 )
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 3.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 1.0, 4.0 )
        
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local accumBuffer = Post:GetUnusedQuarterBuffer()
    accumBuffer:setClearColor()
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)

--OLD & BUSTED
    --Post:CombineFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer)
    --Post:BlitFilter(Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlurFilter(sceneBuffer, finalColorBuffer, .003, kBLEND_ADD, {1,1,1,1})   
    --Post:BlitFilter(Renderer.mUIBuffer, finalColorBuffer, kBLEND_ADD)
    --Post:BlitFilter(Renderer.mGameBuffer, finalColorBuffer, kBLEND_ADD)  
    --Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.95,.95,.95,1}, {.995,.995}, {0.0,0.0})  
    --Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    --Post:BlitFilter(Renderer.mBackgroundBuffer, accumBuffer, kBLEND_ADD)
    --Post:CombineFilter(sceneBuffer, accumBuffer, scratchBuffer, kBLEND_ALPHA, {1,1,1,1}, {1,1}, {0,0}, {2,1,1,1}, {1,1}, {0,0} )      
    --Post:EdgeFilter(sceneBuffer, finalColorBuffer, 1, kBLEND_ADD, {0.75,0.8,1,1})
    --Post:BlitFilter(scratchBuffer, finalColorBuffer, kBLEND_ADD)
-------

--NEW HOTNESS
    Post:Level7Filter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, {1,1}, {1,1})
    Post:Level7BFilter(Renderer.mBackgroundBuffer, accumBuffer, scratchQuarterBuffer, {.96,.96,.96,1}, {.995,.995}, 1)
    Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} ) 
    Post:Level7CFilter(sceneBuffer, accumBuffer, finalColorBuffer, {0,0,0,1}, {1,1})
-------
    
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer

    Post.rScreen:setInBuffer(finalColorBuffer)
    
    --RadEffects:SetEffectLayerToParticleAccumLayer()
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/SoftGlow.particles", false)
    
    PostState_Level7.ChargeFieldOffset = Constants.ValidDrawWidth * 0.02
end

function PostState_Level7.UpdateStart()
    totalTime = totalTime  + Constants.FrameStep
    
    local randomfloat = math.random(-100,100)
    local xParam = math.cos(randomfloat) * 350
    local yParam = math.sin(randomfloat) * 350
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    Wind.mWindDeck:SetPointWidth( Constants.Heartbeat * 6.0 )
    
    local xOffset = math.cos(totalTime) * PostState_Level7.ChargeFieldOffset
    local yOffset = math.sin(totalTime) * PostState_Level7.ChargeFieldOffset
    ChargeField.mChargeFieldProp:setLoc( xOffset, yOffset, 0.0 )
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB   UU    UU  II  LL       DDDDD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBBB  UU    UU  II  LL       DD   DD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBB   UUUUUUUU  II  LLLLLLl  DDDDD
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level7:Build()
    print("PostState_Level7: Build")
    Post:Reset()
    PostState_Level7:Reset()
    
    Post:SetUpdateCallback(PostState_Level7.UpdateBuild)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {0, .3, 1}
    ColorManager.Color2             = {.1, .5, 1} --score
    ColorManager.Color3             = {.1, .1, .15} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(.1,.5,1, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 0.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 400.0, .2) 
    Wind.mWindDeck:SetFrequency( 300.0 )
    
    ChargeField:Enable(true)
    ChargeField:SetColor( Constants.Color1[1]*0.5, Constants.Color1[2]*0.5, Constants.Color1[3]*0.5, 1.0 )
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 3.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 1.0, 4.0 )
        
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local accumBuffer = Post:GetUnusedQuarterBuffer()
    accumBuffer:setClearColor()
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
    
    --Post:CombineFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer)
    --Post:BlitFilter(Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlurFilter(sceneBuffer, finalColorBuffer, .003, kBLEND_ADD, {1,1,1,1})     
    --Post:BlitFilter(Renderer.mUIBuffer, finalColorBuffer, kBLEND_ADD)
    --Post:BlitFilter(Renderer.mGameBuffer, finalColorBuffer, kBLEND_ADD)
    --Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.95,.95,.95,1}, {.99,.99}, {0.0,0.0})
    --Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    --Post:BlitFilter(Renderer.mBackgroundBuffer, accumBuffer, kBLEND_ADD)   
    --Post:CombineFilter(sceneBuffer, accumBuffer, finalColorBuffer, kBLEND_ALPHA, {1,1,1,1}, {1,1}, {0,0}, {2,1,1,1}, {1,1}, {0,0} )
    --noiseFilter = Post:TVNoiseFilter(scratchBuffer, finalColorBuffer, 0.0, kBLEND_ADD, {.01,.02,.2,1})  
    
    Post:Level7Filter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, {1,1}, {1,1})
    Post:Level7BFilter(Renderer.mBackgroundBuffer, accumBuffer, scratchQuarterBuffer, {.95,.95,.95,1}, {.99,.99}, 1)
    Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} ) 
    noiseFilter = Post:Level7CFilter(sceneBuffer, accumBuffer, finalColorBuffer, {.01,.02,.2,1}, {1,1})
    
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer

    Post.rScreen:setInBuffer(finalColorBuffer)
    
    --RadEffects:SetEffectLayerToParticleAccumLayer()
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/SoftGlow.particles", false)
    
    -----------------------------------------------
    -- Setup Props
    -----------------------------------------------  
    local fRR = Constants.GameRingRadius
    PostState_Level7.mDiamondMoveSpeed = 100.0
    PostState_Level7.mNumArcShapes = 8

    RadShapes:SetPropLayer( Renderer.mGameLayer )
    arcSize = math.pi/PostState_Level7.mNumArcShapes
    PostState_Level7.mArcShapes = {}  
    for i = 1, PostState_Level7.mNumArcShapes do
        PostState_Level7.mArcShapes[i] = RadShapes:CreateNewShape() 
        PostState_Level7.mArcShapes[i]:SetType( ShapeProp.DrawModeArc, 64 )
        PostState_Level7.mArcShapes[i]:SetColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
        PostState_Level7.mArcShapes[i]:SetPosition( 0.0, 0.0, -15.0 )
        PostState_Level7.mArcShapes[i]:SetArcRadians( 2 * i * arcSize + arcSize*0.5, 2 * i * arcSize + arcSize + arcSize*0.5)
        PostState_Level7.mArcShapes[i]:SetRadii( fRR * 1.2, fRR * 1.3 )
    end  
end


function PostState_Level7.UpdateBuild()
    local level_intensity = Score.mLevelIntensity
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)

    local randomfloat = math.random(-100,100)
    local xParam = math.cos(randomfloat) * 350
    local yParam = math.sin(randomfloat) * 350
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    Wind.mWindDeck:SetPointWidth( 3.0 + Constants.Heartbeat * 6.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 700.0, .3) 
        
    if noiseFilter then
        noiseFilter:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, totalTime * 100.3)
        noiseFilter:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, {.01*level_intensity,.02*level_intensity,.2*level_intensity,1})
    end
  
    local fRR = Constants.GameRingRadius
    local intensityMirrored = level_intensity
    if intensityMirrored > 0.5 then
        intensityMirrored = 1-intensityMirrored
    end
    for i = 1, PostState_Level7.mNumArcShapes do
        PostState_Level7.mArcShapes[i]:SetArcRadians( 2*i*arcSize + arcSize*0.5, 2*i*arcSize + arcSize + arcSize*0.5)
        PostState_Level7.mArcShapes[i]:SetRadii( fRR *1.02, fRR * (1.02 + intensityMirrored))
    end  
    
    local xOffset = math.cos(totalTime) * PostState_Level7.ChargeFieldOffset
    local yOffset = math.sin(totalTime) * PostState_Level7.ChargeFieldOffset
    ChargeField.mChargeFieldProp:setLoc( xOffset, yOffset, 0.0 )    
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB    OOOOOO  NN    NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NNNN  NN  UU    UU  SS      
-- BBBBBBBB  OO    OO NN NN NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NN  NNNN  UU    UU        SS
-- BBBBBBB    OOOOOO  NN   NNN  UUUUUUUU  SSSSSSSS
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level7:Bonus()
    print("PostState_Level7: Bonus")
    Post:Reset()
    PostState_Level7:Reset()
    
    Post:SetUpdateCallback(PostState_Level7.UpdateBonus)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {0, .3, 1}
    ColorManager.Color2             = {.1, .5, 1} --score
    ColorManager.Color3             = {.1, .1, .15} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(true)
    Wind:SetColor(.1,.5,1, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 0.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 400.0, .2) 
    Wind.mWindDeck:SetFrequency( 300.0 )
    
    ChargeField:Enable(true)
    ChargeField:SetColor( Constants.Color1[1]*0.5, Constants.Color1[2]*0.5, Constants.Color1[3]*0.5, 1.0 )
    ChargeField.mChargeFieldDeck:SetRestoreScalar( 3.0 )
    ChargeField.mChargeFieldDeck:SetPointWidth( 1.0, 4.0 )
        
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local accumBuffer = Post:GetUnusedQuarterBuffer()
    accumBuffer:setClearColor()
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
   
    --gameBlitFilter = Post:BlitFilter(Renderer.mGameBuffer, sceneBuffer, kBLEND_ADD)
    --uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlitFilter(Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlurFilter(sceneBuffer, finalColorBuffer, .003, kBLEND_ADD, {2,2,2,1})       
    --Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.97,.97,.97,1}, {.995,.995}, {0.0,0.0})
    --Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    --Post:BlitFilter(Renderer.mBackgroundBuffer, accumBuffer, kBLEND_ADD)   
    --Post:CombineFilter(sceneBuffer, accumBuffer, finalColorBuffer, kBLEND_ALPHA, {1,1,1,1}, {1,1}, {0,0}, {2,1,1,1}, {1,1}, {0,0} )
    --noiseFilter = Post:TVNoiseFilter(scratchBuffer, finalColorBuffer, 0.0, kBLEND_ADD, {.01,.02,.2,1})  
    --edgeFilter = Post:EdgeFilter(Renderer.mUIBuffer, finalColorBuffer, 1, kBLEND_ADD, {2,2,2,2})
    
    gameBlitFilter = Post:Level7Filter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, {1,1}, {1,1})
    Post:Level7BFilter(Renderer.mBackgroundBuffer, accumBuffer, scratchQuarterBuffer, {.97,.97,.97,1}, {.995,.995}, 1)
    Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} ) 
    noiseFilter = Post:Level7CFilter(sceneBuffer, accumBuffer, finalColorBuffer, {.01,.02,.2,1}, {1,1}) 

    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer
    
    Post.rScreen:setInBuffer(finalColorBuffer)
    
    --RadEffects:SetEffectLayerToParticleAccumLayer()
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/SoftGlow.particles", false)
    
    -----------------------------------------------
    -- Setup Props
    -----------------------------------------------  
    local fRR = Constants.GameRingRadius
    PostState_Level7.mDiamondMoveSpeed = 100.0
    PostState_Level7.mNumArcShapes = 8

    RadShapes:SetPropLayer( Renderer.mGameLayer )
    arcSize = math.pi/PostState_Level7.mNumArcShapes
    PostState_Level7.mArcShapes = {}  
    for i = 1, PostState_Level7.mNumArcShapes do
        PostState_Level7.mArcShapes[i] = RadShapes:CreateNewShape() 
        PostState_Level7.mArcShapes[i]:SetType( ShapeProp.DrawModeArc, 64 )
        PostState_Level7.mArcShapes[i]:SetColor( Constants.Color1[1], Constants.Color1[2], Constants.Color1[3], 1.0 )
        PostState_Level7.mArcShapes[i]:SetPosition( 0.0, 0.0, -15.0 )
        PostState_Level7.mArcShapes[i]:SetArcRadians( 2 * i * arcSize + arcSize*0.5, 2 * i * arcSize + arcSize + arcSize*0.5)
        PostState_Level7.mArcShapes[i]:SetRadii( fRR * 1.2, fRR * 1.3 )
    end  
end


function PostState_Level7.UpdateBonus()
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    local randomfloat = math.random(-100,100)
    local xParam = math.cos(randomfloat) * 350
    local yParam = math.sin(randomfloat) * 350
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )
    Wind.mWindDeck:SetPointWidth( 6.0 + Constants.Heartbeat * 12.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 1000.0, 1000.0, .4) 
                
    if noiseFilter then
        noiseFilter:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, totalTime * 100.3)
    end
    
    local powTime = math.pow(Constants.Heartbeat,3)
    
    if gameBlitFilter then   
        gameBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 - powTime*0.1, 1 - powTime*0.1})
        gameBlitFilter:setShaderValue( "g_vScale2", MOAIMaterial.VALUETYPE_VEC2, {1 + powTime*0.1, 1 + powTime*0.1})
    end
    
    if uiBlitFilter then   
        uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 + powTime*0.1, 1 + powTime*0.1})
    end
    
    if edgeFilter then   
        edgeFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 + powTime*0.1, 1 + powTime*0.1})
    end
    
    local fRR = Constants.GameRingRadius
    local arcOffset = powTime * 0.15
    for i = 1, PostState_Level7.mNumArcShapes do
        PostState_Level7.mArcShapes[i]:SetArcRadians( 2*i*arcSize + arcSize*0.5 + arcOffset, 2*i*arcSize + arcSize + arcSize*0.5 - arcOffset)
        PostState_Level7.mArcShapes[i]:SetRadii( fRR *1.02, fRR * (1.1 +powTime * 0.8))
    end  
    
    local xOffset = math.cos(totalTime) * PostState_Level7.ChargeFieldOffset
    local yOffset = math.sin(totalTime) * PostState_Level7.ChargeFieldOffset
    ChargeField.mChargeFieldProp:setLoc( xOffset, yOffset, 0.0 )        
end

function PostState_Level7.SpawnBubbleChamber(posX, posY, posZ, rTargetProp)
    local rExplosion = RadEffects.SpawnBubbleChamber(posX, posY, posZ, rTargetProp)
    
    local rTracerEffect = rExplosion.rTracerEffect
    local rTracerEffectProp = rTracerEffect:getProp()
    local rTracerEffectMaterial = rTracerEffectProp:getMaterial()
    rTracerEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rRippleEffect = rExplosion.rRippleEffect
    local rRippleEffectProp = rRippleEffect:getProp()
    local rRippleEffectMaterial = rRippleEffectProp:getMaterial()
    rRippleEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rSeekingEffect = rExplosion.rSeekingEffect
    local rSeekingEffectProp = rSeekingEffect:getProp()
    local rSeekingEffectMaterial = rSeekingEffectProp:getMaterial()
    rSeekingEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    return rExplosion
end

return PostState_Level7