local Post = require('postfx.Post')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local Constants = require('Constants')
local ChargeField= require('ChargeField')
local Grid = require('Grid')
local Wind = require('Wind')
local Score = require('Score')
local ColorManager= require('ColorManager')
local DFMath = require('DFCommon.Math')
local ParticleSystem = require("ParticleSystem")
local RadEffects = require('RadEffects')
local InputVisuals = require('InputVisuals')
local RadShapes = require('RadShapes')
local PostState_Level8 = {}

local noiseFilter = nil
local uiBlitFilter = nil
local edgeFilter = nil
local goodEffectColor = {0.1,0.5,0.9,1}
local gameBlitFilter = nil
local totalTime = 0.0
local audioSmoothed = 0.0
local lastHeartbeat = 0.0
local currentHeartbeatDelta = 0.0

function PostState_Level8:Reset()
     noiseFilter = nil
     uiBlitFilter = nil
     edgeFilter = nil
     gameBlitFilter = nil
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- SSSSSSSS  TTTTTTTT  AAAAAAAA  RRRRRR    TTTTTTTT
-- SS           TT     AA    AA  RR    RR     TT
-- SSSSSSSS     TT     AAAAAAAA  RRRRRR       TT
--       SS     TT     AA    AA  RR   RR      TT
-- SSSSSSSS     TT     AA    AA  RR    RR     TT
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level8:Start()
    print("PostState_Level8: Start")
    Post:Reset()
    PostState_Level8:Reset()
    
    Post:SetUpdateCallback(PostState_Level8.UpdateStart)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.8, .4, .1}
    ColorManager.Color2             = {.8, .8, .1} --score
    ColorManager.Color3             = {0.1,0.5,0.9} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()

    Wind:Enable(true)
    Wind:SetColor(.4,.2,.05, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -10, tMax[1], tMax[2], -10 )
    Wind.mWindDeck:SetPointWidth( 3.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 500.0, 200.0, .1) 
    Wind.mWindDeck:SetFrequency( 100.0 )
    
    Grid:Enable(true)
    Grid.mGridDeck:SetPointWidth( 3 )
    Grid.mGridDeck:SetSpringK( 2.0 )
    Grid.mGridDeck:SetSpringDampen( 0.002 )
    Grid:SetColor(0,0.05,0.1,1)
        
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
    
    --Post:CombineFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer)
    --Post:BlitFilter(Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlurFilter(sceneBuffer, finalColorBuffer, .003, kBLEND_ADD, {1,1,1,1})      
    --Post:BlitFilter(Renderer.mUIBuffer, finalColorBuffer, kBLEND_ADD)
    --Post:BlitFilter(Renderer.mGameBuffer, finalColorBuffer, kBLEND_ADD) 
    --Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.93,.93,.93,1}, {.99,.999}, {0.0,0.0})
    --Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    --Post:CombineFilter(sceneBuffer, accumBuffer, scratchBuffer, kBLEND_ALPHA, {1,1,1,1}, {1,1}, {0,0}, {2,1,1,1}, {1,1}, {0,0} )    
    --Post:EdgeFilter(scratchBuffer, finalColorBuffer, 1, kBLEND_ADD, {0.75,0.8,1,1})
    --Post:BlitFilter(scratchBuffer, finalColorBuffer, kBLEND_ADD)
    
    Post:Level7Filter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, {1,1}, {1,1})
    Post:Level7BFilter(Renderer.mBackgroundBuffer, accumBuffer, scratchQuarterBuffer, {.93,.93,.93,1}, {.99,.999}, 0)
    Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} ) 
    Post:Level7CFilter(sceneBuffer, accumBuffer, finalColorBuffer, {0,0,0,1}, {1,1})
    
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer
    
    RadEffects:SetEffectLayerToParticleAccumLayer()
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/Flames.particles", false)

    Post.rScreen:setInBuffer(finalColorBuffer)
end

function PostState_Level8.UpdateStart()
    local level_intensity = Score.mLevelIntensity
    totalTime = totalTime  + Constants.FrameStep
end

------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB   UU    UU  II  LL       DDDDD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBBB  UU    UU  II  LL       DD   DD
-- BB    BB  UU    UU  II  LL       DD   DD
-- BBBBBBB   UUUUUUUU  II  LLLLLLl  DDDDD
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level8:Build()
    print("PostState_Level8: Build")
    Post:Reset()
    PostState_Level8:Reset()
    
    Post:SetUpdateCallback(PostState_Level8.UpdateBuild)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.8, .4, .1}
    ColorManager.Color2             = {.8, .8, .1} --score
    ColorManager.Color3             = {0.1,0.5,0.9} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
    
    Wind:Enable(false)
    Wind:SetColor(.4,.2,.05, 1)
    local tMin = {-Constants.ValidDrawHalfWidth, -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    local tMax = {Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight}
    Wind.mWindDeck:SetDimensions( tMin[1], tMin[2], -100, tMax[1], tMax[2], -100 )
    Wind.mWindDeck:SetPointWidth( 10.0 )
    Wind.mWindDeck:SetDirection( 0.0, 0.0, 500.0, 200.0, .1) 
    Wind.mWindDeck:SetFrequency( 200.0 )
    
    Grid:Enable(true)
    Grid.mGridDeck:SetPointWidth( 3 )
    Grid.mGridDeck:SetSpringK( 1.0 )
    Grid.mGridDeck:SetSpringDampen( 0.002 )
    Grid:SetColor(0,0.05,0.1,1)
        
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)
  
    --Post:CombineFilter(Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer)   
    --Post:BlitFilter(Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlurFilter(sceneBuffer, finalColorBuffer, .003, kBLEND_ADD, {1,1,1,1})    
    --uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer, finalColorBuffer, kBLEND_ADD)
    --Post:BlitFilter(Renderer.mGameBuffer, finalColorBuffer, kBLEND_ADD)
    --Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.93,.93,.93,1}, {.98,.999}, {0.0,0.0})
    --Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    --Post:CombineFilter(sceneBuffer, accumBuffer, finalColorBuffer, kBLEND_ADD, {1,1,1,1}, {1,1}, {0,0}, {2,1,1,1}, {1,1}, {0,0} )
    --noiseFilter = Post:TVNoiseFilter(scratchBuffer, finalColorBuffer, 0.0, kBLEND_ADD, {.1,.05,0,0.3})
    
    Post:Level7Filter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, {1,1}, {1,1})
    Post:Level7BFilter(Renderer.mBackgroundBuffer, accumBuffer, scratchQuarterBuffer, {.93,.93,.93,1}, {.98,.999}, 0)
    Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} ) 
    noiseFilter = Post:Level7CFilter(sceneBuffer, accumBuffer, finalColorBuffer, {.1,.05,0,0.3}, {1,1})
  
    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer
    
    RadEffects:SetEffectLayerToParticleAccumLayer()
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/Flames.particles", false)

    Post.rScreen:setInBuffer(finalColorBuffer)
    
    RadShapes:SetPropLayer( Renderer.mGameLayer )
    
    --diamond location and size
    local fDiamondWidth = Constants.GameRingRadius * 0.07
    PostState_Level8.mNumDiamondShapes = 15
    
    PostState_Level8.mDiamondShapeBatch = RadShapes:CreateNewShapeBatch(ShapeProp.DrawModeDiamond, PostState_Level8.mNumDiamondShapes) 

    for i = 1, PostState_Level8.mNumDiamondShapes do
        local fadeToBlack = 1-(i/PostState_Level8.mNumDiamondShapes)
        PostState_Level8.mDiamondShapeBatch:SetBatchIndex(i)
        PostState_Level8.mDiamondShapeBatch:SetType( ShapeProp.DrawModeDiamond, 1 )
        PostState_Level8.mDiamondShapeBatch:SetColor(Constants.Color1[1]*fadeToBlack, Constants.Color1[2]*fadeToBlack, Constants.Color1[3]*fadeToBlack, 1.0)
        PostState_Level8.mDiamondShapeBatch:SetPosition( 0, 0, -20.0)
        local startRadius = Constants.GameRingRadius*1.5 + (i-1)*fDiamondWidth*2
        local endRadius   = Constants.GameRingRadius*1.5 + (i-1)*fDiamondWidth*2 + fDiamondWidth
        PostState_Level8.mDiamondShapeBatch:SetRadii( startRadius, endRadius )
    end
end


function PostState_Level8.UpdateBuild()
    local level_intensity = Score.mLevelIntensity

    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)

    Wind.mWindDeck:SetFrequency( DFMath.lerp(200, 0, level_intensity) )
    
    Grid:SetColor(ColorManager.Color1[1]*0.1 *(1-level_intensity),ColorManager.Color1[2]*0.1 *(1-level_intensity),ColorManager.Color1[3]*0.1 *(1-level_intensity),1)
    Grid:SetColor(0,0.05*(1-level_intensity),0.1*(1-level_intensity),1)
        
    if noiseFilter then
        noiseFilter:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, totalTime * 100.3)
        noiseFilter:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, {0.1*level_intensity,.05*level_intensity,0,.5})
    end
      
    local scaleFactor = 1+audioSmoothed*100
    if uiBlitFilter then   
        uiBlitFilter:setShaderValue( "g_vScale2", MOAIMaterial.VALUETYPE_VEC2, {scaleFactor, scaleFactor})
    end
    
    local fDiamondWidth = Constants.GameRingRadius * 0.07
    local fDiamondScale = (1-level_intensity)*2-0.5
    if fDiamondScale < 0 then
        fDiamondScale = 0
    end

    for i = 1, PostState_Level8.mNumDiamondShapes do
        PostState_Level8.mDiamondShapeBatch:SetBatchIndex(i)
        local rotationAmount = math.sin((totalTime+ (PostState_Level8.mNumDiamondShapes-i)*0.25))*16
        PostState_Level8.mDiamondShapeBatch:SetRotation(0,0,rotationAmount)
        local startRadius = fDiamondScale*Constants.GameRingRadius*3 + Constants.GameRingRadius*1.5 + (i-1)*fDiamondWidth*2
        local endRadius   = fDiamondScale*Constants.GameRingRadius*3 + Constants.GameRingRadius*1.5 + (i-1)*fDiamondWidth*2 + fDiamondWidth
        PostState_Level8.mDiamondShapeBatch:SetRadii( startRadius, endRadius )
    end 
end


------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------
-- BBBBBBB    OOOOOO  NN    NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NNNN  NN  UU    UU  SS      
-- BBBBBBBB  OO    OO NN NN NN  UU    UU  SSSSSSSS
-- BB    BB  OO    OO NN  NNNN  UU    UU        SS
-- BBBBBBB    OOOOOO  NN   NNN  UUUUUUUU  SSSSSSSS
------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------

function PostState_Level8:Bonus()
    print("PostState_Level8: Bonus")
    Post:Reset()
    PostState_Level8:Reset()
    
    Post:SetUpdateCallback(PostState_Level8.UpdateBonus)   
    
    ColorManager.BGColor            = {0,0,0}
    ColorManager.Color1             = {.8, .4, .1}
    ColorManager.Color2             = {.8, .8, .1} --score
    ColorManager.Color3             = {0.1,0.5,0.9} --multiplier
    ColorManager.BadEffectColor     = {1, 0, 0, 1} 
    ColorManager:SetColor()
     
    local finalColorBuffer = Post:GetUnusedBuffer()
    local sceneBuffer = Post:GetUnusedBuffer()
    local scratchBuffer = Post:GetUnusedBuffer()
    local scratchQuarterBuffer = Post:GetUnusedQuarterBuffer()  
    local accumBuffer = Post:GetUnusedBuffer()
    accumBuffer:setClearColor()
    
    -- create a layer to draw particles into the accumulation buffer
    local particleLayer = Post:GetUnusedScratchLayer()
    particleLayer:setOutBuffer(accumBuffer)

    --gameBlitFilter = Post:BlitFilter(Renderer.mGameBuffer, sceneBuffer, kBLEND_ADD)
    --uiBlitFilter = Post:BlitFilter(Renderer.mUIBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlitFilter(Renderer.mBackgroundBuffer, sceneBuffer, kBLEND_ADD)
    --Post:BlurFilter(sceneBuffer, finalColorBuffer, .003, kBLEND_ADD, {2,2,2,1})        
    --Post:AccumFilter(Post.rBlackTexture, accumBuffer, scratchQuarterBuffer, {.93,.93,.93,1}, {.97,.999}, {0.0,0.0})
    --Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} )
    --Post:CombineFilter(sceneBuffer, accumBuffer, finalColorBuffer, kBLEND_ALPHA, {1,1,1,1}, {1,1}, {0,0}, {2,1,1,1}, {1,1}, {0,0} )
    --noiseFilter = Post:TVNoiseFilter(scratchBuffer, finalColorBuffer, 0.0, kBLEND_ADD, {0,.05,0.1,.3})  
    --edgeFilter = Post:EdgeFilter(Renderer.mUIBuffer, finalColorBuffer, 1, kBLEND_ADD, {2,2,2,2})
    
    gameBlitFilter = Post:Level7Filter(Renderer.mBackgroundBuffer, Renderer.mGameBuffer, Renderer.mUIBuffer, sceneBuffer, {1,1}, {1,1})
    Post:Level7BFilter(Renderer.mBackgroundBuffer, accumBuffer, scratchQuarterBuffer, {.93,.93,.93,1}, {.97,.999}, 0)
    Post:CombineFilter(sceneBuffer, scratchQuarterBuffer, accumBuffer, kBLEND_ALPHA, {.1,.1,.1,1} ) 
    noiseFilter = Post:Level7CFilter(sceneBuffer, accumBuffer, finalColorBuffer, {.1,.05,0,0.3}, {1,1})

    RadEffects:SetGoodExplosionCallback(self.SpawnBubbleChamber)
    RadEffects.particleAccumLayer = particleLayer
    
    RadEffects:SetEffectLayerToParticleAccumLayer()
    InputVisuals:SetFingerEffects("Effects/Particles/Fingers/Flames.particles", false)
    
    Post.rScreen:setInBuffer(finalColorBuffer)
    
    RadShapes:SetPropLayer( Renderer.mGameLayer )
    
    PostState_Level8.mMainDiamond = RadShapes:CreateNewShape() 
    PostState_Level8.mMainDiamond:SetType( ShapeProp.DrawModeDiamond, 1 )
    PostState_Level8.mMainDiamond:SetColor( Constants.Color1[1]*0.2, Constants.Color1[2]*0.2, Constants.Color1[3]*0.2, 1.0 )
    PostState_Level8.mMainDiamond:SetPosition( 0, 0, -20 )
    PostState_Level8.mMainDiamond:SetRadii( Constants.GameRingRadius * 1.6, Constants.GameRingRadius * 1.9 )
    
    --diamond location and size
    local fDiamondWidth = Constants.GameRingRadius * 0.3
    PostState_Level8.mNumDiamondShapes = 15
    PostState_Level8.mDiamondShapeBatch = RadShapes:CreateNewShapeBatch(ShapeProp.DrawModeDiamond, PostState_Level8.mNumDiamondShapes) 
    for i = 1, PostState_Level8.mNumDiamondShapes do
        PostState_Level8.mDiamondShapeBatch:SetBatchIndex(i)
        PostState_Level8.mDiamondShapeBatch:SetType( ShapeProp.DrawModeDiamond, 1 )
        PostState_Level8.mDiamondShapeBatch:SetColor( Constants.Color2[1], Constants.Color2[2], Constants.Color2[3], 1.0 )
        local xPos = math.random( -Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfWidth )
        if xPos < Constants.GameRingRadius then
            if xPos > -Constants.GameRingRadius then
                xPos = math.random( Constants.GameRingRadius, Constants.ValidDrawHalfWidth )
            end
        end
        local yPos = math.random( -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight )
        PostState_Level8.mDiamondShapeBatch:SetPosition( xPos,yPos,-20 )
        PostState_Level8.mDiamondShapeBatch:SetRadii( 0, fDiamondWidth )
    end
end


function PostState_Level8.UpdateBonus()
    totalTime = totalTime  + Constants.FrameStep
    audioSmoothed = DFMath.lerp(audioSmoothed, Constants.AudioSpectrumAverage * 1.0, .3)
    
    local randomfloat = math.random(-100,100)
    local xParam = math.cos(randomfloat) * 350
    local yParam = math.sin(randomfloat) * 350
    Wind.mWindDeck:SetDimensions( xParam, yParam, -200, xParam, yParam, -200 )        
        
    if noiseFilter then
        noiseFilter:setShaderValue( "g_fTime", MOAIMaterial.VALUETYPE_FLOAT, totalTime * 100.3)
    end
 
    local powTime = math.pow(Constants.Heartbeat,3) 
    local inversePowTime = 1-math.pow(Constants.Heartbeat,3) 
    if gameBlitFilter then   
        gameBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 - powTime*0.1, 1 - powTime*0.1})
        gameBlitFilter:setShaderValue( "g_vScale2", MOAIMaterial.VALUETYPE_VEC2, {1 + inversePowTime*0.1, 1+audioSmoothed*10})
    end
    
    if uiBlitFilter then   
        uiBlitFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 + inversePowTime*0.1, 1+audioSmoothed*10})
    end
    
    if edgeFilter then   
        edgeFilter:setShaderValue( "g_vScale", MOAIMaterial.VALUETYPE_VEC2, {1 + inversePowTime*0.1, 1+audioSmoothed*10})
    end
        
    local fDiamondWidth = Constants.GameRingRadius * 0.2
    local fDiamondTime = 1-math.pow(1-Constants.Heartbeat,3) 
    fDiamondTime = fDiamondTime*0.5 + 0.5
    currentHeartbeatDelta = Constants.Heartbeat - lastHeartbeat
    for i = 1, PostState_Level8.mNumDiamondShapes do
        PostState_Level8.mDiamondShapeBatch:SetBatchIndex(i)
        if currentHeartbeatDelta < 0 then -- Heartbeat is increasing
            PostState_Level8.mDiamondShapeBatch:SetRadii( fDiamondWidth*fDiamondTime*0.5, fDiamondWidth*fDiamondTime )
            local x,y,z = PostState_Level8.mDiamondShapeBatch:GetPosition()
            x = x + (x * (1-powTime) * Constants.FrameStep)
            PostState_Level8.mDiamondShapeBatch:SetPosition( x,y,-20 )
        else
            PostState_Level8.mDiamondShapeBatch:SetRadii( 0, 0 )
            local xPos = math.random( -Constants.ValidDrawHalfWidth, Constants.ValidDrawHalfWidth )
            if xPos < Constants.GameRingRadius then
                if xPos > -Constants.GameRingRadius then
                    xPos = math.random( Constants.GameRingRadius, Constants.ValidDrawHalfWidth )
                end
            end
            local yPos = math.random( -Constants.ValidDrawHalfHeight, Constants.ValidDrawHalfHeight )
            PostState_Level8.mDiamondShapeBatch:SetPosition( xPos,yPos,-20 )
        end
    end
    lastHeartbeat = Constants.Heartbeat 
    
    local rotateSpeed = powTime*200
    local x, y, z = PostState_Level8.mMainDiamond:GetRotation()
        z = z - (rotateSpeed * Constants.FrameStep)
    PostState_Level8.mMainDiamond:SetRotation( 0, 0, z )
end

function PostState_Level8.SpawnBubbleChamber(posX, posY, posZ, rTargetProp)
    local rExplosion = RadEffects.SpawnBubbleChamber(posX, posY, posZ, rTargetProp, PostState_Level8.BubbleChamberTargetCallback)
    
    local rTracerEffect = rExplosion.rTracerEffect
    local rTracerEffectProp = rTracerEffect:getProp()
    local rTracerEffectMaterial = rTracerEffectProp:getMaterial()
    rTracerEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rRippleEffect = rExplosion.rRippleEffect
    local rRippleEffectProp = rRippleEffect:getProp()
    local rRippleEffectMaterial = rRippleEffectProp:getMaterial()
    rRippleEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    local rSeekingEffect = rExplosion.rSeekingEffect
    local rSeekingEffectProp = rSeekingEffect:getProp()
    local rSeekingEffectMaterial = rSeekingEffectProp:getMaterial()
    rSeekingEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
    
    return rExplosion
end

function PostState_Level8.BubbleChamberTargetCallback( tParticleEffect )
    local Explosion = {}
    if tParticleEffect ~= nil then
        if tParticleEffect.rTargetProp ~= nil then
            local fTargetX, fTargetY, fTargetZ = tParticleEffect.rTargetProp:getLoc()
            RadEffects:SetEffectTransform( fTargetX, fTargetY, fTargetZ, 0.0, 0.0, 0.0 )   
            RadEffects:SetEffectLayer( RadEffects.tRenderer.mGameLayer )
            local rEffect = RadEffects:CreateEffect( "Effects/Particles/Bursts/BlipFlare.particles", nil, nil, nil, Constants.ParticlePriorityLow )                 
            Explosion.rEffect = rEffect
            
            local rEffectProp = rEffect:getProp()
            local rEffectMaterial = rEffectProp:getMaterial()
            rEffectMaterial:setShaderValue( "g_vColor", MOAIMaterial.VALUETYPE_VEC4, goodEffectColor)
        end
    end
    
    return Explosion
end

return PostState_Level8
